/**
 * Represents the options passed to the `build()` method
 */
export type BuildOptions = {
    /**
     * Default true
     */
    validate?: boolean;
    /**
     * Default true
     */
    normalize?: boolean;
};
/**
 * The type of the underlying function called on `build()` for objects
 */
export type BuildingFunction<TSpec, TBuilt> = (model: Partial<TSpec>, options: BuildOptions) => TBuilt;
/**
 * The type of the underlying function called on `build()` for arrays
 */
export type ArrayBuildingFunction<TSpec, TBuilt> = (model: Array<TSpec>, options: BuildOptions) => TBuilt;
/**
 * Represents a fluent builder proxy for an object
 */
export type Builder<TSpec, TBuilt = TSpec> = {
    build: (option?: BuildOptions) => TBuilt;
} & {
    [K in keyof TSpec]-?: (arg: TSpec[K]) => Builder<TSpec, TBuilt>;
};
/**
 * Represents a fluent builder proxy for an array
 */
export type ArrayBuilder<TSpec, TBuilt> = {
    push: (item: TSpec) => ArrayBuilder<TSpec, TBuilt>;
    build: (option?: BuildOptions) => TBuilt;
};
/**
 * A factory for fluent builders that proxy properties assignations and can validate against schema on build()
 * @param buildingFn The function used to validate and produce the object on build()
 * @returns A fluent builder
 */
export declare function builder<TSpec, TBuilt = TSpec>(model?: Partial<TSpec>, buildingFn?: BuildingFunction<TSpec, TBuilt>): Builder<TSpec, TBuilt>;
/**
 * A factory for fluent builders that proxy properties assignations and can validate against schema on build()
 * @param buildingFn The function used to validate and produce the object on build()
 * @returns A fluent builder
 */
export declare function arrayBuilder<TSpec, TBuilt>(model?: Array<TSpec>, buildingFn?: ArrayBuildingFunction<TSpec, TBuilt>): ArrayBuilder<TSpec, TBuilt>;
//# sourceMappingURL=builder.d.ts.map