import { ObjectHydrator } from '../../hydrator';
import { Specification } from '../definitions';
import { Graph } from '../../graph-builder';
/**
 * Represents the intersection between the Workflow class and type
 */
export type WorkflowIntersection = Workflow & Specification.Workflow;
/**
 * Represents a constructor for the intersection of the Workflow class and type
 */
export interface WorkflowConstructor {
    new (model?: Partial<Specification.Workflow>): WorkflowIntersection;
}
/**
 * Represents a Workflow with methods for validation normalization, and serialization.
 * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
 */
export declare class Workflow extends ObjectHydrator<Specification.Workflow> {
    /**
     * Instanciates a new instance of the Workflow class.
     * Initializes properties based on the provided model if it is an object.
     *
     * @param model - Optional partial model object to initialize the Workflow.
     */
    constructor(model?: Partial<Specification.Workflow>);
    /**
     * Validates the current instance of the Workflow.
     * Throws if invalid.
     */
    validate(workflow?: Partial<Specification.Workflow>): void;
    /**
     * Normalizes the current instance of the Workflow.
     * Creates a copy of the Workflow, invokes normalization hooks if available, and returns the normalized copy.
     *
     * @returns A normalized version of the Workflow instance.
     */
    normalize(): Workflow & Specification.Workflow;
    static deserialize(text: string): WorkflowIntersection;
    static serialize(model: Partial<WorkflowIntersection>, format?: 'yaml' | 'json', normalize?: boolean): string;
    static toGraph(model: Partial<WorkflowIntersection>): Graph;
    static toMermaidCode(model: Partial<WorkflowIntersection>): string;
    /**
     * Serializes the workflow to YAML or JSON
     * @param format The format, 'yaml' or 'json', default is 'yaml'
     * @param normalize If the workflow should be normalized before serialization, default true
     * @returns A string representation of the workflow
     */
    serialize(format?: 'yaml' | 'json', normalize?: boolean): string;
    /**
     * Creates a directed graph representation of the workflow
     * @returns A directed graph of the workflow
     */
    toGraph(): Graph;
    /**
     * Generates the MermaidJS code corresponding to the workflow
     * @returns The MermaidJS code
     */
    toMermaidCode(): string;
}
export declare const _Workflow: WorkflowConstructor & {
    /**
     * Deserializes the provided string as a Workflow
     * @param text The YAML or JSON representation of a workflow
     * @returns A new Workflow instance
     */
    deserialize(text: string): WorkflowIntersection;
    /**
     * Serializes the provided Workflow to YAML or JSON
     * @param workflow The workflow to serialize
     * @param format The format, 'yaml' or 'json', default is 'yaml'
     * @param normalize If the workflow should be normalized before serialization, default true
     * @returns A string representation of the workflow
     */
    serialize(workflow: Partial<WorkflowIntersection>, format?: "yaml" | "json", normalize?: boolean): string;
    /**
     * Creates a directed graph representation of the provided workflow
     * @param workflow The workflow to convert
     * @returns A directed graph of the provided workflow
     */
    toGraph(workflow: Partial<WorkflowIntersection>): Graph;
    /**
     * Generates the MermaidJS code corresponding to the provided workflow
     * @param workflow The workflow to convert
     * @returns The MermaidJS code
     */
    toMermaidCode(workflow: Partial<WorkflowIntersection>): string;
};
//# sourceMappingURL=workflow.d.ts.map