import { Workflow } from './generated/definitions/specification';
/**
 * Represents a generic within a graph.
 * This serves as a base type for nodes, edges, and graphs.
 */
export type GraphElement = {
    /** A unique identifier for this graph element. */
    id: string;
    /** An optional label to provide additional context or naming. */
    label?: string | null;
};
/**
 * Enumeration of possible node types in a graph.
 */
export declare enum GraphNodeType {
    Root = "root",
    Start = "start",
    End = "end",
    Entry = "entry",
    Exit = "exit",
    Call = "call",
    Catch = "catch",
    Do = "do",
    Emit = "emit",
    For = "for",
    Fork = "fork",
    Listen = "listen",
    Raise = "raise",
    Run = "run",
    Set = "set",
    Switch = "switch",
    Try = "try",
    TryCatch = "try-catch",
    Wait = "wait"
}
/**
 * Represents a node within the graph.
 */
export type GraphNode = GraphElement & {
    /** The type of the node. */
    type: GraphNodeType;
};
/**
 * Represents a directed edge connecting two nodes in the graph.
 */
export type GraphEdge = GraphElement & {
    /** The unique identifier of the node where the edge originates. */
    sourceId: string;
    /** The unique identifier of the node where the edge terminates. */
    destinationId: string;
};
/**
 * Represents a graph or a subgraph
 */
export type Graph = GraphNode & {
    /** The parent graph if this is a subgraph, otherwise null. */
    parent?: Graph | null;
    /** A collection of nodes that belong to this graph. */
    nodes: GraphNode[];
    /** A collection of edges that define relationships between nodes. */
    edges: GraphEdge[];
    /** The entry node of the graph. */
    entryNode: GraphNode;
    /** The exit node of the graph. */
    exitNode: GraphNode;
};
/**
 * Constructs a graph representation based on the given workflow.
 *
 * @param workflow The workflow to be converted into a graph structure.
 * @returns A graph representation of the workflow.
 */
export declare function buildGraph(workflow: Workflow): Graph;
//# sourceMappingURL=graph-builder.d.ts.map