(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.serverWorkflowSdk = {}));
})(this, (function (exports) { 'use strict';

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * oUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   *
   */
  /**
   * The default function used to build an object, basically just return the provided object
   * @param model The object to "build"
   * @param options The build options
   * @returns
   */
  function defaultBuildingFn(model, options) {
      // prevents @typescript-eslint/no-unused-vars ...
      if (options.validate == null) {
          options.validate = true;
      }
      if (options.normalize == null) {
          options.normalize = true;
      }
      return model;
  }
  /**
   * A factory for fluent builders that proxy properties assignations and can validate against schema on build()
   * @param buildingFn The function used to validate and produce the object on build()
   * @returns A fluent builder
   */
  function builder(model, buildingFn) {
      if (model === void 0) { model = {}; }
      if (buildingFn === void 0) { buildingFn = defaultBuildingFn; }
      var proxy = new Proxy({}, {
          get: function (_, prop) {
              if (prop === 'build') {
                  return function (options) {
                      options = options || {};
                      if (options.validate == null) {
                          options.validate = true;
                      }
                      if (options.normalize == null) {
                          options.normalize = true;
                      }
                      return buildingFn(model, options);
                  };
              }
              return function (value) {
                  model[prop.toString()] = value;
                  return proxy;
              };
          },
          set: function () {
              return false;
          },
      });
      return proxy;
  }
  /**
   * A factory for fluent builders that proxy properties assignations and can validate against schema on build()
   * @param buildingFn The function used to validate and produce the object on build()
   * @returns A fluent builder
   */
  function arrayBuilder(model, buildingFn) {
      if (model === void 0) { model = []; }
      if (buildingFn === void 0) { buildingFn = defaultBuildingFn; }
      if (model != null && !Array.isArray(model)) {
          throw new Error("The provided model should be an array");
      }
      var proxy = new Proxy({}, {
          get: function (_, prop) {
              if (prop === 'build') {
                  return function (options) {
                      options = options || {};
                      if (options.validate == null) {
                          options.validate = true;
                      }
                      if (options.normalize == null) {
                          options.normalize = true;
                      }
                      return buildingFn(model, options);
                  };
              }
              if (prop === 'push') {
                  return function (value) {
                      model.push(value);
                      return proxy;
                  };
              }
          },
          set: function () {
              return false;
          },
      });
      return proxy;
  }

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */
  /* global Reflect, Promise, SuppressedError, Symbol */

  var extendStatics = function(d, b) {
      extendStatics = Object.setPrototypeOf ||
          ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
          function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
      return extendStatics(d, b);
  };

  function __extends(d, b) {
      if (typeof b !== "function" && b !== null)
          throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
      extendStatics(d, b);
      function __() { this.constructor = d; }
      d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  }

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
      var e = new Error(message);
      return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * oUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   *
   */
  /**
   * Check if the provided value is an object but not an array
   * @param value The value to check
   * @returns True if the value is an object
   */
  var isObject$1 = function (value) {
      if (!value)
          return false;
      return typeof value === 'object' && !Array.isArray(value);
  };
  /**
   * Makes a deep copy of the provided object
   * @param obj
   * @returns
   */
  var deepCopy = function (obj) { return JSON.parse(JSON.stringify(obj)); };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * oUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   *
   */
  /**
   * A base class used for object hydration
   */
  var ObjectHydrator = /** @class */ (function () {
      function ObjectHydrator(model) {
          if (isObject$1(model)) {
              Object.assign(this, deepCopy(model));
          }
      }
      return ObjectHydrator;
  }());
  /**
   * A base class used for array hydration
   */
  var ArrayHydrator = /** @class */ (function (_super) {
      __extends(ArrayHydrator, _super);
      function ArrayHydrator(model) {
          var _this = this;
          if (!isNaN(model)) {
              _this = _super.call(this, model) || this;
          }
          else {
              _this = _super.apply(this, (model || [])) || this;
              if (!model) {
                  model = [];
              }
              if (!Array.isArray(model)) {
                  throw new Error('The provided model should be an array');
              }
          }
          return _this;
      }
      return ArrayHydrator;
  }(Array));

  var schemaVersion = "1.0.0";

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  var WorkflowHooks = {
      preValidation: function (instance) {
          var _a, _b;
          if (((_a = instance === null || instance === void 0 ? void 0 : instance.document) === null || _a === void 0 ? void 0 : _a.dsl) !== schemaVersion) {
              throw new Error("'Workflow' is invalid - The DSL version of the workflow '".concat((_b = instance === null || instance === void 0 ? void 0 : instance.document) === null || _b === void 0 ? void 0 : _b.dsl, "' doesn't match the supported version of the SDK '").concat(schemaVersion, "'."));
          }
          return;
      },
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * oUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   *
   */
  /**
   * A mapping of type names and their lifecycle hooks
   */
  var registeredHooks = new Map();
  /**
   * The function used to register hooks
   * @param typeName The name of the type to register the hooks for
   * @param hooks The hooks to register
   */
  var registerHooks = function (typeName, hooks) {
      registeredHooks.set(typeName, hooks);
  };
  /**
   * Gets the lifecylce hook for the provided type, if any
   * @param typeName The type to get the lifecycle hook for
   * @returns The lifecycle hook, if any
   */
  var getLifecycleHooks = function (typeName) {
      return registeredHooks.get(typeName);
  };
  registerHooks('Workflow', WorkflowHooks);

  function getDefaultExportFromCjs (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  var _2020 = {exports: {}};

  var core$4 = {};

  var validate$1 = {};

  var boolSchema = {};

  var errors = {};

  var codegen = {};

  var code$1 = {};

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.regexpCode = exports.getEsmExportName = exports.getProperty = exports.safeStringify = exports.stringify = exports.strConcat = exports.addCodeArg = exports.str = exports._ = exports.nil = exports._Code = exports.Name = exports.IDENTIFIER = exports._CodeOrName = void 0;
  	// eslint-disable-next-line @typescript-eslint/no-extraneous-class
  	class _CodeOrName {
  	}
  	exports._CodeOrName = _CodeOrName;
  	exports.IDENTIFIER = /^[a-z$_][a-z$_0-9]*$/i;
  	class Name extends _CodeOrName {
  	    constructor(s) {
  	        super();
  	        if (!exports.IDENTIFIER.test(s))
  	            throw new Error("CodeGen: name must be a valid identifier");
  	        this.str = s;
  	    }
  	    toString() {
  	        return this.str;
  	    }
  	    emptyStr() {
  	        return false;
  	    }
  	    get names() {
  	        return { [this.str]: 1 };
  	    }
  	}
  	exports.Name = Name;
  	class _Code extends _CodeOrName {
  	    constructor(code) {
  	        super();
  	        this._items = typeof code === "string" ? [code] : code;
  	    }
  	    toString() {
  	        return this.str;
  	    }
  	    emptyStr() {
  	        if (this._items.length > 1)
  	            return false;
  	        const item = this._items[0];
  	        return item === "" || item === '""';
  	    }
  	    get str() {
  	        var _a;
  	        return ((_a = this._str) !== null && _a !== void 0 ? _a : (this._str = this._items.reduce((s, c) => `${s}${c}`, "")));
  	    }
  	    get names() {
  	        var _a;
  	        return ((_a = this._names) !== null && _a !== void 0 ? _a : (this._names = this._items.reduce((names, c) => {
  	            if (c instanceof Name)
  	                names[c.str] = (names[c.str] || 0) + 1;
  	            return names;
  	        }, {})));
  	    }
  	}
  	exports._Code = _Code;
  	exports.nil = new _Code("");
  	function _(strs, ...args) {
  	    const code = [strs[0]];
  	    let i = 0;
  	    while (i < args.length) {
  	        addCodeArg(code, args[i]);
  	        code.push(strs[++i]);
  	    }
  	    return new _Code(code);
  	}
  	exports._ = _;
  	const plus = new _Code("+");
  	function str(strs, ...args) {
  	    const expr = [safeStringify(strs[0])];
  	    let i = 0;
  	    while (i < args.length) {
  	        expr.push(plus);
  	        addCodeArg(expr, args[i]);
  	        expr.push(plus, safeStringify(strs[++i]));
  	    }
  	    optimize(expr);
  	    return new _Code(expr);
  	}
  	exports.str = str;
  	function addCodeArg(code, arg) {
  	    if (arg instanceof _Code)
  	        code.push(...arg._items);
  	    else if (arg instanceof Name)
  	        code.push(arg);
  	    else
  	        code.push(interpolate(arg));
  	}
  	exports.addCodeArg = addCodeArg;
  	function optimize(expr) {
  	    let i = 1;
  	    while (i < expr.length - 1) {
  	        if (expr[i] === plus) {
  	            const res = mergeExprItems(expr[i - 1], expr[i + 1]);
  	            if (res !== undefined) {
  	                expr.splice(i - 1, 3, res);
  	                continue;
  	            }
  	            expr[i++] = "+";
  	        }
  	        i++;
  	    }
  	}
  	function mergeExprItems(a, b) {
  	    if (b === '""')
  	        return a;
  	    if (a === '""')
  	        return b;
  	    if (typeof a == "string") {
  	        if (b instanceof Name || a[a.length - 1] !== '"')
  	            return;
  	        if (typeof b != "string")
  	            return `${a.slice(0, -1)}${b}"`;
  	        if (b[0] === '"')
  	            return a.slice(0, -1) + b.slice(1);
  	        return;
  	    }
  	    if (typeof b == "string" && b[0] === '"' && !(a instanceof Name))
  	        return `"${a}${b.slice(1)}`;
  	    return;
  	}
  	function strConcat(c1, c2) {
  	    return c2.emptyStr() ? c1 : c1.emptyStr() ? c2 : str `${c1}${c2}`;
  	}
  	exports.strConcat = strConcat;
  	// TODO do not allow arrays here
  	function interpolate(x) {
  	    return typeof x == "number" || typeof x == "boolean" || x === null
  	        ? x
  	        : safeStringify(Array.isArray(x) ? x.join(",") : x);
  	}
  	function stringify(x) {
  	    return new _Code(safeStringify(x));
  	}
  	exports.stringify = stringify;
  	function safeStringify(x) {
  	    return JSON.stringify(x)
  	        .replace(/\u2028/g, "\\u2028")
  	        .replace(/\u2029/g, "\\u2029");
  	}
  	exports.safeStringify = safeStringify;
  	function getProperty(key) {
  	    return typeof key == "string" && exports.IDENTIFIER.test(key) ? new _Code(`.${key}`) : _ `[${key}]`;
  	}
  	exports.getProperty = getProperty;
  	//Does best effort to format the name properly
  	function getEsmExportName(key) {
  	    if (typeof key == "string" && exports.IDENTIFIER.test(key)) {
  	        return new _Code(`${key}`);
  	    }
  	    throw new Error(`CodeGen: invalid export name: ${key}, use explicit $id name mapping`);
  	}
  	exports.getEsmExportName = getEsmExportName;
  	function regexpCode(rx) {
  	    return new _Code(rx.toString());
  	}
  	exports.regexpCode = regexpCode;
  	
  } (code$1));

  var scope = {};

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.ValueScope = exports.ValueScopeName = exports.Scope = exports.varKinds = exports.UsedValueState = void 0;
  	const code_1 = code$1;
  	class ValueError extends Error {
  	    constructor(name) {
  	        super(`CodeGen: "code" for ${name} not defined`);
  	        this.value = name.value;
  	    }
  	}
  	var UsedValueState;
  	(function (UsedValueState) {
  	    UsedValueState[UsedValueState["Started"] = 0] = "Started";
  	    UsedValueState[UsedValueState["Completed"] = 1] = "Completed";
  	})(UsedValueState || (exports.UsedValueState = UsedValueState = {}));
  	exports.varKinds = {
  	    const: new code_1.Name("const"),
  	    let: new code_1.Name("let"),
  	    var: new code_1.Name("var"),
  	};
  	class Scope {
  	    constructor({ prefixes, parent } = {}) {
  	        this._names = {};
  	        this._prefixes = prefixes;
  	        this._parent = parent;
  	    }
  	    toName(nameOrPrefix) {
  	        return nameOrPrefix instanceof code_1.Name ? nameOrPrefix : this.name(nameOrPrefix);
  	    }
  	    name(prefix) {
  	        return new code_1.Name(this._newName(prefix));
  	    }
  	    _newName(prefix) {
  	        const ng = this._names[prefix] || this._nameGroup(prefix);
  	        return `${prefix}${ng.index++}`;
  	    }
  	    _nameGroup(prefix) {
  	        var _a, _b;
  	        if (((_b = (_a = this._parent) === null || _a === void 0 ? void 0 : _a._prefixes) === null || _b === void 0 ? void 0 : _b.has(prefix)) || (this._prefixes && !this._prefixes.has(prefix))) {
  	            throw new Error(`CodeGen: prefix "${prefix}" is not allowed in this scope`);
  	        }
  	        return (this._names[prefix] = { prefix, index: 0 });
  	    }
  	}
  	exports.Scope = Scope;
  	class ValueScopeName extends code_1.Name {
  	    constructor(prefix, nameStr) {
  	        super(nameStr);
  	        this.prefix = prefix;
  	    }
  	    setValue(value, { property, itemIndex }) {
  	        this.value = value;
  	        this.scopePath = (0, code_1._) `.${new code_1.Name(property)}[${itemIndex}]`;
  	    }
  	}
  	exports.ValueScopeName = ValueScopeName;
  	const line = (0, code_1._) `\n`;
  	class ValueScope extends Scope {
  	    constructor(opts) {
  	        super(opts);
  	        this._values = {};
  	        this._scope = opts.scope;
  	        this.opts = { ...opts, _n: opts.lines ? line : code_1.nil };
  	    }
  	    get() {
  	        return this._scope;
  	    }
  	    name(prefix) {
  	        return new ValueScopeName(prefix, this._newName(prefix));
  	    }
  	    value(nameOrPrefix, value) {
  	        var _a;
  	        if (value.ref === undefined)
  	            throw new Error("CodeGen: ref must be passed in value");
  	        const name = this.toName(nameOrPrefix);
  	        const { prefix } = name;
  	        const valueKey = (_a = value.key) !== null && _a !== void 0 ? _a : value.ref;
  	        let vs = this._values[prefix];
  	        if (vs) {
  	            const _name = vs.get(valueKey);
  	            if (_name)
  	                return _name;
  	        }
  	        else {
  	            vs = this._values[prefix] = new Map();
  	        }
  	        vs.set(valueKey, name);
  	        const s = this._scope[prefix] || (this._scope[prefix] = []);
  	        const itemIndex = s.length;
  	        s[itemIndex] = value.ref;
  	        name.setValue(value, { property: prefix, itemIndex });
  	        return name;
  	    }
  	    getValue(prefix, keyOrRef) {
  	        const vs = this._values[prefix];
  	        if (!vs)
  	            return;
  	        return vs.get(keyOrRef);
  	    }
  	    scopeRefs(scopeName, values = this._values) {
  	        return this._reduceValues(values, (name) => {
  	            if (name.scopePath === undefined)
  	                throw new Error(`CodeGen: name "${name}" has no value`);
  	            return (0, code_1._) `${scopeName}${name.scopePath}`;
  	        });
  	    }
  	    scopeCode(values = this._values, usedValues, getCode) {
  	        return this._reduceValues(values, (name) => {
  	            if (name.value === undefined)
  	                throw new Error(`CodeGen: name "${name}" has no value`);
  	            return name.value.code;
  	        }, usedValues, getCode);
  	    }
  	    _reduceValues(values, valueCode, usedValues = {}, getCode) {
  	        let code = code_1.nil;
  	        for (const prefix in values) {
  	            const vs = values[prefix];
  	            if (!vs)
  	                continue;
  	            const nameSet = (usedValues[prefix] = usedValues[prefix] || new Map());
  	            vs.forEach((name) => {
  	                if (nameSet.has(name))
  	                    return;
  	                nameSet.set(name, UsedValueState.Started);
  	                let c = valueCode(name);
  	                if (c) {
  	                    const def = this.opts.es5 ? exports.varKinds.var : exports.varKinds.const;
  	                    code = (0, code_1._) `${code}${def} ${name} = ${c};${this.opts._n}`;
  	                }
  	                else if ((c = getCode === null || getCode === void 0 ? void 0 : getCode(name))) {
  	                    code = (0, code_1._) `${code}${c}${this.opts._n}`;
  	                }
  	                else {
  	                    throw new ValueError(name);
  	                }
  	                nameSet.set(name, UsedValueState.Completed);
  	            });
  	        }
  	        return code;
  	    }
  	}
  	exports.ValueScope = ValueScope;
  	
  } (scope));

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.or = exports.and = exports.not = exports.CodeGen = exports.operators = exports.varKinds = exports.ValueScopeName = exports.ValueScope = exports.Scope = exports.Name = exports.regexpCode = exports.stringify = exports.getProperty = exports.nil = exports.strConcat = exports.str = exports._ = void 0;
  	const code_1 = code$1;
  	const scope_1 = scope;
  	var code_2 = code$1;
  	Object.defineProperty(exports, "_", { enumerable: true, get: function () { return code_2._; } });
  	Object.defineProperty(exports, "str", { enumerable: true, get: function () { return code_2.str; } });
  	Object.defineProperty(exports, "strConcat", { enumerable: true, get: function () { return code_2.strConcat; } });
  	Object.defineProperty(exports, "nil", { enumerable: true, get: function () { return code_2.nil; } });
  	Object.defineProperty(exports, "getProperty", { enumerable: true, get: function () { return code_2.getProperty; } });
  	Object.defineProperty(exports, "stringify", { enumerable: true, get: function () { return code_2.stringify; } });
  	Object.defineProperty(exports, "regexpCode", { enumerable: true, get: function () { return code_2.regexpCode; } });
  	Object.defineProperty(exports, "Name", { enumerable: true, get: function () { return code_2.Name; } });
  	var scope_2 = scope;
  	Object.defineProperty(exports, "Scope", { enumerable: true, get: function () { return scope_2.Scope; } });
  	Object.defineProperty(exports, "ValueScope", { enumerable: true, get: function () { return scope_2.ValueScope; } });
  	Object.defineProperty(exports, "ValueScopeName", { enumerable: true, get: function () { return scope_2.ValueScopeName; } });
  	Object.defineProperty(exports, "varKinds", { enumerable: true, get: function () { return scope_2.varKinds; } });
  	exports.operators = {
  	    GT: new code_1._Code(">"),
  	    GTE: new code_1._Code(">="),
  	    LT: new code_1._Code("<"),
  	    LTE: new code_1._Code("<="),
  	    EQ: new code_1._Code("==="),
  	    NEQ: new code_1._Code("!=="),
  	    NOT: new code_1._Code("!"),
  	    OR: new code_1._Code("||"),
  	    AND: new code_1._Code("&&"),
  	    ADD: new code_1._Code("+"),
  	};
  	class Node {
  	    optimizeNodes() {
  	        return this;
  	    }
  	    optimizeNames(_names, _constants) {
  	        return this;
  	    }
  	}
  	class Def extends Node {
  	    constructor(varKind, name, rhs) {
  	        super();
  	        this.varKind = varKind;
  	        this.name = name;
  	        this.rhs = rhs;
  	    }
  	    render({ es5, _n }) {
  	        const varKind = es5 ? scope_1.varKinds.var : this.varKind;
  	        const rhs = this.rhs === undefined ? "" : ` = ${this.rhs}`;
  	        return `${varKind} ${this.name}${rhs};` + _n;
  	    }
  	    optimizeNames(names, constants) {
  	        if (!names[this.name.str])
  	            return;
  	        if (this.rhs)
  	            this.rhs = optimizeExpr(this.rhs, names, constants);
  	        return this;
  	    }
  	    get names() {
  	        return this.rhs instanceof code_1._CodeOrName ? this.rhs.names : {};
  	    }
  	}
  	class Assign extends Node {
  	    constructor(lhs, rhs, sideEffects) {
  	        super();
  	        this.lhs = lhs;
  	        this.rhs = rhs;
  	        this.sideEffects = sideEffects;
  	    }
  	    render({ _n }) {
  	        return `${this.lhs} = ${this.rhs};` + _n;
  	    }
  	    optimizeNames(names, constants) {
  	        if (this.lhs instanceof code_1.Name && !names[this.lhs.str] && !this.sideEffects)
  	            return;
  	        this.rhs = optimizeExpr(this.rhs, names, constants);
  	        return this;
  	    }
  	    get names() {
  	        const names = this.lhs instanceof code_1.Name ? {} : { ...this.lhs.names };
  	        return addExprNames(names, this.rhs);
  	    }
  	}
  	class AssignOp extends Assign {
  	    constructor(lhs, op, rhs, sideEffects) {
  	        super(lhs, rhs, sideEffects);
  	        this.op = op;
  	    }
  	    render({ _n }) {
  	        return `${this.lhs} ${this.op}= ${this.rhs};` + _n;
  	    }
  	}
  	class Label extends Node {
  	    constructor(label) {
  	        super();
  	        this.label = label;
  	        this.names = {};
  	    }
  	    render({ _n }) {
  	        return `${this.label}:` + _n;
  	    }
  	}
  	class Break extends Node {
  	    constructor(label) {
  	        super();
  	        this.label = label;
  	        this.names = {};
  	    }
  	    render({ _n }) {
  	        const label = this.label ? ` ${this.label}` : "";
  	        return `break${label};` + _n;
  	    }
  	}
  	class Throw extends Node {
  	    constructor(error) {
  	        super();
  	        this.error = error;
  	    }
  	    render({ _n }) {
  	        return `throw ${this.error};` + _n;
  	    }
  	    get names() {
  	        return this.error.names;
  	    }
  	}
  	class AnyCode extends Node {
  	    constructor(code) {
  	        super();
  	        this.code = code;
  	    }
  	    render({ _n }) {
  	        return `${this.code};` + _n;
  	    }
  	    optimizeNodes() {
  	        return `${this.code}` ? this : undefined;
  	    }
  	    optimizeNames(names, constants) {
  	        this.code = optimizeExpr(this.code, names, constants);
  	        return this;
  	    }
  	    get names() {
  	        return this.code instanceof code_1._CodeOrName ? this.code.names : {};
  	    }
  	}
  	class ParentNode extends Node {
  	    constructor(nodes = []) {
  	        super();
  	        this.nodes = nodes;
  	    }
  	    render(opts) {
  	        return this.nodes.reduce((code, n) => code + n.render(opts), "");
  	    }
  	    optimizeNodes() {
  	        const { nodes } = this;
  	        let i = nodes.length;
  	        while (i--) {
  	            const n = nodes[i].optimizeNodes();
  	            if (Array.isArray(n))
  	                nodes.splice(i, 1, ...n);
  	            else if (n)
  	                nodes[i] = n;
  	            else
  	                nodes.splice(i, 1);
  	        }
  	        return nodes.length > 0 ? this : undefined;
  	    }
  	    optimizeNames(names, constants) {
  	        const { nodes } = this;
  	        let i = nodes.length;
  	        while (i--) {
  	            // iterating backwards improves 1-pass optimization
  	            const n = nodes[i];
  	            if (n.optimizeNames(names, constants))
  	                continue;
  	            subtractNames(names, n.names);
  	            nodes.splice(i, 1);
  	        }
  	        return nodes.length > 0 ? this : undefined;
  	    }
  	    get names() {
  	        return this.nodes.reduce((names, n) => addNames(names, n.names), {});
  	    }
  	}
  	class BlockNode extends ParentNode {
  	    render(opts) {
  	        return "{" + opts._n + super.render(opts) + "}" + opts._n;
  	    }
  	}
  	class Root extends ParentNode {
  	}
  	class Else extends BlockNode {
  	}
  	Else.kind = "else";
  	class If extends BlockNode {
  	    constructor(condition, nodes) {
  	        super(nodes);
  	        this.condition = condition;
  	    }
  	    render(opts) {
  	        let code = `if(${this.condition})` + super.render(opts);
  	        if (this.else)
  	            code += "else " + this.else.render(opts);
  	        return code;
  	    }
  	    optimizeNodes() {
  	        super.optimizeNodes();
  	        const cond = this.condition;
  	        if (cond === true)
  	            return this.nodes; // else is ignored here
  	        let e = this.else;
  	        if (e) {
  	            const ns = e.optimizeNodes();
  	            e = this.else = Array.isArray(ns) ? new Else(ns) : ns;
  	        }
  	        if (e) {
  	            if (cond === false)
  	                return e instanceof If ? e : e.nodes;
  	            if (this.nodes.length)
  	                return this;
  	            return new If(not(cond), e instanceof If ? [e] : e.nodes);
  	        }
  	        if (cond === false || !this.nodes.length)
  	            return undefined;
  	        return this;
  	    }
  	    optimizeNames(names, constants) {
  	        var _a;
  	        this.else = (_a = this.else) === null || _a === void 0 ? void 0 : _a.optimizeNames(names, constants);
  	        if (!(super.optimizeNames(names, constants) || this.else))
  	            return;
  	        this.condition = optimizeExpr(this.condition, names, constants);
  	        return this;
  	    }
  	    get names() {
  	        const names = super.names;
  	        addExprNames(names, this.condition);
  	        if (this.else)
  	            addNames(names, this.else.names);
  	        return names;
  	    }
  	}
  	If.kind = "if";
  	class For extends BlockNode {
  	}
  	For.kind = "for";
  	class ForLoop extends For {
  	    constructor(iteration) {
  	        super();
  	        this.iteration = iteration;
  	    }
  	    render(opts) {
  	        return `for(${this.iteration})` + super.render(opts);
  	    }
  	    optimizeNames(names, constants) {
  	        if (!super.optimizeNames(names, constants))
  	            return;
  	        this.iteration = optimizeExpr(this.iteration, names, constants);
  	        return this;
  	    }
  	    get names() {
  	        return addNames(super.names, this.iteration.names);
  	    }
  	}
  	class ForRange extends For {
  	    constructor(varKind, name, from, to) {
  	        super();
  	        this.varKind = varKind;
  	        this.name = name;
  	        this.from = from;
  	        this.to = to;
  	    }
  	    render(opts) {
  	        const varKind = opts.es5 ? scope_1.varKinds.var : this.varKind;
  	        const { name, from, to } = this;
  	        return `for(${varKind} ${name}=${from}; ${name}<${to}; ${name}++)` + super.render(opts);
  	    }
  	    get names() {
  	        const names = addExprNames(super.names, this.from);
  	        return addExprNames(names, this.to);
  	    }
  	}
  	class ForIter extends For {
  	    constructor(loop, varKind, name, iterable) {
  	        super();
  	        this.loop = loop;
  	        this.varKind = varKind;
  	        this.name = name;
  	        this.iterable = iterable;
  	    }
  	    render(opts) {
  	        return `for(${this.varKind} ${this.name} ${this.loop} ${this.iterable})` + super.render(opts);
  	    }
  	    optimizeNames(names, constants) {
  	        if (!super.optimizeNames(names, constants))
  	            return;
  	        this.iterable = optimizeExpr(this.iterable, names, constants);
  	        return this;
  	    }
  	    get names() {
  	        return addNames(super.names, this.iterable.names);
  	    }
  	}
  	class Func extends BlockNode {
  	    constructor(name, args, async) {
  	        super();
  	        this.name = name;
  	        this.args = args;
  	        this.async = async;
  	    }
  	    render(opts) {
  	        const _async = this.async ? "async " : "";
  	        return `${_async}function ${this.name}(${this.args})` + super.render(opts);
  	    }
  	}
  	Func.kind = "func";
  	class Return extends ParentNode {
  	    render(opts) {
  	        return "return " + super.render(opts);
  	    }
  	}
  	Return.kind = "return";
  	class Try extends BlockNode {
  	    render(opts) {
  	        let code = "try" + super.render(opts);
  	        if (this.catch)
  	            code += this.catch.render(opts);
  	        if (this.finally)
  	            code += this.finally.render(opts);
  	        return code;
  	    }
  	    optimizeNodes() {
  	        var _a, _b;
  	        super.optimizeNodes();
  	        (_a = this.catch) === null || _a === void 0 ? void 0 : _a.optimizeNodes();
  	        (_b = this.finally) === null || _b === void 0 ? void 0 : _b.optimizeNodes();
  	        return this;
  	    }
  	    optimizeNames(names, constants) {
  	        var _a, _b;
  	        super.optimizeNames(names, constants);
  	        (_a = this.catch) === null || _a === void 0 ? void 0 : _a.optimizeNames(names, constants);
  	        (_b = this.finally) === null || _b === void 0 ? void 0 : _b.optimizeNames(names, constants);
  	        return this;
  	    }
  	    get names() {
  	        const names = super.names;
  	        if (this.catch)
  	            addNames(names, this.catch.names);
  	        if (this.finally)
  	            addNames(names, this.finally.names);
  	        return names;
  	    }
  	}
  	class Catch extends BlockNode {
  	    constructor(error) {
  	        super();
  	        this.error = error;
  	    }
  	    render(opts) {
  	        return `catch(${this.error})` + super.render(opts);
  	    }
  	}
  	Catch.kind = "catch";
  	class Finally extends BlockNode {
  	    render(opts) {
  	        return "finally" + super.render(opts);
  	    }
  	}
  	Finally.kind = "finally";
  	class CodeGen {
  	    constructor(extScope, opts = {}) {
  	        this._values = {};
  	        this._blockStarts = [];
  	        this._constants = {};
  	        this.opts = { ...opts, _n: opts.lines ? "\n" : "" };
  	        this._extScope = extScope;
  	        this._scope = new scope_1.Scope({ parent: extScope });
  	        this._nodes = [new Root()];
  	    }
  	    toString() {
  	        return this._root.render(this.opts);
  	    }
  	    // returns unique name in the internal scope
  	    name(prefix) {
  	        return this._scope.name(prefix);
  	    }
  	    // reserves unique name in the external scope
  	    scopeName(prefix) {
  	        return this._extScope.name(prefix);
  	    }
  	    // reserves unique name in the external scope and assigns value to it
  	    scopeValue(prefixOrName, value) {
  	        const name = this._extScope.value(prefixOrName, value);
  	        const vs = this._values[name.prefix] || (this._values[name.prefix] = new Set());
  	        vs.add(name);
  	        return name;
  	    }
  	    getScopeValue(prefix, keyOrRef) {
  	        return this._extScope.getValue(prefix, keyOrRef);
  	    }
  	    // return code that assigns values in the external scope to the names that are used internally
  	    // (same names that were returned by gen.scopeName or gen.scopeValue)
  	    scopeRefs(scopeName) {
  	        return this._extScope.scopeRefs(scopeName, this._values);
  	    }
  	    scopeCode() {
  	        return this._extScope.scopeCode(this._values);
  	    }
  	    _def(varKind, nameOrPrefix, rhs, constant) {
  	        const name = this._scope.toName(nameOrPrefix);
  	        if (rhs !== undefined && constant)
  	            this._constants[name.str] = rhs;
  	        this._leafNode(new Def(varKind, name, rhs));
  	        return name;
  	    }
  	    // `const` declaration (`var` in es5 mode)
  	    const(nameOrPrefix, rhs, _constant) {
  	        return this._def(scope_1.varKinds.const, nameOrPrefix, rhs, _constant);
  	    }
  	    // `let` declaration with optional assignment (`var` in es5 mode)
  	    let(nameOrPrefix, rhs, _constant) {
  	        return this._def(scope_1.varKinds.let, nameOrPrefix, rhs, _constant);
  	    }
  	    // `var` declaration with optional assignment
  	    var(nameOrPrefix, rhs, _constant) {
  	        return this._def(scope_1.varKinds.var, nameOrPrefix, rhs, _constant);
  	    }
  	    // assignment code
  	    assign(lhs, rhs, sideEffects) {
  	        return this._leafNode(new Assign(lhs, rhs, sideEffects));
  	    }
  	    // `+=` code
  	    add(lhs, rhs) {
  	        return this._leafNode(new AssignOp(lhs, exports.operators.ADD, rhs));
  	    }
  	    // appends passed SafeExpr to code or executes Block
  	    code(c) {
  	        if (typeof c == "function")
  	            c();
  	        else if (c !== code_1.nil)
  	            this._leafNode(new AnyCode(c));
  	        return this;
  	    }
  	    // returns code for object literal for the passed argument list of key-value pairs
  	    object(...keyValues) {
  	        const code = ["{"];
  	        for (const [key, value] of keyValues) {
  	            if (code.length > 1)
  	                code.push(",");
  	            code.push(key);
  	            if (key !== value || this.opts.es5) {
  	                code.push(":");
  	                (0, code_1.addCodeArg)(code, value);
  	            }
  	        }
  	        code.push("}");
  	        return new code_1._Code(code);
  	    }
  	    // `if` clause (or statement if `thenBody` and, optionally, `elseBody` are passed)
  	    if(condition, thenBody, elseBody) {
  	        this._blockNode(new If(condition));
  	        if (thenBody && elseBody) {
  	            this.code(thenBody).else().code(elseBody).endIf();
  	        }
  	        else if (thenBody) {
  	            this.code(thenBody).endIf();
  	        }
  	        else if (elseBody) {
  	            throw new Error('CodeGen: "else" body without "then" body');
  	        }
  	        return this;
  	    }
  	    // `else if` clause - invalid without `if` or after `else` clauses
  	    elseIf(condition) {
  	        return this._elseNode(new If(condition));
  	    }
  	    // `else` clause - only valid after `if` or `else if` clauses
  	    else() {
  	        return this._elseNode(new Else());
  	    }
  	    // end `if` statement (needed if gen.if was used only with condition)
  	    endIf() {
  	        return this._endBlockNode(If, Else);
  	    }
  	    _for(node, forBody) {
  	        this._blockNode(node);
  	        if (forBody)
  	            this.code(forBody).endFor();
  	        return this;
  	    }
  	    // a generic `for` clause (or statement if `forBody` is passed)
  	    for(iteration, forBody) {
  	        return this._for(new ForLoop(iteration), forBody);
  	    }
  	    // `for` statement for a range of values
  	    forRange(nameOrPrefix, from, to, forBody, varKind = this.opts.es5 ? scope_1.varKinds.var : scope_1.varKinds.let) {
  	        const name = this._scope.toName(nameOrPrefix);
  	        return this._for(new ForRange(varKind, name, from, to), () => forBody(name));
  	    }
  	    // `for-of` statement (in es5 mode replace with a normal for loop)
  	    forOf(nameOrPrefix, iterable, forBody, varKind = scope_1.varKinds.const) {
  	        const name = this._scope.toName(nameOrPrefix);
  	        if (this.opts.es5) {
  	            const arr = iterable instanceof code_1.Name ? iterable : this.var("_arr", iterable);
  	            return this.forRange("_i", 0, (0, code_1._) `${arr}.length`, (i) => {
  	                this.var(name, (0, code_1._) `${arr}[${i}]`);
  	                forBody(name);
  	            });
  	        }
  	        return this._for(new ForIter("of", varKind, name, iterable), () => forBody(name));
  	    }
  	    // `for-in` statement.
  	    // With option `ownProperties` replaced with a `for-of` loop for object keys
  	    forIn(nameOrPrefix, obj, forBody, varKind = this.opts.es5 ? scope_1.varKinds.var : scope_1.varKinds.const) {
  	        if (this.opts.ownProperties) {
  	            return this.forOf(nameOrPrefix, (0, code_1._) `Object.keys(${obj})`, forBody);
  	        }
  	        const name = this._scope.toName(nameOrPrefix);
  	        return this._for(new ForIter("in", varKind, name, obj), () => forBody(name));
  	    }
  	    // end `for` loop
  	    endFor() {
  	        return this._endBlockNode(For);
  	    }
  	    // `label` statement
  	    label(label) {
  	        return this._leafNode(new Label(label));
  	    }
  	    // `break` statement
  	    break(label) {
  	        return this._leafNode(new Break(label));
  	    }
  	    // `return` statement
  	    return(value) {
  	        const node = new Return();
  	        this._blockNode(node);
  	        this.code(value);
  	        if (node.nodes.length !== 1)
  	            throw new Error('CodeGen: "return" should have one node');
  	        return this._endBlockNode(Return);
  	    }
  	    // `try` statement
  	    try(tryBody, catchCode, finallyCode) {
  	        if (!catchCode && !finallyCode)
  	            throw new Error('CodeGen: "try" without "catch" and "finally"');
  	        const node = new Try();
  	        this._blockNode(node);
  	        this.code(tryBody);
  	        if (catchCode) {
  	            const error = this.name("e");
  	            this._currNode = node.catch = new Catch(error);
  	            catchCode(error);
  	        }
  	        if (finallyCode) {
  	            this._currNode = node.finally = new Finally();
  	            this.code(finallyCode);
  	        }
  	        return this._endBlockNode(Catch, Finally);
  	    }
  	    // `throw` statement
  	    throw(error) {
  	        return this._leafNode(new Throw(error));
  	    }
  	    // start self-balancing block
  	    block(body, nodeCount) {
  	        this._blockStarts.push(this._nodes.length);
  	        if (body)
  	            this.code(body).endBlock(nodeCount);
  	        return this;
  	    }
  	    // end the current self-balancing block
  	    endBlock(nodeCount) {
  	        const len = this._blockStarts.pop();
  	        if (len === undefined)
  	            throw new Error("CodeGen: not in self-balancing block");
  	        const toClose = this._nodes.length - len;
  	        if (toClose < 0 || (nodeCount !== undefined && toClose !== nodeCount)) {
  	            throw new Error(`CodeGen: wrong number of nodes: ${toClose} vs ${nodeCount} expected`);
  	        }
  	        this._nodes.length = len;
  	        return this;
  	    }
  	    // `function` heading (or definition if funcBody is passed)
  	    func(name, args = code_1.nil, async, funcBody) {
  	        this._blockNode(new Func(name, args, async));
  	        if (funcBody)
  	            this.code(funcBody).endFunc();
  	        return this;
  	    }
  	    // end function definition
  	    endFunc() {
  	        return this._endBlockNode(Func);
  	    }
  	    optimize(n = 1) {
  	        while (n-- > 0) {
  	            this._root.optimizeNodes();
  	            this._root.optimizeNames(this._root.names, this._constants);
  	        }
  	    }
  	    _leafNode(node) {
  	        this._currNode.nodes.push(node);
  	        return this;
  	    }
  	    _blockNode(node) {
  	        this._currNode.nodes.push(node);
  	        this._nodes.push(node);
  	    }
  	    _endBlockNode(N1, N2) {
  	        const n = this._currNode;
  	        if (n instanceof N1 || (N2 && n instanceof N2)) {
  	            this._nodes.pop();
  	            return this;
  	        }
  	        throw new Error(`CodeGen: not in block "${N2 ? `${N1.kind}/${N2.kind}` : N1.kind}"`);
  	    }
  	    _elseNode(node) {
  	        const n = this._currNode;
  	        if (!(n instanceof If)) {
  	            throw new Error('CodeGen: "else" without "if"');
  	        }
  	        this._currNode = n.else = node;
  	        return this;
  	    }
  	    get _root() {
  	        return this._nodes[0];
  	    }
  	    get _currNode() {
  	        const ns = this._nodes;
  	        return ns[ns.length - 1];
  	    }
  	    set _currNode(node) {
  	        const ns = this._nodes;
  	        ns[ns.length - 1] = node;
  	    }
  	}
  	exports.CodeGen = CodeGen;
  	function addNames(names, from) {
  	    for (const n in from)
  	        names[n] = (names[n] || 0) + (from[n] || 0);
  	    return names;
  	}
  	function addExprNames(names, from) {
  	    return from instanceof code_1._CodeOrName ? addNames(names, from.names) : names;
  	}
  	function optimizeExpr(expr, names, constants) {
  	    if (expr instanceof code_1.Name)
  	        return replaceName(expr);
  	    if (!canOptimize(expr))
  	        return expr;
  	    return new code_1._Code(expr._items.reduce((items, c) => {
  	        if (c instanceof code_1.Name)
  	            c = replaceName(c);
  	        if (c instanceof code_1._Code)
  	            items.push(...c._items);
  	        else
  	            items.push(c);
  	        return items;
  	    }, []));
  	    function replaceName(n) {
  	        const c = constants[n.str];
  	        if (c === undefined || names[n.str] !== 1)
  	            return n;
  	        delete names[n.str];
  	        return c;
  	    }
  	    function canOptimize(e) {
  	        return (e instanceof code_1._Code &&
  	            e._items.some((c) => c instanceof code_1.Name && names[c.str] === 1 && constants[c.str] !== undefined));
  	    }
  	}
  	function subtractNames(names, from) {
  	    for (const n in from)
  	        names[n] = (names[n] || 0) - (from[n] || 0);
  	}
  	function not(x) {
  	    return typeof x == "boolean" || typeof x == "number" || x === null ? !x : (0, code_1._) `!${par(x)}`;
  	}
  	exports.not = not;
  	const andCode = mappend(exports.operators.AND);
  	// boolean AND (&&) expression with the passed arguments
  	function and(...args) {
  	    return args.reduce(andCode);
  	}
  	exports.and = and;
  	const orCode = mappend(exports.operators.OR);
  	// boolean OR (||) expression with the passed arguments
  	function or(...args) {
  	    return args.reduce(orCode);
  	}
  	exports.or = or;
  	function mappend(op) {
  	    return (x, y) => (x === code_1.nil ? y : y === code_1.nil ? x : (0, code_1._) `${par(x)} ${op} ${par(y)}`);
  	}
  	function par(x) {
  	    return x instanceof code_1.Name ? x : (0, code_1._) `(${x})`;
  	}
  	
  } (codegen));

  var util = {};

  Object.defineProperty(util, "__esModule", { value: true });
  util.checkStrictMode = util.getErrorPath = util.Type = util.useFunc = util.setEvaluated = util.evaluatedPropsToName = util.mergeEvaluated = util.eachItem = util.unescapeJsonPointer = util.escapeJsonPointer = util.escapeFragment = util.unescapeFragment = util.schemaRefOrVal = util.schemaHasRulesButRef = util.schemaHasRules = util.checkUnknownRules = util.alwaysValidSchema = util.toHash = void 0;
  const codegen_1$s = codegen;
  const code_1$9 = code$1;
  // TODO refactor to use Set
  function toHash(arr) {
      const hash = {};
      for (const item of arr)
          hash[item] = true;
      return hash;
  }
  util.toHash = toHash;
  function alwaysValidSchema(it, schema) {
      if (typeof schema == "boolean")
          return schema;
      if (Object.keys(schema).length === 0)
          return true;
      checkUnknownRules(it, schema);
      return !schemaHasRules(schema, it.self.RULES.all);
  }
  util.alwaysValidSchema = alwaysValidSchema;
  function checkUnknownRules(it, schema = it.schema) {
      const { opts, self } = it;
      if (!opts.strictSchema)
          return;
      if (typeof schema === "boolean")
          return;
      const rules = self.RULES.keywords;
      for (const key in schema) {
          if (!rules[key])
              checkStrictMode(it, `unknown keyword: "${key}"`);
      }
  }
  util.checkUnknownRules = checkUnknownRules;
  function schemaHasRules(schema, rules) {
      if (typeof schema == "boolean")
          return !schema;
      for (const key in schema)
          if (rules[key])
              return true;
      return false;
  }
  util.schemaHasRules = schemaHasRules;
  function schemaHasRulesButRef(schema, RULES) {
      if (typeof schema == "boolean")
          return !schema;
      for (const key in schema)
          if (key !== "$ref" && RULES.all[key])
              return true;
      return false;
  }
  util.schemaHasRulesButRef = schemaHasRulesButRef;
  function schemaRefOrVal({ topSchemaRef, schemaPath }, schema, keyword, $data) {
      if (!$data) {
          if (typeof schema == "number" || typeof schema == "boolean")
              return schema;
          if (typeof schema == "string")
              return (0, codegen_1$s._) `${schema}`;
      }
      return (0, codegen_1$s._) `${topSchemaRef}${schemaPath}${(0, codegen_1$s.getProperty)(keyword)}`;
  }
  util.schemaRefOrVal = schemaRefOrVal;
  function unescapeFragment(str) {
      return unescapeJsonPointer(decodeURIComponent(str));
  }
  util.unescapeFragment = unescapeFragment;
  function escapeFragment(str) {
      return encodeURIComponent(escapeJsonPointer(str));
  }
  util.escapeFragment = escapeFragment;
  function escapeJsonPointer(str) {
      if (typeof str == "number")
          return `${str}`;
      return str.replace(/~/g, "~0").replace(/\//g, "~1");
  }
  util.escapeJsonPointer = escapeJsonPointer;
  function unescapeJsonPointer(str) {
      return str.replace(/~1/g, "/").replace(/~0/g, "~");
  }
  util.unescapeJsonPointer = unescapeJsonPointer;
  function eachItem(xs, f) {
      if (Array.isArray(xs)) {
          for (const x of xs)
              f(x);
      }
      else {
          f(xs);
      }
  }
  util.eachItem = eachItem;
  function makeMergeEvaluated({ mergeNames, mergeToName, mergeValues, resultToName, }) {
      return (gen, from, to, toName) => {
          const res = to === undefined
              ? from
              : to instanceof codegen_1$s.Name
                  ? (from instanceof codegen_1$s.Name ? mergeNames(gen, from, to) : mergeToName(gen, from, to), to)
                  : from instanceof codegen_1$s.Name
                      ? (mergeToName(gen, to, from), from)
                      : mergeValues(from, to);
          return toName === codegen_1$s.Name && !(res instanceof codegen_1$s.Name) ? resultToName(gen, res) : res;
      };
  }
  util.mergeEvaluated = {
      props: makeMergeEvaluated({
          mergeNames: (gen, from, to) => gen.if((0, codegen_1$s._) `${to} !== true && ${from} !== undefined`, () => {
              gen.if((0, codegen_1$s._) `${from} === true`, () => gen.assign(to, true), () => gen.assign(to, (0, codegen_1$s._) `${to} || {}`).code((0, codegen_1$s._) `Object.assign(${to}, ${from})`));
          }),
          mergeToName: (gen, from, to) => gen.if((0, codegen_1$s._) `${to} !== true`, () => {
              if (from === true) {
                  gen.assign(to, true);
              }
              else {
                  gen.assign(to, (0, codegen_1$s._) `${to} || {}`);
                  setEvaluated(gen, to, from);
              }
          }),
          mergeValues: (from, to) => (from === true ? true : { ...from, ...to }),
          resultToName: evaluatedPropsToName,
      }),
      items: makeMergeEvaluated({
          mergeNames: (gen, from, to) => gen.if((0, codegen_1$s._) `${to} !== true && ${from} !== undefined`, () => gen.assign(to, (0, codegen_1$s._) `${from} === true ? true : ${to} > ${from} ? ${to} : ${from}`)),
          mergeToName: (gen, from, to) => gen.if((0, codegen_1$s._) `${to} !== true`, () => gen.assign(to, from === true ? true : (0, codegen_1$s._) `${to} > ${from} ? ${to} : ${from}`)),
          mergeValues: (from, to) => (from === true ? true : Math.max(from, to)),
          resultToName: (gen, items) => gen.var("items", items),
      }),
  };
  function evaluatedPropsToName(gen, ps) {
      if (ps === true)
          return gen.var("props", true);
      const props = gen.var("props", (0, codegen_1$s._) `{}`);
      if (ps !== undefined)
          setEvaluated(gen, props, ps);
      return props;
  }
  util.evaluatedPropsToName = evaluatedPropsToName;
  function setEvaluated(gen, props, ps) {
      Object.keys(ps).forEach((p) => gen.assign((0, codegen_1$s._) `${props}${(0, codegen_1$s.getProperty)(p)}`, true));
  }
  util.setEvaluated = setEvaluated;
  const snippets = {};
  function useFunc(gen, f) {
      return gen.scopeValue("func", {
          ref: f,
          code: snippets[f.code] || (snippets[f.code] = new code_1$9._Code(f.code)),
      });
  }
  util.useFunc = useFunc;
  var Type;
  (function (Type) {
      Type[Type["Num"] = 0] = "Num";
      Type[Type["Str"] = 1] = "Str";
  })(Type || (util.Type = Type = {}));
  function getErrorPath(dataProp, dataPropType, jsPropertySyntax) {
      // let path
      if (dataProp instanceof codegen_1$s.Name) {
          const isNumber = dataPropType === Type.Num;
          return jsPropertySyntax
              ? isNumber
                  ? (0, codegen_1$s._) `"[" + ${dataProp} + "]"`
                  : (0, codegen_1$s._) `"['" + ${dataProp} + "']"`
              : isNumber
                  ? (0, codegen_1$s._) `"/" + ${dataProp}`
                  : (0, codegen_1$s._) `"/" + ${dataProp}.replace(/~/g, "~0").replace(/\\//g, "~1")`; // TODO maybe use global escapePointer
      }
      return jsPropertySyntax ? (0, codegen_1$s.getProperty)(dataProp).toString() : "/" + escapeJsonPointer(dataProp);
  }
  util.getErrorPath = getErrorPath;
  function checkStrictMode(it, msg, mode = it.opts.strictSchema) {
      if (!mode)
          return;
      msg = `strict mode: ${msg}`;
      if (mode === true)
          throw new Error(msg);
      it.self.logger.warn(msg);
  }
  util.checkStrictMode = checkStrictMode;

  var names = {};

  var hasRequiredNames;

  function requireNames () {
  	if (hasRequiredNames) return names;
  	hasRequiredNames = 1;
  	Object.defineProperty(names, "__esModule", { value: true });
  	const codegen_1 = codegen;
  	const names$1 = {
  	    // validation function arguments
  	    data: new codegen_1.Name("data"), // data passed to validation function
  	    // args passed from referencing schema
  	    valCxt: new codegen_1.Name("valCxt"), // validation/data context - should not be used directly, it is destructured to the names below
  	    instancePath: new codegen_1.Name("instancePath"),
  	    parentData: new codegen_1.Name("parentData"),
  	    parentDataProperty: new codegen_1.Name("parentDataProperty"),
  	    rootData: new codegen_1.Name("rootData"), // root data - same as the data passed to the first/top validation function
  	    dynamicAnchors: new codegen_1.Name("dynamicAnchors"), // used to support recursiveRef and dynamicRef
  	    // function scoped variables
  	    vErrors: new codegen_1.Name("vErrors"), // null or array of validation errors
  	    errors: new codegen_1.Name("errors"), // counter of validation errors
  	    this: new codegen_1.Name("this"),
  	    // "globals"
  	    self: new codegen_1.Name("self"),
  	    scope: new codegen_1.Name("scope"),
  	    // JTD serialize/parse name for JSON string and position
  	    json: new codegen_1.Name("json"),
  	    jsonPos: new codegen_1.Name("jsonPos"),
  	    jsonLen: new codegen_1.Name("jsonLen"),
  	    jsonPart: new codegen_1.Name("jsonPart"),
  	};
  	names.default = names$1;
  	
  	return names;
  }

  var hasRequiredErrors;

  function requireErrors () {
  	if (hasRequiredErrors) return errors;
  	hasRequiredErrors = 1;
  	(function (exports) {
  		Object.defineProperty(exports, "__esModule", { value: true });
  		exports.extendErrors = exports.resetErrorsCount = exports.reportExtraError = exports.reportError = exports.keyword$DataError = exports.keywordError = void 0;
  		const codegen_1 = codegen;
  		const util_1 = util;
  		const names_1 = requireNames();
  		exports.keywordError = {
  		    message: ({ keyword }) => (0, codegen_1.str) `must pass "${keyword}" keyword validation`,
  		};
  		exports.keyword$DataError = {
  		    message: ({ keyword, schemaType }) => schemaType
  		        ? (0, codegen_1.str) `"${keyword}" keyword must be ${schemaType} ($data)`
  		        : (0, codegen_1.str) `"${keyword}" keyword is invalid ($data)`,
  		};
  		function reportError(cxt, error = exports.keywordError, errorPaths, overrideAllErrors) {
  		    const { it } = cxt;
  		    const { gen, compositeRule, allErrors } = it;
  		    const errObj = errorObjectCode(cxt, error, errorPaths);
  		    if (overrideAllErrors !== null && overrideAllErrors !== void 0 ? overrideAllErrors : (compositeRule || allErrors)) {
  		        addError(gen, errObj);
  		    }
  		    else {
  		        returnErrors(it, (0, codegen_1._) `[${errObj}]`);
  		    }
  		}
  		exports.reportError = reportError;
  		function reportExtraError(cxt, error = exports.keywordError, errorPaths) {
  		    const { it } = cxt;
  		    const { gen, compositeRule, allErrors } = it;
  		    const errObj = errorObjectCode(cxt, error, errorPaths);
  		    addError(gen, errObj);
  		    if (!(compositeRule || allErrors)) {
  		        returnErrors(it, names_1.default.vErrors);
  		    }
  		}
  		exports.reportExtraError = reportExtraError;
  		function resetErrorsCount(gen, errsCount) {
  		    gen.assign(names_1.default.errors, errsCount);
  		    gen.if((0, codegen_1._) `${names_1.default.vErrors} !== null`, () => gen.if(errsCount, () => gen.assign((0, codegen_1._) `${names_1.default.vErrors}.length`, errsCount), () => gen.assign(names_1.default.vErrors, null)));
  		}
  		exports.resetErrorsCount = resetErrorsCount;
  		function extendErrors({ gen, keyword, schemaValue, data, errsCount, it, }) {
  		    /* istanbul ignore if */
  		    if (errsCount === undefined)
  		        throw new Error("ajv implementation error");
  		    const err = gen.name("err");
  		    gen.forRange("i", errsCount, names_1.default.errors, (i) => {
  		        gen.const(err, (0, codegen_1._) `${names_1.default.vErrors}[${i}]`);
  		        gen.if((0, codegen_1._) `${err}.instancePath === undefined`, () => gen.assign((0, codegen_1._) `${err}.instancePath`, (0, codegen_1.strConcat)(names_1.default.instancePath, it.errorPath)));
  		        gen.assign((0, codegen_1._) `${err}.schemaPath`, (0, codegen_1.str) `${it.errSchemaPath}/${keyword}`);
  		        if (it.opts.verbose) {
  		            gen.assign((0, codegen_1._) `${err}.schema`, schemaValue);
  		            gen.assign((0, codegen_1._) `${err}.data`, data);
  		        }
  		    });
  		}
  		exports.extendErrors = extendErrors;
  		function addError(gen, errObj) {
  		    const err = gen.const("err", errObj);
  		    gen.if((0, codegen_1._) `${names_1.default.vErrors} === null`, () => gen.assign(names_1.default.vErrors, (0, codegen_1._) `[${err}]`), (0, codegen_1._) `${names_1.default.vErrors}.push(${err})`);
  		    gen.code((0, codegen_1._) `${names_1.default.errors}++`);
  		}
  		function returnErrors(it, errs) {
  		    const { gen, validateName, schemaEnv } = it;
  		    if (schemaEnv.$async) {
  		        gen.throw((0, codegen_1._) `new ${it.ValidationError}(${errs})`);
  		    }
  		    else {
  		        gen.assign((0, codegen_1._) `${validateName}.errors`, errs);
  		        gen.return(false);
  		    }
  		}
  		const E = {
  		    keyword: new codegen_1.Name("keyword"),
  		    schemaPath: new codegen_1.Name("schemaPath"), // also used in JTD errors
  		    params: new codegen_1.Name("params"),
  		    propertyName: new codegen_1.Name("propertyName"),
  		    message: new codegen_1.Name("message"),
  		    schema: new codegen_1.Name("schema"),
  		    parentSchema: new codegen_1.Name("parentSchema"),
  		};
  		function errorObjectCode(cxt, error, errorPaths) {
  		    const { createErrors } = cxt.it;
  		    if (createErrors === false)
  		        return (0, codegen_1._) `{}`;
  		    return errorObject(cxt, error, errorPaths);
  		}
  		function errorObject(cxt, error, errorPaths = {}) {
  		    const { gen, it } = cxt;
  		    const keyValues = [
  		        errorInstancePath(it, errorPaths),
  		        errorSchemaPath(cxt, errorPaths),
  		    ];
  		    extraErrorProps(cxt, error, keyValues);
  		    return gen.object(...keyValues);
  		}
  		function errorInstancePath({ errorPath }, { instancePath }) {
  		    const instPath = instancePath
  		        ? (0, codegen_1.str) `${errorPath}${(0, util_1.getErrorPath)(instancePath, util_1.Type.Str)}`
  		        : errorPath;
  		    return [names_1.default.instancePath, (0, codegen_1.strConcat)(names_1.default.instancePath, instPath)];
  		}
  		function errorSchemaPath({ keyword, it: { errSchemaPath } }, { schemaPath, parentSchema }) {
  		    let schPath = parentSchema ? errSchemaPath : (0, codegen_1.str) `${errSchemaPath}/${keyword}`;
  		    if (schemaPath) {
  		        schPath = (0, codegen_1.str) `${schPath}${(0, util_1.getErrorPath)(schemaPath, util_1.Type.Str)}`;
  		    }
  		    return [E.schemaPath, schPath];
  		}
  		function extraErrorProps(cxt, { params, message }, keyValues) {
  		    const { keyword, data, schemaValue, it } = cxt;
  		    const { opts, propertyName, topSchemaRef, schemaPath } = it;
  		    keyValues.push([E.keyword, keyword], [E.params, typeof params == "function" ? params(cxt) : params || (0, codegen_1._) `{}`]);
  		    if (opts.messages) {
  		        keyValues.push([E.message, typeof message == "function" ? message(cxt) : message]);
  		    }
  		    if (opts.verbose) {
  		        keyValues.push([E.schema, schemaValue], [E.parentSchema, (0, codegen_1._) `${topSchemaRef}${schemaPath}`], [names_1.default.data, data]);
  		    }
  		    if (propertyName)
  		        keyValues.push([E.propertyName, propertyName]);
  		}
  		
  	} (errors));
  	return errors;
  }

  var hasRequiredBoolSchema;

  function requireBoolSchema () {
  	if (hasRequiredBoolSchema) return boolSchema;
  	hasRequiredBoolSchema = 1;
  	Object.defineProperty(boolSchema, "__esModule", { value: true });
  	boolSchema.boolOrEmptySchema = boolSchema.topBoolOrEmptySchema = void 0;
  	const errors_1 = requireErrors();
  	const codegen_1 = codegen;
  	const names_1 = requireNames();
  	const boolError = {
  	    message: "boolean schema is false",
  	};
  	function topBoolOrEmptySchema(it) {
  	    const { gen, schema, validateName } = it;
  	    if (schema === false) {
  	        falseSchemaError(it, false);
  	    }
  	    else if (typeof schema == "object" && schema.$async === true) {
  	        gen.return(names_1.default.data);
  	    }
  	    else {
  	        gen.assign((0, codegen_1._) `${validateName}.errors`, null);
  	        gen.return(true);
  	    }
  	}
  	boolSchema.topBoolOrEmptySchema = topBoolOrEmptySchema;
  	function boolOrEmptySchema(it, valid) {
  	    const { gen, schema } = it;
  	    if (schema === false) {
  	        gen.var(valid, false); // TODO var
  	        falseSchemaError(it);
  	    }
  	    else {
  	        gen.var(valid, true); // TODO var
  	    }
  	}
  	boolSchema.boolOrEmptySchema = boolOrEmptySchema;
  	function falseSchemaError(it, overrideAllErrors) {
  	    const { gen, data } = it;
  	    // TODO maybe some other interface should be used for non-keyword validation errors...
  	    const cxt = {
  	        gen,
  	        keyword: "false schema",
  	        data,
  	        schema: false,
  	        schemaCode: false,
  	        schemaValue: false,
  	        params: {},
  	        it,
  	    };
  	    (0, errors_1.reportError)(cxt, boolError, undefined, overrideAllErrors);
  	}
  	
  	return boolSchema;
  }

  var dataType = {};

  var rules = {};

  Object.defineProperty(rules, "__esModule", { value: true });
  rules.getRules = rules.isJSONType = void 0;
  const _jsonTypes = ["string", "number", "integer", "boolean", "null", "object", "array"];
  const jsonTypes = new Set(_jsonTypes);
  function isJSONType(x) {
      return typeof x == "string" && jsonTypes.has(x);
  }
  rules.isJSONType = isJSONType;
  function getRules() {
      const groups = {
          number: { type: "number", rules: [] },
          string: { type: "string", rules: [] },
          array: { type: "array", rules: [] },
          object: { type: "object", rules: [] },
      };
      return {
          types: { ...groups, integer: true, boolean: true, null: true },
          rules: [{ rules: [] }, groups.number, groups.string, groups.array, groups.object],
          post: { rules: [] },
          all: {},
          keywords: {},
      };
  }
  rules.getRules = getRules;

  var applicability = {};

  var hasRequiredApplicability;

  function requireApplicability () {
  	if (hasRequiredApplicability) return applicability;
  	hasRequiredApplicability = 1;
  	Object.defineProperty(applicability, "__esModule", { value: true });
  	applicability.shouldUseRule = applicability.shouldUseGroup = applicability.schemaHasRulesForType = void 0;
  	function schemaHasRulesForType({ schema, self }, type) {
  	    const group = self.RULES.types[type];
  	    return group && group !== true && shouldUseGroup(schema, group);
  	}
  	applicability.schemaHasRulesForType = schemaHasRulesForType;
  	function shouldUseGroup(schema, group) {
  	    return group.rules.some((rule) => shouldUseRule(schema, rule));
  	}
  	applicability.shouldUseGroup = shouldUseGroup;
  	function shouldUseRule(schema, rule) {
  	    var _a;
  	    return (schema[rule.keyword] !== undefined ||
  	        ((_a = rule.definition.implements) === null || _a === void 0 ? void 0 : _a.some((kwd) => schema[kwd] !== undefined)));
  	}
  	applicability.shouldUseRule = shouldUseRule;
  	
  	return applicability;
  }

  Object.defineProperty(dataType, "__esModule", { value: true });
  dataType.reportTypeError = dataType.checkDataTypes = dataType.checkDataType = dataType.coerceAndCheckDataType = dataType.getJSONTypes = dataType.getSchemaTypes = dataType.DataType = void 0;
  const rules_1 = rules;
  const applicability_1 = requireApplicability();
  const errors_1 = requireErrors();
  const codegen_1$r = codegen;
  const util_1$q = util;
  var DataType;
  (function (DataType) {
      DataType[DataType["Correct"] = 0] = "Correct";
      DataType[DataType["Wrong"] = 1] = "Wrong";
  })(DataType || (dataType.DataType = DataType = {}));
  function getSchemaTypes(schema) {
      const types = getJSONTypes(schema.type);
      const hasNull = types.includes("null");
      if (hasNull) {
          if (schema.nullable === false)
              throw new Error("type: null contradicts nullable: false");
      }
      else {
          if (!types.length && schema.nullable !== undefined) {
              throw new Error('"nullable" cannot be used without "type"');
          }
          if (schema.nullable === true)
              types.push("null");
      }
      return types;
  }
  dataType.getSchemaTypes = getSchemaTypes;
  // eslint-disable-next-line @typescript-eslint/no-redundant-type-constituents
  function getJSONTypes(ts) {
      const types = Array.isArray(ts) ? ts : ts ? [ts] : [];
      if (types.every(rules_1.isJSONType))
          return types;
      throw new Error("type must be JSONType or JSONType[]: " + types.join(","));
  }
  dataType.getJSONTypes = getJSONTypes;
  function coerceAndCheckDataType(it, types) {
      const { gen, data, opts } = it;
      const coerceTo = coerceToTypes(types, opts.coerceTypes);
      const checkTypes = types.length > 0 &&
          !(coerceTo.length === 0 && types.length === 1 && (0, applicability_1.schemaHasRulesForType)(it, types[0]));
      if (checkTypes) {
          const wrongType = checkDataTypes(types, data, opts.strictNumbers, DataType.Wrong);
          gen.if(wrongType, () => {
              if (coerceTo.length)
                  coerceData(it, types, coerceTo);
              else
                  reportTypeError(it);
          });
      }
      return checkTypes;
  }
  dataType.coerceAndCheckDataType = coerceAndCheckDataType;
  const COERCIBLE = new Set(["string", "number", "integer", "boolean", "null"]);
  function coerceToTypes(types, coerceTypes) {
      return coerceTypes
          ? types.filter((t) => COERCIBLE.has(t) || (coerceTypes === "array" && t === "array"))
          : [];
  }
  function coerceData(it, types, coerceTo) {
      const { gen, data, opts } = it;
      const dataType = gen.let("dataType", (0, codegen_1$r._) `typeof ${data}`);
      const coerced = gen.let("coerced", (0, codegen_1$r._) `undefined`);
      if (opts.coerceTypes === "array") {
          gen.if((0, codegen_1$r._) `${dataType} == 'object' && Array.isArray(${data}) && ${data}.length == 1`, () => gen
              .assign(data, (0, codegen_1$r._) `${data}[0]`)
              .assign(dataType, (0, codegen_1$r._) `typeof ${data}`)
              .if(checkDataTypes(types, data, opts.strictNumbers), () => gen.assign(coerced, data)));
      }
      gen.if((0, codegen_1$r._) `${coerced} !== undefined`);
      for (const t of coerceTo) {
          if (COERCIBLE.has(t) || (t === "array" && opts.coerceTypes === "array")) {
              coerceSpecificType(t);
          }
      }
      gen.else();
      reportTypeError(it);
      gen.endIf();
      gen.if((0, codegen_1$r._) `${coerced} !== undefined`, () => {
          gen.assign(data, coerced);
          assignParentData(it, coerced);
      });
      function coerceSpecificType(t) {
          switch (t) {
              case "string":
                  gen
                      .elseIf((0, codegen_1$r._) `${dataType} == "number" || ${dataType} == "boolean"`)
                      .assign(coerced, (0, codegen_1$r._) `"" + ${data}`)
                      .elseIf((0, codegen_1$r._) `${data} === null`)
                      .assign(coerced, (0, codegen_1$r._) `""`);
                  return;
              case "number":
                  gen
                      .elseIf((0, codegen_1$r._) `${dataType} == "boolean" || ${data} === null
              || (${dataType} == "string" && ${data} && ${data} == +${data})`)
                      .assign(coerced, (0, codegen_1$r._) `+${data}`);
                  return;
              case "integer":
                  gen
                      .elseIf((0, codegen_1$r._) `${dataType} === "boolean" || ${data} === null
              || (${dataType} === "string" && ${data} && ${data} == +${data} && !(${data} % 1))`)
                      .assign(coerced, (0, codegen_1$r._) `+${data}`);
                  return;
              case "boolean":
                  gen
                      .elseIf((0, codegen_1$r._) `${data} === "false" || ${data} === 0 || ${data} === null`)
                      .assign(coerced, false)
                      .elseIf((0, codegen_1$r._) `${data} === "true" || ${data} === 1`)
                      .assign(coerced, true);
                  return;
              case "null":
                  gen.elseIf((0, codegen_1$r._) `${data} === "" || ${data} === 0 || ${data} === false`);
                  gen.assign(coerced, null);
                  return;
              case "array":
                  gen
                      .elseIf((0, codegen_1$r._) `${dataType} === "string" || ${dataType} === "number"
              || ${dataType} === "boolean" || ${data} === null`)
                      .assign(coerced, (0, codegen_1$r._) `[${data}]`);
          }
      }
  }
  function assignParentData({ gen, parentData, parentDataProperty }, expr) {
      // TODO use gen.property
      gen.if((0, codegen_1$r._) `${parentData} !== undefined`, () => gen.assign((0, codegen_1$r._) `${parentData}[${parentDataProperty}]`, expr));
  }
  function checkDataType(dataType, data, strictNums, correct = DataType.Correct) {
      const EQ = correct === DataType.Correct ? codegen_1$r.operators.EQ : codegen_1$r.operators.NEQ;
      let cond;
      switch (dataType) {
          case "null":
              return (0, codegen_1$r._) `${data} ${EQ} null`;
          case "array":
              cond = (0, codegen_1$r._) `Array.isArray(${data})`;
              break;
          case "object":
              cond = (0, codegen_1$r._) `${data} && typeof ${data} == "object" && !Array.isArray(${data})`;
              break;
          case "integer":
              cond = numCond((0, codegen_1$r._) `!(${data} % 1) && !isNaN(${data})`);
              break;
          case "number":
              cond = numCond();
              break;
          default:
              return (0, codegen_1$r._) `typeof ${data} ${EQ} ${dataType}`;
      }
      return correct === DataType.Correct ? cond : (0, codegen_1$r.not)(cond);
      function numCond(_cond = codegen_1$r.nil) {
          return (0, codegen_1$r.and)((0, codegen_1$r._) `typeof ${data} == "number"`, _cond, strictNums ? (0, codegen_1$r._) `isFinite(${data})` : codegen_1$r.nil);
      }
  }
  dataType.checkDataType = checkDataType;
  function checkDataTypes(dataTypes, data, strictNums, correct) {
      if (dataTypes.length === 1) {
          return checkDataType(dataTypes[0], data, strictNums, correct);
      }
      let cond;
      const types = (0, util_1$q.toHash)(dataTypes);
      if (types.array && types.object) {
          const notObj = (0, codegen_1$r._) `typeof ${data} != "object"`;
          cond = types.null ? notObj : (0, codegen_1$r._) `!${data} || ${notObj}`;
          delete types.null;
          delete types.array;
          delete types.object;
      }
      else {
          cond = codegen_1$r.nil;
      }
      if (types.number)
          delete types.integer;
      for (const t in types)
          cond = (0, codegen_1$r.and)(cond, checkDataType(t, data, strictNums, correct));
      return cond;
  }
  dataType.checkDataTypes = checkDataTypes;
  const typeError = {
      message: ({ schema }) => `must be ${schema}`,
      params: ({ schema, schemaValue }) => typeof schema == "string" ? (0, codegen_1$r._) `{type: ${schema}}` : (0, codegen_1$r._) `{type: ${schemaValue}}`,
  };
  function reportTypeError(it) {
      const cxt = getTypeErrorContext(it);
      (0, errors_1.reportError)(cxt, typeError);
  }
  dataType.reportTypeError = reportTypeError;
  function getTypeErrorContext(it) {
      const { gen, data, schema } = it;
      const schemaCode = (0, util_1$q.schemaRefOrVal)(it, schema, "type");
      return {
          gen,
          keyword: "type",
          data,
          schema: schema.type,
          schemaCode,
          schemaValue: schemaCode,
          parentSchema: schema,
          params: {},
          it,
      };
  }

  var defaults = {};

  var hasRequiredDefaults;

  function requireDefaults () {
  	if (hasRequiredDefaults) return defaults;
  	hasRequiredDefaults = 1;
  	Object.defineProperty(defaults, "__esModule", { value: true });
  	defaults.assignDefaults = void 0;
  	const codegen_1 = codegen;
  	const util_1 = util;
  	function assignDefaults(it, ty) {
  	    const { properties, items } = it.schema;
  	    if (ty === "object" && properties) {
  	        for (const key in properties) {
  	            assignDefault(it, key, properties[key].default);
  	        }
  	    }
  	    else if (ty === "array" && Array.isArray(items)) {
  	        items.forEach((sch, i) => assignDefault(it, i, sch.default));
  	    }
  	}
  	defaults.assignDefaults = assignDefaults;
  	function assignDefault(it, prop, defaultValue) {
  	    const { gen, compositeRule, data, opts } = it;
  	    if (defaultValue === undefined)
  	        return;
  	    const childData = (0, codegen_1._) `${data}${(0, codegen_1.getProperty)(prop)}`;
  	    if (compositeRule) {
  	        (0, util_1.checkStrictMode)(it, `default is ignored for: ${childData}`);
  	        return;
  	    }
  	    let condition = (0, codegen_1._) `${childData} === undefined`;
  	    if (opts.useDefaults === "empty") {
  	        condition = (0, codegen_1._) `${condition} || ${childData} === null || ${childData} === ""`;
  	    }
  	    // `${childData} === undefined` +
  	    // (opts.useDefaults === "empty" ? ` || ${childData} === null || ${childData} === ""` : "")
  	    gen.if(condition, (0, codegen_1._) `${childData} = ${(0, codegen_1.stringify)(defaultValue)}`);
  	}
  	
  	return defaults;
  }

  var keyword = {};

  var code = {};

  var hasRequiredCode;

  function requireCode () {
  	if (hasRequiredCode) return code;
  	hasRequiredCode = 1;
  	Object.defineProperty(code, "__esModule", { value: true });
  	code.validateUnion = code.validateArray = code.usePattern = code.callValidateCode = code.schemaProperties = code.allSchemaProperties = code.noPropertyInData = code.propertyInData = code.isOwnProperty = code.hasPropFunc = code.reportMissingProp = code.checkMissingProp = code.checkReportMissingProp = void 0;
  	const codegen_1 = codegen;
  	const util_1 = util;
  	const names_1 = requireNames();
  	const util_2 = util;
  	function checkReportMissingProp(cxt, prop) {
  	    const { gen, data, it } = cxt;
  	    gen.if(noPropertyInData(gen, data, prop, it.opts.ownProperties), () => {
  	        cxt.setParams({ missingProperty: (0, codegen_1._) `${prop}` }, true);
  	        cxt.error();
  	    });
  	}
  	code.checkReportMissingProp = checkReportMissingProp;
  	function checkMissingProp({ gen, data, it: { opts } }, properties, missing) {
  	    return (0, codegen_1.or)(...properties.map((prop) => (0, codegen_1.and)(noPropertyInData(gen, data, prop, opts.ownProperties), (0, codegen_1._) `${missing} = ${prop}`)));
  	}
  	code.checkMissingProp = checkMissingProp;
  	function reportMissingProp(cxt, missing) {
  	    cxt.setParams({ missingProperty: missing }, true);
  	    cxt.error();
  	}
  	code.reportMissingProp = reportMissingProp;
  	function hasPropFunc(gen) {
  	    return gen.scopeValue("func", {
  	        // eslint-disable-next-line @typescript-eslint/unbound-method
  	        ref: Object.prototype.hasOwnProperty,
  	        code: (0, codegen_1._) `Object.prototype.hasOwnProperty`,
  	    });
  	}
  	code.hasPropFunc = hasPropFunc;
  	function isOwnProperty(gen, data, property) {
  	    return (0, codegen_1._) `${hasPropFunc(gen)}.call(${data}, ${property})`;
  	}
  	code.isOwnProperty = isOwnProperty;
  	function propertyInData(gen, data, property, ownProperties) {
  	    const cond = (0, codegen_1._) `${data}${(0, codegen_1.getProperty)(property)} !== undefined`;
  	    return ownProperties ? (0, codegen_1._) `${cond} && ${isOwnProperty(gen, data, property)}` : cond;
  	}
  	code.propertyInData = propertyInData;
  	function noPropertyInData(gen, data, property, ownProperties) {
  	    const cond = (0, codegen_1._) `${data}${(0, codegen_1.getProperty)(property)} === undefined`;
  	    return ownProperties ? (0, codegen_1.or)(cond, (0, codegen_1.not)(isOwnProperty(gen, data, property))) : cond;
  	}
  	code.noPropertyInData = noPropertyInData;
  	function allSchemaProperties(schemaMap) {
  	    return schemaMap ? Object.keys(schemaMap).filter((p) => p !== "__proto__") : [];
  	}
  	code.allSchemaProperties = allSchemaProperties;
  	function schemaProperties(it, schemaMap) {
  	    return allSchemaProperties(schemaMap).filter((p) => !(0, util_1.alwaysValidSchema)(it, schemaMap[p]));
  	}
  	code.schemaProperties = schemaProperties;
  	function callValidateCode({ schemaCode, data, it: { gen, topSchemaRef, schemaPath, errorPath }, it }, func, context, passSchema) {
  	    const dataAndSchema = passSchema ? (0, codegen_1._) `${schemaCode}, ${data}, ${topSchemaRef}${schemaPath}` : data;
  	    const valCxt = [
  	        [names_1.default.instancePath, (0, codegen_1.strConcat)(names_1.default.instancePath, errorPath)],
  	        [names_1.default.parentData, it.parentData],
  	        [names_1.default.parentDataProperty, it.parentDataProperty],
  	        [names_1.default.rootData, names_1.default.rootData],
  	    ];
  	    if (it.opts.dynamicRef)
  	        valCxt.push([names_1.default.dynamicAnchors, names_1.default.dynamicAnchors]);
  	    const args = (0, codegen_1._) `${dataAndSchema}, ${gen.object(...valCxt)}`;
  	    return context !== codegen_1.nil ? (0, codegen_1._) `${func}.call(${context}, ${args})` : (0, codegen_1._) `${func}(${args})`;
  	}
  	code.callValidateCode = callValidateCode;
  	const newRegExp = (0, codegen_1._) `new RegExp`;
  	function usePattern({ gen, it: { opts } }, pattern) {
  	    const u = opts.unicodeRegExp ? "u" : "";
  	    const { regExp } = opts.code;
  	    const rx = regExp(pattern, u);
  	    return gen.scopeValue("pattern", {
  	        key: rx.toString(),
  	        ref: rx,
  	        code: (0, codegen_1._) `${regExp.code === "new RegExp" ? newRegExp : (0, util_2.useFunc)(gen, regExp)}(${pattern}, ${u})`,
  	    });
  	}
  	code.usePattern = usePattern;
  	function validateArray(cxt) {
  	    const { gen, data, keyword, it } = cxt;
  	    const valid = gen.name("valid");
  	    if (it.allErrors) {
  	        const validArr = gen.let("valid", true);
  	        validateItems(() => gen.assign(validArr, false));
  	        return validArr;
  	    }
  	    gen.var(valid, true);
  	    validateItems(() => gen.break());
  	    return valid;
  	    function validateItems(notValid) {
  	        const len = gen.const("len", (0, codegen_1._) `${data}.length`);
  	        gen.forRange("i", 0, len, (i) => {
  	            cxt.subschema({
  	                keyword,
  	                dataProp: i,
  	                dataPropType: util_1.Type.Num,
  	            }, valid);
  	            gen.if((0, codegen_1.not)(valid), notValid);
  	        });
  	    }
  	}
  	code.validateArray = validateArray;
  	function validateUnion(cxt) {
  	    const { gen, schema, keyword, it } = cxt;
  	    /* istanbul ignore if */
  	    if (!Array.isArray(schema))
  	        throw new Error("ajv implementation error");
  	    const alwaysValid = schema.some((sch) => (0, util_1.alwaysValidSchema)(it, sch));
  	    if (alwaysValid && !it.opts.unevaluated)
  	        return;
  	    const valid = gen.let("valid", false);
  	    const schValid = gen.name("_valid");
  	    gen.block(() => schema.forEach((_sch, i) => {
  	        const schCxt = cxt.subschema({
  	            keyword,
  	            schemaProp: i,
  	            compositeRule: true,
  	        }, schValid);
  	        gen.assign(valid, (0, codegen_1._) `${valid} || ${schValid}`);
  	        const merged = cxt.mergeValidEvaluated(schCxt, schValid);
  	        // can short-circuit if `unevaluatedProperties/Items` not supported (opts.unevaluated !== true)
  	        // or if all properties and items were evaluated (it.props === true && it.items === true)
  	        if (!merged)
  	            gen.if((0, codegen_1.not)(valid));
  	    }));
  	    cxt.result(valid, () => cxt.reset(), () => cxt.error(true));
  	}
  	code.validateUnion = validateUnion;
  	
  	return code;
  }

  var hasRequiredKeyword;

  function requireKeyword () {
  	if (hasRequiredKeyword) return keyword;
  	hasRequiredKeyword = 1;
  	Object.defineProperty(keyword, "__esModule", { value: true });
  	keyword.validateKeywordUsage = keyword.validSchemaType = keyword.funcKeywordCode = keyword.macroKeywordCode = void 0;
  	const codegen_1 = codegen;
  	const names_1 = requireNames();
  	const code_1 = requireCode();
  	const errors_1 = requireErrors();
  	function macroKeywordCode(cxt, def) {
  	    const { gen, keyword, schema, parentSchema, it } = cxt;
  	    const macroSchema = def.macro.call(it.self, schema, parentSchema, it);
  	    const schemaRef = useKeyword(gen, keyword, macroSchema);
  	    if (it.opts.validateSchema !== false)
  	        it.self.validateSchema(macroSchema, true);
  	    const valid = gen.name("valid");
  	    cxt.subschema({
  	        schema: macroSchema,
  	        schemaPath: codegen_1.nil,
  	        errSchemaPath: `${it.errSchemaPath}/${keyword}`,
  	        topSchemaRef: schemaRef,
  	        compositeRule: true,
  	    }, valid);
  	    cxt.pass(valid, () => cxt.error(true));
  	}
  	keyword.macroKeywordCode = macroKeywordCode;
  	function funcKeywordCode(cxt, def) {
  	    var _a;
  	    const { gen, keyword, schema, parentSchema, $data, it } = cxt;
  	    checkAsyncKeyword(it, def);
  	    const validate = !$data && def.compile ? def.compile.call(it.self, schema, parentSchema, it) : def.validate;
  	    const validateRef = useKeyword(gen, keyword, validate);
  	    const valid = gen.let("valid");
  	    cxt.block$data(valid, validateKeyword);
  	    cxt.ok((_a = def.valid) !== null && _a !== void 0 ? _a : valid);
  	    function validateKeyword() {
  	        if (def.errors === false) {
  	            assignValid();
  	            if (def.modifying)
  	                modifyData(cxt);
  	            reportErrs(() => cxt.error());
  	        }
  	        else {
  	            const ruleErrs = def.async ? validateAsync() : validateSync();
  	            if (def.modifying)
  	                modifyData(cxt);
  	            reportErrs(() => addErrs(cxt, ruleErrs));
  	        }
  	    }
  	    function validateAsync() {
  	        const ruleErrs = gen.let("ruleErrs", null);
  	        gen.try(() => assignValid((0, codegen_1._) `await `), (e) => gen.assign(valid, false).if((0, codegen_1._) `${e} instanceof ${it.ValidationError}`, () => gen.assign(ruleErrs, (0, codegen_1._) `${e}.errors`), () => gen.throw(e)));
  	        return ruleErrs;
  	    }
  	    function validateSync() {
  	        const validateErrs = (0, codegen_1._) `${validateRef}.errors`;
  	        gen.assign(validateErrs, null);
  	        assignValid(codegen_1.nil);
  	        return validateErrs;
  	    }
  	    function assignValid(_await = def.async ? (0, codegen_1._) `await ` : codegen_1.nil) {
  	        const passCxt = it.opts.passContext ? names_1.default.this : names_1.default.self;
  	        const passSchema = !(("compile" in def && !$data) || def.schema === false);
  	        gen.assign(valid, (0, codegen_1._) `${_await}${(0, code_1.callValidateCode)(cxt, validateRef, passCxt, passSchema)}`, def.modifying);
  	    }
  	    function reportErrs(errors) {
  	        var _a;
  	        gen.if((0, codegen_1.not)((_a = def.valid) !== null && _a !== void 0 ? _a : valid), errors);
  	    }
  	}
  	keyword.funcKeywordCode = funcKeywordCode;
  	function modifyData(cxt) {
  	    const { gen, data, it } = cxt;
  	    gen.if(it.parentData, () => gen.assign(data, (0, codegen_1._) `${it.parentData}[${it.parentDataProperty}]`));
  	}
  	function addErrs(cxt, errs) {
  	    const { gen } = cxt;
  	    gen.if((0, codegen_1._) `Array.isArray(${errs})`, () => {
  	        gen
  	            .assign(names_1.default.vErrors, (0, codegen_1._) `${names_1.default.vErrors} === null ? ${errs} : ${names_1.default.vErrors}.concat(${errs})`)
  	            .assign(names_1.default.errors, (0, codegen_1._) `${names_1.default.vErrors}.length`);
  	        (0, errors_1.extendErrors)(cxt);
  	    }, () => cxt.error());
  	}
  	function checkAsyncKeyword({ schemaEnv }, def) {
  	    if (def.async && !schemaEnv.$async)
  	        throw new Error("async keyword in sync schema");
  	}
  	function useKeyword(gen, keyword, result) {
  	    if (result === undefined)
  	        throw new Error(`keyword "${keyword}" failed to compile`);
  	    return gen.scopeValue("keyword", typeof result == "function" ? { ref: result } : { ref: result, code: (0, codegen_1.stringify)(result) });
  	}
  	function validSchemaType(schema, schemaType, allowUndefined = false) {
  	    // TODO add tests
  	    return (!schemaType.length ||
  	        schemaType.some((st) => st === "array"
  	            ? Array.isArray(schema)
  	            : st === "object"
  	                ? schema && typeof schema == "object" && !Array.isArray(schema)
  	                : typeof schema == st || (allowUndefined && typeof schema == "undefined")));
  	}
  	keyword.validSchemaType = validSchemaType;
  	function validateKeywordUsage({ schema, opts, self, errSchemaPath }, def, keyword) {
  	    /* istanbul ignore if */
  	    if (Array.isArray(def.keyword) ? !def.keyword.includes(keyword) : def.keyword !== keyword) {
  	        throw new Error("ajv implementation error");
  	    }
  	    const deps = def.dependencies;
  	    if (deps === null || deps === void 0 ? void 0 : deps.some((kwd) => !Object.prototype.hasOwnProperty.call(schema, kwd))) {
  	        throw new Error(`parent schema must have dependencies of ${keyword}: ${deps.join(",")}`);
  	    }
  	    if (def.validateSchema) {
  	        const valid = def.validateSchema(schema[keyword]);
  	        if (!valid) {
  	            const msg = `keyword "${keyword}" value is invalid at path "${errSchemaPath}": ` +
  	                self.errorsText(def.validateSchema.errors);
  	            if (opts.validateSchema === "log")
  	                self.logger.error(msg);
  	            else
  	                throw new Error(msg);
  	        }
  	    }
  	}
  	keyword.validateKeywordUsage = validateKeywordUsage;
  	
  	return keyword;
  }

  var subschema = {};

  var hasRequiredSubschema;

  function requireSubschema () {
  	if (hasRequiredSubschema) return subschema;
  	hasRequiredSubschema = 1;
  	Object.defineProperty(subschema, "__esModule", { value: true });
  	subschema.extendSubschemaMode = subschema.extendSubschemaData = subschema.getSubschema = void 0;
  	const codegen_1 = codegen;
  	const util_1 = util;
  	function getSubschema(it, { keyword, schemaProp, schema, schemaPath, errSchemaPath, topSchemaRef }) {
  	    if (keyword !== undefined && schema !== undefined) {
  	        throw new Error('both "keyword" and "schema" passed, only one allowed');
  	    }
  	    if (keyword !== undefined) {
  	        const sch = it.schema[keyword];
  	        return schemaProp === undefined
  	            ? {
  	                schema: sch,
  	                schemaPath: (0, codegen_1._) `${it.schemaPath}${(0, codegen_1.getProperty)(keyword)}`,
  	                errSchemaPath: `${it.errSchemaPath}/${keyword}`,
  	            }
  	            : {
  	                schema: sch[schemaProp],
  	                schemaPath: (0, codegen_1._) `${it.schemaPath}${(0, codegen_1.getProperty)(keyword)}${(0, codegen_1.getProperty)(schemaProp)}`,
  	                errSchemaPath: `${it.errSchemaPath}/${keyword}/${(0, util_1.escapeFragment)(schemaProp)}`,
  	            };
  	    }
  	    if (schema !== undefined) {
  	        if (schemaPath === undefined || errSchemaPath === undefined || topSchemaRef === undefined) {
  	            throw new Error('"schemaPath", "errSchemaPath" and "topSchemaRef" are required with "schema"');
  	        }
  	        return {
  	            schema,
  	            schemaPath,
  	            topSchemaRef,
  	            errSchemaPath,
  	        };
  	    }
  	    throw new Error('either "keyword" or "schema" must be passed');
  	}
  	subschema.getSubschema = getSubschema;
  	function extendSubschemaData(subschema, it, { dataProp, dataPropType: dpType, data, dataTypes, propertyName }) {
  	    if (data !== undefined && dataProp !== undefined) {
  	        throw new Error('both "data" and "dataProp" passed, only one allowed');
  	    }
  	    const { gen } = it;
  	    if (dataProp !== undefined) {
  	        const { errorPath, dataPathArr, opts } = it;
  	        const nextData = gen.let("data", (0, codegen_1._) `${it.data}${(0, codegen_1.getProperty)(dataProp)}`, true);
  	        dataContextProps(nextData);
  	        subschema.errorPath = (0, codegen_1.str) `${errorPath}${(0, util_1.getErrorPath)(dataProp, dpType, opts.jsPropertySyntax)}`;
  	        subschema.parentDataProperty = (0, codegen_1._) `${dataProp}`;
  	        subschema.dataPathArr = [...dataPathArr, subschema.parentDataProperty];
  	    }
  	    if (data !== undefined) {
  	        const nextData = data instanceof codegen_1.Name ? data : gen.let("data", data, true); // replaceable if used once?
  	        dataContextProps(nextData);
  	        if (propertyName !== undefined)
  	            subschema.propertyName = propertyName;
  	        // TODO something is possibly wrong here with not changing parentDataProperty and not appending dataPathArr
  	    }
  	    if (dataTypes)
  	        subschema.dataTypes = dataTypes;
  	    function dataContextProps(_nextData) {
  	        subschema.data = _nextData;
  	        subschema.dataLevel = it.dataLevel + 1;
  	        subschema.dataTypes = [];
  	        it.definedProperties = new Set();
  	        subschema.parentData = it.data;
  	        subschema.dataNames = [...it.dataNames, _nextData];
  	    }
  	}
  	subschema.extendSubschemaData = extendSubschemaData;
  	function extendSubschemaMode(subschema, { jtdDiscriminator, jtdMetadata, compositeRule, createErrors, allErrors }) {
  	    if (compositeRule !== undefined)
  	        subschema.compositeRule = compositeRule;
  	    if (createErrors !== undefined)
  	        subschema.createErrors = createErrors;
  	    if (allErrors !== undefined)
  	        subschema.allErrors = allErrors;
  	    subschema.jtdDiscriminator = jtdDiscriminator; // not inherited
  	    subschema.jtdMetadata = jtdMetadata; // not inherited
  	}
  	subschema.extendSubschemaMode = extendSubschemaMode;
  	
  	return subschema;
  }

  var resolve$2 = {};

  // do not edit .js files directly - edit src/index.jst



  var fastDeepEqual = function equal(a, b) {
    if (a === b) return true;

    if (a && b && typeof a == 'object' && typeof b == 'object') {
      if (a.constructor !== b.constructor) return false;

      var length, i, keys;
      if (Array.isArray(a)) {
        length = a.length;
        if (length != b.length) return false;
        for (i = length; i-- !== 0;)
          if (!equal(a[i], b[i])) return false;
        return true;
      }



      if (a.constructor === RegExp) return a.source === b.source && a.flags === b.flags;
      if (a.valueOf !== Object.prototype.valueOf) return a.valueOf() === b.valueOf();
      if (a.toString !== Object.prototype.toString) return a.toString() === b.toString();

      keys = Object.keys(a);
      length = keys.length;
      if (length !== Object.keys(b).length) return false;

      for (i = length; i-- !== 0;)
        if (!Object.prototype.hasOwnProperty.call(b, keys[i])) return false;

      for (i = length; i-- !== 0;) {
        var key = keys[i];

        if (!equal(a[key], b[key])) return false;
      }

      return true;
    }

    // true if both NaN, false otherwise
    return a!==a && b!==b;
  };

  var jsonSchemaTraverse = {exports: {}};

  var traverse$1 = jsonSchemaTraverse.exports = function (schema, opts, cb) {
    // Legacy support for v0.3.1 and earlier.
    if (typeof opts == 'function') {
      cb = opts;
      opts = {};
    }

    cb = opts.cb || cb;
    var pre = (typeof cb == 'function') ? cb : cb.pre || function() {};
    var post = cb.post || function() {};

    _traverse(opts, pre, post, schema, '', schema);
  };


  traverse$1.keywords = {
    additionalItems: true,
    items: true,
    contains: true,
    additionalProperties: true,
    propertyNames: true,
    not: true,
    if: true,
    then: true,
    else: true
  };

  traverse$1.arrayKeywords = {
    items: true,
    allOf: true,
    anyOf: true,
    oneOf: true
  };

  traverse$1.propsKeywords = {
    $defs: true,
    definitions: true,
    properties: true,
    patternProperties: true,
    dependencies: true
  };

  traverse$1.skipKeywords = {
    default: true,
    enum: true,
    const: true,
    required: true,
    maximum: true,
    minimum: true,
    exclusiveMaximum: true,
    exclusiveMinimum: true,
    multipleOf: true,
    maxLength: true,
    minLength: true,
    pattern: true,
    format: true,
    maxItems: true,
    minItems: true,
    uniqueItems: true,
    maxProperties: true,
    minProperties: true
  };


  function _traverse(opts, pre, post, schema, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex) {
    if (schema && typeof schema == 'object' && !Array.isArray(schema)) {
      pre(schema, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex);
      for (var key in schema) {
        var sch = schema[key];
        if (Array.isArray(sch)) {
          if (key in traverse$1.arrayKeywords) {
            for (var i=0; i<sch.length; i++)
              _traverse(opts, pre, post, sch[i], jsonPtr + '/' + key + '/' + i, rootSchema, jsonPtr, key, schema, i);
          }
        } else if (key in traverse$1.propsKeywords) {
          if (sch && typeof sch == 'object') {
            for (var prop in sch)
              _traverse(opts, pre, post, sch[prop], jsonPtr + '/' + key + '/' + escapeJsonPtr(prop), rootSchema, jsonPtr, key, schema, prop);
          }
        } else if (key in traverse$1.keywords || (opts.allKeys && !(key in traverse$1.skipKeywords))) {
          _traverse(opts, pre, post, sch, jsonPtr + '/' + key, rootSchema, jsonPtr, key, schema);
        }
      }
      post(schema, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex);
    }
  }


  function escapeJsonPtr(str) {
    return str.replace(/~/g, '~0').replace(/\//g, '~1');
  }

  var jsonSchemaTraverseExports = jsonSchemaTraverse.exports;

  Object.defineProperty(resolve$2, "__esModule", { value: true });
  resolve$2.getSchemaRefs = resolve$2.resolveUrl = resolve$2.normalizeId = resolve$2._getFullPath = resolve$2.getFullPath = resolve$2.inlineRef = void 0;
  const util_1$p = util;
  const equal$3 = fastDeepEqual;
  const traverse = jsonSchemaTraverseExports;
  // TODO refactor to use keyword definitions
  const SIMPLE_INLINED = new Set([
      "type",
      "format",
      "pattern",
      "maxLength",
      "minLength",
      "maxProperties",
      "minProperties",
      "maxItems",
      "minItems",
      "maximum",
      "minimum",
      "uniqueItems",
      "multipleOf",
      "required",
      "enum",
      "const",
  ]);
  function inlineRef(schema, limit = true) {
      if (typeof schema == "boolean")
          return true;
      if (limit === true)
          return !hasRef(schema);
      if (!limit)
          return false;
      return countKeys(schema) <= limit;
  }
  resolve$2.inlineRef = inlineRef;
  const REF_KEYWORDS = new Set([
      "$ref",
      "$recursiveRef",
      "$recursiveAnchor",
      "$dynamicRef",
      "$dynamicAnchor",
  ]);
  function hasRef(schema) {
      for (const key in schema) {
          if (REF_KEYWORDS.has(key))
              return true;
          const sch = schema[key];
          if (Array.isArray(sch) && sch.some(hasRef))
              return true;
          if (typeof sch == "object" && hasRef(sch))
              return true;
      }
      return false;
  }
  function countKeys(schema) {
      let count = 0;
      for (const key in schema) {
          if (key === "$ref")
              return Infinity;
          count++;
          if (SIMPLE_INLINED.has(key))
              continue;
          if (typeof schema[key] == "object") {
              (0, util_1$p.eachItem)(schema[key], (sch) => (count += countKeys(sch)));
          }
          if (count === Infinity)
              return Infinity;
      }
      return count;
  }
  function getFullPath(resolver, id = "", normalize) {
      if (normalize !== false)
          id = normalizeId(id);
      const p = resolver.parse(id);
      return _getFullPath(resolver, p);
  }
  resolve$2.getFullPath = getFullPath;
  function _getFullPath(resolver, p) {
      const serialized = resolver.serialize(p);
      return serialized.split("#")[0] + "#";
  }
  resolve$2._getFullPath = _getFullPath;
  const TRAILING_SLASH_HASH = /#\/?$/;
  function normalizeId(id) {
      return id ? id.replace(TRAILING_SLASH_HASH, "") : "";
  }
  resolve$2.normalizeId = normalizeId;
  function resolveUrl(resolver, baseId, id) {
      id = normalizeId(id);
      return resolver.resolve(baseId, id);
  }
  resolve$2.resolveUrl = resolveUrl;
  const ANCHOR = /^[a-z_][-a-z0-9._]*$/i;
  function getSchemaRefs(schema, baseId) {
      if (typeof schema == "boolean")
          return {};
      const { schemaId, uriResolver } = this.opts;
      const schId = normalizeId(schema[schemaId] || baseId);
      const baseIds = { "": schId };
      const pathPrefix = getFullPath(uriResolver, schId, false);
      const localRefs = {};
      const schemaRefs = new Set();
      traverse(schema, { allKeys: true }, (sch, jsonPtr, _, parentJsonPtr) => {
          if (parentJsonPtr === undefined)
              return;
          const fullPath = pathPrefix + jsonPtr;
          let innerBaseId = baseIds[parentJsonPtr];
          if (typeof sch[schemaId] == "string")
              innerBaseId = addRef.call(this, sch[schemaId]);
          addAnchor.call(this, sch.$anchor);
          addAnchor.call(this, sch.$dynamicAnchor);
          baseIds[jsonPtr] = innerBaseId;
          function addRef(ref) {
              // eslint-disable-next-line @typescript-eslint/unbound-method
              const _resolve = this.opts.uriResolver.resolve;
              ref = normalizeId(innerBaseId ? _resolve(innerBaseId, ref) : ref);
              if (schemaRefs.has(ref))
                  throw ambiguos(ref);
              schemaRefs.add(ref);
              let schOrRef = this.refs[ref];
              if (typeof schOrRef == "string")
                  schOrRef = this.refs[schOrRef];
              if (typeof schOrRef == "object") {
                  checkAmbiguosRef(sch, schOrRef.schema, ref);
              }
              else if (ref !== normalizeId(fullPath)) {
                  if (ref[0] === "#") {
                      checkAmbiguosRef(sch, localRefs[ref], ref);
                      localRefs[ref] = sch;
                  }
                  else {
                      this.refs[ref] = fullPath;
                  }
              }
              return ref;
          }
          function addAnchor(anchor) {
              if (typeof anchor == "string") {
                  if (!ANCHOR.test(anchor))
                      throw new Error(`invalid anchor "${anchor}"`);
                  addRef.call(this, `#${anchor}`);
              }
          }
      });
      return localRefs;
      function checkAmbiguosRef(sch1, sch2, ref) {
          if (sch2 !== undefined && !equal$3(sch1, sch2))
              throw ambiguos(ref);
      }
      function ambiguos(ref) {
          return new Error(`reference "${ref}" resolves to more than one schema`);
      }
  }
  resolve$2.getSchemaRefs = getSchemaRefs;

  var hasRequiredValidate;

  function requireValidate () {
  	if (hasRequiredValidate) return validate$1;
  	hasRequiredValidate = 1;
  	Object.defineProperty(validate$1, "__esModule", { value: true });
  	validate$1.getData = validate$1.KeywordCxt = validate$1.validateFunctionCode = void 0;
  	const boolSchema_1 = requireBoolSchema();
  	const dataType_1 = dataType;
  	const applicability_1 = requireApplicability();
  	const dataType_2 = dataType;
  	const defaults_1 = requireDefaults();
  	const keyword_1 = requireKeyword();
  	const subschema_1 = requireSubschema();
  	const codegen_1 = codegen;
  	const names_1 = requireNames();
  	const resolve_1 = resolve$2;
  	const util_1 = util;
  	const errors_1 = requireErrors();
  	// schema compilation - generates validation function, subschemaCode (below) is used for subschemas
  	function validateFunctionCode(it) {
  	    if (isSchemaObj(it)) {
  	        checkKeywords(it);
  	        if (schemaCxtHasRules(it)) {
  	            topSchemaObjCode(it);
  	            return;
  	        }
  	    }
  	    validateFunction(it, () => (0, boolSchema_1.topBoolOrEmptySchema)(it));
  	}
  	validate$1.validateFunctionCode = validateFunctionCode;
  	function validateFunction({ gen, validateName, schema, schemaEnv, opts }, body) {
  	    if (opts.code.es5) {
  	        gen.func(validateName, (0, codegen_1._) `${names_1.default.data}, ${names_1.default.valCxt}`, schemaEnv.$async, () => {
  	            gen.code((0, codegen_1._) `"use strict"; ${funcSourceUrl(schema, opts)}`);
  	            destructureValCxtES5(gen, opts);
  	            gen.code(body);
  	        });
  	    }
  	    else {
  	        gen.func(validateName, (0, codegen_1._) `${names_1.default.data}, ${destructureValCxt(opts)}`, schemaEnv.$async, () => gen.code(funcSourceUrl(schema, opts)).code(body));
  	    }
  	}
  	function destructureValCxt(opts) {
  	    return (0, codegen_1._) `{${names_1.default.instancePath}="", ${names_1.default.parentData}, ${names_1.default.parentDataProperty}, ${names_1.default.rootData}=${names_1.default.data}${opts.dynamicRef ? (0, codegen_1._) `, ${names_1.default.dynamicAnchors}={}` : codegen_1.nil}}={}`;
  	}
  	function destructureValCxtES5(gen, opts) {
  	    gen.if(names_1.default.valCxt, () => {
  	        gen.var(names_1.default.instancePath, (0, codegen_1._) `${names_1.default.valCxt}.${names_1.default.instancePath}`);
  	        gen.var(names_1.default.parentData, (0, codegen_1._) `${names_1.default.valCxt}.${names_1.default.parentData}`);
  	        gen.var(names_1.default.parentDataProperty, (0, codegen_1._) `${names_1.default.valCxt}.${names_1.default.parentDataProperty}`);
  	        gen.var(names_1.default.rootData, (0, codegen_1._) `${names_1.default.valCxt}.${names_1.default.rootData}`);
  	        if (opts.dynamicRef)
  	            gen.var(names_1.default.dynamicAnchors, (0, codegen_1._) `${names_1.default.valCxt}.${names_1.default.dynamicAnchors}`);
  	    }, () => {
  	        gen.var(names_1.default.instancePath, (0, codegen_1._) `""`);
  	        gen.var(names_1.default.parentData, (0, codegen_1._) `undefined`);
  	        gen.var(names_1.default.parentDataProperty, (0, codegen_1._) `undefined`);
  	        gen.var(names_1.default.rootData, names_1.default.data);
  	        if (opts.dynamicRef)
  	            gen.var(names_1.default.dynamicAnchors, (0, codegen_1._) `{}`);
  	    });
  	}
  	function topSchemaObjCode(it) {
  	    const { schema, opts, gen } = it;
  	    validateFunction(it, () => {
  	        if (opts.$comment && schema.$comment)
  	            commentKeyword(it);
  	        checkNoDefault(it);
  	        gen.let(names_1.default.vErrors, null);
  	        gen.let(names_1.default.errors, 0);
  	        if (opts.unevaluated)
  	            resetEvaluated(it);
  	        typeAndKeywords(it);
  	        returnResults(it);
  	    });
  	    return;
  	}
  	function resetEvaluated(it) {
  	    // TODO maybe some hook to execute it in the end to check whether props/items are Name, as in assignEvaluated
  	    const { gen, validateName } = it;
  	    it.evaluated = gen.const("evaluated", (0, codegen_1._) `${validateName}.evaluated`);
  	    gen.if((0, codegen_1._) `${it.evaluated}.dynamicProps`, () => gen.assign((0, codegen_1._) `${it.evaluated}.props`, (0, codegen_1._) `undefined`));
  	    gen.if((0, codegen_1._) `${it.evaluated}.dynamicItems`, () => gen.assign((0, codegen_1._) `${it.evaluated}.items`, (0, codegen_1._) `undefined`));
  	}
  	function funcSourceUrl(schema, opts) {
  	    const schId = typeof schema == "object" && schema[opts.schemaId];
  	    return schId && (opts.code.source || opts.code.process) ? (0, codegen_1._) `/*# sourceURL=${schId} */` : codegen_1.nil;
  	}
  	// schema compilation - this function is used recursively to generate code for sub-schemas
  	function subschemaCode(it, valid) {
  	    if (isSchemaObj(it)) {
  	        checkKeywords(it);
  	        if (schemaCxtHasRules(it)) {
  	            subSchemaObjCode(it, valid);
  	            return;
  	        }
  	    }
  	    (0, boolSchema_1.boolOrEmptySchema)(it, valid);
  	}
  	function schemaCxtHasRules({ schema, self }) {
  	    if (typeof schema == "boolean")
  	        return !schema;
  	    for (const key in schema)
  	        if (self.RULES.all[key])
  	            return true;
  	    return false;
  	}
  	function isSchemaObj(it) {
  	    return typeof it.schema != "boolean";
  	}
  	function subSchemaObjCode(it, valid) {
  	    const { schema, gen, opts } = it;
  	    if (opts.$comment && schema.$comment)
  	        commentKeyword(it);
  	    updateContext(it);
  	    checkAsyncSchema(it);
  	    const errsCount = gen.const("_errs", names_1.default.errors);
  	    typeAndKeywords(it, errsCount);
  	    // TODO var
  	    gen.var(valid, (0, codegen_1._) `${errsCount} === ${names_1.default.errors}`);
  	}
  	function checkKeywords(it) {
  	    (0, util_1.checkUnknownRules)(it);
  	    checkRefsAndKeywords(it);
  	}
  	function typeAndKeywords(it, errsCount) {
  	    if (it.opts.jtd)
  	        return schemaKeywords(it, [], false, errsCount);
  	    const types = (0, dataType_1.getSchemaTypes)(it.schema);
  	    const checkedTypes = (0, dataType_1.coerceAndCheckDataType)(it, types);
  	    schemaKeywords(it, types, !checkedTypes, errsCount);
  	}
  	function checkRefsAndKeywords(it) {
  	    const { schema, errSchemaPath, opts, self } = it;
  	    if (schema.$ref && opts.ignoreKeywordsWithRef && (0, util_1.schemaHasRulesButRef)(schema, self.RULES)) {
  	        self.logger.warn(`$ref: keywords ignored in schema at path "${errSchemaPath}"`);
  	    }
  	}
  	function checkNoDefault(it) {
  	    const { schema, opts } = it;
  	    if (schema.default !== undefined && opts.useDefaults && opts.strictSchema) {
  	        (0, util_1.checkStrictMode)(it, "default is ignored in the schema root");
  	    }
  	}
  	function updateContext(it) {
  	    const schId = it.schema[it.opts.schemaId];
  	    if (schId)
  	        it.baseId = (0, resolve_1.resolveUrl)(it.opts.uriResolver, it.baseId, schId);
  	}
  	function checkAsyncSchema(it) {
  	    if (it.schema.$async && !it.schemaEnv.$async)
  	        throw new Error("async schema in sync schema");
  	}
  	function commentKeyword({ gen, schemaEnv, schema, errSchemaPath, opts }) {
  	    const msg = schema.$comment;
  	    if (opts.$comment === true) {
  	        gen.code((0, codegen_1._) `${names_1.default.self}.logger.log(${msg})`);
  	    }
  	    else if (typeof opts.$comment == "function") {
  	        const schemaPath = (0, codegen_1.str) `${errSchemaPath}/$comment`;
  	        const rootName = gen.scopeValue("root", { ref: schemaEnv.root });
  	        gen.code((0, codegen_1._) `${names_1.default.self}.opts.$comment(${msg}, ${schemaPath}, ${rootName}.schema)`);
  	    }
  	}
  	function returnResults(it) {
  	    const { gen, schemaEnv, validateName, ValidationError, opts } = it;
  	    if (schemaEnv.$async) {
  	        // TODO assign unevaluated
  	        gen.if((0, codegen_1._) `${names_1.default.errors} === 0`, () => gen.return(names_1.default.data), () => gen.throw((0, codegen_1._) `new ${ValidationError}(${names_1.default.vErrors})`));
  	    }
  	    else {
  	        gen.assign((0, codegen_1._) `${validateName}.errors`, names_1.default.vErrors);
  	        if (opts.unevaluated)
  	            assignEvaluated(it);
  	        gen.return((0, codegen_1._) `${names_1.default.errors} === 0`);
  	    }
  	}
  	function assignEvaluated({ gen, evaluated, props, items }) {
  	    if (props instanceof codegen_1.Name)
  	        gen.assign((0, codegen_1._) `${evaluated}.props`, props);
  	    if (items instanceof codegen_1.Name)
  	        gen.assign((0, codegen_1._) `${evaluated}.items`, items);
  	}
  	function schemaKeywords(it, types, typeErrors, errsCount) {
  	    const { gen, schema, data, allErrors, opts, self } = it;
  	    const { RULES } = self;
  	    if (schema.$ref && (opts.ignoreKeywordsWithRef || !(0, util_1.schemaHasRulesButRef)(schema, RULES))) {
  	        gen.block(() => keywordCode(it, "$ref", RULES.all.$ref.definition)); // TODO typecast
  	        return;
  	    }
  	    if (!opts.jtd)
  	        checkStrictTypes(it, types);
  	    gen.block(() => {
  	        for (const group of RULES.rules)
  	            groupKeywords(group);
  	        groupKeywords(RULES.post);
  	    });
  	    function groupKeywords(group) {
  	        if (!(0, applicability_1.shouldUseGroup)(schema, group))
  	            return;
  	        if (group.type) {
  	            gen.if((0, dataType_2.checkDataType)(group.type, data, opts.strictNumbers));
  	            iterateKeywords(it, group);
  	            if (types.length === 1 && types[0] === group.type && typeErrors) {
  	                gen.else();
  	                (0, dataType_2.reportTypeError)(it);
  	            }
  	            gen.endIf();
  	        }
  	        else {
  	            iterateKeywords(it, group);
  	        }
  	        // TODO make it "ok" call?
  	        if (!allErrors)
  	            gen.if((0, codegen_1._) `${names_1.default.errors} === ${errsCount || 0}`);
  	    }
  	}
  	function iterateKeywords(it, group) {
  	    const { gen, schema, opts: { useDefaults }, } = it;
  	    if (useDefaults)
  	        (0, defaults_1.assignDefaults)(it, group.type);
  	    gen.block(() => {
  	        for (const rule of group.rules) {
  	            if ((0, applicability_1.shouldUseRule)(schema, rule)) {
  	                keywordCode(it, rule.keyword, rule.definition, group.type);
  	            }
  	        }
  	    });
  	}
  	function checkStrictTypes(it, types) {
  	    if (it.schemaEnv.meta || !it.opts.strictTypes)
  	        return;
  	    checkContextTypes(it, types);
  	    if (!it.opts.allowUnionTypes)
  	        checkMultipleTypes(it, types);
  	    checkKeywordTypes(it, it.dataTypes);
  	}
  	function checkContextTypes(it, types) {
  	    if (!types.length)
  	        return;
  	    if (!it.dataTypes.length) {
  	        it.dataTypes = types;
  	        return;
  	    }
  	    types.forEach((t) => {
  	        if (!includesType(it.dataTypes, t)) {
  	            strictTypesError(it, `type "${t}" not allowed by context "${it.dataTypes.join(",")}"`);
  	        }
  	    });
  	    narrowSchemaTypes(it, types);
  	}
  	function checkMultipleTypes(it, ts) {
  	    if (ts.length > 1 && !(ts.length === 2 && ts.includes("null"))) {
  	        strictTypesError(it, "use allowUnionTypes to allow union type keyword");
  	    }
  	}
  	function checkKeywordTypes(it, ts) {
  	    const rules = it.self.RULES.all;
  	    for (const keyword in rules) {
  	        const rule = rules[keyword];
  	        if (typeof rule == "object" && (0, applicability_1.shouldUseRule)(it.schema, rule)) {
  	            const { type } = rule.definition;
  	            if (type.length && !type.some((t) => hasApplicableType(ts, t))) {
  	                strictTypesError(it, `missing type "${type.join(",")}" for keyword "${keyword}"`);
  	            }
  	        }
  	    }
  	}
  	function hasApplicableType(schTs, kwdT) {
  	    return schTs.includes(kwdT) || (kwdT === "number" && schTs.includes("integer"));
  	}
  	function includesType(ts, t) {
  	    return ts.includes(t) || (t === "integer" && ts.includes("number"));
  	}
  	function narrowSchemaTypes(it, withTypes) {
  	    const ts = [];
  	    for (const t of it.dataTypes) {
  	        if (includesType(withTypes, t))
  	            ts.push(t);
  	        else if (withTypes.includes("integer") && t === "number")
  	            ts.push("integer");
  	    }
  	    it.dataTypes = ts;
  	}
  	function strictTypesError(it, msg) {
  	    const schemaPath = it.schemaEnv.baseId + it.errSchemaPath;
  	    msg += ` at "${schemaPath}" (strictTypes)`;
  	    (0, util_1.checkStrictMode)(it, msg, it.opts.strictTypes);
  	}
  	class KeywordCxt {
  	    constructor(it, def, keyword) {
  	        (0, keyword_1.validateKeywordUsage)(it, def, keyword);
  	        this.gen = it.gen;
  	        this.allErrors = it.allErrors;
  	        this.keyword = keyword;
  	        this.data = it.data;
  	        this.schema = it.schema[keyword];
  	        this.$data = def.$data && it.opts.$data && this.schema && this.schema.$data;
  	        this.schemaValue = (0, util_1.schemaRefOrVal)(it, this.schema, keyword, this.$data);
  	        this.schemaType = def.schemaType;
  	        this.parentSchema = it.schema;
  	        this.params = {};
  	        this.it = it;
  	        this.def = def;
  	        if (this.$data) {
  	            this.schemaCode = it.gen.const("vSchema", getData(this.$data, it));
  	        }
  	        else {
  	            this.schemaCode = this.schemaValue;
  	            if (!(0, keyword_1.validSchemaType)(this.schema, def.schemaType, def.allowUndefined)) {
  	                throw new Error(`${keyword} value must be ${JSON.stringify(def.schemaType)}`);
  	            }
  	        }
  	        if ("code" in def ? def.trackErrors : def.errors !== false) {
  	            this.errsCount = it.gen.const("_errs", names_1.default.errors);
  	        }
  	    }
  	    result(condition, successAction, failAction) {
  	        this.failResult((0, codegen_1.not)(condition), successAction, failAction);
  	    }
  	    failResult(condition, successAction, failAction) {
  	        this.gen.if(condition);
  	        if (failAction)
  	            failAction();
  	        else
  	            this.error();
  	        if (successAction) {
  	            this.gen.else();
  	            successAction();
  	            if (this.allErrors)
  	                this.gen.endIf();
  	        }
  	        else {
  	            if (this.allErrors)
  	                this.gen.endIf();
  	            else
  	                this.gen.else();
  	        }
  	    }
  	    pass(condition, failAction) {
  	        this.failResult((0, codegen_1.not)(condition), undefined, failAction);
  	    }
  	    fail(condition) {
  	        if (condition === undefined) {
  	            this.error();
  	            if (!this.allErrors)
  	                this.gen.if(false); // this branch will be removed by gen.optimize
  	            return;
  	        }
  	        this.gen.if(condition);
  	        this.error();
  	        if (this.allErrors)
  	            this.gen.endIf();
  	        else
  	            this.gen.else();
  	    }
  	    fail$data(condition) {
  	        if (!this.$data)
  	            return this.fail(condition);
  	        const { schemaCode } = this;
  	        this.fail((0, codegen_1._) `${schemaCode} !== undefined && (${(0, codegen_1.or)(this.invalid$data(), condition)})`);
  	    }
  	    error(append, errorParams, errorPaths) {
  	        if (errorParams) {
  	            this.setParams(errorParams);
  	            this._error(append, errorPaths);
  	            this.setParams({});
  	            return;
  	        }
  	        this._error(append, errorPaths);
  	    }
  	    _error(append, errorPaths) {
  	        (append ? errors_1.reportExtraError : errors_1.reportError)(this, this.def.error, errorPaths);
  	    }
  	    $dataError() {
  	        (0, errors_1.reportError)(this, this.def.$dataError || errors_1.keyword$DataError);
  	    }
  	    reset() {
  	        if (this.errsCount === undefined)
  	            throw new Error('add "trackErrors" to keyword definition');
  	        (0, errors_1.resetErrorsCount)(this.gen, this.errsCount);
  	    }
  	    ok(cond) {
  	        if (!this.allErrors)
  	            this.gen.if(cond);
  	    }
  	    setParams(obj, assign) {
  	        if (assign)
  	            Object.assign(this.params, obj);
  	        else
  	            this.params = obj;
  	    }
  	    block$data(valid, codeBlock, $dataValid = codegen_1.nil) {
  	        this.gen.block(() => {
  	            this.check$data(valid, $dataValid);
  	            codeBlock();
  	        });
  	    }
  	    check$data(valid = codegen_1.nil, $dataValid = codegen_1.nil) {
  	        if (!this.$data)
  	            return;
  	        const { gen, schemaCode, schemaType, def } = this;
  	        gen.if((0, codegen_1.or)((0, codegen_1._) `${schemaCode} === undefined`, $dataValid));
  	        if (valid !== codegen_1.nil)
  	            gen.assign(valid, true);
  	        if (schemaType.length || def.validateSchema) {
  	            gen.elseIf(this.invalid$data());
  	            this.$dataError();
  	            if (valid !== codegen_1.nil)
  	                gen.assign(valid, false);
  	        }
  	        gen.else();
  	    }
  	    invalid$data() {
  	        const { gen, schemaCode, schemaType, def, it } = this;
  	        return (0, codegen_1.or)(wrong$DataType(), invalid$DataSchema());
  	        function wrong$DataType() {
  	            if (schemaType.length) {
  	                /* istanbul ignore if */
  	                if (!(schemaCode instanceof codegen_1.Name))
  	                    throw new Error("ajv implementation error");
  	                const st = Array.isArray(schemaType) ? schemaType : [schemaType];
  	                return (0, codegen_1._) `${(0, dataType_2.checkDataTypes)(st, schemaCode, it.opts.strictNumbers, dataType_2.DataType.Wrong)}`;
  	            }
  	            return codegen_1.nil;
  	        }
  	        function invalid$DataSchema() {
  	            if (def.validateSchema) {
  	                const validateSchemaRef = gen.scopeValue("validate$data", { ref: def.validateSchema }); // TODO value.code for standalone
  	                return (0, codegen_1._) `!${validateSchemaRef}(${schemaCode})`;
  	            }
  	            return codegen_1.nil;
  	        }
  	    }
  	    subschema(appl, valid) {
  	        const subschema = (0, subschema_1.getSubschema)(this.it, appl);
  	        (0, subschema_1.extendSubschemaData)(subschema, this.it, appl);
  	        (0, subschema_1.extendSubschemaMode)(subschema, appl);
  	        const nextContext = { ...this.it, ...subschema, items: undefined, props: undefined };
  	        subschemaCode(nextContext, valid);
  	        return nextContext;
  	    }
  	    mergeEvaluated(schemaCxt, toName) {
  	        const { it, gen } = this;
  	        if (!it.opts.unevaluated)
  	            return;
  	        if (it.props !== true && schemaCxt.props !== undefined) {
  	            it.props = util_1.mergeEvaluated.props(gen, schemaCxt.props, it.props, toName);
  	        }
  	        if (it.items !== true && schemaCxt.items !== undefined) {
  	            it.items = util_1.mergeEvaluated.items(gen, schemaCxt.items, it.items, toName);
  	        }
  	    }
  	    mergeValidEvaluated(schemaCxt, valid) {
  	        const { it, gen } = this;
  	        if (it.opts.unevaluated && (it.props !== true || it.items !== true)) {
  	            gen.if(valid, () => this.mergeEvaluated(schemaCxt, codegen_1.Name));
  	            return true;
  	        }
  	    }
  	}
  	validate$1.KeywordCxt = KeywordCxt;
  	function keywordCode(it, keyword, def, ruleType) {
  	    const cxt = new KeywordCxt(it, def, keyword);
  	    if ("code" in def) {
  	        def.code(cxt, ruleType);
  	    }
  	    else if (cxt.$data && def.validate) {
  	        (0, keyword_1.funcKeywordCode)(cxt, def);
  	    }
  	    else if ("macro" in def) {
  	        (0, keyword_1.macroKeywordCode)(cxt, def);
  	    }
  	    else if (def.compile || def.validate) {
  	        (0, keyword_1.funcKeywordCode)(cxt, def);
  	    }
  	}
  	const JSON_POINTER = /^\/(?:[^~]|~0|~1)*$/;
  	const RELATIVE_JSON_POINTER = /^([0-9]+)(#|\/(?:[^~]|~0|~1)*)?$/;
  	function getData($data, { dataLevel, dataNames, dataPathArr }) {
  	    let jsonPointer;
  	    let data;
  	    if ($data === "")
  	        return names_1.default.rootData;
  	    if ($data[0] === "/") {
  	        if (!JSON_POINTER.test($data))
  	            throw new Error(`Invalid JSON-pointer: ${$data}`);
  	        jsonPointer = $data;
  	        data = names_1.default.rootData;
  	    }
  	    else {
  	        const matches = RELATIVE_JSON_POINTER.exec($data);
  	        if (!matches)
  	            throw new Error(`Invalid JSON-pointer: ${$data}`);
  	        const up = +matches[1];
  	        jsonPointer = matches[2];
  	        if (jsonPointer === "#") {
  	            if (up >= dataLevel)
  	                throw new Error(errorMsg("property/index", up));
  	            return dataPathArr[dataLevel - up];
  	        }
  	        if (up > dataLevel)
  	            throw new Error(errorMsg("data", up));
  	        data = dataNames[dataLevel - up];
  	        if (!jsonPointer)
  	            return data;
  	    }
  	    let expr = data;
  	    const segments = jsonPointer.split("/");
  	    for (const segment of segments) {
  	        if (segment) {
  	            data = (0, codegen_1._) `${data}${(0, codegen_1.getProperty)((0, util_1.unescapeJsonPointer)(segment))}`;
  	            expr = (0, codegen_1._) `${expr} && ${data}`;
  	        }
  	    }
  	    return expr;
  	    function errorMsg(pointerType, up) {
  	        return `Cannot access ${pointerType} ${up} levels up, current level is ${dataLevel}`;
  	    }
  	}
  	validate$1.getData = getData;
  	
  	return validate$1;
  }

  var validation_error = {};

  var hasRequiredValidation_error;

  function requireValidation_error () {
  	if (hasRequiredValidation_error) return validation_error;
  	hasRequiredValidation_error = 1;
  	Object.defineProperty(validation_error, "__esModule", { value: true });
  	class ValidationError extends Error {
  	    constructor(errors) {
  	        super("validation failed");
  	        this.errors = errors;
  	        this.ajv = this.validation = true;
  	    }
  	}
  	validation_error.default = ValidationError;
  	
  	return validation_error;
  }

  var ref_error = {};

  var hasRequiredRef_error;

  function requireRef_error () {
  	if (hasRequiredRef_error) return ref_error;
  	hasRequiredRef_error = 1;
  	Object.defineProperty(ref_error, "__esModule", { value: true });
  	const resolve_1 = resolve$2;
  	class MissingRefError extends Error {
  	    constructor(resolver, baseId, ref, msg) {
  	        super(msg || `can't resolve reference ${ref} from id ${baseId}`);
  	        this.missingRef = (0, resolve_1.resolveUrl)(resolver, baseId, ref);
  	        this.missingSchema = (0, resolve_1.normalizeId)((0, resolve_1.getFullPath)(resolver, this.missingRef));
  	    }
  	}
  	ref_error.default = MissingRefError;
  	
  	return ref_error;
  }

  var compile = {};

  Object.defineProperty(compile, "__esModule", { value: true });
  compile.resolveSchema = compile.getCompilingSchema = compile.resolveRef = compile.compileSchema = compile.SchemaEnv = void 0;
  const codegen_1$q = codegen;
  const validation_error_1 = requireValidation_error();
  const names_1$5 = requireNames();
  const resolve_1 = resolve$2;
  const util_1$o = util;
  const validate_1$1 = requireValidate();
  class SchemaEnv {
      constructor(env) {
          var _a;
          this.refs = {};
          this.dynamicAnchors = {};
          let schema;
          if (typeof env.schema == "object")
              schema = env.schema;
          this.schema = env.schema;
          this.schemaId = env.schemaId;
          this.root = env.root || this;
          this.baseId = (_a = env.baseId) !== null && _a !== void 0 ? _a : (0, resolve_1.normalizeId)(schema === null || schema === void 0 ? void 0 : schema[env.schemaId || "$id"]);
          this.schemaPath = env.schemaPath;
          this.localRefs = env.localRefs;
          this.meta = env.meta;
          this.$async = schema === null || schema === void 0 ? void 0 : schema.$async;
          this.refs = {};
      }
  }
  compile.SchemaEnv = SchemaEnv;
  // let codeSize = 0
  // let nodeCount = 0
  // Compiles schema in SchemaEnv
  function compileSchema(sch) {
      // TODO refactor - remove compilations
      const _sch = getCompilingSchema.call(this, sch);
      if (_sch)
          return _sch;
      const rootId = (0, resolve_1.getFullPath)(this.opts.uriResolver, sch.root.baseId); // TODO if getFullPath removed 1 tests fails
      const { es5, lines } = this.opts.code;
      const { ownProperties } = this.opts;
      const gen = new codegen_1$q.CodeGen(this.scope, { es5, lines, ownProperties });
      let _ValidationError;
      if (sch.$async) {
          _ValidationError = gen.scopeValue("Error", {
              ref: validation_error_1.default,
              code: (0, codegen_1$q._) `require("ajv/dist/runtime/validation_error").default`,
          });
      }
      const validateName = gen.scopeName("validate");
      sch.validateName = validateName;
      const schemaCxt = {
          gen,
          allErrors: this.opts.allErrors,
          data: names_1$5.default.data,
          parentData: names_1$5.default.parentData,
          parentDataProperty: names_1$5.default.parentDataProperty,
          dataNames: [names_1$5.default.data],
          dataPathArr: [codegen_1$q.nil], // TODO can its length be used as dataLevel if nil is removed?
          dataLevel: 0,
          dataTypes: [],
          definedProperties: new Set(),
          topSchemaRef: gen.scopeValue("schema", this.opts.code.source === true
              ? { ref: sch.schema, code: (0, codegen_1$q.stringify)(sch.schema) }
              : { ref: sch.schema }),
          validateName,
          ValidationError: _ValidationError,
          schema: sch.schema,
          schemaEnv: sch,
          rootId,
          baseId: sch.baseId || rootId,
          schemaPath: codegen_1$q.nil,
          errSchemaPath: sch.schemaPath || (this.opts.jtd ? "" : "#"),
          errorPath: (0, codegen_1$q._) `""`,
          opts: this.opts,
          self: this,
      };
      let sourceCode;
      try {
          this._compilations.add(sch);
          (0, validate_1$1.validateFunctionCode)(schemaCxt);
          gen.optimize(this.opts.code.optimize);
          // gen.optimize(1)
          const validateCode = gen.toString();
          sourceCode = `${gen.scopeRefs(names_1$5.default.scope)}return ${validateCode}`;
          // console.log((codeSize += sourceCode.length), (nodeCount += gen.nodeCount))
          if (this.opts.code.process)
              sourceCode = this.opts.code.process(sourceCode, sch);
          // console.log("\n\n\n *** \n", sourceCode)
          const makeValidate = new Function(`${names_1$5.default.self}`, `${names_1$5.default.scope}`, sourceCode);
          const validate = makeValidate(this, this.scope.get());
          this.scope.value(validateName, { ref: validate });
          validate.errors = null;
          validate.schema = sch.schema;
          validate.schemaEnv = sch;
          if (sch.$async)
              validate.$async = true;
          if (this.opts.code.source === true) {
              validate.source = { validateName, validateCode, scopeValues: gen._values };
          }
          if (this.opts.unevaluated) {
              const { props, items } = schemaCxt;
              validate.evaluated = {
                  props: props instanceof codegen_1$q.Name ? undefined : props,
                  items: items instanceof codegen_1$q.Name ? undefined : items,
                  dynamicProps: props instanceof codegen_1$q.Name,
                  dynamicItems: items instanceof codegen_1$q.Name,
              };
              if (validate.source)
                  validate.source.evaluated = (0, codegen_1$q.stringify)(validate.evaluated);
          }
          sch.validate = validate;
          return sch;
      }
      catch (e) {
          delete sch.validate;
          delete sch.validateName;
          if (sourceCode)
              this.logger.error("Error compiling schema, function code:", sourceCode);
          // console.log("\n\n\n *** \n", sourceCode, this.opts)
          throw e;
      }
      finally {
          this._compilations.delete(sch);
      }
  }
  compile.compileSchema = compileSchema;
  function resolveRef(root, baseId, ref) {
      var _a;
      ref = (0, resolve_1.resolveUrl)(this.opts.uriResolver, baseId, ref);
      const schOrFunc = root.refs[ref];
      if (schOrFunc)
          return schOrFunc;
      let _sch = resolve$1.call(this, root, ref);
      if (_sch === undefined) {
          const schema = (_a = root.localRefs) === null || _a === void 0 ? void 0 : _a[ref]; // TODO maybe localRefs should hold SchemaEnv
          const { schemaId } = this.opts;
          if (schema)
              _sch = new SchemaEnv({ schema, schemaId, root, baseId });
      }
      if (_sch === undefined)
          return;
      return (root.refs[ref] = inlineOrCompile.call(this, _sch));
  }
  compile.resolveRef = resolveRef;
  function inlineOrCompile(sch) {
      if ((0, resolve_1.inlineRef)(sch.schema, this.opts.inlineRefs))
          return sch.schema;
      return sch.validate ? sch : compileSchema.call(this, sch);
  }
  // Index of schema compilation in the currently compiled list
  function getCompilingSchema(schEnv) {
      for (const sch of this._compilations) {
          if (sameSchemaEnv(sch, schEnv))
              return sch;
      }
  }
  compile.getCompilingSchema = getCompilingSchema;
  function sameSchemaEnv(s1, s2) {
      return s1.schema === s2.schema && s1.root === s2.root && s1.baseId === s2.baseId;
  }
  // resolve and compile the references ($ref)
  // TODO returns AnySchemaObject (if the schema can be inlined) or validation function
  function resolve$1(root, // information about the root schema for the current schema
  ref // reference to resolve
  ) {
      let sch;
      while (typeof (sch = this.refs[ref]) == "string")
          ref = sch;
      return sch || this.schemas[ref] || resolveSchema.call(this, root, ref);
  }
  // Resolve schema, its root and baseId
  function resolveSchema(root, // root object with properties schema, refs TODO below SchemaEnv is assigned to it
  ref // reference to resolve
  ) {
      const p = this.opts.uriResolver.parse(ref);
      const refPath = (0, resolve_1._getFullPath)(this.opts.uriResolver, p);
      let baseId = (0, resolve_1.getFullPath)(this.opts.uriResolver, root.baseId, undefined);
      // TODO `Object.keys(root.schema).length > 0` should not be needed - but removing breaks 2 tests
      if (Object.keys(root.schema).length > 0 && refPath === baseId) {
          return getJsonPointer.call(this, p, root);
      }
      const id = (0, resolve_1.normalizeId)(refPath);
      const schOrRef = this.refs[id] || this.schemas[id];
      if (typeof schOrRef == "string") {
          const sch = resolveSchema.call(this, root, schOrRef);
          if (typeof (sch === null || sch === void 0 ? void 0 : sch.schema) !== "object")
              return;
          return getJsonPointer.call(this, p, sch);
      }
      if (typeof (schOrRef === null || schOrRef === void 0 ? void 0 : schOrRef.schema) !== "object")
          return;
      if (!schOrRef.validate)
          compileSchema.call(this, schOrRef);
      if (id === (0, resolve_1.normalizeId)(ref)) {
          const { schema } = schOrRef;
          const { schemaId } = this.opts;
          const schId = schema[schemaId];
          if (schId)
              baseId = (0, resolve_1.resolveUrl)(this.opts.uriResolver, baseId, schId);
          return new SchemaEnv({ schema, schemaId, root, baseId });
      }
      return getJsonPointer.call(this, p, schOrRef);
  }
  compile.resolveSchema = resolveSchema;
  const PREVENT_SCOPE_CHANGE = new Set([
      "properties",
      "patternProperties",
      "enum",
      "dependencies",
      "definitions",
  ]);
  function getJsonPointer(parsedRef, { baseId, schema, root }) {
      var _a;
      if (((_a = parsedRef.fragment) === null || _a === void 0 ? void 0 : _a[0]) !== "/")
          return;
      for (const part of parsedRef.fragment.slice(1).split("/")) {
          if (typeof schema === "boolean")
              return;
          const partSchema = schema[(0, util_1$o.unescapeFragment)(part)];
          if (partSchema === undefined)
              return;
          schema = partSchema;
          // TODO PREVENT_SCOPE_CHANGE could be defined in keyword def?
          const schId = typeof schema === "object" && schema[this.opts.schemaId];
          if (!PREVENT_SCOPE_CHANGE.has(part) && schId) {
              baseId = (0, resolve_1.resolveUrl)(this.opts.uriResolver, baseId, schId);
          }
      }
      let env;
      if (typeof schema != "boolean" && schema.$ref && !(0, util_1$o.schemaHasRulesButRef)(schema, this.RULES)) {
          const $ref = (0, resolve_1.resolveUrl)(this.opts.uriResolver, baseId, schema.$ref);
          env = resolveSchema.call(this, root, $ref);
      }
      // even though resolution failed we need to return SchemaEnv to throw exception
      // so that compileAsync loads missing schema.
      const { schemaId } = this.opts;
      env = env || new SchemaEnv({ schema, schemaId, root, baseId });
      if (env.schema !== env.root.schema)
          return env;
      return undefined;
  }

  var $id$a = "https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#";
  var description$1 = "Meta-schema for $data reference (JSON AnySchema extension proposal)";
  var type$b = "object";
  var required$2 = [
  	"$data"
  ];
  var properties$b = {
  	$data: {
  		type: "string",
  		anyOf: [
  			{
  				format: "relative-json-pointer"
  			},
  			{
  				format: "json-pointer"
  			}
  		]
  	}
  };
  var additionalProperties$1 = false;
  var require$$9 = {
  	$id: $id$a,
  	description: description$1,
  	type: type$b,
  	required: required$2,
  	properties: properties$b,
  	additionalProperties: additionalProperties$1
  };

  var uri$1 = {};

  var fastUri$1 = {exports: {}};

  const HEX$1 = {
    0: 0,
    1: 1,
    2: 2,
    3: 3,
    4: 4,
    5: 5,
    6: 6,
    7: 7,
    8: 8,
    9: 9,
    a: 10,
    A: 10,
    b: 11,
    B: 11,
    c: 12,
    C: 12,
    d: 13,
    D: 13,
    e: 14,
    E: 14,
    f: 15,
    F: 15
  };

  var scopedChars = {
    HEX: HEX$1
  };

  const { HEX } = scopedChars;

  function normalizeIPv4$1 (host) {
    if (findToken(host, '.') < 3) { return { host, isIPV4: false } }
    const matches = host.match(/^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])$/u) || [];
    const [address] = matches;
    if (address) {
      return { host: stripLeadingZeros(address, '.'), isIPV4: true }
    } else {
      return { host, isIPV4: false }
    }
  }

  /**
   * @param {string[]} input
   * @param {boolean} [keepZero=false]
   * @returns {string|undefined}
   */
  function stringArrayToHexStripped (input, keepZero = false) {
    let acc = '';
    let strip = true;
    for (const c of input) {
      if (HEX[c] === undefined) return undefined
      if (c !== '0' && strip === true) strip = false;
      if (!strip) acc += c;
    }
    if (keepZero && acc.length === 0) acc = '0';
    return acc
  }

  function getIPV6 (input) {
    let tokenCount = 0;
    const output = { error: false, address: '', zone: '' };
    const address = [];
    const buffer = [];
    let isZone = false;
    let endipv6Encountered = false;
    let endIpv6 = false;

    function consume () {
      if (buffer.length) {
        if (isZone === false) {
          const hex = stringArrayToHexStripped(buffer);
          if (hex !== undefined) {
            address.push(hex);
          } else {
            output.error = true;
            return false
          }
        }
        buffer.length = 0;
      }
      return true
    }

    for (let i = 0; i < input.length; i++) {
      const cursor = input[i];
      if (cursor === '[' || cursor === ']') { continue }
      if (cursor === ':') {
        if (endipv6Encountered === true) {
          endIpv6 = true;
        }
        if (!consume()) { break }
        tokenCount++;
        address.push(':');
        if (tokenCount > 7) {
          // not valid
          output.error = true;
          break
        }
        if (i - 1 >= 0 && input[i - 1] === ':') {
          endipv6Encountered = true;
        }
        continue
      } else if (cursor === '%') {
        if (!consume()) { break }
        // switch to zone detection
        isZone = true;
      } else {
        buffer.push(cursor);
        continue
      }
    }
    if (buffer.length) {
      if (isZone) {
        output.zone = buffer.join('');
      } else if (endIpv6) {
        address.push(buffer.join(''));
      } else {
        address.push(stringArrayToHexStripped(buffer));
      }
    }
    output.address = address.join('');
    return output
  }

  function normalizeIPv6$1 (host, opts = {}) {
    if (findToken(host, ':') < 2) { return { host, isIPV6: false } }
    const ipv6 = getIPV6(host);

    if (!ipv6.error) {
      let newHost = ipv6.address;
      let escapedHost = ipv6.address;
      if (ipv6.zone) {
        newHost += '%' + ipv6.zone;
        escapedHost += '%25' + ipv6.zone;
      }
      return { host: newHost, escapedHost, isIPV6: true }
    } else {
      return { host, isIPV6: false }
    }
  }

  function stripLeadingZeros (str, token) {
    let out = '';
    let skip = true;
    const l = str.length;
    for (let i = 0; i < l; i++) {
      const c = str[i];
      if (c === '0' && skip) {
        if ((i + 1 <= l && str[i + 1] === token) || i + 1 === l) {
          out += c;
          skip = false;
        }
      } else {
        if (c === token) {
          skip = true;
        } else {
          skip = false;
        }
        out += c;
      }
    }
    return out
  }

  function findToken (str, token) {
    let ind = 0;
    for (let i = 0; i < str.length; i++) {
      if (str[i] === token) ind++;
    }
    return ind
  }

  const RDS1 = /^\.\.?\//u;
  const RDS2 = /^\/\.(?:\/|$)/u;
  const RDS3 = /^\/\.\.(?:\/|$)/u;
  const RDS5 = /^\/?(?:.|\n)*?(?=\/|$)/u;

  function removeDotSegments$1 (input) {
    const output = [];

    while (input.length) {
      if (input.match(RDS1)) {
        input = input.replace(RDS1, '');
      } else if (input.match(RDS2)) {
        input = input.replace(RDS2, '/');
      } else if (input.match(RDS3)) {
        input = input.replace(RDS3, '/');
        output.pop();
      } else if (input === '.' || input === '..') {
        input = '';
      } else {
        const im = input.match(RDS5);
        if (im) {
          const s = im[0];
          input = input.slice(s.length);
          output.push(s);
        } else {
          throw new Error('Unexpected dot segment condition')
        }
      }
    }
    return output.join('')
  }

  function normalizeComponentEncoding$1 (components, esc) {
    const func = esc !== true ? escape : unescape;
    if (components.scheme !== undefined) {
      components.scheme = func(components.scheme);
    }
    if (components.userinfo !== undefined) {
      components.userinfo = func(components.userinfo);
    }
    if (components.host !== undefined) {
      components.host = func(components.host);
    }
    if (components.path !== undefined) {
      components.path = func(components.path);
    }
    if (components.query !== undefined) {
      components.query = func(components.query);
    }
    if (components.fragment !== undefined) {
      components.fragment = func(components.fragment);
    }
    return components
  }

  function recomposeAuthority$1 (components, options) {
    const uriTokens = [];

    if (components.userinfo !== undefined) {
      uriTokens.push(components.userinfo);
      uriTokens.push('@');
    }

    if (components.host !== undefined) {
      let host = unescape(components.host);
      const ipV4res = normalizeIPv4$1(host);

      if (ipV4res.isIPV4) {
        host = ipV4res.host;
      } else {
        const ipV6res = normalizeIPv6$1(ipV4res.host, { isIPV4: false });
        if (ipV6res.isIPV6 === true) {
          host = `[${ipV6res.escapedHost}]`;
        } else {
          host = components.host;
        }
      }
      uriTokens.push(host);
    }

    if (typeof components.port === 'number' || typeof components.port === 'string') {
      uriTokens.push(':');
      uriTokens.push(String(components.port));
    }

    return uriTokens.length ? uriTokens.join('') : undefined
  }
  var utils = {
    recomposeAuthority: recomposeAuthority$1,
    normalizeComponentEncoding: normalizeComponentEncoding$1,
    removeDotSegments: removeDotSegments$1,
    normalizeIPv4: normalizeIPv4$1,
    normalizeIPv6: normalizeIPv6$1,
    stringArrayToHexStripped
  };

  const UUID_REG = /^[\da-f]{8}\b-[\da-f]{4}\b-[\da-f]{4}\b-[\da-f]{4}\b-[\da-f]{12}$/iu;
  const URN_REG = /([\da-z][\d\-a-z]{0,31}):((?:[\w!$'()*+,\-.:;=@]|%[\da-f]{2})+)/iu;

  function isSecure (wsComponents) {
    return typeof wsComponents.secure === 'boolean' ? wsComponents.secure : String(wsComponents.scheme).toLowerCase() === 'wss'
  }

  function httpParse (components) {
    if (!components.host) {
      components.error = components.error || 'HTTP URIs must have a host.';
    }

    return components
  }

  function httpSerialize (components) {
    const secure = String(components.scheme).toLowerCase() === 'https';

    // normalize the default port
    if (components.port === (secure ? 443 : 80) || components.port === '') {
      components.port = undefined;
    }

    // normalize the empty path
    if (!components.path) {
      components.path = '/';
    }

    // NOTE: We do not parse query strings for HTTP URIs
    // as WWW Form Url Encoded query strings are part of the HTML4+ spec,
    // and not the HTTP spec.

    return components
  }

  function wsParse (wsComponents) {
  // indicate if the secure flag is set
    wsComponents.secure = isSecure(wsComponents);

    // construct resouce name
    wsComponents.resourceName = (wsComponents.path || '/') + (wsComponents.query ? '?' + wsComponents.query : '');
    wsComponents.path = undefined;
    wsComponents.query = undefined;

    return wsComponents
  }

  function wsSerialize (wsComponents) {
  // normalize the default port
    if (wsComponents.port === (isSecure(wsComponents) ? 443 : 80) || wsComponents.port === '') {
      wsComponents.port = undefined;
    }

    // ensure scheme matches secure flag
    if (typeof wsComponents.secure === 'boolean') {
      wsComponents.scheme = (wsComponents.secure ? 'wss' : 'ws');
      wsComponents.secure = undefined;
    }

    // reconstruct path from resource name
    if (wsComponents.resourceName) {
      const [path, query] = wsComponents.resourceName.split('?');
      wsComponents.path = (path && path !== '/' ? path : undefined);
      wsComponents.query = query;
      wsComponents.resourceName = undefined;
    }

    // forbid fragment component
    wsComponents.fragment = undefined;

    return wsComponents
  }

  function urnParse (urnComponents, options) {
    if (!urnComponents.path) {
      urnComponents.error = 'URN can not be parsed';
      return urnComponents
    }
    const matches = urnComponents.path.match(URN_REG);
    if (matches) {
      const scheme = options.scheme || urnComponents.scheme || 'urn';
      urnComponents.nid = matches[1].toLowerCase();
      urnComponents.nss = matches[2];
      const urnScheme = `${scheme}:${options.nid || urnComponents.nid}`;
      const schemeHandler = SCHEMES$1[urnScheme];
      urnComponents.path = undefined;

      if (schemeHandler) {
        urnComponents = schemeHandler.parse(urnComponents, options);
      }
    } else {
      urnComponents.error = urnComponents.error || 'URN can not be parsed.';
    }

    return urnComponents
  }

  function urnSerialize (urnComponents, options) {
    const scheme = options.scheme || urnComponents.scheme || 'urn';
    const nid = urnComponents.nid.toLowerCase();
    const urnScheme = `${scheme}:${options.nid || nid}`;
    const schemeHandler = SCHEMES$1[urnScheme];

    if (schemeHandler) {
      urnComponents = schemeHandler.serialize(urnComponents, options);
    }

    const uriComponents = urnComponents;
    const nss = urnComponents.nss;
    uriComponents.path = `${nid || options.nid}:${nss}`;

    options.skipEscape = true;
    return uriComponents
  }

  function urnuuidParse (urnComponents, options) {
    const uuidComponents = urnComponents;
    uuidComponents.uuid = uuidComponents.nss;
    uuidComponents.nss = undefined;

    if (!options.tolerant && (!uuidComponents.uuid || !UUID_REG.test(uuidComponents.uuid))) {
      uuidComponents.error = uuidComponents.error || 'UUID is not valid.';
    }

    return uuidComponents
  }

  function urnuuidSerialize (uuidComponents) {
    const urnComponents = uuidComponents;
    // normalize UUID
    urnComponents.nss = (uuidComponents.uuid || '').toLowerCase();
    return urnComponents
  }

  const http = {
    scheme: 'http',
    domainHost: true,
    parse: httpParse,
    serialize: httpSerialize
  };

  const https = {
    scheme: 'https',
    domainHost: http.domainHost,
    parse: httpParse,
    serialize: httpSerialize
  };

  const ws = {
    scheme: 'ws',
    domainHost: true,
    parse: wsParse,
    serialize: wsSerialize
  };

  const wss = {
    scheme: 'wss',
    domainHost: ws.domainHost,
    parse: ws.parse,
    serialize: ws.serialize
  };

  const urn = {
    scheme: 'urn',
    parse: urnParse,
    serialize: urnSerialize,
    skipNormalize: true
  };

  const urnuuid = {
    scheme: 'urn:uuid',
    parse: urnuuidParse,
    serialize: urnuuidSerialize,
    skipNormalize: true
  };

  const SCHEMES$1 = {
    http,
    https,
    ws,
    wss,
    urn,
    'urn:uuid': urnuuid
  };

  var schemes = SCHEMES$1;

  const { normalizeIPv6, normalizeIPv4, removeDotSegments, recomposeAuthority, normalizeComponentEncoding } = utils;
  const SCHEMES = schemes;

  function normalize (uri, options) {
    if (typeof uri === 'string') {
      uri = serialize(parse(uri, options), options);
    } else if (typeof uri === 'object') {
      uri = parse(serialize(uri, options), options);
    }
    return uri
  }

  function resolve (baseURI, relativeURI, options) {
    const schemelessOptions = Object.assign({ scheme: 'null' }, options);
    const resolved = resolveComponents(parse(baseURI, schemelessOptions), parse(relativeURI, schemelessOptions), schemelessOptions, true);
    return serialize(resolved, { ...schemelessOptions, skipEscape: true })
  }

  function resolveComponents (base, relative, options, skipNormalization) {
    const target = {};
    if (!skipNormalization) {
      base = parse(serialize(base, options), options); // normalize base components
      relative = parse(serialize(relative, options), options); // normalize relative components
    }
    options = options || {};

    if (!options.tolerant && relative.scheme) {
      target.scheme = relative.scheme;
      // target.authority = relative.authority;
      target.userinfo = relative.userinfo;
      target.host = relative.host;
      target.port = relative.port;
      target.path = removeDotSegments(relative.path || '');
      target.query = relative.query;
    } else {
      if (relative.userinfo !== undefined || relative.host !== undefined || relative.port !== undefined) {
        // target.authority = relative.authority;
        target.userinfo = relative.userinfo;
        target.host = relative.host;
        target.port = relative.port;
        target.path = removeDotSegments(relative.path || '');
        target.query = relative.query;
      } else {
        if (!relative.path) {
          target.path = base.path;
          if (relative.query !== undefined) {
            target.query = relative.query;
          } else {
            target.query = base.query;
          }
        } else {
          if (relative.path.charAt(0) === '/') {
            target.path = removeDotSegments(relative.path);
          } else {
            if ((base.userinfo !== undefined || base.host !== undefined || base.port !== undefined) && !base.path) {
              target.path = '/' + relative.path;
            } else if (!base.path) {
              target.path = relative.path;
            } else {
              target.path = base.path.slice(0, base.path.lastIndexOf('/') + 1) + relative.path;
            }
            target.path = removeDotSegments(target.path);
          }
          target.query = relative.query;
        }
        // target.authority = base.authority;
        target.userinfo = base.userinfo;
        target.host = base.host;
        target.port = base.port;
      }
      target.scheme = base.scheme;
    }

    target.fragment = relative.fragment;

    return target
  }

  function equal$2 (uriA, uriB, options) {
    if (typeof uriA === 'string') {
      uriA = unescape(uriA);
      uriA = serialize(normalizeComponentEncoding(parse(uriA, options), true), { ...options, skipEscape: true });
    } else if (typeof uriA === 'object') {
      uriA = serialize(normalizeComponentEncoding(uriA, true), { ...options, skipEscape: true });
    }

    if (typeof uriB === 'string') {
      uriB = unescape(uriB);
      uriB = serialize(normalizeComponentEncoding(parse(uriB, options), true), { ...options, skipEscape: true });
    } else if (typeof uriB === 'object') {
      uriB = serialize(normalizeComponentEncoding(uriB, true), { ...options, skipEscape: true });
    }

    return uriA.toLowerCase() === uriB.toLowerCase()
  }

  function serialize (cmpts, opts) {
    const components = {
      host: cmpts.host,
      scheme: cmpts.scheme,
      userinfo: cmpts.userinfo,
      port: cmpts.port,
      path: cmpts.path,
      query: cmpts.query,
      nid: cmpts.nid,
      nss: cmpts.nss,
      uuid: cmpts.uuid,
      fragment: cmpts.fragment,
      reference: cmpts.reference,
      resourceName: cmpts.resourceName,
      secure: cmpts.secure,
      error: ''
    };
    const options = Object.assign({}, opts);
    const uriTokens = [];

    // find scheme handler
    const schemeHandler = SCHEMES[(options.scheme || components.scheme || '').toLowerCase()];

    // perform scheme specific serialization
    if (schemeHandler && schemeHandler.serialize) schemeHandler.serialize(components, options);

    if (components.path !== undefined) {
      if (!options.skipEscape) {
        components.path = escape(components.path);

        if (components.scheme !== undefined) {
          components.path = components.path.split('%3A').join(':');
        }
      } else {
        components.path = unescape(components.path);
      }
    }

    if (options.reference !== 'suffix' && components.scheme) {
      uriTokens.push(components.scheme);
      uriTokens.push(':');
    }

    const authority = recomposeAuthority(components, options);
    if (authority !== undefined) {
      if (options.reference !== 'suffix') {
        uriTokens.push('//');
      }

      uriTokens.push(authority);

      if (components.path && components.path.charAt(0) !== '/') {
        uriTokens.push('/');
      }
    }
    if (components.path !== undefined) {
      let s = components.path;

      if (!options.absolutePath && (!schemeHandler || !schemeHandler.absolutePath)) {
        s = removeDotSegments(s);
      }

      if (authority === undefined) {
        s = s.replace(/^\/\//u, '/%2F'); // don't allow the path to start with "//"
      }

      uriTokens.push(s);
    }

    if (components.query !== undefined) {
      uriTokens.push('?');
      uriTokens.push(components.query);
    }

    if (components.fragment !== undefined) {
      uriTokens.push('#');
      uriTokens.push(components.fragment);
    }
    return uriTokens.join('')
  }

  const hexLookUp = Array.from({ length: 127 }, (v, k) => /[^!"$&'()*+,\-.;=_`a-z{}~]/u.test(String.fromCharCode(k)));

  function nonSimpleDomain (value) {
    let code = 0;
    for (let i = 0, len = value.length; i < len; ++i) {
      code = value.charCodeAt(i);
      if (code > 126 || hexLookUp[code]) {
        return true
      }
    }
    return false
  }

  const URI_PARSE = /^(?:([^#/:?]+):)?(?:\/\/((?:([^#/?@]*)@)?(\[[^#/?\]]+\]|[^#/:?]*)(?::(\d*))?))?([^#?]*)(?:\?([^#]*))?(?:#((?:.|[\n\r])*))?/u;

  function parse (uri, opts) {
    const options = Object.assign({}, opts);
    const parsed = {
      scheme: undefined,
      userinfo: undefined,
      host: '',
      port: undefined,
      path: '',
      query: undefined,
      fragment: undefined
    };
    const gotEncoding = uri.indexOf('%') !== -1;
    let isIP = false;
    if (options.reference === 'suffix') uri = (options.scheme ? options.scheme + ':' : '') + '//' + uri;

    const matches = uri.match(URI_PARSE);

    if (matches) {
      // store each component
      parsed.scheme = matches[1];
      parsed.userinfo = matches[3];
      parsed.host = matches[4];
      parsed.port = parseInt(matches[5], 10);
      parsed.path = matches[6] || '';
      parsed.query = matches[7];
      parsed.fragment = matches[8];

      // fix port number
      if (isNaN(parsed.port)) {
        parsed.port = matches[5];
      }
      if (parsed.host) {
        const ipv4result = normalizeIPv4(parsed.host);
        if (ipv4result.isIPV4 === false) {
          const ipv6result = normalizeIPv6(ipv4result.host, { isIPV4: false });
          parsed.host = ipv6result.host.toLowerCase();
          isIP = ipv6result.isIPV6;
        } else {
          parsed.host = ipv4result.host;
          isIP = true;
        }
      }
      if (parsed.scheme === undefined && parsed.userinfo === undefined && parsed.host === undefined && parsed.port === undefined && !parsed.path && parsed.query === undefined) {
        parsed.reference = 'same-document';
      } else if (parsed.scheme === undefined) {
        parsed.reference = 'relative';
      } else if (parsed.fragment === undefined) {
        parsed.reference = 'absolute';
      } else {
        parsed.reference = 'uri';
      }

      // check for reference errors
      if (options.reference && options.reference !== 'suffix' && options.reference !== parsed.reference) {
        parsed.error = parsed.error || 'URI is not a ' + options.reference + ' reference.';
      }

      // find scheme handler
      const schemeHandler = SCHEMES[(options.scheme || parsed.scheme || '').toLowerCase()];

      // check if scheme can't handle IRIs
      if (!options.unicodeSupport && (!schemeHandler || !schemeHandler.unicodeSupport)) {
        // if host component is a domain name
        if (parsed.host && (options.domainHost || (schemeHandler && schemeHandler.domainHost)) && isIP === false && nonSimpleDomain(parsed.host)) {
          // convert Unicode IDN -> ASCII IDN
          try {
            parsed.host = URL.domainToASCII(parsed.host.toLowerCase());
          } catch (e) {
            parsed.error = parsed.error || "Host's domain name can not be converted to ASCII: " + e;
          }
        }
        // convert IRI -> URI
      }

      if (!schemeHandler || (schemeHandler && !schemeHandler.skipNormalize)) {
        if (gotEncoding && parsed.scheme !== undefined) {
          parsed.scheme = unescape(parsed.scheme);
        }
        if (gotEncoding && parsed.userinfo !== undefined) {
          parsed.userinfo = unescape(parsed.userinfo);
        }
        if (gotEncoding && parsed.host !== undefined) {
          parsed.host = unescape(parsed.host);
        }
        if (parsed.path !== undefined && parsed.path.length) {
          parsed.path = escape(unescape(parsed.path));
        }
        if (parsed.fragment !== undefined && parsed.fragment.length) {
          parsed.fragment = encodeURI(decodeURIComponent(parsed.fragment));
        }
      }

      // perform scheme specific parsing
      if (schemeHandler && schemeHandler.parse) {
        schemeHandler.parse(parsed, options);
      }
    } else {
      parsed.error = parsed.error || 'URI can not be parsed.';
    }
    return parsed
  }

  const fastUri = {
    SCHEMES,
    normalize,
    resolve,
    resolveComponents,
    equal: equal$2,
    serialize,
    parse
  };

  fastUri$1.exports = fastUri;
  fastUri$1.exports.default = fastUri;
  fastUri$1.exports.fastUri = fastUri;

  var fastUriExports = fastUri$1.exports;

  Object.defineProperty(uri$1, "__esModule", { value: true });
  const uri = fastUriExports;
  uri.code = 'require("ajv/dist/runtime/uri").default';
  uri$1.default = uri;

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.CodeGen = exports.Name = exports.nil = exports.stringify = exports.str = exports._ = exports.KeywordCxt = void 0;
  	var validate_1 = requireValidate();
  	Object.defineProperty(exports, "KeywordCxt", { enumerable: true, get: function () { return validate_1.KeywordCxt; } });
  	var codegen_1 = codegen;
  	Object.defineProperty(exports, "_", { enumerable: true, get: function () { return codegen_1._; } });
  	Object.defineProperty(exports, "str", { enumerable: true, get: function () { return codegen_1.str; } });
  	Object.defineProperty(exports, "stringify", { enumerable: true, get: function () { return codegen_1.stringify; } });
  	Object.defineProperty(exports, "nil", { enumerable: true, get: function () { return codegen_1.nil; } });
  	Object.defineProperty(exports, "Name", { enumerable: true, get: function () { return codegen_1.Name; } });
  	Object.defineProperty(exports, "CodeGen", { enumerable: true, get: function () { return codegen_1.CodeGen; } });
  	const validation_error_1 = requireValidation_error();
  	const ref_error_1 = requireRef_error();
  	const rules_1 = rules;
  	const compile_1 = compile;
  	const codegen_2 = codegen;
  	const resolve_1 = resolve$2;
  	const dataType_1 = dataType;
  	const util_1 = util;
  	const $dataRefSchema = require$$9;
  	const uri_1 = uri$1;
  	const defaultRegExp = (str, flags) => new RegExp(str, flags);
  	defaultRegExp.code = "new RegExp";
  	const META_IGNORE_OPTIONS = ["removeAdditional", "useDefaults", "coerceTypes"];
  	const EXT_SCOPE_NAMES = new Set([
  	    "validate",
  	    "serialize",
  	    "parse",
  	    "wrapper",
  	    "root",
  	    "schema",
  	    "keyword",
  	    "pattern",
  	    "formats",
  	    "validate$data",
  	    "func",
  	    "obj",
  	    "Error",
  	]);
  	const removedOptions = {
  	    errorDataPath: "",
  	    format: "`validateFormats: false` can be used instead.",
  	    nullable: '"nullable" keyword is supported by default.',
  	    jsonPointers: "Deprecated jsPropertySyntax can be used instead.",
  	    extendRefs: "Deprecated ignoreKeywordsWithRef can be used instead.",
  	    missingRefs: "Pass empty schema with $id that should be ignored to ajv.addSchema.",
  	    processCode: "Use option `code: {process: (code, schemaEnv: object) => string}`",
  	    sourceCode: "Use option `code: {source: true}`",
  	    strictDefaults: "It is default now, see option `strict`.",
  	    strictKeywords: "It is default now, see option `strict`.",
  	    uniqueItems: '"uniqueItems" keyword is always validated.',
  	    unknownFormats: "Disable strict mode or pass `true` to `ajv.addFormat` (or `formats` option).",
  	    cache: "Map is used as cache, schema object as key.",
  	    serialize: "Map is used as cache, schema object as key.",
  	    ajvErrors: "It is default now.",
  	};
  	const deprecatedOptions = {
  	    ignoreKeywordsWithRef: "",
  	    jsPropertySyntax: "",
  	    unicode: '"minLength"/"maxLength" account for unicode characters by default.',
  	};
  	const MAX_EXPRESSION = 200;
  	// eslint-disable-next-line complexity
  	function requiredOptions(o) {
  	    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0;
  	    const s = o.strict;
  	    const _optz = (_a = o.code) === null || _a === void 0 ? void 0 : _a.optimize;
  	    const optimize = _optz === true || _optz === undefined ? 1 : _optz || 0;
  	    const regExp = (_c = (_b = o.code) === null || _b === void 0 ? void 0 : _b.regExp) !== null && _c !== void 0 ? _c : defaultRegExp;
  	    const uriResolver = (_d = o.uriResolver) !== null && _d !== void 0 ? _d : uri_1.default;
  	    return {
  	        strictSchema: (_f = (_e = o.strictSchema) !== null && _e !== void 0 ? _e : s) !== null && _f !== void 0 ? _f : true,
  	        strictNumbers: (_h = (_g = o.strictNumbers) !== null && _g !== void 0 ? _g : s) !== null && _h !== void 0 ? _h : true,
  	        strictTypes: (_k = (_j = o.strictTypes) !== null && _j !== void 0 ? _j : s) !== null && _k !== void 0 ? _k : "log",
  	        strictTuples: (_m = (_l = o.strictTuples) !== null && _l !== void 0 ? _l : s) !== null && _m !== void 0 ? _m : "log",
  	        strictRequired: (_p = (_o = o.strictRequired) !== null && _o !== void 0 ? _o : s) !== null && _p !== void 0 ? _p : false,
  	        code: o.code ? { ...o.code, optimize, regExp } : { optimize, regExp },
  	        loopRequired: (_q = o.loopRequired) !== null && _q !== void 0 ? _q : MAX_EXPRESSION,
  	        loopEnum: (_r = o.loopEnum) !== null && _r !== void 0 ? _r : MAX_EXPRESSION,
  	        meta: (_s = o.meta) !== null && _s !== void 0 ? _s : true,
  	        messages: (_t = o.messages) !== null && _t !== void 0 ? _t : true,
  	        inlineRefs: (_u = o.inlineRefs) !== null && _u !== void 0 ? _u : true,
  	        schemaId: (_v = o.schemaId) !== null && _v !== void 0 ? _v : "$id",
  	        addUsedSchema: (_w = o.addUsedSchema) !== null && _w !== void 0 ? _w : true,
  	        validateSchema: (_x = o.validateSchema) !== null && _x !== void 0 ? _x : true,
  	        validateFormats: (_y = o.validateFormats) !== null && _y !== void 0 ? _y : true,
  	        unicodeRegExp: (_z = o.unicodeRegExp) !== null && _z !== void 0 ? _z : true,
  	        int32range: (_0 = o.int32range) !== null && _0 !== void 0 ? _0 : true,
  	        uriResolver: uriResolver,
  	    };
  	}
  	class Ajv {
  	    constructor(opts = {}) {
  	        this.schemas = {};
  	        this.refs = {};
  	        this.formats = {};
  	        this._compilations = new Set();
  	        this._loading = {};
  	        this._cache = new Map();
  	        opts = this.opts = { ...opts, ...requiredOptions(opts) };
  	        const { es5, lines } = this.opts.code;
  	        this.scope = new codegen_2.ValueScope({ scope: {}, prefixes: EXT_SCOPE_NAMES, es5, lines });
  	        this.logger = getLogger(opts.logger);
  	        const formatOpt = opts.validateFormats;
  	        opts.validateFormats = false;
  	        this.RULES = (0, rules_1.getRules)();
  	        checkOptions.call(this, removedOptions, opts, "NOT SUPPORTED");
  	        checkOptions.call(this, deprecatedOptions, opts, "DEPRECATED", "warn");
  	        this._metaOpts = getMetaSchemaOptions.call(this);
  	        if (opts.formats)
  	            addInitialFormats.call(this);
  	        this._addVocabularies();
  	        this._addDefaultMetaSchema();
  	        if (opts.keywords)
  	            addInitialKeywords.call(this, opts.keywords);
  	        if (typeof opts.meta == "object")
  	            this.addMetaSchema(opts.meta);
  	        addInitialSchemas.call(this);
  	        opts.validateFormats = formatOpt;
  	    }
  	    _addVocabularies() {
  	        this.addKeyword("$async");
  	    }
  	    _addDefaultMetaSchema() {
  	        const { $data, meta, schemaId } = this.opts;
  	        let _dataRefSchema = $dataRefSchema;
  	        if (schemaId === "id") {
  	            _dataRefSchema = { ...$dataRefSchema };
  	            _dataRefSchema.id = _dataRefSchema.$id;
  	            delete _dataRefSchema.$id;
  	        }
  	        if (meta && $data)
  	            this.addMetaSchema(_dataRefSchema, _dataRefSchema[schemaId], false);
  	    }
  	    defaultMeta() {
  	        const { meta, schemaId } = this.opts;
  	        return (this.opts.defaultMeta = typeof meta == "object" ? meta[schemaId] || meta : undefined);
  	    }
  	    validate(schemaKeyRef, // key, ref or schema object
  	    // eslint-disable-next-line @typescript-eslint/no-redundant-type-constituents
  	    data // to be validated
  	    ) {
  	        let v;
  	        if (typeof schemaKeyRef == "string") {
  	            v = this.getSchema(schemaKeyRef);
  	            if (!v)
  	                throw new Error(`no schema with key or ref "${schemaKeyRef}"`);
  	        }
  	        else {
  	            v = this.compile(schemaKeyRef);
  	        }
  	        const valid = v(data);
  	        if (!("$async" in v))
  	            this.errors = v.errors;
  	        return valid;
  	    }
  	    compile(schema, _meta) {
  	        const sch = this._addSchema(schema, _meta);
  	        return (sch.validate || this._compileSchemaEnv(sch));
  	    }
  	    compileAsync(schema, meta) {
  	        if (typeof this.opts.loadSchema != "function") {
  	            throw new Error("options.loadSchema should be a function");
  	        }
  	        const { loadSchema } = this.opts;
  	        return runCompileAsync.call(this, schema, meta);
  	        async function runCompileAsync(_schema, _meta) {
  	            await loadMetaSchema.call(this, _schema.$schema);
  	            const sch = this._addSchema(_schema, _meta);
  	            return sch.validate || _compileAsync.call(this, sch);
  	        }
  	        async function loadMetaSchema($ref) {
  	            if ($ref && !this.getSchema($ref)) {
  	                await runCompileAsync.call(this, { $ref }, true);
  	            }
  	        }
  	        async function _compileAsync(sch) {
  	            try {
  	                return this._compileSchemaEnv(sch);
  	            }
  	            catch (e) {
  	                if (!(e instanceof ref_error_1.default))
  	                    throw e;
  	                checkLoaded.call(this, e);
  	                await loadMissingSchema.call(this, e.missingSchema);
  	                return _compileAsync.call(this, sch);
  	            }
  	        }
  	        function checkLoaded({ missingSchema: ref, missingRef }) {
  	            if (this.refs[ref]) {
  	                throw new Error(`AnySchema ${ref} is loaded but ${missingRef} cannot be resolved`);
  	            }
  	        }
  	        async function loadMissingSchema(ref) {
  	            const _schema = await _loadSchema.call(this, ref);
  	            if (!this.refs[ref])
  	                await loadMetaSchema.call(this, _schema.$schema);
  	            if (!this.refs[ref])
  	                this.addSchema(_schema, ref, meta);
  	        }
  	        async function _loadSchema(ref) {
  	            const p = this._loading[ref];
  	            if (p)
  	                return p;
  	            try {
  	                return await (this._loading[ref] = loadSchema(ref));
  	            }
  	            finally {
  	                delete this._loading[ref];
  	            }
  	        }
  	    }
  	    // Adds schema to the instance
  	    addSchema(schema, // If array is passed, `key` will be ignored
  	    key, // Optional schema key. Can be passed to `validate` method instead of schema object or id/ref. One schema per instance can have empty `id` and `key`.
  	    _meta, // true if schema is a meta-schema. Used internally, addMetaSchema should be used instead.
  	    _validateSchema = this.opts.validateSchema // false to skip schema validation. Used internally, option validateSchema should be used instead.
  	    ) {
  	        if (Array.isArray(schema)) {
  	            for (const sch of schema)
  	                this.addSchema(sch, undefined, _meta, _validateSchema);
  	            return this;
  	        }
  	        let id;
  	        if (typeof schema === "object") {
  	            const { schemaId } = this.opts;
  	            id = schema[schemaId];
  	            if (id !== undefined && typeof id != "string") {
  	                throw new Error(`schema ${schemaId} must be string`);
  	            }
  	        }
  	        key = (0, resolve_1.normalizeId)(key || id);
  	        this._checkUnique(key);
  	        this.schemas[key] = this._addSchema(schema, _meta, key, _validateSchema, true);
  	        return this;
  	    }
  	    // Add schema that will be used to validate other schemas
  	    // options in META_IGNORE_OPTIONS are alway set to false
  	    addMetaSchema(schema, key, // schema key
  	    _validateSchema = this.opts.validateSchema // false to skip schema validation, can be used to override validateSchema option for meta-schema
  	    ) {
  	        this.addSchema(schema, key, true, _validateSchema);
  	        return this;
  	    }
  	    //  Validate schema against its meta-schema
  	    validateSchema(schema, throwOrLogError) {
  	        if (typeof schema == "boolean")
  	            return true;
  	        let $schema;
  	        $schema = schema.$schema;
  	        if ($schema !== undefined && typeof $schema != "string") {
  	            throw new Error("$schema must be a string");
  	        }
  	        $schema = $schema || this.opts.defaultMeta || this.defaultMeta();
  	        if (!$schema) {
  	            this.logger.warn("meta-schema not available");
  	            this.errors = null;
  	            return true;
  	        }
  	        const valid = this.validate($schema, schema);
  	        if (!valid && throwOrLogError) {
  	            const message = "schema is invalid: " + this.errorsText();
  	            if (this.opts.validateSchema === "log")
  	                this.logger.error(message);
  	            else
  	                throw new Error(message);
  	        }
  	        return valid;
  	    }
  	    // Get compiled schema by `key` or `ref`.
  	    // (`key` that was passed to `addSchema` or full schema reference - `schema.$id` or resolved id)
  	    getSchema(keyRef) {
  	        let sch;
  	        while (typeof (sch = getSchEnv.call(this, keyRef)) == "string")
  	            keyRef = sch;
  	        if (sch === undefined) {
  	            const { schemaId } = this.opts;
  	            const root = new compile_1.SchemaEnv({ schema: {}, schemaId });
  	            sch = compile_1.resolveSchema.call(this, root, keyRef);
  	            if (!sch)
  	                return;
  	            this.refs[keyRef] = sch;
  	        }
  	        return (sch.validate || this._compileSchemaEnv(sch));
  	    }
  	    // Remove cached schema(s).
  	    // If no parameter is passed all schemas but meta-schemas are removed.
  	    // If RegExp is passed all schemas with key/id matching pattern but meta-schemas are removed.
  	    // Even if schema is referenced by other schemas it still can be removed as other schemas have local references.
  	    removeSchema(schemaKeyRef) {
  	        if (schemaKeyRef instanceof RegExp) {
  	            this._removeAllSchemas(this.schemas, schemaKeyRef);
  	            this._removeAllSchemas(this.refs, schemaKeyRef);
  	            return this;
  	        }
  	        switch (typeof schemaKeyRef) {
  	            case "undefined":
  	                this._removeAllSchemas(this.schemas);
  	                this._removeAllSchemas(this.refs);
  	                this._cache.clear();
  	                return this;
  	            case "string": {
  	                const sch = getSchEnv.call(this, schemaKeyRef);
  	                if (typeof sch == "object")
  	                    this._cache.delete(sch.schema);
  	                delete this.schemas[schemaKeyRef];
  	                delete this.refs[schemaKeyRef];
  	                return this;
  	            }
  	            case "object": {
  	                const cacheKey = schemaKeyRef;
  	                this._cache.delete(cacheKey);
  	                let id = schemaKeyRef[this.opts.schemaId];
  	                if (id) {
  	                    id = (0, resolve_1.normalizeId)(id);
  	                    delete this.schemas[id];
  	                    delete this.refs[id];
  	                }
  	                return this;
  	            }
  	            default:
  	                throw new Error("ajv.removeSchema: invalid parameter");
  	        }
  	    }
  	    // add "vocabulary" - a collection of keywords
  	    addVocabulary(definitions) {
  	        for (const def of definitions)
  	            this.addKeyword(def);
  	        return this;
  	    }
  	    addKeyword(kwdOrDef, def // deprecated
  	    ) {
  	        let keyword;
  	        if (typeof kwdOrDef == "string") {
  	            keyword = kwdOrDef;
  	            if (typeof def == "object") {
  	                this.logger.warn("these parameters are deprecated, see docs for addKeyword");
  	                def.keyword = keyword;
  	            }
  	        }
  	        else if (typeof kwdOrDef == "object" && def === undefined) {
  	            def = kwdOrDef;
  	            keyword = def.keyword;
  	            if (Array.isArray(keyword) && !keyword.length) {
  	                throw new Error("addKeywords: keyword must be string or non-empty array");
  	            }
  	        }
  	        else {
  	            throw new Error("invalid addKeywords parameters");
  	        }
  	        checkKeyword.call(this, keyword, def);
  	        if (!def) {
  	            (0, util_1.eachItem)(keyword, (kwd) => addRule.call(this, kwd));
  	            return this;
  	        }
  	        keywordMetaschema.call(this, def);
  	        const definition = {
  	            ...def,
  	            type: (0, dataType_1.getJSONTypes)(def.type),
  	            schemaType: (0, dataType_1.getJSONTypes)(def.schemaType),
  	        };
  	        (0, util_1.eachItem)(keyword, definition.type.length === 0
  	            ? (k) => addRule.call(this, k, definition)
  	            : (k) => definition.type.forEach((t) => addRule.call(this, k, definition, t)));
  	        return this;
  	    }
  	    getKeyword(keyword) {
  	        const rule = this.RULES.all[keyword];
  	        return typeof rule == "object" ? rule.definition : !!rule;
  	    }
  	    // Remove keyword
  	    removeKeyword(keyword) {
  	        // TODO return type should be Ajv
  	        const { RULES } = this;
  	        delete RULES.keywords[keyword];
  	        delete RULES.all[keyword];
  	        for (const group of RULES.rules) {
  	            const i = group.rules.findIndex((rule) => rule.keyword === keyword);
  	            if (i >= 0)
  	                group.rules.splice(i, 1);
  	        }
  	        return this;
  	    }
  	    // Add format
  	    addFormat(name, format) {
  	        if (typeof format == "string")
  	            format = new RegExp(format);
  	        this.formats[name] = format;
  	        return this;
  	    }
  	    errorsText(errors = this.errors, // optional array of validation errors
  	    { separator = ", ", dataVar = "data" } = {} // optional options with properties `separator` and `dataVar`
  	    ) {
  	        if (!errors || errors.length === 0)
  	            return "No errors";
  	        return errors
  	            .map((e) => `${dataVar}${e.instancePath} ${e.message}`)
  	            .reduce((text, msg) => text + separator + msg);
  	    }
  	    $dataMetaSchema(metaSchema, keywordsJsonPointers) {
  	        const rules = this.RULES.all;
  	        metaSchema = JSON.parse(JSON.stringify(metaSchema));
  	        for (const jsonPointer of keywordsJsonPointers) {
  	            const segments = jsonPointer.split("/").slice(1); // first segment is an empty string
  	            let keywords = metaSchema;
  	            for (const seg of segments)
  	                keywords = keywords[seg];
  	            for (const key in rules) {
  	                const rule = rules[key];
  	                if (typeof rule != "object")
  	                    continue;
  	                const { $data } = rule.definition;
  	                const schema = keywords[key];
  	                if ($data && schema)
  	                    keywords[key] = schemaOrData(schema);
  	            }
  	        }
  	        return metaSchema;
  	    }
  	    _removeAllSchemas(schemas, regex) {
  	        for (const keyRef in schemas) {
  	            const sch = schemas[keyRef];
  	            if (!regex || regex.test(keyRef)) {
  	                if (typeof sch == "string") {
  	                    delete schemas[keyRef];
  	                }
  	                else if (sch && !sch.meta) {
  	                    this._cache.delete(sch.schema);
  	                    delete schemas[keyRef];
  	                }
  	            }
  	        }
  	    }
  	    _addSchema(schema, meta, baseId, validateSchema = this.opts.validateSchema, addSchema = this.opts.addUsedSchema) {
  	        let id;
  	        const { schemaId } = this.opts;
  	        if (typeof schema == "object") {
  	            id = schema[schemaId];
  	        }
  	        else {
  	            if (this.opts.jtd)
  	                throw new Error("schema must be object");
  	            else if (typeof schema != "boolean")
  	                throw new Error("schema must be object or boolean");
  	        }
  	        let sch = this._cache.get(schema);
  	        if (sch !== undefined)
  	            return sch;
  	        baseId = (0, resolve_1.normalizeId)(id || baseId);
  	        const localRefs = resolve_1.getSchemaRefs.call(this, schema, baseId);
  	        sch = new compile_1.SchemaEnv({ schema, schemaId, meta, baseId, localRefs });
  	        this._cache.set(sch.schema, sch);
  	        if (addSchema && !baseId.startsWith("#")) {
  	            // TODO atm it is allowed to overwrite schemas without id (instead of not adding them)
  	            if (baseId)
  	                this._checkUnique(baseId);
  	            this.refs[baseId] = sch;
  	        }
  	        if (validateSchema)
  	            this.validateSchema(schema, true);
  	        return sch;
  	    }
  	    _checkUnique(id) {
  	        if (this.schemas[id] || this.refs[id]) {
  	            throw new Error(`schema with key or id "${id}" already exists`);
  	        }
  	    }
  	    _compileSchemaEnv(sch) {
  	        if (sch.meta)
  	            this._compileMetaSchema(sch);
  	        else
  	            compile_1.compileSchema.call(this, sch);
  	        /* istanbul ignore if */
  	        if (!sch.validate)
  	            throw new Error("ajv implementation error");
  	        return sch.validate;
  	    }
  	    _compileMetaSchema(sch) {
  	        const currentOpts = this.opts;
  	        this.opts = this._metaOpts;
  	        try {
  	            compile_1.compileSchema.call(this, sch);
  	        }
  	        finally {
  	            this.opts = currentOpts;
  	        }
  	    }
  	}
  	Ajv.ValidationError = validation_error_1.default;
  	Ajv.MissingRefError = ref_error_1.default;
  	exports.default = Ajv;
  	function checkOptions(checkOpts, options, msg, log = "error") {
  	    for (const key in checkOpts) {
  	        const opt = key;
  	        if (opt in options)
  	            this.logger[log](`${msg}: option ${key}. ${checkOpts[opt]}`);
  	    }
  	}
  	function getSchEnv(keyRef) {
  	    keyRef = (0, resolve_1.normalizeId)(keyRef); // TODO tests fail without this line
  	    return this.schemas[keyRef] || this.refs[keyRef];
  	}
  	function addInitialSchemas() {
  	    const optsSchemas = this.opts.schemas;
  	    if (!optsSchemas)
  	        return;
  	    if (Array.isArray(optsSchemas))
  	        this.addSchema(optsSchemas);
  	    else
  	        for (const key in optsSchemas)
  	            this.addSchema(optsSchemas[key], key);
  	}
  	function addInitialFormats() {
  	    for (const name in this.opts.formats) {
  	        const format = this.opts.formats[name];
  	        if (format)
  	            this.addFormat(name, format);
  	    }
  	}
  	function addInitialKeywords(defs) {
  	    if (Array.isArray(defs)) {
  	        this.addVocabulary(defs);
  	        return;
  	    }
  	    this.logger.warn("keywords option as map is deprecated, pass array");
  	    for (const keyword in defs) {
  	        const def = defs[keyword];
  	        if (!def.keyword)
  	            def.keyword = keyword;
  	        this.addKeyword(def);
  	    }
  	}
  	function getMetaSchemaOptions() {
  	    const metaOpts = { ...this.opts };
  	    for (const opt of META_IGNORE_OPTIONS)
  	        delete metaOpts[opt];
  	    return metaOpts;
  	}
  	const noLogs = { log() { }, warn() { }, error() { } };
  	function getLogger(logger) {
  	    if (logger === false)
  	        return noLogs;
  	    if (logger === undefined)
  	        return console;
  	    if (logger.log && logger.warn && logger.error)
  	        return logger;
  	    throw new Error("logger must implement log, warn and error methods");
  	}
  	const KEYWORD_NAME = /^[a-z_$][a-z0-9_$:-]*$/i;
  	function checkKeyword(keyword, def) {
  	    const { RULES } = this;
  	    (0, util_1.eachItem)(keyword, (kwd) => {
  	        if (RULES.keywords[kwd])
  	            throw new Error(`Keyword ${kwd} is already defined`);
  	        if (!KEYWORD_NAME.test(kwd))
  	            throw new Error(`Keyword ${kwd} has invalid name`);
  	    });
  	    if (!def)
  	        return;
  	    if (def.$data && !("code" in def || "validate" in def)) {
  	        throw new Error('$data keyword must have "code" or "validate" function');
  	    }
  	}
  	function addRule(keyword, definition, dataType) {
  	    var _a;
  	    const post = definition === null || definition === void 0 ? void 0 : definition.post;
  	    if (dataType && post)
  	        throw new Error('keyword with "post" flag cannot have "type"');
  	    const { RULES } = this;
  	    let ruleGroup = post ? RULES.post : RULES.rules.find(({ type: t }) => t === dataType);
  	    if (!ruleGroup) {
  	        ruleGroup = { type: dataType, rules: [] };
  	        RULES.rules.push(ruleGroup);
  	    }
  	    RULES.keywords[keyword] = true;
  	    if (!definition)
  	        return;
  	    const rule = {
  	        keyword,
  	        definition: {
  	            ...definition,
  	            type: (0, dataType_1.getJSONTypes)(definition.type),
  	            schemaType: (0, dataType_1.getJSONTypes)(definition.schemaType),
  	        },
  	    };
  	    if (definition.before)
  	        addBeforeRule.call(this, ruleGroup, rule, definition.before);
  	    else
  	        ruleGroup.rules.push(rule);
  	    RULES.all[keyword] = rule;
  	    (_a = definition.implements) === null || _a === void 0 ? void 0 : _a.forEach((kwd) => this.addKeyword(kwd));
  	}
  	function addBeforeRule(ruleGroup, rule, before) {
  	    const i = ruleGroup.rules.findIndex((_rule) => _rule.keyword === before);
  	    if (i >= 0) {
  	        ruleGroup.rules.splice(i, 0, rule);
  	    }
  	    else {
  	        ruleGroup.rules.push(rule);
  	        this.logger.warn(`rule ${before} is not defined`);
  	    }
  	}
  	function keywordMetaschema(def) {
  	    let { metaSchema } = def;
  	    if (metaSchema === undefined)
  	        return;
  	    if (def.$data && this.opts.$data)
  	        metaSchema = schemaOrData(metaSchema);
  	    def.validateSchema = this.compile(metaSchema, true);
  	}
  	const $dataRef = {
  	    $ref: "https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#",
  	};
  	function schemaOrData(schema) {
  	    return { anyOf: [schema, $dataRef] };
  	}
  	
  } (core$4));

  var draft2020 = {};

  var core$3 = {};

  var id = {};

  Object.defineProperty(id, "__esModule", { value: true });
  const def$B = {
      keyword: "id",
      code() {
          throw new Error('NOT SUPPORTED: keyword "id", use "$id" for schema ID');
      },
  };
  id.default = def$B;

  var ref = {};

  Object.defineProperty(ref, "__esModule", { value: true });
  ref.callRef = ref.getValidate = void 0;
  const ref_error_1$1 = requireRef_error();
  const code_1$8 = requireCode();
  const codegen_1$p = codegen;
  const names_1$4 = requireNames();
  const compile_1$2 = compile;
  const util_1$n = util;
  const def$A = {
      keyword: "$ref",
      schemaType: "string",
      code(cxt) {
          const { gen, schema: $ref, it } = cxt;
          const { baseId, schemaEnv: env, validateName, opts, self } = it;
          const { root } = env;
          if (($ref === "#" || $ref === "#/") && baseId === root.baseId)
              return callRootRef();
          const schOrEnv = compile_1$2.resolveRef.call(self, root, baseId, $ref);
          if (schOrEnv === undefined)
              throw new ref_error_1$1.default(it.opts.uriResolver, baseId, $ref);
          if (schOrEnv instanceof compile_1$2.SchemaEnv)
              return callValidate(schOrEnv);
          return inlineRefSchema(schOrEnv);
          function callRootRef() {
              if (env === root)
                  return callRef(cxt, validateName, env, env.$async);
              const rootName = gen.scopeValue("root", { ref: root });
              return callRef(cxt, (0, codegen_1$p._) `${rootName}.validate`, root, root.$async);
          }
          function callValidate(sch) {
              const v = getValidate(cxt, sch);
              callRef(cxt, v, sch, sch.$async);
          }
          function inlineRefSchema(sch) {
              const schName = gen.scopeValue("schema", opts.code.source === true ? { ref: sch, code: (0, codegen_1$p.stringify)(sch) } : { ref: sch });
              const valid = gen.name("valid");
              const schCxt = cxt.subschema({
                  schema: sch,
                  dataTypes: [],
                  schemaPath: codegen_1$p.nil,
                  topSchemaRef: schName,
                  errSchemaPath: $ref,
              }, valid);
              cxt.mergeEvaluated(schCxt);
              cxt.ok(valid);
          }
      },
  };
  function getValidate(cxt, sch) {
      const { gen } = cxt;
      return sch.validate
          ? gen.scopeValue("validate", { ref: sch.validate })
          : (0, codegen_1$p._) `${gen.scopeValue("wrapper", { ref: sch })}.validate`;
  }
  ref.getValidate = getValidate;
  function callRef(cxt, v, sch, $async) {
      const { gen, it } = cxt;
      const { allErrors, schemaEnv: env, opts } = it;
      const passCxt = opts.passContext ? names_1$4.default.this : codegen_1$p.nil;
      if ($async)
          callAsyncRef();
      else
          callSyncRef();
      function callAsyncRef() {
          if (!env.$async)
              throw new Error("async schema referenced by sync schema");
          const valid = gen.let("valid");
          gen.try(() => {
              gen.code((0, codegen_1$p._) `await ${(0, code_1$8.callValidateCode)(cxt, v, passCxt)}`);
              addEvaluatedFrom(v); // TODO will not work with async, it has to be returned with the result
              if (!allErrors)
                  gen.assign(valid, true);
          }, (e) => {
              gen.if((0, codegen_1$p._) `!(${e} instanceof ${it.ValidationError})`, () => gen.throw(e));
              addErrorsFrom(e);
              if (!allErrors)
                  gen.assign(valid, false);
          });
          cxt.ok(valid);
      }
      function callSyncRef() {
          cxt.result((0, code_1$8.callValidateCode)(cxt, v, passCxt), () => addEvaluatedFrom(v), () => addErrorsFrom(v));
      }
      function addErrorsFrom(source) {
          const errs = (0, codegen_1$p._) `${source}.errors`;
          gen.assign(names_1$4.default.vErrors, (0, codegen_1$p._) `${names_1$4.default.vErrors} === null ? ${errs} : ${names_1$4.default.vErrors}.concat(${errs})`); // TODO tagged
          gen.assign(names_1$4.default.errors, (0, codegen_1$p._) `${names_1$4.default.vErrors}.length`);
      }
      function addEvaluatedFrom(source) {
          var _a;
          if (!it.opts.unevaluated)
              return;
          const schEvaluated = (_a = sch === null || sch === void 0 ? void 0 : sch.validate) === null || _a === void 0 ? void 0 : _a.evaluated;
          // TODO refactor
          if (it.props !== true) {
              if (schEvaluated && !schEvaluated.dynamicProps) {
                  if (schEvaluated.props !== undefined) {
                      it.props = util_1$n.mergeEvaluated.props(gen, schEvaluated.props, it.props);
                  }
              }
              else {
                  const props = gen.var("props", (0, codegen_1$p._) `${source}.evaluated.props`);
                  it.props = util_1$n.mergeEvaluated.props(gen, props, it.props, codegen_1$p.Name);
              }
          }
          if (it.items !== true) {
              if (schEvaluated && !schEvaluated.dynamicItems) {
                  if (schEvaluated.items !== undefined) {
                      it.items = util_1$n.mergeEvaluated.items(gen, schEvaluated.items, it.items);
                  }
              }
              else {
                  const items = gen.var("items", (0, codegen_1$p._) `${source}.evaluated.items`);
                  it.items = util_1$n.mergeEvaluated.items(gen, items, it.items, codegen_1$p.Name);
              }
          }
      }
  }
  ref.callRef = callRef;
  ref.default = def$A;

  Object.defineProperty(core$3, "__esModule", { value: true });
  const id_1 = id;
  const ref_1$2 = ref;
  const core$2 = [
      "$schema",
      "$id",
      "$defs",
      "$vocabulary",
      { keyword: "$comment" },
      "definitions",
      id_1.default,
      ref_1$2.default,
  ];
  core$3.default = core$2;

  var validation$2 = {};

  var limitNumber = {};

  Object.defineProperty(limitNumber, "__esModule", { value: true });
  const codegen_1$o = codegen;
  const ops = codegen_1$o.operators;
  const KWDs = {
      maximum: { okStr: "<=", ok: ops.LTE, fail: ops.GT },
      minimum: { okStr: ">=", ok: ops.GTE, fail: ops.LT },
      exclusiveMaximum: { okStr: "<", ok: ops.LT, fail: ops.GTE },
      exclusiveMinimum: { okStr: ">", ok: ops.GT, fail: ops.LTE },
  };
  const error$k = {
      message: ({ keyword, schemaCode }) => (0, codegen_1$o.str) `must be ${KWDs[keyword].okStr} ${schemaCode}`,
      params: ({ keyword, schemaCode }) => (0, codegen_1$o._) `{comparison: ${KWDs[keyword].okStr}, limit: ${schemaCode}}`,
  };
  const def$z = {
      keyword: Object.keys(KWDs),
      type: "number",
      schemaType: "number",
      $data: true,
      error: error$k,
      code(cxt) {
          const { keyword, data, schemaCode } = cxt;
          cxt.fail$data((0, codegen_1$o._) `${data} ${KWDs[keyword].fail} ${schemaCode} || isNaN(${data})`);
      },
  };
  limitNumber.default = def$z;

  var multipleOf = {};

  Object.defineProperty(multipleOf, "__esModule", { value: true });
  const codegen_1$n = codegen;
  const error$j = {
      message: ({ schemaCode }) => (0, codegen_1$n.str) `must be multiple of ${schemaCode}`,
      params: ({ schemaCode }) => (0, codegen_1$n._) `{multipleOf: ${schemaCode}}`,
  };
  const def$y = {
      keyword: "multipleOf",
      type: "number",
      schemaType: "number",
      $data: true,
      error: error$j,
      code(cxt) {
          const { gen, data, schemaCode, it } = cxt;
          // const bdt = bad$DataType(schemaCode, <string>def.schemaType, $data)
          const prec = it.opts.multipleOfPrecision;
          const res = gen.let("res");
          const invalid = prec
              ? (0, codegen_1$n._) `Math.abs(Math.round(${res}) - ${res}) > 1e-${prec}`
              : (0, codegen_1$n._) `${res} !== parseInt(${res})`;
          cxt.fail$data((0, codegen_1$n._) `(${schemaCode} === 0 || (${res} = ${data}/${schemaCode}, ${invalid}))`);
      },
  };
  multipleOf.default = def$y;

  var limitLength = {};

  var ucs2length$1 = {};

  Object.defineProperty(ucs2length$1, "__esModule", { value: true });
  // https://mathiasbynens.be/notes/javascript-encoding
  // https://github.com/bestiejs/punycode.js - punycode.ucs2.decode
  function ucs2length(str) {
      const len = str.length;
      let length = 0;
      let pos = 0;
      let value;
      while (pos < len) {
          length++;
          value = str.charCodeAt(pos++);
          if (value >= 0xd800 && value <= 0xdbff && pos < len) {
              // high surrogate, and there is a next character
              value = str.charCodeAt(pos);
              if ((value & 0xfc00) === 0xdc00)
                  pos++; // low surrogate
          }
      }
      return length;
  }
  ucs2length$1.default = ucs2length;
  ucs2length.code = 'require("ajv/dist/runtime/ucs2length").default';

  Object.defineProperty(limitLength, "__esModule", { value: true });
  const codegen_1$m = codegen;
  const util_1$m = util;
  const ucs2length_1 = ucs2length$1;
  const error$i = {
      message({ keyword, schemaCode }) {
          const comp = keyword === "maxLength" ? "more" : "fewer";
          return (0, codegen_1$m.str) `must NOT have ${comp} than ${schemaCode} characters`;
      },
      params: ({ schemaCode }) => (0, codegen_1$m._) `{limit: ${schemaCode}}`,
  };
  const def$x = {
      keyword: ["maxLength", "minLength"],
      type: "string",
      schemaType: "number",
      $data: true,
      error: error$i,
      code(cxt) {
          const { keyword, data, schemaCode, it } = cxt;
          const op = keyword === "maxLength" ? codegen_1$m.operators.GT : codegen_1$m.operators.LT;
          const len = it.opts.unicode === false ? (0, codegen_1$m._) `${data}.length` : (0, codegen_1$m._) `${(0, util_1$m.useFunc)(cxt.gen, ucs2length_1.default)}(${data})`;
          cxt.fail$data((0, codegen_1$m._) `${len} ${op} ${schemaCode}`);
      },
  };
  limitLength.default = def$x;

  var pattern = {};

  Object.defineProperty(pattern, "__esModule", { value: true });
  const code_1$7 = requireCode();
  const codegen_1$l = codegen;
  const error$h = {
      message: ({ schemaCode }) => (0, codegen_1$l.str) `must match pattern "${schemaCode}"`,
      params: ({ schemaCode }) => (0, codegen_1$l._) `{pattern: ${schemaCode}}`,
  };
  const def$w = {
      keyword: "pattern",
      type: "string",
      schemaType: "string",
      $data: true,
      error: error$h,
      code(cxt) {
          const { data, $data, schema, schemaCode, it } = cxt;
          // TODO regexp should be wrapped in try/catchs
          const u = it.opts.unicodeRegExp ? "u" : "";
          const regExp = $data ? (0, codegen_1$l._) `(new RegExp(${schemaCode}, ${u}))` : (0, code_1$7.usePattern)(cxt, schema);
          cxt.fail$data((0, codegen_1$l._) `!${regExp}.test(${data})`);
      },
  };
  pattern.default = def$w;

  var limitProperties = {};

  Object.defineProperty(limitProperties, "__esModule", { value: true });
  const codegen_1$k = codegen;
  const error$g = {
      message({ keyword, schemaCode }) {
          const comp = keyword === "maxProperties" ? "more" : "fewer";
          return (0, codegen_1$k.str) `must NOT have ${comp} than ${schemaCode} properties`;
      },
      params: ({ schemaCode }) => (0, codegen_1$k._) `{limit: ${schemaCode}}`,
  };
  const def$v = {
      keyword: ["maxProperties", "minProperties"],
      type: "object",
      schemaType: "number",
      $data: true,
      error: error$g,
      code(cxt) {
          const { keyword, data, schemaCode } = cxt;
          const op = keyword === "maxProperties" ? codegen_1$k.operators.GT : codegen_1$k.operators.LT;
          cxt.fail$data((0, codegen_1$k._) `Object.keys(${data}).length ${op} ${schemaCode}`);
      },
  };
  limitProperties.default = def$v;

  var required$1 = {};

  Object.defineProperty(required$1, "__esModule", { value: true });
  const code_1$6 = requireCode();
  const codegen_1$j = codegen;
  const util_1$l = util;
  const error$f = {
      message: ({ params: { missingProperty } }) => (0, codegen_1$j.str) `must have required property '${missingProperty}'`,
      params: ({ params: { missingProperty } }) => (0, codegen_1$j._) `{missingProperty: ${missingProperty}}`,
  };
  const def$u = {
      keyword: "required",
      type: "object",
      schemaType: "array",
      $data: true,
      error: error$f,
      code(cxt) {
          const { gen, schema, schemaCode, data, $data, it } = cxt;
          const { opts } = it;
          if (!$data && schema.length === 0)
              return;
          const useLoop = schema.length >= opts.loopRequired;
          if (it.allErrors)
              allErrorsMode();
          else
              exitOnErrorMode();
          if (opts.strictRequired) {
              const props = cxt.parentSchema.properties;
              const { definedProperties } = cxt.it;
              for (const requiredKey of schema) {
                  if ((props === null || props === void 0 ? void 0 : props[requiredKey]) === undefined && !definedProperties.has(requiredKey)) {
                      const schemaPath = it.schemaEnv.baseId + it.errSchemaPath;
                      const msg = `required property "${requiredKey}" is not defined at "${schemaPath}" (strictRequired)`;
                      (0, util_1$l.checkStrictMode)(it, msg, it.opts.strictRequired);
                  }
              }
          }
          function allErrorsMode() {
              if (useLoop || $data) {
                  cxt.block$data(codegen_1$j.nil, loopAllRequired);
              }
              else {
                  for (const prop of schema) {
                      (0, code_1$6.checkReportMissingProp)(cxt, prop);
                  }
              }
          }
          function exitOnErrorMode() {
              const missing = gen.let("missing");
              if (useLoop || $data) {
                  const valid = gen.let("valid", true);
                  cxt.block$data(valid, () => loopUntilMissing(missing, valid));
                  cxt.ok(valid);
              }
              else {
                  gen.if((0, code_1$6.checkMissingProp)(cxt, schema, missing));
                  (0, code_1$6.reportMissingProp)(cxt, missing);
                  gen.else();
              }
          }
          function loopAllRequired() {
              gen.forOf("prop", schemaCode, (prop) => {
                  cxt.setParams({ missingProperty: prop });
                  gen.if((0, code_1$6.noPropertyInData)(gen, data, prop, opts.ownProperties), () => cxt.error());
              });
          }
          function loopUntilMissing(missing, valid) {
              cxt.setParams({ missingProperty: missing });
              gen.forOf(missing, schemaCode, () => {
                  gen.assign(valid, (0, code_1$6.propertyInData)(gen, data, missing, opts.ownProperties));
                  gen.if((0, codegen_1$j.not)(valid), () => {
                      cxt.error();
                      gen.break();
                  });
              }, codegen_1$j.nil);
          }
      },
  };
  required$1.default = def$u;

  var limitItems = {};

  Object.defineProperty(limitItems, "__esModule", { value: true });
  const codegen_1$i = codegen;
  const error$e = {
      message({ keyword, schemaCode }) {
          const comp = keyword === "maxItems" ? "more" : "fewer";
          return (0, codegen_1$i.str) `must NOT have ${comp} than ${schemaCode} items`;
      },
      params: ({ schemaCode }) => (0, codegen_1$i._) `{limit: ${schemaCode}}`,
  };
  const def$t = {
      keyword: ["maxItems", "minItems"],
      type: "array",
      schemaType: "number",
      $data: true,
      error: error$e,
      code(cxt) {
          const { keyword, data, schemaCode } = cxt;
          const op = keyword === "maxItems" ? codegen_1$i.operators.GT : codegen_1$i.operators.LT;
          cxt.fail$data((0, codegen_1$i._) `${data}.length ${op} ${schemaCode}`);
      },
  };
  limitItems.default = def$t;

  var uniqueItems = {};

  var equal$1 = {};

  Object.defineProperty(equal$1, "__esModule", { value: true });
  // https://github.com/ajv-validator/ajv/issues/889
  const equal = fastDeepEqual;
  equal.code = 'require("ajv/dist/runtime/equal").default';
  equal$1.default = equal;

  Object.defineProperty(uniqueItems, "__esModule", { value: true });
  const dataType_1 = dataType;
  const codegen_1$h = codegen;
  const util_1$k = util;
  const equal_1$2 = equal$1;
  const error$d = {
      message: ({ params: { i, j } }) => (0, codegen_1$h.str) `must NOT have duplicate items (items ## ${j} and ${i} are identical)`,
      params: ({ params: { i, j } }) => (0, codegen_1$h._) `{i: ${i}, j: ${j}}`,
  };
  const def$s = {
      keyword: "uniqueItems",
      type: "array",
      schemaType: "boolean",
      $data: true,
      error: error$d,
      code(cxt) {
          const { gen, data, $data, schema, parentSchema, schemaCode, it } = cxt;
          if (!$data && !schema)
              return;
          const valid = gen.let("valid");
          const itemTypes = parentSchema.items ? (0, dataType_1.getSchemaTypes)(parentSchema.items) : [];
          cxt.block$data(valid, validateUniqueItems, (0, codegen_1$h._) `${schemaCode} === false`);
          cxt.ok(valid);
          function validateUniqueItems() {
              const i = gen.let("i", (0, codegen_1$h._) `${data}.length`);
              const j = gen.let("j");
              cxt.setParams({ i, j });
              gen.assign(valid, true);
              gen.if((0, codegen_1$h._) `${i} > 1`, () => (canOptimize() ? loopN : loopN2)(i, j));
          }
          function canOptimize() {
              return itemTypes.length > 0 && !itemTypes.some((t) => t === "object" || t === "array");
          }
          function loopN(i, j) {
              const item = gen.name("item");
              const wrongType = (0, dataType_1.checkDataTypes)(itemTypes, item, it.opts.strictNumbers, dataType_1.DataType.Wrong);
              const indices = gen.const("indices", (0, codegen_1$h._) `{}`);
              gen.for((0, codegen_1$h._) `;${i}--;`, () => {
                  gen.let(item, (0, codegen_1$h._) `${data}[${i}]`);
                  gen.if(wrongType, (0, codegen_1$h._) `continue`);
                  if (itemTypes.length > 1)
                      gen.if((0, codegen_1$h._) `typeof ${item} == "string"`, (0, codegen_1$h._) `${item} += "_"`);
                  gen
                      .if((0, codegen_1$h._) `typeof ${indices}[${item}] == "number"`, () => {
                      gen.assign(j, (0, codegen_1$h._) `${indices}[${item}]`);
                      cxt.error();
                      gen.assign(valid, false).break();
                  })
                      .code((0, codegen_1$h._) `${indices}[${item}] = ${i}`);
              });
          }
          function loopN2(i, j) {
              const eql = (0, util_1$k.useFunc)(gen, equal_1$2.default);
              const outer = gen.name("outer");
              gen.label(outer).for((0, codegen_1$h._) `;${i}--;`, () => gen.for((0, codegen_1$h._) `${j} = ${i}; ${j}--;`, () => gen.if((0, codegen_1$h._) `${eql}(${data}[${i}], ${data}[${j}])`, () => {
                  cxt.error();
                  gen.assign(valid, false).break(outer);
              })));
          }
      },
  };
  uniqueItems.default = def$s;

  var _const = {};

  Object.defineProperty(_const, "__esModule", { value: true });
  const codegen_1$g = codegen;
  const util_1$j = util;
  const equal_1$1 = equal$1;
  const error$c = {
      message: "must be equal to constant",
      params: ({ schemaCode }) => (0, codegen_1$g._) `{allowedValue: ${schemaCode}}`,
  };
  const def$r = {
      keyword: "const",
      $data: true,
      error: error$c,
      code(cxt) {
          const { gen, data, $data, schemaCode, schema } = cxt;
          if ($data || (schema && typeof schema == "object")) {
              cxt.fail$data((0, codegen_1$g._) `!${(0, util_1$j.useFunc)(gen, equal_1$1.default)}(${data}, ${schemaCode})`);
          }
          else {
              cxt.fail((0, codegen_1$g._) `${schema} !== ${data}`);
          }
      },
  };
  _const.default = def$r;

  var _enum = {};

  Object.defineProperty(_enum, "__esModule", { value: true });
  const codegen_1$f = codegen;
  const util_1$i = util;
  const equal_1 = equal$1;
  const error$b = {
      message: "must be equal to one of the allowed values",
      params: ({ schemaCode }) => (0, codegen_1$f._) `{allowedValues: ${schemaCode}}`,
  };
  const def$q = {
      keyword: "enum",
      schemaType: "array",
      $data: true,
      error: error$b,
      code(cxt) {
          const { gen, data, $data, schema, schemaCode, it } = cxt;
          if (!$data && schema.length === 0)
              throw new Error("enum must have non-empty array");
          const useLoop = schema.length >= it.opts.loopEnum;
          let eql;
          const getEql = () => (eql !== null && eql !== void 0 ? eql : (eql = (0, util_1$i.useFunc)(gen, equal_1.default)));
          let valid;
          if (useLoop || $data) {
              valid = gen.let("valid");
              cxt.block$data(valid, loopEnum);
          }
          else {
              /* istanbul ignore if */
              if (!Array.isArray(schema))
                  throw new Error("ajv implementation error");
              const vSchema = gen.const("vSchema", schemaCode);
              valid = (0, codegen_1$f.or)(...schema.map((_x, i) => equalCode(vSchema, i)));
          }
          cxt.pass(valid);
          function loopEnum() {
              gen.assign(valid, false);
              gen.forOf("v", schemaCode, (v) => gen.if((0, codegen_1$f._) `${getEql()}(${data}, ${v})`, () => gen.assign(valid, true).break()));
          }
          function equalCode(vSchema, i) {
              const sch = schema[i];
              return typeof sch === "object" && sch !== null
                  ? (0, codegen_1$f._) `${getEql()}(${data}, ${vSchema}[${i}])`
                  : (0, codegen_1$f._) `${data} === ${sch}`;
          }
      },
  };
  _enum.default = def$q;

  Object.defineProperty(validation$2, "__esModule", { value: true });
  const limitNumber_1 = limitNumber;
  const multipleOf_1 = multipleOf;
  const limitLength_1 = limitLength;
  const pattern_1 = pattern;
  const limitProperties_1 = limitProperties;
  const required_1 = required$1;
  const limitItems_1 = limitItems;
  const uniqueItems_1 = uniqueItems;
  const const_1 = _const;
  const enum_1 = _enum;
  const validation$1 = [
      // number
      limitNumber_1.default,
      multipleOf_1.default,
      // string
      limitLength_1.default,
      pattern_1.default,
      // object
      limitProperties_1.default,
      required_1.default,
      // array
      limitItems_1.default,
      uniqueItems_1.default,
      // any
      { keyword: "type", schemaType: ["string", "array"] },
      { keyword: "nullable", schemaType: "boolean" },
      const_1.default,
      enum_1.default,
  ];
  validation$2.default = validation$1;

  var applicator$1 = {};

  var additionalItems = {};

  Object.defineProperty(additionalItems, "__esModule", { value: true });
  additionalItems.validateAdditionalItems = void 0;
  const codegen_1$e = codegen;
  const util_1$h = util;
  const error$a = {
      message: ({ params: { len } }) => (0, codegen_1$e.str) `must NOT have more than ${len} items`,
      params: ({ params: { len } }) => (0, codegen_1$e._) `{limit: ${len}}`,
  };
  const def$p = {
      keyword: "additionalItems",
      type: "array",
      schemaType: ["boolean", "object"],
      before: "uniqueItems",
      error: error$a,
      code(cxt) {
          const { parentSchema, it } = cxt;
          const { items } = parentSchema;
          if (!Array.isArray(items)) {
              (0, util_1$h.checkStrictMode)(it, '"additionalItems" is ignored when "items" is not an array of schemas');
              return;
          }
          validateAdditionalItems(cxt, items);
      },
  };
  function validateAdditionalItems(cxt, items) {
      const { gen, schema, data, keyword, it } = cxt;
      it.items = true;
      const len = gen.const("len", (0, codegen_1$e._) `${data}.length`);
      if (schema === false) {
          cxt.setParams({ len: items.length });
          cxt.pass((0, codegen_1$e._) `${len} <= ${items.length}`);
      }
      else if (typeof schema == "object" && !(0, util_1$h.alwaysValidSchema)(it, schema)) {
          const valid = gen.var("valid", (0, codegen_1$e._) `${len} <= ${items.length}`); // TODO var
          gen.if((0, codegen_1$e.not)(valid), () => validateItems(valid));
          cxt.ok(valid);
      }
      function validateItems(valid) {
          gen.forRange("i", items.length, len, (i) => {
              cxt.subschema({ keyword, dataProp: i, dataPropType: util_1$h.Type.Num }, valid);
              if (!it.allErrors)
                  gen.if((0, codegen_1$e.not)(valid), () => gen.break());
          });
      }
  }
  additionalItems.validateAdditionalItems = validateAdditionalItems;
  additionalItems.default = def$p;

  var prefixItems = {};

  var items = {};

  Object.defineProperty(items, "__esModule", { value: true });
  items.validateTuple = void 0;
  const codegen_1$d = codegen;
  const util_1$g = util;
  const code_1$5 = requireCode();
  const def$o = {
      keyword: "items",
      type: "array",
      schemaType: ["object", "array", "boolean"],
      before: "uniqueItems",
      code(cxt) {
          const { schema, it } = cxt;
          if (Array.isArray(schema))
              return validateTuple(cxt, "additionalItems", schema);
          it.items = true;
          if ((0, util_1$g.alwaysValidSchema)(it, schema))
              return;
          cxt.ok((0, code_1$5.validateArray)(cxt));
      },
  };
  function validateTuple(cxt, extraItems, schArr = cxt.schema) {
      const { gen, parentSchema, data, keyword, it } = cxt;
      checkStrictTuple(parentSchema);
      if (it.opts.unevaluated && schArr.length && it.items !== true) {
          it.items = util_1$g.mergeEvaluated.items(gen, schArr.length, it.items);
      }
      const valid = gen.name("valid");
      const len = gen.const("len", (0, codegen_1$d._) `${data}.length`);
      schArr.forEach((sch, i) => {
          if ((0, util_1$g.alwaysValidSchema)(it, sch))
              return;
          gen.if((0, codegen_1$d._) `${len} > ${i}`, () => cxt.subschema({
              keyword,
              schemaProp: i,
              dataProp: i,
          }, valid));
          cxt.ok(valid);
      });
      function checkStrictTuple(sch) {
          const { opts, errSchemaPath } = it;
          const l = schArr.length;
          const fullTuple = l === sch.minItems && (l === sch.maxItems || sch[extraItems] === false);
          if (opts.strictTuples && !fullTuple) {
              const msg = `"${keyword}" is ${l}-tuple, but minItems or maxItems/${extraItems} are not specified or different at path "${errSchemaPath}"`;
              (0, util_1$g.checkStrictMode)(it, msg, opts.strictTuples);
          }
      }
  }
  items.validateTuple = validateTuple;
  items.default = def$o;

  Object.defineProperty(prefixItems, "__esModule", { value: true });
  const items_1$1 = items;
  const def$n = {
      keyword: "prefixItems",
      type: "array",
      schemaType: ["array"],
      before: "uniqueItems",
      code: (cxt) => (0, items_1$1.validateTuple)(cxt, "items"),
  };
  prefixItems.default = def$n;

  var items2020 = {};

  Object.defineProperty(items2020, "__esModule", { value: true });
  const codegen_1$c = codegen;
  const util_1$f = util;
  const code_1$4 = requireCode();
  const additionalItems_1$1 = additionalItems;
  const error$9 = {
      message: ({ params: { len } }) => (0, codegen_1$c.str) `must NOT have more than ${len} items`,
      params: ({ params: { len } }) => (0, codegen_1$c._) `{limit: ${len}}`,
  };
  const def$m = {
      keyword: "items",
      type: "array",
      schemaType: ["object", "boolean"],
      before: "uniqueItems",
      error: error$9,
      code(cxt) {
          const { schema, parentSchema, it } = cxt;
          const { prefixItems } = parentSchema;
          it.items = true;
          if ((0, util_1$f.alwaysValidSchema)(it, schema))
              return;
          if (prefixItems)
              (0, additionalItems_1$1.validateAdditionalItems)(cxt, prefixItems);
          else
              cxt.ok((0, code_1$4.validateArray)(cxt));
      },
  };
  items2020.default = def$m;

  var contains = {};

  Object.defineProperty(contains, "__esModule", { value: true });
  const codegen_1$b = codegen;
  const util_1$e = util;
  const error$8 = {
      message: ({ params: { min, max } }) => max === undefined
          ? (0, codegen_1$b.str) `must contain at least ${min} valid item(s)`
          : (0, codegen_1$b.str) `must contain at least ${min} and no more than ${max} valid item(s)`,
      params: ({ params: { min, max } }) => max === undefined ? (0, codegen_1$b._) `{minContains: ${min}}` : (0, codegen_1$b._) `{minContains: ${min}, maxContains: ${max}}`,
  };
  const def$l = {
      keyword: "contains",
      type: "array",
      schemaType: ["object", "boolean"],
      before: "uniqueItems",
      trackErrors: true,
      error: error$8,
      code(cxt) {
          const { gen, schema, parentSchema, data, it } = cxt;
          let min;
          let max;
          const { minContains, maxContains } = parentSchema;
          if (it.opts.next) {
              min = minContains === undefined ? 1 : minContains;
              max = maxContains;
          }
          else {
              min = 1;
          }
          const len = gen.const("len", (0, codegen_1$b._) `${data}.length`);
          cxt.setParams({ min, max });
          if (max === undefined && min === 0) {
              (0, util_1$e.checkStrictMode)(it, `"minContains" == 0 without "maxContains": "contains" keyword ignored`);
              return;
          }
          if (max !== undefined && min > max) {
              (0, util_1$e.checkStrictMode)(it, `"minContains" > "maxContains" is always invalid`);
              cxt.fail();
              return;
          }
          if ((0, util_1$e.alwaysValidSchema)(it, schema)) {
              let cond = (0, codegen_1$b._) `${len} >= ${min}`;
              if (max !== undefined)
                  cond = (0, codegen_1$b._) `${cond} && ${len} <= ${max}`;
              cxt.pass(cond);
              return;
          }
          it.items = true;
          const valid = gen.name("valid");
          if (max === undefined && min === 1) {
              validateItems(valid, () => gen.if(valid, () => gen.break()));
          }
          else if (min === 0) {
              gen.let(valid, true);
              if (max !== undefined)
                  gen.if((0, codegen_1$b._) `${data}.length > 0`, validateItemsWithCount);
          }
          else {
              gen.let(valid, false);
              validateItemsWithCount();
          }
          cxt.result(valid, () => cxt.reset());
          function validateItemsWithCount() {
              const schValid = gen.name("_valid");
              const count = gen.let("count", 0);
              validateItems(schValid, () => gen.if(schValid, () => checkLimits(count)));
          }
          function validateItems(_valid, block) {
              gen.forRange("i", 0, len, (i) => {
                  cxt.subschema({
                      keyword: "contains",
                      dataProp: i,
                      dataPropType: util_1$e.Type.Num,
                      compositeRule: true,
                  }, _valid);
                  block();
              });
          }
          function checkLimits(count) {
              gen.code((0, codegen_1$b._) `${count}++`);
              if (max === undefined) {
                  gen.if((0, codegen_1$b._) `${count} >= ${min}`, () => gen.assign(valid, true).break());
              }
              else {
                  gen.if((0, codegen_1$b._) `${count} > ${max}`, () => gen.assign(valid, false).break());
                  if (min === 1)
                      gen.assign(valid, true);
                  else
                      gen.if((0, codegen_1$b._) `${count} >= ${min}`, () => gen.assign(valid, true));
              }
          }
      },
  };
  contains.default = def$l;

  var dependencies = {};

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.validateSchemaDeps = exports.validatePropertyDeps = exports.error = void 0;
  	const codegen_1 = codegen;
  	const util_1 = util;
  	const code_1 = requireCode();
  	exports.error = {
  	    message: ({ params: { property, depsCount, deps } }) => {
  	        const property_ies = depsCount === 1 ? "property" : "properties";
  	        return (0, codegen_1.str) `must have ${property_ies} ${deps} when property ${property} is present`;
  	    },
  	    params: ({ params: { property, depsCount, deps, missingProperty } }) => (0, codegen_1._) `{property: ${property},
    missingProperty: ${missingProperty},
    depsCount: ${depsCount},
    deps: ${deps}}`, // TODO change to reference
  	};
  	const def = {
  	    keyword: "dependencies",
  	    type: "object",
  	    schemaType: "object",
  	    error: exports.error,
  	    code(cxt) {
  	        const [propDeps, schDeps] = splitDependencies(cxt);
  	        validatePropertyDeps(cxt, propDeps);
  	        validateSchemaDeps(cxt, schDeps);
  	    },
  	};
  	function splitDependencies({ schema }) {
  	    const propertyDeps = {};
  	    const schemaDeps = {};
  	    for (const key in schema) {
  	        if (key === "__proto__")
  	            continue;
  	        const deps = Array.isArray(schema[key]) ? propertyDeps : schemaDeps;
  	        deps[key] = schema[key];
  	    }
  	    return [propertyDeps, schemaDeps];
  	}
  	function validatePropertyDeps(cxt, propertyDeps = cxt.schema) {
  	    const { gen, data, it } = cxt;
  	    if (Object.keys(propertyDeps).length === 0)
  	        return;
  	    const missing = gen.let("missing");
  	    for (const prop in propertyDeps) {
  	        const deps = propertyDeps[prop];
  	        if (deps.length === 0)
  	            continue;
  	        const hasProperty = (0, code_1.propertyInData)(gen, data, prop, it.opts.ownProperties);
  	        cxt.setParams({
  	            property: prop,
  	            depsCount: deps.length,
  	            deps: deps.join(", "),
  	        });
  	        if (it.allErrors) {
  	            gen.if(hasProperty, () => {
  	                for (const depProp of deps) {
  	                    (0, code_1.checkReportMissingProp)(cxt, depProp);
  	                }
  	            });
  	        }
  	        else {
  	            gen.if((0, codegen_1._) `${hasProperty} && (${(0, code_1.checkMissingProp)(cxt, deps, missing)})`);
  	            (0, code_1.reportMissingProp)(cxt, missing);
  	            gen.else();
  	        }
  	    }
  	}
  	exports.validatePropertyDeps = validatePropertyDeps;
  	function validateSchemaDeps(cxt, schemaDeps = cxt.schema) {
  	    const { gen, data, keyword, it } = cxt;
  	    const valid = gen.name("valid");
  	    for (const prop in schemaDeps) {
  	        if ((0, util_1.alwaysValidSchema)(it, schemaDeps[prop]))
  	            continue;
  	        gen.if((0, code_1.propertyInData)(gen, data, prop, it.opts.ownProperties), () => {
  	            const schCxt = cxt.subschema({ keyword, schemaProp: prop }, valid);
  	            cxt.mergeValidEvaluated(schCxt, valid);
  	        }, () => gen.var(valid, true) // TODO var
  	        );
  	        cxt.ok(valid);
  	    }
  	}
  	exports.validateSchemaDeps = validateSchemaDeps;
  	exports.default = def;
  	
  } (dependencies));

  var propertyNames = {};

  Object.defineProperty(propertyNames, "__esModule", { value: true });
  const codegen_1$a = codegen;
  const util_1$d = util;
  const error$7 = {
      message: "property name must be valid",
      params: ({ params }) => (0, codegen_1$a._) `{propertyName: ${params.propertyName}}`,
  };
  const def$k = {
      keyword: "propertyNames",
      type: "object",
      schemaType: ["object", "boolean"],
      error: error$7,
      code(cxt) {
          const { gen, schema, data, it } = cxt;
          if ((0, util_1$d.alwaysValidSchema)(it, schema))
              return;
          const valid = gen.name("valid");
          gen.forIn("key", data, (key) => {
              cxt.setParams({ propertyName: key });
              cxt.subschema({
                  keyword: "propertyNames",
                  data: key,
                  dataTypes: ["string"],
                  propertyName: key,
                  compositeRule: true,
              }, valid);
              gen.if((0, codegen_1$a.not)(valid), () => {
                  cxt.error(true);
                  if (!it.allErrors)
                      gen.break();
              });
          });
          cxt.ok(valid);
      },
  };
  propertyNames.default = def$k;

  var additionalProperties = {};

  Object.defineProperty(additionalProperties, "__esModule", { value: true });
  const code_1$3 = requireCode();
  const codegen_1$9 = codegen;
  const names_1$3 = requireNames();
  const util_1$c = util;
  const error$6 = {
      message: "must NOT have additional properties",
      params: ({ params }) => (0, codegen_1$9._) `{additionalProperty: ${params.additionalProperty}}`,
  };
  const def$j = {
      keyword: "additionalProperties",
      type: ["object"],
      schemaType: ["boolean", "object"],
      allowUndefined: true,
      trackErrors: true,
      error: error$6,
      code(cxt) {
          const { gen, schema, parentSchema, data, errsCount, it } = cxt;
          /* istanbul ignore if */
          if (!errsCount)
              throw new Error("ajv implementation error");
          const { allErrors, opts } = it;
          it.props = true;
          if (opts.removeAdditional !== "all" && (0, util_1$c.alwaysValidSchema)(it, schema))
              return;
          const props = (0, code_1$3.allSchemaProperties)(parentSchema.properties);
          const patProps = (0, code_1$3.allSchemaProperties)(parentSchema.patternProperties);
          checkAdditionalProperties();
          cxt.ok((0, codegen_1$9._) `${errsCount} === ${names_1$3.default.errors}`);
          function checkAdditionalProperties() {
              gen.forIn("key", data, (key) => {
                  if (!props.length && !patProps.length)
                      additionalPropertyCode(key);
                  else
                      gen.if(isAdditional(key), () => additionalPropertyCode(key));
              });
          }
          function isAdditional(key) {
              let definedProp;
              if (props.length > 8) {
                  // TODO maybe an option instead of hard-coded 8?
                  const propsSchema = (0, util_1$c.schemaRefOrVal)(it, parentSchema.properties, "properties");
                  definedProp = (0, code_1$3.isOwnProperty)(gen, propsSchema, key);
              }
              else if (props.length) {
                  definedProp = (0, codegen_1$9.or)(...props.map((p) => (0, codegen_1$9._) `${key} === ${p}`));
              }
              else {
                  definedProp = codegen_1$9.nil;
              }
              if (patProps.length) {
                  definedProp = (0, codegen_1$9.or)(definedProp, ...patProps.map((p) => (0, codegen_1$9._) `${(0, code_1$3.usePattern)(cxt, p)}.test(${key})`));
              }
              return (0, codegen_1$9.not)(definedProp);
          }
          function deleteAdditional(key) {
              gen.code((0, codegen_1$9._) `delete ${data}[${key}]`);
          }
          function additionalPropertyCode(key) {
              if (opts.removeAdditional === "all" || (opts.removeAdditional && schema === false)) {
                  deleteAdditional(key);
                  return;
              }
              if (schema === false) {
                  cxt.setParams({ additionalProperty: key });
                  cxt.error();
                  if (!allErrors)
                      gen.break();
                  return;
              }
              if (typeof schema == "object" && !(0, util_1$c.alwaysValidSchema)(it, schema)) {
                  const valid = gen.name("valid");
                  if (opts.removeAdditional === "failing") {
                      applyAdditionalSchema(key, valid, false);
                      gen.if((0, codegen_1$9.not)(valid), () => {
                          cxt.reset();
                          deleteAdditional(key);
                      });
                  }
                  else {
                      applyAdditionalSchema(key, valid);
                      if (!allErrors)
                          gen.if((0, codegen_1$9.not)(valid), () => gen.break());
                  }
              }
          }
          function applyAdditionalSchema(key, valid, errors) {
              const subschema = {
                  keyword: "additionalProperties",
                  dataProp: key,
                  dataPropType: util_1$c.Type.Str,
              };
              if (errors === false) {
                  Object.assign(subschema, {
                      compositeRule: true,
                      createErrors: false,
                      allErrors: false,
                  });
              }
              cxt.subschema(subschema, valid);
          }
      },
  };
  additionalProperties.default = def$j;

  var properties$a = {};

  Object.defineProperty(properties$a, "__esModule", { value: true });
  const validate_1 = requireValidate();
  const code_1$2 = requireCode();
  const util_1$b = util;
  const additionalProperties_1$1 = additionalProperties;
  const def$i = {
      keyword: "properties",
      type: "object",
      schemaType: "object",
      code(cxt) {
          const { gen, schema, parentSchema, data, it } = cxt;
          if (it.opts.removeAdditional === "all" && parentSchema.additionalProperties === undefined) {
              additionalProperties_1$1.default.code(new validate_1.KeywordCxt(it, additionalProperties_1$1.default, "additionalProperties"));
          }
          const allProps = (0, code_1$2.allSchemaProperties)(schema);
          for (const prop of allProps) {
              it.definedProperties.add(prop);
          }
          if (it.opts.unevaluated && allProps.length && it.props !== true) {
              it.props = util_1$b.mergeEvaluated.props(gen, (0, util_1$b.toHash)(allProps), it.props);
          }
          const properties = allProps.filter((p) => !(0, util_1$b.alwaysValidSchema)(it, schema[p]));
          if (properties.length === 0)
              return;
          const valid = gen.name("valid");
          for (const prop of properties) {
              if (hasDefault(prop)) {
                  applyPropertySchema(prop);
              }
              else {
                  gen.if((0, code_1$2.propertyInData)(gen, data, prop, it.opts.ownProperties));
                  applyPropertySchema(prop);
                  if (!it.allErrors)
                      gen.else().var(valid, true);
                  gen.endIf();
              }
              cxt.it.definedProperties.add(prop);
              cxt.ok(valid);
          }
          function hasDefault(prop) {
              return it.opts.useDefaults && !it.compositeRule && schema[prop].default !== undefined;
          }
          function applyPropertySchema(prop) {
              cxt.subschema({
                  keyword: "properties",
                  schemaProp: prop,
                  dataProp: prop,
              }, valid);
          }
      },
  };
  properties$a.default = def$i;

  var patternProperties = {};

  Object.defineProperty(patternProperties, "__esModule", { value: true });
  const code_1$1 = requireCode();
  const codegen_1$8 = codegen;
  const util_1$a = util;
  const util_2 = util;
  const def$h = {
      keyword: "patternProperties",
      type: "object",
      schemaType: "object",
      code(cxt) {
          const { gen, schema, data, parentSchema, it } = cxt;
          const { opts } = it;
          const patterns = (0, code_1$1.allSchemaProperties)(schema);
          const alwaysValidPatterns = patterns.filter((p) => (0, util_1$a.alwaysValidSchema)(it, schema[p]));
          if (patterns.length === 0 ||
              (alwaysValidPatterns.length === patterns.length &&
                  (!it.opts.unevaluated || it.props === true))) {
              return;
          }
          const checkProperties = opts.strictSchema && !opts.allowMatchingProperties && parentSchema.properties;
          const valid = gen.name("valid");
          if (it.props !== true && !(it.props instanceof codegen_1$8.Name)) {
              it.props = (0, util_2.evaluatedPropsToName)(gen, it.props);
          }
          const { props } = it;
          validatePatternProperties();
          function validatePatternProperties() {
              for (const pat of patterns) {
                  if (checkProperties)
                      checkMatchingProperties(pat);
                  if (it.allErrors) {
                      validateProperties(pat);
                  }
                  else {
                      gen.var(valid, true); // TODO var
                      validateProperties(pat);
                      gen.if(valid);
                  }
              }
          }
          function checkMatchingProperties(pat) {
              for (const prop in checkProperties) {
                  if (new RegExp(pat).test(prop)) {
                      (0, util_1$a.checkStrictMode)(it, `property ${prop} matches pattern ${pat} (use allowMatchingProperties)`);
                  }
              }
          }
          function validateProperties(pat) {
              gen.forIn("key", data, (key) => {
                  gen.if((0, codegen_1$8._) `${(0, code_1$1.usePattern)(cxt, pat)}.test(${key})`, () => {
                      const alwaysValid = alwaysValidPatterns.includes(pat);
                      if (!alwaysValid) {
                          cxt.subschema({
                              keyword: "patternProperties",
                              schemaProp: pat,
                              dataProp: key,
                              dataPropType: util_2.Type.Str,
                          }, valid);
                      }
                      if (it.opts.unevaluated && props !== true) {
                          gen.assign((0, codegen_1$8._) `${props}[${key}]`, true);
                      }
                      else if (!alwaysValid && !it.allErrors) {
                          // can short-circuit if `unevaluatedProperties` is not supported (opts.next === false)
                          // or if all properties were evaluated (props === true)
                          gen.if((0, codegen_1$8.not)(valid), () => gen.break());
                      }
                  });
              });
          }
      },
  };
  patternProperties.default = def$h;

  var not = {};

  Object.defineProperty(not, "__esModule", { value: true });
  const util_1$9 = util;
  const def$g = {
      keyword: "not",
      schemaType: ["object", "boolean"],
      trackErrors: true,
      code(cxt) {
          const { gen, schema, it } = cxt;
          if ((0, util_1$9.alwaysValidSchema)(it, schema)) {
              cxt.fail();
              return;
          }
          const valid = gen.name("valid");
          cxt.subschema({
              keyword: "not",
              compositeRule: true,
              createErrors: false,
              allErrors: false,
          }, valid);
          cxt.failResult(valid, () => cxt.reset(), () => cxt.error());
      },
      error: { message: "must NOT be valid" },
  };
  not.default = def$g;

  var anyOf = {};

  Object.defineProperty(anyOf, "__esModule", { value: true });
  const code_1 = requireCode();
  const def$f = {
      keyword: "anyOf",
      schemaType: "array",
      trackErrors: true,
      code: code_1.validateUnion,
      error: { message: "must match a schema in anyOf" },
  };
  anyOf.default = def$f;

  var oneOf = {};

  Object.defineProperty(oneOf, "__esModule", { value: true });
  const codegen_1$7 = codegen;
  const util_1$8 = util;
  const error$5 = {
      message: "must match exactly one schema in oneOf",
      params: ({ params }) => (0, codegen_1$7._) `{passingSchemas: ${params.passing}}`,
  };
  const def$e = {
      keyword: "oneOf",
      schemaType: "array",
      trackErrors: true,
      error: error$5,
      code(cxt) {
          const { gen, schema, parentSchema, it } = cxt;
          /* istanbul ignore if */
          if (!Array.isArray(schema))
              throw new Error("ajv implementation error");
          if (it.opts.discriminator && parentSchema.discriminator)
              return;
          const schArr = schema;
          const valid = gen.let("valid", false);
          const passing = gen.let("passing", null);
          const schValid = gen.name("_valid");
          cxt.setParams({ passing });
          // TODO possibly fail straight away (with warning or exception) if there are two empty always valid schemas
          gen.block(validateOneOf);
          cxt.result(valid, () => cxt.reset(), () => cxt.error(true));
          function validateOneOf() {
              schArr.forEach((sch, i) => {
                  let schCxt;
                  if ((0, util_1$8.alwaysValidSchema)(it, sch)) {
                      gen.var(schValid, true);
                  }
                  else {
                      schCxt = cxt.subschema({
                          keyword: "oneOf",
                          schemaProp: i,
                          compositeRule: true,
                      }, schValid);
                  }
                  if (i > 0) {
                      gen
                          .if((0, codegen_1$7._) `${schValid} && ${valid}`)
                          .assign(valid, false)
                          .assign(passing, (0, codegen_1$7._) `[${passing}, ${i}]`)
                          .else();
                  }
                  gen.if(schValid, () => {
                      gen.assign(valid, true);
                      gen.assign(passing, i);
                      if (schCxt)
                          cxt.mergeEvaluated(schCxt, codegen_1$7.Name);
                  });
              });
          }
      },
  };
  oneOf.default = def$e;

  var allOf$1 = {};

  Object.defineProperty(allOf$1, "__esModule", { value: true });
  const util_1$7 = util;
  const def$d = {
      keyword: "allOf",
      schemaType: "array",
      code(cxt) {
          const { gen, schema, it } = cxt;
          /* istanbul ignore if */
          if (!Array.isArray(schema))
              throw new Error("ajv implementation error");
          const valid = gen.name("valid");
          schema.forEach((sch, i) => {
              if ((0, util_1$7.alwaysValidSchema)(it, sch))
                  return;
              const schCxt = cxt.subschema({ keyword: "allOf", schemaProp: i }, valid);
              cxt.ok(valid);
              cxt.mergeEvaluated(schCxt);
          });
      },
  };
  allOf$1.default = def$d;

  var _if = {};

  Object.defineProperty(_if, "__esModule", { value: true });
  const codegen_1$6 = codegen;
  const util_1$6 = util;
  const error$4 = {
      message: ({ params }) => (0, codegen_1$6.str) `must match "${params.ifClause}" schema`,
      params: ({ params }) => (0, codegen_1$6._) `{failingKeyword: ${params.ifClause}}`,
  };
  const def$c = {
      keyword: "if",
      schemaType: ["object", "boolean"],
      trackErrors: true,
      error: error$4,
      code(cxt) {
          const { gen, parentSchema, it } = cxt;
          if (parentSchema.then === undefined && parentSchema.else === undefined) {
              (0, util_1$6.checkStrictMode)(it, '"if" without "then" and "else" is ignored');
          }
          const hasThen = hasSchema(it, "then");
          const hasElse = hasSchema(it, "else");
          if (!hasThen && !hasElse)
              return;
          const valid = gen.let("valid", true);
          const schValid = gen.name("_valid");
          validateIf();
          cxt.reset();
          if (hasThen && hasElse) {
              const ifClause = gen.let("ifClause");
              cxt.setParams({ ifClause });
              gen.if(schValid, validateClause("then", ifClause), validateClause("else", ifClause));
          }
          else if (hasThen) {
              gen.if(schValid, validateClause("then"));
          }
          else {
              gen.if((0, codegen_1$6.not)(schValid), validateClause("else"));
          }
          cxt.pass(valid, () => cxt.error(true));
          function validateIf() {
              const schCxt = cxt.subschema({
                  keyword: "if",
                  compositeRule: true,
                  createErrors: false,
                  allErrors: false,
              }, schValid);
              cxt.mergeEvaluated(schCxt);
          }
          function validateClause(keyword, ifClause) {
              return () => {
                  const schCxt = cxt.subschema({ keyword }, schValid);
                  gen.assign(valid, schValid);
                  cxt.mergeValidEvaluated(schCxt, valid);
                  if (ifClause)
                      gen.assign(ifClause, (0, codegen_1$6._) `${keyword}`);
                  else
                      cxt.setParams({ ifClause: keyword });
              };
          }
      },
  };
  function hasSchema(it, keyword) {
      const schema = it.schema[keyword];
      return schema !== undefined && !(0, util_1$6.alwaysValidSchema)(it, schema);
  }
  _if.default = def$c;

  var thenElse = {};

  Object.defineProperty(thenElse, "__esModule", { value: true });
  const util_1$5 = util;
  const def$b = {
      keyword: ["then", "else"],
      schemaType: ["object", "boolean"],
      code({ keyword, parentSchema, it }) {
          if (parentSchema.if === undefined)
              (0, util_1$5.checkStrictMode)(it, `"${keyword}" without "if" is ignored`);
      },
  };
  thenElse.default = def$b;

  Object.defineProperty(applicator$1, "__esModule", { value: true });
  const additionalItems_1 = additionalItems;
  const prefixItems_1 = prefixItems;
  const items_1 = items;
  const items2020_1 = items2020;
  const contains_1 = contains;
  const dependencies_1$2 = dependencies;
  const propertyNames_1 = propertyNames;
  const additionalProperties_1 = additionalProperties;
  const properties_1 = properties$a;
  const patternProperties_1 = patternProperties;
  const not_1 = not;
  const anyOf_1 = anyOf;
  const oneOf_1 = oneOf;
  const allOf_1 = allOf$1;
  const if_1 = _if;
  const thenElse_1 = thenElse;
  function getApplicator(draft2020 = false) {
      const applicator = [
          // any
          not_1.default,
          anyOf_1.default,
          oneOf_1.default,
          allOf_1.default,
          if_1.default,
          thenElse_1.default,
          // object
          propertyNames_1.default,
          additionalProperties_1.default,
          dependencies_1$2.default,
          properties_1.default,
          patternProperties_1.default,
      ];
      // array
      if (draft2020)
          applicator.push(prefixItems_1.default, items2020_1.default);
      else
          applicator.push(additionalItems_1.default, items_1.default);
      applicator.push(contains_1.default);
      return applicator;
  }
  applicator$1.default = getApplicator;

  var dynamic$1 = {};

  var dynamicAnchor$1 = {};

  Object.defineProperty(dynamicAnchor$1, "__esModule", { value: true });
  dynamicAnchor$1.dynamicAnchor = void 0;
  const codegen_1$5 = codegen;
  const names_1$2 = requireNames();
  const compile_1$1 = compile;
  const ref_1$1 = ref;
  const def$a = {
      keyword: "$dynamicAnchor",
      schemaType: "string",
      code: (cxt) => dynamicAnchor(cxt, cxt.schema),
  };
  function dynamicAnchor(cxt, anchor) {
      const { gen, it } = cxt;
      it.schemaEnv.root.dynamicAnchors[anchor] = true;
      const v = (0, codegen_1$5._) `${names_1$2.default.dynamicAnchors}${(0, codegen_1$5.getProperty)(anchor)}`;
      const validate = it.errSchemaPath === "#" ? it.validateName : _getValidate(cxt);
      gen.if((0, codegen_1$5._) `!${v}`, () => gen.assign(v, validate));
  }
  dynamicAnchor$1.dynamicAnchor = dynamicAnchor;
  function _getValidate(cxt) {
      const { schemaEnv, schema, self } = cxt.it;
      const { root, baseId, localRefs, meta } = schemaEnv.root;
      const { schemaId } = self.opts;
      const sch = new compile_1$1.SchemaEnv({ schema, schemaId, root, baseId, localRefs, meta });
      compile_1$1.compileSchema.call(self, sch);
      return (0, ref_1$1.getValidate)(cxt, sch);
  }
  dynamicAnchor$1.default = def$a;

  var dynamicRef$1 = {};

  Object.defineProperty(dynamicRef$1, "__esModule", { value: true });
  dynamicRef$1.dynamicRef = void 0;
  const codegen_1$4 = codegen;
  const names_1$1 = requireNames();
  const ref_1 = ref;
  const def$9 = {
      keyword: "$dynamicRef",
      schemaType: "string",
      code: (cxt) => dynamicRef(cxt, cxt.schema),
  };
  function dynamicRef(cxt, ref) {
      const { gen, keyword, it } = cxt;
      if (ref[0] !== "#")
          throw new Error(`"${keyword}" only supports hash fragment reference`);
      const anchor = ref.slice(1);
      if (it.allErrors) {
          _dynamicRef();
      }
      else {
          const valid = gen.let("valid", false);
          _dynamicRef(valid);
          cxt.ok(valid);
      }
      function _dynamicRef(valid) {
          // TODO the assumption here is that `recursiveRef: #` always points to the root
          // of the schema object, which is not correct, because there may be $id that
          // makes # point to it, and the target schema may not contain dynamic/recursiveAnchor.
          // Because of that 2 tests in recursiveRef.json fail.
          // This is a similar problem to #815 (`$id` doesn't alter resolution scope for `{ "$ref": "#" }`).
          // (This problem is not tested in JSON-Schema-Test-Suite)
          if (it.schemaEnv.root.dynamicAnchors[anchor]) {
              const v = gen.let("_v", (0, codegen_1$4._) `${names_1$1.default.dynamicAnchors}${(0, codegen_1$4.getProperty)(anchor)}`);
              gen.if(v, _callRef(v, valid), _callRef(it.validateName, valid));
          }
          else {
              _callRef(it.validateName, valid)();
          }
      }
      function _callRef(validate, valid) {
          return valid
              ? () => gen.block(() => {
                  (0, ref_1.callRef)(cxt, validate);
                  gen.let(valid, true);
              })
              : () => (0, ref_1.callRef)(cxt, validate);
      }
  }
  dynamicRef$1.dynamicRef = dynamicRef;
  dynamicRef$1.default = def$9;

  var recursiveAnchor = {};

  Object.defineProperty(recursiveAnchor, "__esModule", { value: true });
  const dynamicAnchor_1$1 = dynamicAnchor$1;
  const util_1$4 = util;
  const def$8 = {
      keyword: "$recursiveAnchor",
      schemaType: "boolean",
      code(cxt) {
          if (cxt.schema)
              (0, dynamicAnchor_1$1.dynamicAnchor)(cxt, "");
          else
              (0, util_1$4.checkStrictMode)(cxt.it, "$recursiveAnchor: false is ignored");
      },
  };
  recursiveAnchor.default = def$8;

  var recursiveRef = {};

  Object.defineProperty(recursiveRef, "__esModule", { value: true });
  const dynamicRef_1$1 = dynamicRef$1;
  const def$7 = {
      keyword: "$recursiveRef",
      schemaType: "string",
      code: (cxt) => (0, dynamicRef_1$1.dynamicRef)(cxt, cxt.schema),
  };
  recursiveRef.default = def$7;

  Object.defineProperty(dynamic$1, "__esModule", { value: true });
  const dynamicAnchor_1 = dynamicAnchor$1;
  const dynamicRef_1 = dynamicRef$1;
  const recursiveAnchor_1 = recursiveAnchor;
  const recursiveRef_1 = recursiveRef;
  const dynamic = [dynamicAnchor_1.default, dynamicRef_1.default, recursiveAnchor_1.default, recursiveRef_1.default];
  dynamic$1.default = dynamic;

  var next$1 = {};

  var dependentRequired = {};

  Object.defineProperty(dependentRequired, "__esModule", { value: true });
  const dependencies_1$1 = dependencies;
  const def$6 = {
      keyword: "dependentRequired",
      type: "object",
      schemaType: "object",
      error: dependencies_1$1.error,
      code: (cxt) => (0, dependencies_1$1.validatePropertyDeps)(cxt),
  };
  dependentRequired.default = def$6;

  var dependentSchemas = {};

  Object.defineProperty(dependentSchemas, "__esModule", { value: true });
  const dependencies_1 = dependencies;
  const def$5 = {
      keyword: "dependentSchemas",
      type: "object",
      schemaType: "object",
      code: (cxt) => (0, dependencies_1.validateSchemaDeps)(cxt),
  };
  dependentSchemas.default = def$5;

  var limitContains = {};

  Object.defineProperty(limitContains, "__esModule", { value: true });
  const util_1$3 = util;
  const def$4 = {
      keyword: ["maxContains", "minContains"],
      type: "array",
      schemaType: "number",
      code({ keyword, parentSchema, it }) {
          if (parentSchema.contains === undefined) {
              (0, util_1$3.checkStrictMode)(it, `"${keyword}" without "contains" is ignored`);
          }
      },
  };
  limitContains.default = def$4;

  Object.defineProperty(next$1, "__esModule", { value: true });
  const dependentRequired_1 = dependentRequired;
  const dependentSchemas_1 = dependentSchemas;
  const limitContains_1 = limitContains;
  const next = [dependentRequired_1.default, dependentSchemas_1.default, limitContains_1.default];
  next$1.default = next;

  var unevaluated$2 = {};

  var unevaluatedProperties = {};

  Object.defineProperty(unevaluatedProperties, "__esModule", { value: true });
  const codegen_1$3 = codegen;
  const util_1$2 = util;
  const names_1 = requireNames();
  const error$3 = {
      message: "must NOT have unevaluated properties",
      params: ({ params }) => (0, codegen_1$3._) `{unevaluatedProperty: ${params.unevaluatedProperty}}`,
  };
  const def$3 = {
      keyword: "unevaluatedProperties",
      type: "object",
      schemaType: ["boolean", "object"],
      trackErrors: true,
      error: error$3,
      code(cxt) {
          const { gen, schema, data, errsCount, it } = cxt;
          /* istanbul ignore if */
          if (!errsCount)
              throw new Error("ajv implementation error");
          const { allErrors, props } = it;
          if (props instanceof codegen_1$3.Name) {
              gen.if((0, codegen_1$3._) `${props} !== true`, () => gen.forIn("key", data, (key) => gen.if(unevaluatedDynamic(props, key), () => unevaluatedPropCode(key))));
          }
          else if (props !== true) {
              gen.forIn("key", data, (key) => props === undefined
                  ? unevaluatedPropCode(key)
                  : gen.if(unevaluatedStatic(props, key), () => unevaluatedPropCode(key)));
          }
          it.props = true;
          cxt.ok((0, codegen_1$3._) `${errsCount} === ${names_1.default.errors}`);
          function unevaluatedPropCode(key) {
              if (schema === false) {
                  cxt.setParams({ unevaluatedProperty: key });
                  cxt.error();
                  if (!allErrors)
                      gen.break();
                  return;
              }
              if (!(0, util_1$2.alwaysValidSchema)(it, schema)) {
                  const valid = gen.name("valid");
                  cxt.subschema({
                      keyword: "unevaluatedProperties",
                      dataProp: key,
                      dataPropType: util_1$2.Type.Str,
                  }, valid);
                  if (!allErrors)
                      gen.if((0, codegen_1$3.not)(valid), () => gen.break());
              }
          }
          function unevaluatedDynamic(evaluatedProps, key) {
              return (0, codegen_1$3._) `!${evaluatedProps} || !${evaluatedProps}[${key}]`;
          }
          function unevaluatedStatic(evaluatedProps, key) {
              const ps = [];
              for (const p in evaluatedProps) {
                  if (evaluatedProps[p] === true)
                      ps.push((0, codegen_1$3._) `${key} !== ${p}`);
              }
              return (0, codegen_1$3.and)(...ps);
          }
      },
  };
  unevaluatedProperties.default = def$3;

  var unevaluatedItems = {};

  Object.defineProperty(unevaluatedItems, "__esModule", { value: true });
  const codegen_1$2 = codegen;
  const util_1$1 = util;
  const error$2 = {
      message: ({ params: { len } }) => (0, codegen_1$2.str) `must NOT have more than ${len} items`,
      params: ({ params: { len } }) => (0, codegen_1$2._) `{limit: ${len}}`,
  };
  const def$2 = {
      keyword: "unevaluatedItems",
      type: "array",
      schemaType: ["boolean", "object"],
      error: error$2,
      code(cxt) {
          const { gen, schema, data, it } = cxt;
          const items = it.items || 0;
          if (items === true)
              return;
          const len = gen.const("len", (0, codegen_1$2._) `${data}.length`);
          if (schema === false) {
              cxt.setParams({ len: items });
              cxt.fail((0, codegen_1$2._) `${len} > ${items}`);
          }
          else if (typeof schema == "object" && !(0, util_1$1.alwaysValidSchema)(it, schema)) {
              const valid = gen.var("valid", (0, codegen_1$2._) `${len} <= ${items}`);
              gen.if((0, codegen_1$2.not)(valid), () => validateItems(valid, items));
              cxt.ok(valid);
          }
          it.items = true;
          function validateItems(valid, from) {
              gen.forRange("i", from, len, (i) => {
                  cxt.subschema({ keyword: "unevaluatedItems", dataProp: i, dataPropType: util_1$1.Type.Num }, valid);
                  if (!it.allErrors)
                      gen.if((0, codegen_1$2.not)(valid), () => gen.break());
              });
          }
      },
  };
  unevaluatedItems.default = def$2;

  Object.defineProperty(unevaluated$2, "__esModule", { value: true });
  const unevaluatedProperties_1 = unevaluatedProperties;
  const unevaluatedItems_1 = unevaluatedItems;
  const unevaluated$1 = [unevaluatedProperties_1.default, unevaluatedItems_1.default];
  unevaluated$2.default = unevaluated$1;

  var format$3 = {};

  var format$2 = {};

  Object.defineProperty(format$2, "__esModule", { value: true });
  const codegen_1$1 = codegen;
  const error$1 = {
      message: ({ schemaCode }) => (0, codegen_1$1.str) `must match format "${schemaCode}"`,
      params: ({ schemaCode }) => (0, codegen_1$1._) `{format: ${schemaCode}}`,
  };
  const def$1 = {
      keyword: "format",
      type: ["number", "string"],
      schemaType: "string",
      $data: true,
      error: error$1,
      code(cxt, ruleType) {
          const { gen, data, $data, schema, schemaCode, it } = cxt;
          const { opts, errSchemaPath, schemaEnv, self } = it;
          if (!opts.validateFormats)
              return;
          if ($data)
              validate$DataFormat();
          else
              validateFormat();
          function validate$DataFormat() {
              const fmts = gen.scopeValue("formats", {
                  ref: self.formats,
                  code: opts.code.formats,
              });
              const fDef = gen.const("fDef", (0, codegen_1$1._) `${fmts}[${schemaCode}]`);
              const fType = gen.let("fType");
              const format = gen.let("format");
              // TODO simplify
              gen.if((0, codegen_1$1._) `typeof ${fDef} == "object" && !(${fDef} instanceof RegExp)`, () => gen.assign(fType, (0, codegen_1$1._) `${fDef}.type || "string"`).assign(format, (0, codegen_1$1._) `${fDef}.validate`), () => gen.assign(fType, (0, codegen_1$1._) `"string"`).assign(format, fDef));
              cxt.fail$data((0, codegen_1$1.or)(unknownFmt(), invalidFmt()));
              function unknownFmt() {
                  if (opts.strictSchema === false)
                      return codegen_1$1.nil;
                  return (0, codegen_1$1._) `${schemaCode} && !${format}`;
              }
              function invalidFmt() {
                  const callFormat = schemaEnv.$async
                      ? (0, codegen_1$1._) `(${fDef}.async ? await ${format}(${data}) : ${format}(${data}))`
                      : (0, codegen_1$1._) `${format}(${data})`;
                  const validData = (0, codegen_1$1._) `(typeof ${format} == "function" ? ${callFormat} : ${format}.test(${data}))`;
                  return (0, codegen_1$1._) `${format} && ${format} !== true && ${fType} === ${ruleType} && !${validData}`;
              }
          }
          function validateFormat() {
              const formatDef = self.formats[schema];
              if (!formatDef) {
                  unknownFormat();
                  return;
              }
              if (formatDef === true)
                  return;
              const [fmtType, format, fmtRef] = getFormat(formatDef);
              if (fmtType === ruleType)
                  cxt.pass(validCondition());
              function unknownFormat() {
                  if (opts.strictSchema === false) {
                      self.logger.warn(unknownMsg());
                      return;
                  }
                  throw new Error(unknownMsg());
                  function unknownMsg() {
                      return `unknown format "${schema}" ignored in schema at path "${errSchemaPath}"`;
                  }
              }
              function getFormat(fmtDef) {
                  const code = fmtDef instanceof RegExp
                      ? (0, codegen_1$1.regexpCode)(fmtDef)
                      : opts.code.formats
                          ? (0, codegen_1$1._) `${opts.code.formats}${(0, codegen_1$1.getProperty)(schema)}`
                          : undefined;
                  const fmt = gen.scopeValue("formats", { key: schema, ref: fmtDef, code });
                  if (typeof fmtDef == "object" && !(fmtDef instanceof RegExp)) {
                      return [fmtDef.type || "string", fmtDef.validate, (0, codegen_1$1._) `${fmt}.validate`];
                  }
                  return ["string", fmtDef, fmt];
              }
              function validCondition() {
                  if (typeof formatDef == "object" && !(formatDef instanceof RegExp) && formatDef.async) {
                      if (!schemaEnv.$async)
                          throw new Error("async format in sync schema");
                      return (0, codegen_1$1._) `await ${fmtRef}(${data})`;
                  }
                  return typeof format == "function" ? (0, codegen_1$1._) `${fmtRef}(${data})` : (0, codegen_1$1._) `${fmtRef}.test(${data})`;
              }
          }
      },
  };
  format$2.default = def$1;

  Object.defineProperty(format$3, "__esModule", { value: true });
  const format_1$2 = format$2;
  const format$1 = [format_1$2.default];
  format$3.default = format$1;

  var metadata$1 = {};

  Object.defineProperty(metadata$1, "__esModule", { value: true });
  metadata$1.contentVocabulary = metadata$1.metadataVocabulary = void 0;
  metadata$1.metadataVocabulary = [
      "title",
      "description",
      "default",
      "deprecated",
      "readOnly",
      "writeOnly",
      "examples",
  ];
  metadata$1.contentVocabulary = [
      "contentMediaType",
      "contentEncoding",
      "contentSchema",
  ];

  Object.defineProperty(draft2020, "__esModule", { value: true });
  const core_1$1 = core$3;
  const validation_1$1 = validation$2;
  const applicator_1$1 = applicator$1;
  const dynamic_1 = dynamic$1;
  const next_1 = next$1;
  const unevaluated_1 = unevaluated$2;
  const format_1$1 = format$3;
  const metadata_1$1 = metadata$1;
  const draft2020Vocabularies = [
      dynamic_1.default,
      core_1$1.default,
      validation_1$1.default,
      (0, applicator_1$1.default)(true),
      format_1$1.default,
      metadata_1$1.metadataVocabulary,
      metadata_1$1.contentVocabulary,
      next_1.default,
      unevaluated_1.default,
  ];
  draft2020.default = draft2020Vocabularies;

  var discriminator = {};

  var types = {};

  Object.defineProperty(types, "__esModule", { value: true });
  types.DiscrError = void 0;
  var DiscrError;
  (function (DiscrError) {
      DiscrError["Tag"] = "tag";
      DiscrError["Mapping"] = "mapping";
  })(DiscrError || (types.DiscrError = DiscrError = {}));

  Object.defineProperty(discriminator, "__esModule", { value: true });
  const codegen_1 = codegen;
  const types_1 = types;
  const compile_1 = compile;
  const ref_error_1 = requireRef_error();
  const util_1 = util;
  const error = {
      message: ({ params: { discrError, tagName } }) => discrError === types_1.DiscrError.Tag
          ? `tag "${tagName}" must be string`
          : `value of tag "${tagName}" must be in oneOf`,
      params: ({ params: { discrError, tag, tagName } }) => (0, codegen_1._) `{error: ${discrError}, tag: ${tagName}, tagValue: ${tag}}`,
  };
  const def = {
      keyword: "discriminator",
      type: "object",
      schemaType: "object",
      error,
      code(cxt) {
          const { gen, data, schema, parentSchema, it } = cxt;
          const { oneOf } = parentSchema;
          if (!it.opts.discriminator) {
              throw new Error("discriminator: requires discriminator option");
          }
          const tagName = schema.propertyName;
          if (typeof tagName != "string")
              throw new Error("discriminator: requires propertyName");
          if (schema.mapping)
              throw new Error("discriminator: mapping is not supported");
          if (!oneOf)
              throw new Error("discriminator: requires oneOf keyword");
          const valid = gen.let("valid", false);
          const tag = gen.const("tag", (0, codegen_1._) `${data}${(0, codegen_1.getProperty)(tagName)}`);
          gen.if((0, codegen_1._) `typeof ${tag} == "string"`, () => validateMapping(), () => cxt.error(false, { discrError: types_1.DiscrError.Tag, tag, tagName }));
          cxt.ok(valid);
          function validateMapping() {
              const mapping = getMapping();
              gen.if(false);
              for (const tagValue in mapping) {
                  gen.elseIf((0, codegen_1._) `${tag} === ${tagValue}`);
                  gen.assign(valid, applyTagSchema(mapping[tagValue]));
              }
              gen.else();
              cxt.error(false, { discrError: types_1.DiscrError.Mapping, tag, tagName });
              gen.endIf();
          }
          function applyTagSchema(schemaProp) {
              const _valid = gen.name("valid");
              const schCxt = cxt.subschema({ keyword: "oneOf", schemaProp }, _valid);
              cxt.mergeEvaluated(schCxt, codegen_1.Name);
              return _valid;
          }
          function getMapping() {
              var _a;
              const oneOfMapping = {};
              const topRequired = hasRequired(parentSchema);
              let tagRequired = true;
              for (let i = 0; i < oneOf.length; i++) {
                  let sch = oneOf[i];
                  if ((sch === null || sch === void 0 ? void 0 : sch.$ref) && !(0, util_1.schemaHasRulesButRef)(sch, it.self.RULES)) {
                      const ref = sch.$ref;
                      sch = compile_1.resolveRef.call(it.self, it.schemaEnv.root, it.baseId, ref);
                      if (sch instanceof compile_1.SchemaEnv)
                          sch = sch.schema;
                      if (sch === undefined)
                          throw new ref_error_1.default(it.opts.uriResolver, it.baseId, ref);
                  }
                  const propSch = (_a = sch === null || sch === void 0 ? void 0 : sch.properties) === null || _a === void 0 ? void 0 : _a[tagName];
                  if (typeof propSch != "object") {
                      throw new Error(`discriminator: oneOf subschemas (or referenced schemas) must have "properties/${tagName}"`);
                  }
                  tagRequired = tagRequired && (topRequired || hasRequired(sch));
                  addMappings(propSch, i);
              }
              if (!tagRequired)
                  throw new Error(`discriminator: "${tagName}" must be required`);
              return oneOfMapping;
              function hasRequired({ required }) {
                  return Array.isArray(required) && required.includes(tagName);
              }
              function addMappings(sch, i) {
                  if (sch.const) {
                      addMapping(sch.const, i);
                  }
                  else if (sch.enum) {
                      for (const tagValue of sch.enum) {
                          addMapping(tagValue, i);
                      }
                  }
                  else {
                      throw new Error(`discriminator: "properties/${tagName}" must have "const" or "enum"`);
                  }
              }
              function addMapping(tagValue, i) {
                  if (typeof tagValue != "string" || tagValue in oneOfMapping) {
                      throw new Error(`discriminator: "${tagName}" values must be unique strings`);
                  }
                  oneOfMapping[tagValue] = i;
              }
          }
      },
  };
  discriminator.default = def;

  var jsonSchema202012 = {};

  var $schema$9 = "https://json-schema.org/draft/2020-12/schema";
  var $id$9 = "https://json-schema.org/draft/2020-12/schema";
  var $vocabulary$7 = {
  	"https://json-schema.org/draft/2020-12/vocab/core": true,
  	"https://json-schema.org/draft/2020-12/vocab/applicator": true,
  	"https://json-schema.org/draft/2020-12/vocab/unevaluated": true,
  	"https://json-schema.org/draft/2020-12/vocab/validation": true,
  	"https://json-schema.org/draft/2020-12/vocab/meta-data": true,
  	"https://json-schema.org/draft/2020-12/vocab/format-annotation": true,
  	"https://json-schema.org/draft/2020-12/vocab/content": true
  };
  var $dynamicAnchor$7 = "meta";
  var title$8 = "Core and Validation specifications meta-schema";
  var allOf = [
  	{
  		$ref: "meta/core"
  	},
  	{
  		$ref: "meta/applicator"
  	},
  	{
  		$ref: "meta/unevaluated"
  	},
  	{
  		$ref: "meta/validation"
  	},
  	{
  		$ref: "meta/meta-data"
  	},
  	{
  		$ref: "meta/format-annotation"
  	},
  	{
  		$ref: "meta/content"
  	}
  ];
  var type$a = [
  	"object",
  	"boolean"
  ];
  var $comment = "This meta-schema also defines keywords that have appeared in previous drafts in order to prevent incompatible extensions as they remain in common use.";
  var properties$9 = {
  	definitions: {
  		$comment: "\"definitions\" has been replaced by \"$defs\".",
  		type: "object",
  		additionalProperties: {
  			$dynamicRef: "#meta"
  		},
  		deprecated: true,
  		"default": {
  		}
  	},
  	dependencies: {
  		$comment: "\"dependencies\" has been split and replaced by \"dependentSchemas\" and \"dependentRequired\" in order to serve their differing semantics.",
  		type: "object",
  		additionalProperties: {
  			anyOf: [
  				{
  					$dynamicRef: "#meta"
  				},
  				{
  					$ref: "meta/validation#/$defs/stringArray"
  				}
  			]
  		},
  		deprecated: true,
  		"default": {
  		}
  	},
  	$recursiveAnchor: {
  		$comment: "\"$recursiveAnchor\" has been replaced by \"$dynamicAnchor\".",
  		$ref: "meta/core#/$defs/anchorString",
  		deprecated: true
  	},
  	$recursiveRef: {
  		$comment: "\"$recursiveRef\" has been replaced by \"$dynamicRef\".",
  		$ref: "meta/core#/$defs/uriReferenceString",
  		deprecated: true
  	}
  };
  var require$$0 = {
  	$schema: $schema$9,
  	$id: $id$9,
  	$vocabulary: $vocabulary$7,
  	$dynamicAnchor: $dynamicAnchor$7,
  	title: title$8,
  	allOf: allOf,
  	type: type$a,
  	$comment: $comment,
  	properties: properties$9
  };

  var $schema$8 = "https://json-schema.org/draft/2020-12/schema";
  var $id$8 = "https://json-schema.org/draft/2020-12/meta/applicator";
  var $vocabulary$6 = {
  	"https://json-schema.org/draft/2020-12/vocab/applicator": true
  };
  var $dynamicAnchor$6 = "meta";
  var title$7 = "Applicator vocabulary meta-schema";
  var type$9 = [
  	"object",
  	"boolean"
  ];
  var properties$8 = {
  	prefixItems: {
  		$ref: "#/$defs/schemaArray"
  	},
  	items: {
  		$dynamicRef: "#meta"
  	},
  	contains: {
  		$dynamicRef: "#meta"
  	},
  	additionalProperties: {
  		$dynamicRef: "#meta"
  	},
  	properties: {
  		type: "object",
  		additionalProperties: {
  			$dynamicRef: "#meta"
  		},
  		"default": {
  		}
  	},
  	patternProperties: {
  		type: "object",
  		additionalProperties: {
  			$dynamicRef: "#meta"
  		},
  		propertyNames: {
  			format: "regex"
  		},
  		"default": {
  		}
  	},
  	dependentSchemas: {
  		type: "object",
  		additionalProperties: {
  			$dynamicRef: "#meta"
  		},
  		"default": {
  		}
  	},
  	propertyNames: {
  		$dynamicRef: "#meta"
  	},
  	"if": {
  		$dynamicRef: "#meta"
  	},
  	then: {
  		$dynamicRef: "#meta"
  	},
  	"else": {
  		$dynamicRef: "#meta"
  	},
  	allOf: {
  		$ref: "#/$defs/schemaArray"
  	},
  	anyOf: {
  		$ref: "#/$defs/schemaArray"
  	},
  	oneOf: {
  		$ref: "#/$defs/schemaArray"
  	},
  	not: {
  		$dynamicRef: "#meta"
  	}
  };
  var $defs$3 = {
  	schemaArray: {
  		type: "array",
  		minItems: 1,
  		items: {
  			$dynamicRef: "#meta"
  		}
  	}
  };
  var require$$1 = {
  	$schema: $schema$8,
  	$id: $id$8,
  	$vocabulary: $vocabulary$6,
  	$dynamicAnchor: $dynamicAnchor$6,
  	title: title$7,
  	type: type$9,
  	properties: properties$8,
  	$defs: $defs$3
  };

  var $schema$7 = "https://json-schema.org/draft/2020-12/schema";
  var $id$7 = "https://json-schema.org/draft/2020-12/meta/unevaluated";
  var $vocabulary$5 = {
  	"https://json-schema.org/draft/2020-12/vocab/unevaluated": true
  };
  var $dynamicAnchor$5 = "meta";
  var title$6 = "Unevaluated applicator vocabulary meta-schema";
  var type$8 = [
  	"object",
  	"boolean"
  ];
  var properties$7 = {
  	unevaluatedItems: {
  		$dynamicRef: "#meta"
  	},
  	unevaluatedProperties: {
  		$dynamicRef: "#meta"
  	}
  };
  var require$$2 = {
  	$schema: $schema$7,
  	$id: $id$7,
  	$vocabulary: $vocabulary$5,
  	$dynamicAnchor: $dynamicAnchor$5,
  	title: title$6,
  	type: type$8,
  	properties: properties$7
  };

  var $schema$6 = "https://json-schema.org/draft/2020-12/schema";
  var $id$6 = "https://json-schema.org/draft/2020-12/meta/content";
  var $vocabulary$4 = {
  	"https://json-schema.org/draft/2020-12/vocab/content": true
  };
  var $dynamicAnchor$4 = "meta";
  var title$5 = "Content vocabulary meta-schema";
  var type$7 = [
  	"object",
  	"boolean"
  ];
  var properties$6 = {
  	contentEncoding: {
  		type: "string"
  	},
  	contentMediaType: {
  		type: "string"
  	},
  	contentSchema: {
  		$dynamicRef: "#meta"
  	}
  };
  var require$$3$1 = {
  	$schema: $schema$6,
  	$id: $id$6,
  	$vocabulary: $vocabulary$4,
  	$dynamicAnchor: $dynamicAnchor$4,
  	title: title$5,
  	type: type$7,
  	properties: properties$6
  };

  var $schema$5 = "https://json-schema.org/draft/2020-12/schema";
  var $id$5 = "https://json-schema.org/draft/2020-12/meta/core";
  var $vocabulary$3 = {
  	"https://json-schema.org/draft/2020-12/vocab/core": true
  };
  var $dynamicAnchor$3 = "meta";
  var title$4 = "Core vocabulary meta-schema";
  var type$6 = [
  	"object",
  	"boolean"
  ];
  var properties$5 = {
  	$id: {
  		$ref: "#/$defs/uriReferenceString",
  		$comment: "Non-empty fragments not allowed.",
  		pattern: "^[^#]*#?$"
  	},
  	$schema: {
  		$ref: "#/$defs/uriString"
  	},
  	$ref: {
  		$ref: "#/$defs/uriReferenceString"
  	},
  	$anchor: {
  		$ref: "#/$defs/anchorString"
  	},
  	$dynamicRef: {
  		$ref: "#/$defs/uriReferenceString"
  	},
  	$dynamicAnchor: {
  		$ref: "#/$defs/anchorString"
  	},
  	$vocabulary: {
  		type: "object",
  		propertyNames: {
  			$ref: "#/$defs/uriString"
  		},
  		additionalProperties: {
  			type: "boolean"
  		}
  	},
  	$comment: {
  		type: "string"
  	},
  	$defs: {
  		type: "object",
  		additionalProperties: {
  			$dynamicRef: "#meta"
  		}
  	}
  };
  var $defs$2 = {
  	anchorString: {
  		type: "string",
  		pattern: "^[A-Za-z_][-A-Za-z0-9._]*$"
  	},
  	uriString: {
  		type: "string",
  		format: "uri"
  	},
  	uriReferenceString: {
  		type: "string",
  		format: "uri-reference"
  	}
  };
  var require$$4 = {
  	$schema: $schema$5,
  	$id: $id$5,
  	$vocabulary: $vocabulary$3,
  	$dynamicAnchor: $dynamicAnchor$3,
  	title: title$4,
  	type: type$6,
  	properties: properties$5,
  	$defs: $defs$2
  };

  var $schema$4 = "https://json-schema.org/draft/2020-12/schema";
  var $id$4 = "https://json-schema.org/draft/2020-12/meta/format-annotation";
  var $vocabulary$2 = {
  	"https://json-schema.org/draft/2020-12/vocab/format-annotation": true
  };
  var $dynamicAnchor$2 = "meta";
  var title$3 = "Format vocabulary meta-schema for annotation results";
  var type$5 = [
  	"object",
  	"boolean"
  ];
  var properties$4 = {
  	format: {
  		type: "string"
  	}
  };
  var require$$5 = {
  	$schema: $schema$4,
  	$id: $id$4,
  	$vocabulary: $vocabulary$2,
  	$dynamicAnchor: $dynamicAnchor$2,
  	title: title$3,
  	type: type$5,
  	properties: properties$4
  };

  var $schema$3 = "https://json-schema.org/draft/2020-12/schema";
  var $id$3 = "https://json-schema.org/draft/2020-12/meta/meta-data";
  var $vocabulary$1 = {
  	"https://json-schema.org/draft/2020-12/vocab/meta-data": true
  };
  var $dynamicAnchor$1 = "meta";
  var title$2 = "Meta-data vocabulary meta-schema";
  var type$4 = [
  	"object",
  	"boolean"
  ];
  var properties$3 = {
  	title: {
  		type: "string"
  	},
  	description: {
  		type: "string"
  	},
  	"default": true,
  	deprecated: {
  		type: "boolean",
  		"default": false
  	},
  	readOnly: {
  		type: "boolean",
  		"default": false
  	},
  	writeOnly: {
  		type: "boolean",
  		"default": false
  	},
  	examples: {
  		type: "array",
  		items: true
  	}
  };
  var require$$6 = {
  	$schema: $schema$3,
  	$id: $id$3,
  	$vocabulary: $vocabulary$1,
  	$dynamicAnchor: $dynamicAnchor$1,
  	title: title$2,
  	type: type$4,
  	properties: properties$3
  };

  var $schema$2 = "https://json-schema.org/draft/2020-12/schema";
  var $id$2 = "https://json-schema.org/draft/2020-12/meta/validation";
  var $vocabulary = {
  	"https://json-schema.org/draft/2020-12/vocab/validation": true
  };
  var $dynamicAnchor = "meta";
  var title$1 = "Validation vocabulary meta-schema";
  var type$3 = [
  	"object",
  	"boolean"
  ];
  var properties$2 = {
  	type: {
  		anyOf: [
  			{
  				$ref: "#/$defs/simpleTypes"
  			},
  			{
  				type: "array",
  				items: {
  					$ref: "#/$defs/simpleTypes"
  				},
  				minItems: 1,
  				uniqueItems: true
  			}
  		]
  	},
  	"const": true,
  	"enum": {
  		type: "array",
  		items: true
  	},
  	multipleOf: {
  		type: "number",
  		exclusiveMinimum: 0
  	},
  	maximum: {
  		type: "number"
  	},
  	exclusiveMaximum: {
  		type: "number"
  	},
  	minimum: {
  		type: "number"
  	},
  	exclusiveMinimum: {
  		type: "number"
  	},
  	maxLength: {
  		$ref: "#/$defs/nonNegativeInteger"
  	},
  	minLength: {
  		$ref: "#/$defs/nonNegativeIntegerDefault0"
  	},
  	pattern: {
  		type: "string",
  		format: "regex"
  	},
  	maxItems: {
  		$ref: "#/$defs/nonNegativeInteger"
  	},
  	minItems: {
  		$ref: "#/$defs/nonNegativeIntegerDefault0"
  	},
  	uniqueItems: {
  		type: "boolean",
  		"default": false
  	},
  	maxContains: {
  		$ref: "#/$defs/nonNegativeInteger"
  	},
  	minContains: {
  		$ref: "#/$defs/nonNegativeInteger",
  		"default": 1
  	},
  	maxProperties: {
  		$ref: "#/$defs/nonNegativeInteger"
  	},
  	minProperties: {
  		$ref: "#/$defs/nonNegativeIntegerDefault0"
  	},
  	required: {
  		$ref: "#/$defs/stringArray"
  	},
  	dependentRequired: {
  		type: "object",
  		additionalProperties: {
  			$ref: "#/$defs/stringArray"
  		}
  	}
  };
  var $defs$1 = {
  	nonNegativeInteger: {
  		type: "integer",
  		minimum: 0
  	},
  	nonNegativeIntegerDefault0: {
  		$ref: "#/$defs/nonNegativeInteger",
  		"default": 0
  	},
  	simpleTypes: {
  		"enum": [
  			"array",
  			"boolean",
  			"integer",
  			"null",
  			"number",
  			"object",
  			"string"
  		]
  	},
  	stringArray: {
  		type: "array",
  		items: {
  			type: "string"
  		},
  		uniqueItems: true,
  		"default": [
  		]
  	}
  };
  var require$$7 = {
  	$schema: $schema$2,
  	$id: $id$2,
  	$vocabulary: $vocabulary,
  	$dynamicAnchor: $dynamicAnchor,
  	title: title$1,
  	type: type$3,
  	properties: properties$2,
  	$defs: $defs$1
  };

  Object.defineProperty(jsonSchema202012, "__esModule", { value: true });
  const metaSchema = require$$0;
  const applicator = require$$1;
  const unevaluated = require$$2;
  const content = require$$3$1;
  const core$1 = require$$4;
  const format = require$$5;
  const metadata = require$$6;
  const validation = require$$7;
  const META_SUPPORT_DATA = ["/properties"];
  function addMetaSchema2020($data) {
      [
          metaSchema,
          applicator,
          unevaluated,
          content,
          core$1,
          with$data(this, format),
          metadata,
          with$data(this, validation),
      ].forEach((sch) => this.addMetaSchema(sch, undefined, false));
      return this;
      function with$data(ajv, sch) {
          return $data ? ajv.$dataMetaSchema(sch, META_SUPPORT_DATA) : sch;
      }
  }
  jsonSchema202012.default = addMetaSchema2020;

  _2020.exports;

  (function (module, exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.MissingRefError = exports.ValidationError = exports.CodeGen = exports.Name = exports.nil = exports.stringify = exports.str = exports._ = exports.KeywordCxt = exports.Ajv2020 = void 0;
  	const core_1 = core$4;
  	const draft2020_1 = draft2020;
  	const discriminator_1 = discriminator;
  	const json_schema_2020_12_1 = jsonSchema202012;
  	const META_SCHEMA_ID = "https://json-schema.org/draft/2020-12/schema";
  	class Ajv2020 extends core_1.default {
  	    constructor(opts = {}) {
  	        super({
  	            ...opts,
  	            dynamicRef: true,
  	            next: true,
  	            unevaluated: true,
  	        });
  	    }
  	    _addVocabularies() {
  	        super._addVocabularies();
  	        draft2020_1.default.forEach((v) => this.addVocabulary(v));
  	        if (this.opts.discriminator)
  	            this.addKeyword(discriminator_1.default);
  	    }
  	    _addDefaultMetaSchema() {
  	        super._addDefaultMetaSchema();
  	        const { $data, meta } = this.opts;
  	        if (!meta)
  	            return;
  	        json_schema_2020_12_1.default.call(this, $data);
  	        this.refs["http://json-schema.org/schema"] = META_SCHEMA_ID;
  	    }
  	    defaultMeta() {
  	        return (this.opts.defaultMeta =
  	            super.defaultMeta() || (this.getSchema(META_SCHEMA_ID) ? META_SCHEMA_ID : undefined));
  	    }
  	}
  	exports.Ajv2020 = Ajv2020;
  	module.exports = exports = Ajv2020;
  	module.exports.Ajv2020 = Ajv2020;
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.default = Ajv2020;
  	var validate_1 = requireValidate();
  	Object.defineProperty(exports, "KeywordCxt", { enumerable: true, get: function () { return validate_1.KeywordCxt; } });
  	var codegen_1 = codegen;
  	Object.defineProperty(exports, "_", { enumerable: true, get: function () { return codegen_1._; } });
  	Object.defineProperty(exports, "str", { enumerable: true, get: function () { return codegen_1.str; } });
  	Object.defineProperty(exports, "stringify", { enumerable: true, get: function () { return codegen_1.stringify; } });
  	Object.defineProperty(exports, "nil", { enumerable: true, get: function () { return codegen_1.nil; } });
  	Object.defineProperty(exports, "Name", { enumerable: true, get: function () { return codegen_1.Name; } });
  	Object.defineProperty(exports, "CodeGen", { enumerable: true, get: function () { return codegen_1.CodeGen; } });
  	var validation_error_1 = requireValidation_error();
  	Object.defineProperty(exports, "ValidationError", { enumerable: true, get: function () { return validation_error_1.default; } });
  	var ref_error_1 = requireRef_error();
  	Object.defineProperty(exports, "MissingRefError", { enumerable: true, get: function () { return ref_error_1.default; } });
  	
  } (_2020, _2020.exports));

  var _2020Exports = _2020.exports;
  var Ajv = /*@__PURE__*/getDefaultExportFromCjs(_2020Exports);

  var dist = {exports: {}};

  var formats = {};

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.formatNames = exports.fastFormats = exports.fullFormats = void 0;
  	function fmtDef(validate, compare) {
  	    return { validate, compare };
  	}
  	exports.fullFormats = {
  	    // date: http://tools.ietf.org/html/rfc3339#section-5.6
  	    date: fmtDef(date, compareDate),
  	    // date-time: http://tools.ietf.org/html/rfc3339#section-5.6
  	    time: fmtDef(getTime(true), compareTime),
  	    "date-time": fmtDef(getDateTime(true), compareDateTime),
  	    "iso-time": fmtDef(getTime(), compareIsoTime),
  	    "iso-date-time": fmtDef(getDateTime(), compareIsoDateTime),
  	    // duration: https://tools.ietf.org/html/rfc3339#appendix-A
  	    duration: /^P(?!$)((\d+Y)?(\d+M)?(\d+D)?(T(?=\d)(\d+H)?(\d+M)?(\d+S)?)?|(\d+W)?)$/,
  	    uri,
  	    "uri-reference": /^(?:[a-z][a-z0-9+\-.]*:)?(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'"()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?(?:\?(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i,
  	    // uri-template: https://tools.ietf.org/html/rfc6570
  	    "uri-template": /^(?:(?:[^\x00-\x20"'<>%\\^`{|}]|%[0-9a-f]{2})|\{[+#./;?&=,!@|]?(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?(?:,(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?)*\})*$/i,
  	    // For the source: https://gist.github.com/dperini/729294
  	    // For test cases: https://mathiasbynens.be/demo/url-regex
  	    url: /^(?:https?|ftp):\/\/(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z0-9\u{00a1}-\u{ffff}]+-)*[a-z0-9\u{00a1}-\u{ffff}]+)(?:\.(?:[a-z0-9\u{00a1}-\u{ffff}]+-)*[a-z0-9\u{00a1}-\u{ffff}]+)*(?:\.(?:[a-z\u{00a1}-\u{ffff}]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?$/iu,
  	    email: /^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i,
  	    hostname: /^(?=.{1,253}\.?$)[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[-0-9a-z]{0,61}[0-9a-z])?)*\.?$/i,
  	    // optimized https://www.safaribooksonline.com/library/view/regular-expressions-cookbook/9780596802837/ch07s16.html
  	    ipv4: /^(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)\.){3}(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)$/,
  	    ipv6: /^((([0-9a-f]{1,4}:){7}([0-9a-f]{1,4}|:))|(([0-9a-f]{1,4}:){6}(:[0-9a-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9a-f]{1,4}:){5}(((:[0-9a-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9a-f]{1,4}:){4}(((:[0-9a-f]{1,4}){1,3})|((:[0-9a-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9a-f]{1,4}:){3}(((:[0-9a-f]{1,4}){1,4})|((:[0-9a-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9a-f]{1,4}:){2}(((:[0-9a-f]{1,4}){1,5})|((:[0-9a-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9a-f]{1,4}:){1}(((:[0-9a-f]{1,4}){1,6})|((:[0-9a-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9a-f]{1,4}){1,7})|((:[0-9a-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))$/i,
  	    regex,
  	    // uuid: http://tools.ietf.org/html/rfc4122
  	    uuid: /^(?:urn:uuid:)?[0-9a-f]{8}-(?:[0-9a-f]{4}-){3}[0-9a-f]{12}$/i,
  	    // JSON-pointer: https://tools.ietf.org/html/rfc6901
  	    // uri fragment: https://tools.ietf.org/html/rfc3986#appendix-A
  	    "json-pointer": /^(?:\/(?:[^~/]|~0|~1)*)*$/,
  	    "json-pointer-uri-fragment": /^#(?:\/(?:[a-z0-9_\-.!$&'()*+,;:=@]|%[0-9a-f]{2}|~0|~1)*)*$/i,
  	    // relative JSON-pointer: http://tools.ietf.org/html/draft-luff-relative-json-pointer-00
  	    "relative-json-pointer": /^(?:0|[1-9][0-9]*)(?:#|(?:\/(?:[^~/]|~0|~1)*)*)$/,
  	    // the following formats are used by the openapi specification: https://spec.openapis.org/oas/v3.0.0#data-types
  	    // byte: https://github.com/miguelmota/is-base64
  	    byte,
  	    // signed 32 bit integer
  	    int32: { type: "number", validate: validateInt32 },
  	    // signed 64 bit integer
  	    int64: { type: "number", validate: validateInt64 },
  	    // C-type float
  	    float: { type: "number", validate: validateNumber },
  	    // C-type double
  	    double: { type: "number", validate: validateNumber },
  	    // hint to the UI to hide input strings
  	    password: true,
  	    // unchecked string payload
  	    binary: true,
  	};
  	exports.fastFormats = {
  	    ...exports.fullFormats,
  	    date: fmtDef(/^\d\d\d\d-[0-1]\d-[0-3]\d$/, compareDate),
  	    time: fmtDef(/^(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)$/i, compareTime),
  	    "date-time": fmtDef(/^\d\d\d\d-[0-1]\d-[0-3]\dt(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)$/i, compareDateTime),
  	    "iso-time": fmtDef(/^(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)?$/i, compareIsoTime),
  	    "iso-date-time": fmtDef(/^\d\d\d\d-[0-1]\d-[0-3]\d[t\s](?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)?$/i, compareIsoDateTime),
  	    // uri: https://github.com/mafintosh/is-my-json-valid/blob/master/formats.js
  	    uri: /^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/)?[^\s]*$/i,
  	    "uri-reference": /^(?:(?:[a-z][a-z0-9+\-.]*:)?\/?\/)?(?:[^\\\s#][^\s#]*)?(?:#[^\\\s]*)?$/i,
  	    // email (sources from jsen validator):
  	    // http://stackoverflow.com/questions/201323/using-a-regular-expression-to-validate-an-email-address#answer-8829363
  	    // http://www.w3.org/TR/html5/forms.html#valid-e-mail-address (search for 'wilful violation')
  	    email: /^[a-z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?)*$/i,
  	};
  	exports.formatNames = Object.keys(exports.fullFormats);
  	function isLeapYear(year) {
  	    // https://tools.ietf.org/html/rfc3339#appendix-C
  	    return year % 4 === 0 && (year % 100 !== 0 || year % 400 === 0);
  	}
  	const DATE = /^(\d\d\d\d)-(\d\d)-(\d\d)$/;
  	const DAYS = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
  	function date(str) {
  	    // full-date from http://tools.ietf.org/html/rfc3339#section-5.6
  	    const matches = DATE.exec(str);
  	    if (!matches)
  	        return false;
  	    const year = +matches[1];
  	    const month = +matches[2];
  	    const day = +matches[3];
  	    return (month >= 1 &&
  	        month <= 12 &&
  	        day >= 1 &&
  	        day <= (month === 2 && isLeapYear(year) ? 29 : DAYS[month]));
  	}
  	function compareDate(d1, d2) {
  	    if (!(d1 && d2))
  	        return undefined;
  	    if (d1 > d2)
  	        return 1;
  	    if (d1 < d2)
  	        return -1;
  	    return 0;
  	}
  	const TIME = /^(\d\d):(\d\d):(\d\d(?:\.\d+)?)(z|([+-])(\d\d)(?::?(\d\d))?)?$/i;
  	function getTime(strictTimeZone) {
  	    return function time(str) {
  	        const matches = TIME.exec(str);
  	        if (!matches)
  	            return false;
  	        const hr = +matches[1];
  	        const min = +matches[2];
  	        const sec = +matches[3];
  	        const tz = matches[4];
  	        const tzSign = matches[5] === "-" ? -1 : 1;
  	        const tzH = +(matches[6] || 0);
  	        const tzM = +(matches[7] || 0);
  	        if (tzH > 23 || tzM > 59 || (strictTimeZone && !tz))
  	            return false;
  	        if (hr <= 23 && min <= 59 && sec < 60)
  	            return true;
  	        // leap second
  	        const utcMin = min - tzM * tzSign;
  	        const utcHr = hr - tzH * tzSign - (utcMin < 0 ? 1 : 0);
  	        return (utcHr === 23 || utcHr === -1) && (utcMin === 59 || utcMin === -1) && sec < 61;
  	    };
  	}
  	function compareTime(s1, s2) {
  	    if (!(s1 && s2))
  	        return undefined;
  	    const t1 = new Date("2020-01-01T" + s1).valueOf();
  	    const t2 = new Date("2020-01-01T" + s2).valueOf();
  	    if (!(t1 && t2))
  	        return undefined;
  	    return t1 - t2;
  	}
  	function compareIsoTime(t1, t2) {
  	    if (!(t1 && t2))
  	        return undefined;
  	    const a1 = TIME.exec(t1);
  	    const a2 = TIME.exec(t2);
  	    if (!(a1 && a2))
  	        return undefined;
  	    t1 = a1[1] + a1[2] + a1[3];
  	    t2 = a2[1] + a2[2] + a2[3];
  	    if (t1 > t2)
  	        return 1;
  	    if (t1 < t2)
  	        return -1;
  	    return 0;
  	}
  	const DATE_TIME_SEPARATOR = /t|\s/i;
  	function getDateTime(strictTimeZone) {
  	    const time = getTime(strictTimeZone);
  	    return function date_time(str) {
  	        // http://tools.ietf.org/html/rfc3339#section-5.6
  	        const dateTime = str.split(DATE_TIME_SEPARATOR);
  	        return dateTime.length === 2 && date(dateTime[0]) && time(dateTime[1]);
  	    };
  	}
  	function compareDateTime(dt1, dt2) {
  	    if (!(dt1 && dt2))
  	        return undefined;
  	    const d1 = new Date(dt1).valueOf();
  	    const d2 = new Date(dt2).valueOf();
  	    if (!(d1 && d2))
  	        return undefined;
  	    return d1 - d2;
  	}
  	function compareIsoDateTime(dt1, dt2) {
  	    if (!(dt1 && dt2))
  	        return undefined;
  	    const [d1, t1] = dt1.split(DATE_TIME_SEPARATOR);
  	    const [d2, t2] = dt2.split(DATE_TIME_SEPARATOR);
  	    const res = compareDate(d1, d2);
  	    if (res === undefined)
  	        return undefined;
  	    return res || compareTime(t1, t2);
  	}
  	const NOT_URI_FRAGMENT = /\/|:/;
  	const URI = /^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)(?:\?(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i;
  	function uri(str) {
  	    // http://jmrware.com/articles/2009/uri_regexp/URI_regex.html + optional protocol + required "."
  	    return NOT_URI_FRAGMENT.test(str) && URI.test(str);
  	}
  	const BYTE = /^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$/gm;
  	function byte(str) {
  	    BYTE.lastIndex = 0;
  	    return BYTE.test(str);
  	}
  	const MIN_INT32 = -(2 ** 31);
  	const MAX_INT32 = 2 ** 31 - 1;
  	function validateInt32(value) {
  	    return Number.isInteger(value) && value <= MAX_INT32 && value >= MIN_INT32;
  	}
  	function validateInt64(value) {
  	    // JSON and javascript max Int is 2**53, so any int that passes isInteger is valid for Int64
  	    return Number.isInteger(value);
  	}
  	function validateNumber() {
  	    return true;
  	}
  	const Z_ANCHOR = /[^\\]\\Z/;
  	function regex(str) {
  	    if (Z_ANCHOR.test(str))
  	        return false;
  	    try {
  	        new RegExp(str);
  	        return true;
  	    }
  	    catch (e) {
  	        return false;
  	    }
  	}
  	
  } (formats));

  var limit = {};

  var ajv$1 = {exports: {}};

  var draft7 = {};

  Object.defineProperty(draft7, "__esModule", { value: true });
  const core_1 = core$3;
  const validation_1 = validation$2;
  const applicator_1 = applicator$1;
  const format_1 = format$3;
  const metadata_1 = metadata$1;
  const draft7Vocabularies = [
      core_1.default,
      validation_1.default,
      (0, applicator_1.default)(),
      format_1.default,
      metadata_1.metadataVocabulary,
      metadata_1.contentVocabulary,
  ];
  draft7.default = draft7Vocabularies;

  var $schema$1 = "http://json-schema.org/draft-07/schema#";
  var $id$1 = "http://json-schema.org/draft-07/schema#";
  var title = "Core schema meta-schema";
  var definitions = {
  	schemaArray: {
  		type: "array",
  		minItems: 1,
  		items: {
  			$ref: "#"
  		}
  	},
  	nonNegativeInteger: {
  		type: "integer",
  		minimum: 0
  	},
  	nonNegativeIntegerDefault0: {
  		allOf: [
  			{
  				$ref: "#/definitions/nonNegativeInteger"
  			},
  			{
  				"default": 0
  			}
  		]
  	},
  	simpleTypes: {
  		"enum": [
  			"array",
  			"boolean",
  			"integer",
  			"null",
  			"number",
  			"object",
  			"string"
  		]
  	},
  	stringArray: {
  		type: "array",
  		items: {
  			type: "string"
  		},
  		uniqueItems: true,
  		"default": [
  		]
  	}
  };
  var type$2 = [
  	"object",
  	"boolean"
  ];
  var properties$1 = {
  	$id: {
  		type: "string",
  		format: "uri-reference"
  	},
  	$schema: {
  		type: "string",
  		format: "uri"
  	},
  	$ref: {
  		type: "string",
  		format: "uri-reference"
  	},
  	$comment: {
  		type: "string"
  	},
  	title: {
  		type: "string"
  	},
  	description: {
  		type: "string"
  	},
  	"default": true,
  	readOnly: {
  		type: "boolean",
  		"default": false
  	},
  	examples: {
  		type: "array",
  		items: true
  	},
  	multipleOf: {
  		type: "number",
  		exclusiveMinimum: 0
  	},
  	maximum: {
  		type: "number"
  	},
  	exclusiveMaximum: {
  		type: "number"
  	},
  	minimum: {
  		type: "number"
  	},
  	exclusiveMinimum: {
  		type: "number"
  	},
  	maxLength: {
  		$ref: "#/definitions/nonNegativeInteger"
  	},
  	minLength: {
  		$ref: "#/definitions/nonNegativeIntegerDefault0"
  	},
  	pattern: {
  		type: "string",
  		format: "regex"
  	},
  	additionalItems: {
  		$ref: "#"
  	},
  	items: {
  		anyOf: [
  			{
  				$ref: "#"
  			},
  			{
  				$ref: "#/definitions/schemaArray"
  			}
  		],
  		"default": true
  	},
  	maxItems: {
  		$ref: "#/definitions/nonNegativeInteger"
  	},
  	minItems: {
  		$ref: "#/definitions/nonNegativeIntegerDefault0"
  	},
  	uniqueItems: {
  		type: "boolean",
  		"default": false
  	},
  	contains: {
  		$ref: "#"
  	},
  	maxProperties: {
  		$ref: "#/definitions/nonNegativeInteger"
  	},
  	minProperties: {
  		$ref: "#/definitions/nonNegativeIntegerDefault0"
  	},
  	required: {
  		$ref: "#/definitions/stringArray"
  	},
  	additionalProperties: {
  		$ref: "#"
  	},
  	definitions: {
  		type: "object",
  		additionalProperties: {
  			$ref: "#"
  		},
  		"default": {
  		}
  	},
  	properties: {
  		type: "object",
  		additionalProperties: {
  			$ref: "#"
  		},
  		"default": {
  		}
  	},
  	patternProperties: {
  		type: "object",
  		additionalProperties: {
  			$ref: "#"
  		},
  		propertyNames: {
  			format: "regex"
  		},
  		"default": {
  		}
  	},
  	dependencies: {
  		type: "object",
  		additionalProperties: {
  			anyOf: [
  				{
  					$ref: "#"
  				},
  				{
  					$ref: "#/definitions/stringArray"
  				}
  			]
  		}
  	},
  	propertyNames: {
  		$ref: "#"
  	},
  	"const": true,
  	"enum": {
  		type: "array",
  		items: true,
  		minItems: 1,
  		uniqueItems: true
  	},
  	type: {
  		anyOf: [
  			{
  				$ref: "#/definitions/simpleTypes"
  			},
  			{
  				type: "array",
  				items: {
  					$ref: "#/definitions/simpleTypes"
  				},
  				minItems: 1,
  				uniqueItems: true
  			}
  		]
  	},
  	format: {
  		type: "string"
  	},
  	contentMediaType: {
  		type: "string"
  	},
  	contentEncoding: {
  		type: "string"
  	},
  	"if": {
  		$ref: "#"
  	},
  	then: {
  		$ref: "#"
  	},
  	"else": {
  		$ref: "#"
  	},
  	allOf: {
  		$ref: "#/definitions/schemaArray"
  	},
  	anyOf: {
  		$ref: "#/definitions/schemaArray"
  	},
  	oneOf: {
  		$ref: "#/definitions/schemaArray"
  	},
  	not: {
  		$ref: "#"
  	}
  };
  var require$$3 = {
  	$schema: $schema$1,
  	$id: $id$1,
  	title: title,
  	definitions: definitions,
  	type: type$2,
  	properties: properties$1,
  	"default": true
  };

  ajv$1.exports;

  (function (module, exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.MissingRefError = exports.ValidationError = exports.CodeGen = exports.Name = exports.nil = exports.stringify = exports.str = exports._ = exports.KeywordCxt = exports.Ajv = void 0;
  	const core_1 = core$4;
  	const draft7_1 = draft7;
  	const discriminator_1 = discriminator;
  	const draft7MetaSchema = require$$3;
  	const META_SUPPORT_DATA = ["/properties"];
  	const META_SCHEMA_ID = "http://json-schema.org/draft-07/schema";
  	class Ajv extends core_1.default {
  	    _addVocabularies() {
  	        super._addVocabularies();
  	        draft7_1.default.forEach((v) => this.addVocabulary(v));
  	        if (this.opts.discriminator)
  	            this.addKeyword(discriminator_1.default);
  	    }
  	    _addDefaultMetaSchema() {
  	        super._addDefaultMetaSchema();
  	        if (!this.opts.meta)
  	            return;
  	        const metaSchema = this.opts.$data
  	            ? this.$dataMetaSchema(draft7MetaSchema, META_SUPPORT_DATA)
  	            : draft7MetaSchema;
  	        this.addMetaSchema(metaSchema, META_SCHEMA_ID, false);
  	        this.refs["http://json-schema.org/schema"] = META_SCHEMA_ID;
  	    }
  	    defaultMeta() {
  	        return (this.opts.defaultMeta =
  	            super.defaultMeta() || (this.getSchema(META_SCHEMA_ID) ? META_SCHEMA_ID : undefined));
  	    }
  	}
  	exports.Ajv = Ajv;
  	module.exports = exports = Ajv;
  	module.exports.Ajv = Ajv;
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.default = Ajv;
  	var validate_1 = requireValidate();
  	Object.defineProperty(exports, "KeywordCxt", { enumerable: true, get: function () { return validate_1.KeywordCxt; } });
  	var codegen_1 = codegen;
  	Object.defineProperty(exports, "_", { enumerable: true, get: function () { return codegen_1._; } });
  	Object.defineProperty(exports, "str", { enumerable: true, get: function () { return codegen_1.str; } });
  	Object.defineProperty(exports, "stringify", { enumerable: true, get: function () { return codegen_1.stringify; } });
  	Object.defineProperty(exports, "nil", { enumerable: true, get: function () { return codegen_1.nil; } });
  	Object.defineProperty(exports, "Name", { enumerable: true, get: function () { return codegen_1.Name; } });
  	Object.defineProperty(exports, "CodeGen", { enumerable: true, get: function () { return codegen_1.CodeGen; } });
  	var validation_error_1 = requireValidation_error();
  	Object.defineProperty(exports, "ValidationError", { enumerable: true, get: function () { return validation_error_1.default; } });
  	var ref_error_1 = requireRef_error();
  	Object.defineProperty(exports, "MissingRefError", { enumerable: true, get: function () { return ref_error_1.default; } });
  	
  } (ajv$1, ajv$1.exports));

  var ajvExports = ajv$1.exports;

  (function (exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.formatLimitDefinition = void 0;
  	const ajv_1 = ajvExports;
  	const codegen_1 = codegen;
  	const ops = codegen_1.operators;
  	const KWDs = {
  	    formatMaximum: { okStr: "<=", ok: ops.LTE, fail: ops.GT },
  	    formatMinimum: { okStr: ">=", ok: ops.GTE, fail: ops.LT },
  	    formatExclusiveMaximum: { okStr: "<", ok: ops.LT, fail: ops.GTE },
  	    formatExclusiveMinimum: { okStr: ">", ok: ops.GT, fail: ops.LTE },
  	};
  	const error = {
  	    message: ({ keyword, schemaCode }) => (0, codegen_1.str) `should be ${KWDs[keyword].okStr} ${schemaCode}`,
  	    params: ({ keyword, schemaCode }) => (0, codegen_1._) `{comparison: ${KWDs[keyword].okStr}, limit: ${schemaCode}}`,
  	};
  	exports.formatLimitDefinition = {
  	    keyword: Object.keys(KWDs),
  	    type: "string",
  	    schemaType: "string",
  	    $data: true,
  	    error,
  	    code(cxt) {
  	        const { gen, data, schemaCode, keyword, it } = cxt;
  	        const { opts, self } = it;
  	        if (!opts.validateFormats)
  	            return;
  	        const fCxt = new ajv_1.KeywordCxt(it, self.RULES.all.format.definition, "format");
  	        if (fCxt.$data)
  	            validate$DataFormat();
  	        else
  	            validateFormat();
  	        function validate$DataFormat() {
  	            const fmts = gen.scopeValue("formats", {
  	                ref: self.formats,
  	                code: opts.code.formats,
  	            });
  	            const fmt = gen.const("fmt", (0, codegen_1._) `${fmts}[${fCxt.schemaCode}]`);
  	            cxt.fail$data((0, codegen_1.or)((0, codegen_1._) `typeof ${fmt} != "object"`, (0, codegen_1._) `${fmt} instanceof RegExp`, (0, codegen_1._) `typeof ${fmt}.compare != "function"`, compareCode(fmt)));
  	        }
  	        function validateFormat() {
  	            const format = fCxt.schema;
  	            const fmtDef = self.formats[format];
  	            if (!fmtDef || fmtDef === true)
  	                return;
  	            if (typeof fmtDef != "object" ||
  	                fmtDef instanceof RegExp ||
  	                typeof fmtDef.compare != "function") {
  	                throw new Error(`"${keyword}": format "${format}" does not define "compare" function`);
  	            }
  	            const fmt = gen.scopeValue("formats", {
  	                key: format,
  	                ref: fmtDef,
  	                code: opts.code.formats ? (0, codegen_1._) `${opts.code.formats}${(0, codegen_1.getProperty)(format)}` : undefined,
  	            });
  	            cxt.fail$data(compareCode(fmt));
  	        }
  	        function compareCode(fmt) {
  	            return (0, codegen_1._) `${fmt}.compare(${data}, ${schemaCode}) ${KWDs[keyword].fail} 0`;
  	        }
  	    },
  	    dependencies: ["format"],
  	};
  	const formatLimitPlugin = (ajv) => {
  	    ajv.addKeyword(exports.formatLimitDefinition);
  	    return ajv;
  	};
  	exports.default = formatLimitPlugin;
  	
  } (limit));

  dist.exports;

  (function (module, exports) {
  	Object.defineProperty(exports, "__esModule", { value: true });
  	const formats_1 = formats;
  	const limit_1 = limit;
  	const codegen_1 = codegen;
  	const fullName = new codegen_1.Name("fullFormats");
  	const fastName = new codegen_1.Name("fastFormats");
  	const formatsPlugin = (ajv, opts = { keywords: true }) => {
  	    if (Array.isArray(opts)) {
  	        addFormats(ajv, opts, formats_1.fullFormats, fullName);
  	        return ajv;
  	    }
  	    const [formats, exportName] = opts.mode === "fast" ? [formats_1.fastFormats, fastName] : [formats_1.fullFormats, fullName];
  	    const list = opts.formats || formats_1.formatNames;
  	    addFormats(ajv, list, formats, exportName);
  	    if (opts.keywords)
  	        (0, limit_1.default)(ajv);
  	    return ajv;
  	};
  	formatsPlugin.get = (name, mode = "full") => {
  	    const formats = mode === "fast" ? formats_1.fastFormats : formats_1.fullFormats;
  	    const f = formats[name];
  	    if (!f)
  	        throw new Error(`Unknown format "${name}"`);
  	    return f;
  	};
  	function addFormats(ajv, list, fs, exportName) {
  	    var _a;
  	    var _b;
  	    (_a = (_b = ajv.opts.code).formats) !== null && _a !== void 0 ? _a : (_b.formats = (0, codegen_1._) `require("ajv-formats/dist/formats").${exportName}`);
  	    for (const f of list)
  	        ajv.addFormat(f, fs[f]);
  	}
  	module.exports = exports = formatsPlugin;
  	Object.defineProperty(exports, "__esModule", { value: true });
  	exports.default = formatsPlugin;
  	
  } (dist, dist.exports));

  var distExports = dist.exports;
  var addFormats = /*@__PURE__*/getDefaultExportFromCjs(distExports);

  var $id = "https://serverlessworkflow.io/schemas/1.0.0/workflow.json";
  var $schema = "https://json-schema.org/draft/2020-12/schema";
  var description = "Serverless Workflow DSL - Workflow Schema.";
  var type$1 = "object";
  var required = [
  	"document",
  	"do"
  ];
  var properties = {
  	document: {
  		type: "object",
  		title: "Document",
  		description: "Documents the workflow.",
  		unevaluatedProperties: false,
  		properties: {
  			dsl: {
  				type: "string",
  				pattern: "^(0|[1-9]\\d*)\\.(0|[1-9]\\d*)\\.(0|[1-9]\\d*)(?:-((?:0|[1-9]\\d*|\\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\\.(?:0|[1-9]\\d*|\\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\\+([0-9a-zA-Z-]+(?:\\.[0-9a-zA-Z-]+)*))?$",
  				title: "WorkflowDSL",
  				description: "The version of the DSL used by the workflow."
  			},
  			namespace: {
  				type: "string",
  				pattern: "^[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?$",
  				title: "WorkflowNamespace",
  				description: "The workflow's namespace."
  			},
  			name: {
  				type: "string",
  				pattern: "^[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?$",
  				title: "WorkflowName",
  				description: "The workflow's name."
  			},
  			version: {
  				type: "string",
  				pattern: "^(0|[1-9]\\d*)\\.(0|[1-9]\\d*)\\.(0|[1-9]\\d*)(?:-((?:0|[1-9]\\d*|\\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\\.(?:0|[1-9]\\d*|\\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\\+([0-9a-zA-Z-]+(?:\\.[0-9a-zA-Z-]+)*))?$",
  				title: "WorkflowVersion",
  				description: "The workflow's semantic version."
  			},
  			title: {
  				type: "string",
  				title: "WorkflowTitle",
  				description: "The workflow's title."
  			},
  			summary: {
  				type: "string",
  				title: "WorkflowSummary",
  				description: "The workflow's Markdown summary."
  			},
  			tags: {
  				type: "object",
  				title: "WorkflowTags",
  				description: "A key/value mapping of the workflow's tags, if any.",
  				additionalProperties: true
  			},
  			metadata: {
  				type: "object",
  				title: "WorkflowMetadata",
  				description: "Holds additional information about the workflow.",
  				additionalProperties: true
  			}
  		},
  		required: [
  			"dsl",
  			"namespace",
  			"name",
  			"version"
  		]
  	},
  	input: {
  		$ref: "#/$defs/input",
  		title: "Input",
  		description: "Configures the workflow's input."
  	},
  	use: {
  		type: "object",
  		title: "Use",
  		description: "Defines the workflow's reusable components.",
  		unevaluatedProperties: false,
  		properties: {
  			authentications: {
  				type: "object",
  				title: "UseAuthentications",
  				description: "The workflow's reusable authentication policies.",
  				additionalProperties: {
  					$ref: "#/$defs/authenticationPolicy"
  				}
  			},
  			errors: {
  				type: "object",
  				title: "UseErrors",
  				description: "The workflow's reusable errors.",
  				additionalProperties: {
  					$ref: "#/$defs/error"
  				}
  			},
  			extensions: {
  				type: "array",
  				title: "UseExtensions",
  				description: "The workflow's extensions.",
  				items: {
  					type: "object",
  					title: "ExtensionItem",
  					minProperties: 1,
  					maxProperties: 1,
  					additionalProperties: {
  						$ref: "#/$defs/extension"
  					}
  				}
  			},
  			functions: {
  				type: "object",
  				title: "UseFunctions",
  				description: "The workflow's reusable functions.",
  				additionalProperties: {
  					$ref: "#/$defs/task"
  				}
  			},
  			retries: {
  				type: "object",
  				title: "UseRetries",
  				description: "The workflow's reusable retry policies.",
  				additionalProperties: {
  					$ref: "#/$defs/retryPolicy"
  				}
  			},
  			secrets: {
  				type: "array",
  				title: "UseSecrets",
  				description: "The workflow's reusable secrets.",
  				items: {
  					type: "string",
  					description: "The workflow's secrets."
  				}
  			},
  			timeouts: {
  				type: "object",
  				title: "UseTimeouts",
  				description: "The workflow's reusable timeouts.",
  				additionalProperties: {
  					$ref: "#/$defs/timeout"
  				}
  			},
  			catalogs: {
  				type: "object",
  				title: "UseCatalogs",
  				description: "The workflow's reusable catalogs.",
  				additionalProperties: {
  					$ref: "#/$defs/catalog"
  				}
  			}
  		}
  	},
  	"do": {
  		$ref: "#/$defs/taskList",
  		title: "Do",
  		description: "Defines the task(s) the workflow must perform."
  	},
  	timeout: {
  		title: "DoTimeout",
  		oneOf: [
  			{
  				$ref: "#/$defs/timeout",
  				title: "TimeoutDefinition",
  				description: "The workflow's timeout configuration, if any."
  			},
  			{
  				type: "string",
  				title: "TimeoutReference",
  				description: "The name of the workflow's timeout, if any."
  			}
  		]
  	},
  	output: {
  		$ref: "#/$defs/output",
  		title: "Output",
  		description: "Configures the workflow's output."
  	},
  	schedule: {
  		type: "object",
  		title: "Schedule",
  		description: "Schedules the workflow.",
  		unevaluatedProperties: false,
  		properties: {
  			every: {
  				$ref: "#/$defs/duration",
  				title: "ScheduleEvery",
  				description: "Specifies the duration of the interval at which the workflow should be executed."
  			},
  			cron: {
  				type: "string",
  				title: "ScheduleCron",
  				description: "Specifies the schedule using a cron expression, e.g., '0 0 * * *' for daily at midnight."
  			},
  			after: {
  				$ref: "#/$defs/duration",
  				title: "ScheduleAfter",
  				description: "Specifies a delay duration that the workflow must wait before starting again after it completes."
  			},
  			on: {
  				$ref: "#/$defs/eventConsumptionStrategy",
  				title: "ScheduleOn",
  				description: "Specifies the events that trigger the workflow execution."
  			}
  		}
  	}
  };
  var $defs = {
  	taskList: {
  		title: "TaskList",
  		description: "List of named tasks to perform.",
  		type: "array",
  		items: {
  			type: "object",
  			title: "TaskItem",
  			minProperties: 1,
  			maxProperties: 1,
  			additionalProperties: {
  				$ref: "#/$defs/task"
  			}
  		}
  	},
  	taskBase: {
  		type: "object",
  		title: "TaskBase",
  		description: "An object inherited by all tasks.",
  		properties: {
  			"if": {
  				type: "string",
  				title: "TaskBaseIf",
  				description: "A runtime expression, if any, used to determine whether or not the task should be run."
  			},
  			input: {
  				$ref: "#/$defs/input",
  				title: "TaskBaseInput",
  				description: "Configure the task's input."
  			},
  			output: {
  				$ref: "#/$defs/output",
  				title: "TaskBaseOutput",
  				description: "Configure the task's output."
  			},
  			"export": {
  				$ref: "#/$defs/export",
  				title: "TaskBaseExport",
  				description: "Export task output to context."
  			},
  			timeout: {
  				title: "TaskTimeout",
  				oneOf: [
  					{
  						$ref: "#/$defs/timeout",
  						title: "TaskTimeoutDefinition",
  						description: "The task's timeout configuration, if any."
  					},
  					{
  						type: "string",
  						title: "TaskTimeoutReference",
  						description: "The name of the task's timeout, if any."
  					}
  				]
  			},
  			then: {
  				$ref: "#/$defs/flowDirective",
  				title: "TaskBaseThen",
  				description: "The flow directive to be performed upon completion of the task."
  			},
  			metadata: {
  				type: "object",
  				title: "TaskMetadata",
  				description: "Holds additional information about the task.",
  				additionalProperties: true
  			}
  		}
  	},
  	task: {
  		title: "Task",
  		description: "A discrete unit of work that contributes to achieving the overall objectives defined by the workflow.",
  		unevaluatedProperties: false,
  		oneOf: [
  			{
  				$ref: "#/$defs/callTask"
  			},
  			{
  				$ref: "#/$defs/doTask"
  			},
  			{
  				$ref: "#/$defs/forkTask"
  			},
  			{
  				$ref: "#/$defs/emitTask"
  			},
  			{
  				$ref: "#/$defs/forTask"
  			},
  			{
  				$ref: "#/$defs/listenTask"
  			},
  			{
  				$ref: "#/$defs/raiseTask"
  			},
  			{
  				$ref: "#/$defs/runTask"
  			},
  			{
  				$ref: "#/$defs/setTask"
  			},
  			{
  				$ref: "#/$defs/switchTask"
  			},
  			{
  				$ref: "#/$defs/tryTask"
  			},
  			{
  				$ref: "#/$defs/waitTask"
  			}
  		]
  	},
  	callTask: {
  		title: "CallTask",
  		description: "Defines the call to perform.",
  		oneOf: [
  			{
  				title: "CallAsyncAPI",
  				description: "Defines the AsyncAPI call to perform.",
  				$ref: "#/$defs/taskBase",
  				type: "object",
  				required: [
  					"call",
  					"with"
  				],
  				unevaluatedProperties: false,
  				properties: {
  					call: {
  						type: "string",
  						"const": "asyncapi"
  					},
  					"with": {
  						type: "object",
  						title: "AsyncApiArguments",
  						description: "The Async API call arguments.",
  						properties: {
  							document: {
  								$ref: "#/$defs/externalResource",
  								title: "AsyncAPIDocument",
  								description: "The document that defines the AsyncAPI operation to call."
  							},
  							channel: {
  								type: "string",
  								title: "With",
  								description: "The name of the channel on which to perform the operation. Used only in case the referenced document uses AsyncAPI v2.6.0."
  							},
  							operation: {
  								type: "string",
  								title: "AsyncAPIOperation",
  								description: "A reference to the AsyncAPI operation to call."
  							},
  							server: {
  								$ref: "#/$defs/asyncApiServer",
  								title: "AsyncAPIServer",
  								description: "An object used to configure to the server to call the specified AsyncAPI operation on."
  							},
  							protocol: {
  								type: "string",
  								title: "AsyncApiProtocol",
  								description: "The protocol to use to select the target server.",
  								"enum": [
  									"amqp",
  									"amqp1",
  									"anypointmq",
  									"googlepubsub",
  									"http",
  									"ibmmq",
  									"jms",
  									"kafka",
  									"mercure",
  									"mqtt",
  									"mqtt5",
  									"nats",
  									"pulsar",
  									"redis",
  									"sns",
  									"solace",
  									"sqs",
  									"stomp",
  									"ws"
  								]
  							},
  							message: {
  								$ref: "#/$defs/asyncApiOutboundMessage",
  								title: "AsyncApiMessage",
  								description: "An object used to configure the message to publish using the target operation."
  							},
  							subscription: {
  								$ref: "#/$defs/asyncApiSubscription",
  								title: "AsyncApiSubscription",
  								description: "An object used to configure the subscription to messages consumed using the target operation."
  							},
  							authentication: {
  								$ref: "#/$defs/referenceableAuthenticationPolicy",
  								title: "AsyncAPIAuthentication",
  								description: "The authentication policy, if any, to use when calling the AsyncAPI operation."
  							}
  						},
  						oneOf: [
  							{
  								required: [
  									"document",
  									"operation",
  									"message"
  								]
  							},
  							{
  								required: [
  									"document",
  									"operation",
  									"subscription"
  								]
  							},
  							{
  								required: [
  									"document",
  									"channel",
  									"message"
  								]
  							},
  							{
  								required: [
  									"document",
  									"channel",
  									"subscription"
  								]
  							}
  						],
  						unevaluatedProperties: false
  					}
  				}
  			},
  			{
  				title: "CallGRPC",
  				description: "Defines the GRPC call to perform.",
  				$ref: "#/$defs/taskBase",
  				type: "object",
  				unevaluatedProperties: false,
  				required: [
  					"call",
  					"with"
  				],
  				properties: {
  					call: {
  						type: "string",
  						"const": "grpc"
  					},
  					"with": {
  						type: "object",
  						title: "GRPCArguments",
  						description: "The GRPC call arguments.",
  						properties: {
  							proto: {
  								$ref: "#/$defs/externalResource",
  								title: "WithGRPCProto",
  								description: "The proto resource that describes the GRPC service to call."
  							},
  							service: {
  								type: "object",
  								title: "WithGRPCService",
  								unevaluatedProperties: false,
  								properties: {
  									name: {
  										type: "string",
  										title: "WithGRPCServiceName",
  										description: "The name of the GRPC service to call."
  									},
  									host: {
  										type: "string",
  										title: "WithGRPCServiceHost",
  										description: "The hostname of the GRPC service to call.",
  										pattern: "^[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?$"
  									},
  									port: {
  										type: "integer",
  										title: "WithGRPCServicePost",
  										description: "The port number of the GRPC service to call.",
  										minimum: 0,
  										maximum: 65535
  									},
  									authentication: {
  										$ref: "#/$defs/referenceableAuthenticationPolicy",
  										title: "WithGRPCServiceAuthentication",
  										description: "The endpoint's authentication policy, if any."
  									}
  								},
  								required: [
  									"name",
  									"host"
  								]
  							},
  							method: {
  								type: "string",
  								title: "WithGRPCMethod",
  								description: "The name of the method to call on the defined GRPC service."
  							},
  							"arguments": {
  								type: "object",
  								title: "WithGRPCArguments",
  								description: "The arguments, if any, to call the method with.",
  								additionalProperties: true
  							}
  						},
  						required: [
  							"proto",
  							"service",
  							"method"
  						],
  						unevaluatedProperties: false
  					}
  				}
  			},
  			{
  				title: "CallHTTP",
  				description: "Defines the HTTP call to perform.",
  				$ref: "#/$defs/taskBase",
  				type: "object",
  				unevaluatedProperties: false,
  				required: [
  					"call",
  					"with"
  				],
  				properties: {
  					call: {
  						type: "string",
  						"const": "http"
  					},
  					"with": {
  						type: "object",
  						title: "HTTPArguments",
  						description: "The HTTP call arguments.",
  						properties: {
  							method: {
  								type: "string",
  								title: "HTTPMethod",
  								description: "The HTTP method of the HTTP request to perform."
  							},
  							endpoint: {
  								title: "HTTPEndpoint",
  								description: "The HTTP endpoint to send the request to.",
  								$ref: "#/$defs/endpoint"
  							},
  							headers: {
  								type: "object",
  								title: "HTTPHeaders",
  								description: "A name/value mapping of the headers, if any, of the HTTP request to perform."
  							},
  							body: {
  								title: "HTTPBody",
  								description: "The body, if any, of the HTTP request to perform."
  							},
  							query: {
  								type: "object",
  								title: "HTTPQuery",
  								description: "A name/value mapping of the query parameters, if any, of the HTTP request to perform.",
  								additionalProperties: true
  							},
  							output: {
  								type: "string",
  								title: "HTTPOutput",
  								description: "The http call output format. Defaults to 'content'.",
  								"enum": [
  									"raw",
  									"content",
  									"response"
  								]
  							},
  							redirect: {
  								type: "boolean",
  								title: "HttpRedirect",
  								description: "Specifies whether redirection status codes (`300–399`) should be treated as errors."
  							}
  						},
  						required: [
  							"method",
  							"endpoint"
  						],
  						unevaluatedProperties: false
  					}
  				}
  			},
  			{
  				title: "CallOpenAPI",
  				description: "Defines the OpenAPI call to perform.",
  				$ref: "#/$defs/taskBase",
  				type: "object",
  				unevaluatedProperties: false,
  				required: [
  					"call",
  					"with"
  				],
  				properties: {
  					call: {
  						type: "string",
  						"const": "openapi"
  					},
  					"with": {
  						type: "object",
  						title: "OpenAPIArguments",
  						description: "The OpenAPI call arguments.",
  						properties: {
  							document: {
  								$ref: "#/$defs/externalResource",
  								title: "WithOpenAPIDocument",
  								description: "The document that defines the OpenAPI operation to call."
  							},
  							operationId: {
  								type: "string",
  								title: "WithOpenAPIOperation",
  								description: "The id of the OpenAPI operation to call."
  							},
  							parameters: {
  								type: "object",
  								title: "WithOpenAPIParameters",
  								description: "A name/value mapping of the parameters of the OpenAPI operation to call.",
  								additionalProperties: true
  							},
  							authentication: {
  								$ref: "#/$defs/referenceableAuthenticationPolicy",
  								title: "WithOpenAPIAuthentication",
  								description: "The authentication policy, if any, to use when calling the OpenAPI operation."
  							},
  							output: {
  								type: "string",
  								"enum": [
  									"raw",
  									"content",
  									"response"
  								],
  								title: "WithOpenAPIOutput",
  								description: "The http call output format. Defaults to 'content'."
  							},
  							redirect: {
  								type: "boolean",
  								title: "HttpRedirect",
  								description: "Specifies whether redirection status codes (`300–399`) should be treated as errors."
  							}
  						},
  						required: [
  							"document",
  							"operationId"
  						],
  						unevaluatedProperties: false
  					}
  				}
  			},
  			{
  				title: "CallFunction",
  				description: "Defines the function call to perform.",
  				$ref: "#/$defs/taskBase",
  				type: "object",
  				unevaluatedProperties: false,
  				required: [
  					"call"
  				],
  				properties: {
  					call: {
  						type: "string",
  						not: {
  							"enum": [
  								"asyncapi",
  								"grpc",
  								"http",
  								"openapi"
  							]
  						},
  						description: "The name of the function to call."
  					},
  					"with": {
  						type: "object",
  						title: "FunctionArguments",
  						description: "A name/value mapping of the parameters, if any, to call the function with.",
  						additionalProperties: true
  					}
  				}
  			}
  		]
  	},
  	forkTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "ForkTask",
  		description: "Allows workflows to execute multiple tasks concurrently and optionally race them against each other, with a single possible winner, which sets the task's output.",
  		unevaluatedProperties: false,
  		required: [
  			"fork"
  		],
  		properties: {
  			fork: {
  				type: "object",
  				title: "ForkTaskConfiguration",
  				description: "The configuration of the branches to perform concurrently.",
  				unevaluatedProperties: false,
  				required: [
  					"branches"
  				],
  				properties: {
  					branches: {
  						$ref: "#/$defs/taskList",
  						title: "ForkBranches"
  					},
  					compete: {
  						type: "boolean",
  						title: "ForkCompete",
  						description: "Indicates whether or not the concurrent tasks are racing against each other, with a single possible winner, which sets the composite task's output.",
  						"default": false
  					}
  				}
  			}
  		}
  	},
  	doTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "DoTask",
  		description: "Allows to execute a list of tasks in sequence.",
  		unevaluatedProperties: false,
  		required: [
  			"do"
  		],
  		properties: {
  			"do": {
  				$ref: "#/$defs/taskList",
  				title: "DoTaskConfiguration",
  				description: "The configuration of the tasks to perform sequentially."
  			}
  		}
  	},
  	emitTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "EmitTask",
  		description: "Allows workflows to publish events to event brokers or messaging systems, facilitating communication and coordination between different components and services.",
  		required: [
  			"emit"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			emit: {
  				type: "object",
  				title: "EmitTaskConfiguration",
  				description: "The configuration of an event's emission.",
  				unevaluatedProperties: false,
  				properties: {
  					event: {
  						type: "object",
  						title: "EmitEventDefinition",
  						description: "The definition of the event to emit.",
  						properties: {
  							"with": {
  								$ref: "#/$defs/eventProperties",
  								title: "EmitEventWith",
  								description: "Defines the properties of event to emit.",
  								required: [
  									"source",
  									"type"
  								]
  							}
  						},
  						additionalProperties: true
  					}
  				},
  				required: [
  					"event"
  				]
  			}
  		}
  	},
  	forTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "ForTask",
  		description: "Allows workflows to iterate over a collection of items, executing a defined set of subtasks for each item in the collection. This task type is instrumental in handling scenarios such as batch processing, data transformation, and repetitive operations across datasets.",
  		required: [
  			"for",
  			"do"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			"for": {
  				type: "object",
  				title: "ForTaskConfiguration",
  				description: "The definition of the loop that iterates over a range of values.",
  				unevaluatedProperties: false,
  				properties: {
  					each: {
  						type: "string",
  						title: "ForEach",
  						description: "The name of the variable used to store the current item being enumerated.",
  						"default": "item"
  					},
  					"in": {
  						type: "string",
  						title: "ForIn",
  						description: "A runtime expression used to get the collection to enumerate."
  					},
  					at: {
  						type: "string",
  						title: "ForAt",
  						description: "The name of the variable used to store the index of the current item being enumerated.",
  						"default": "index"
  					}
  				},
  				required: [
  					"in"
  				]
  			},
  			"while": {
  				type: "string",
  				title: "While",
  				description: "A runtime expression that represents the condition, if any, that must be met for the iteration to continue."
  			},
  			"do": {
  				$ref: "#/$defs/taskList",
  				title: "ForTaskDo"
  			}
  		}
  	},
  	listenTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "ListenTask",
  		description: "Provides a mechanism for workflows to await and react to external events, enabling event-driven behavior within workflow systems.",
  		required: [
  			"listen"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			listen: {
  				type: "object",
  				title: "ListenTaskConfiguration",
  				description: "The configuration of the listener to use.",
  				unevaluatedProperties: false,
  				properties: {
  					to: {
  						$ref: "#/$defs/eventConsumptionStrategy",
  						title: "ListenTo",
  						description: "Defines the event(s) to listen to."
  					},
  					read: {
  						type: "string",
  						"enum": [
  							"data",
  							"envelope",
  							"raw"
  						],
  						"default": "data",
  						title: "ListenAndReadAs",
  						description: "Specifies how events are read during the listen operation."
  					}
  				},
  				required: [
  					"to"
  				]
  			},
  			foreach: {
  				$ref: "#/$defs/subscriptionIterator",
  				title: "ListenIterator",
  				description: "Configures the iterator, if any, for processing consumed event(s)."
  			}
  		}
  	},
  	raiseTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "RaiseTask",
  		description: "Intentionally triggers and propagates errors.",
  		required: [
  			"raise"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			raise: {
  				type: "object",
  				title: "RaiseTaskConfiguration",
  				description: "The definition of the error to raise.",
  				unevaluatedProperties: false,
  				properties: {
  					error: {
  						title: "RaiseTaskError",
  						oneOf: [
  							{
  								$ref: "#/$defs/error",
  								title: "RaiseErrorDefinition",
  								description: "Defines the error to raise."
  							},
  							{
  								type: "string",
  								title: "RaiseErrorReference",
  								description: "The name of the error to raise"
  							}
  						]
  					}
  				},
  				required: [
  					"error"
  				]
  			}
  		}
  	},
  	runTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "RunTask",
  		description: "Provides the capability to execute external containers, shell commands, scripts, or workflows.",
  		required: [
  			"run"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			run: {
  				type: "object",
  				title: "RunTaskConfiguration",
  				description: "The configuration of the process to execute.",
  				unevaluatedProperties: false,
  				properties: {
  					"await": {
  						type: "boolean",
  						"default": true,
  						title: "AwaitProcessCompletion",
  						description: "Whether to await the process completion before continuing."
  					},
  					"return": {
  						type: "string",
  						title: "ProcessReturnType",
  						description: "Configures the output of the process.",
  						"enum": [
  							"stdout",
  							"stderr",
  							"code",
  							"all",
  							"none"
  						],
  						"default": "stdout"
  					}
  				},
  				oneOf: [
  					{
  						title: "RunContainer",
  						description: "Enables the execution of external processes encapsulated within a containerized environment.",
  						properties: {
  							container: {
  								type: "object",
  								title: "Container",
  								description: "The configuration of the container to run.",
  								unevaluatedProperties: false,
  								properties: {
  									image: {
  										type: "string",
  										title: "ContainerImage",
  										description: "The name of the container image to run."
  									},
  									name: {
  										type: "string",
  										title: "ContainerName",
  										description: "A runtime expression, if any, used to give specific name to the container."
  									},
  									command: {
  										type: "string",
  										title: "ContainerCommand",
  										description: "The command, if any, to execute on the container."
  									},
  									ports: {
  										type: "object",
  										title: "ContainerPorts",
  										description: "The container's port mappings, if any."
  									},
  									volumes: {
  										type: "object",
  										title: "ContainerVolumes",
  										description: "The container's volume mappings, if any."
  									},
  									environment: {
  										type: "object",
  										title: "ContainerEnvironment",
  										description: "A key/value mapping of the environment variables, if any, to use when running the configured process."
  									},
  									lifetime: {
  										$ref: "#/$defs/containerLifetime",
  										title: "ContainerLifetime",
  										description: "An object, if any, used to configure the container's lifetime"
  									}
  								},
  								required: [
  									"image"
  								]
  							}
  						},
  						required: [
  							"container"
  						]
  					},
  					{
  						title: "RunScript",
  						description: "Enables the execution of custom scripts or code within a workflow, empowering workflows to perform specialized logic, data processing, or integration tasks by executing user-defined scripts written in various programming languages.",
  						properties: {
  							script: {
  								type: "object",
  								title: "Script",
  								description: "The configuration of the script to run.",
  								unevaluatedProperties: false,
  								properties: {
  									language: {
  										type: "string",
  										title: "ScriptLanguage",
  										description: "The language of the script to run."
  									},
  									"arguments": {
  										type: "object",
  										title: "ScriptArguments",
  										description: "A key/value mapping of the arguments, if any, to use when running the configured script.",
  										additionalProperties: true
  									},
  									environment: {
  										type: "object",
  										title: "ScriptEnvironment",
  										description: "A key/value mapping of the environment variables, if any, to use when running the configured script process.",
  										additionalProperties: true
  									}
  								},
  								oneOf: [
  									{
  										title: "InlineScript",
  										type: "object",
  										description: "The script's code.",
  										properties: {
  											code: {
  												type: "string",
  												title: "InlineScriptCode"
  											}
  										},
  										required: [
  											"code"
  										]
  									},
  									{
  										title: "ExternalScript",
  										type: "object",
  										description: "The script's resource.",
  										properties: {
  											source: {
  												$ref: "#/$defs/externalResource",
  												title: "ExternalScriptResource"
  											}
  										},
  										required: [
  											"source"
  										]
  									}
  								],
  								required: [
  									"language"
  								]
  							}
  						},
  						required: [
  							"script"
  						]
  					},
  					{
  						title: "RunShell",
  						description: "Enables the execution of shell commands within a workflow, enabling workflows to interact with the underlying operating system and perform system-level operations, such as file manipulation, environment configuration, or system administration tasks.",
  						properties: {
  							shell: {
  								type: "object",
  								title: "Shell",
  								description: "The configuration of the shell command to run.",
  								unevaluatedProperties: false,
  								properties: {
  									command: {
  										type: "string",
  										title: "ShellCommand",
  										description: "The shell command to run."
  									},
  									"arguments": {
  										type: "object",
  										title: "ShellArguments",
  										description: "A list of the arguments of the shell command to run.",
  										additionalProperties: true
  									},
  									environment: {
  										type: "object",
  										title: "ShellEnvironment",
  										description: "A key/value mapping of the environment variables, if any, to use when running the configured process.",
  										additionalProperties: true
  									}
  								},
  								required: [
  									"command"
  								]
  							}
  						},
  						required: [
  							"shell"
  						]
  					},
  					{
  						title: "RunWorkflow",
  						description: "Enables the invocation and execution of nested workflows within a parent workflow, facilitating modularization, reusability, and abstraction of complex logic or business processes by encapsulating them into standalone workflow units.",
  						properties: {
  							workflow: {
  								type: "object",
  								title: "SubflowConfiguration",
  								description: "The configuration of the workflow to run.",
  								unevaluatedProperties: false,
  								properties: {
  									namespace: {
  										type: "string",
  										title: "SubflowNamespace",
  										description: "The namespace the workflow to run belongs to."
  									},
  									name: {
  										type: "string",
  										title: "SubflowName",
  										description: "The name of the workflow to run."
  									},
  									version: {
  										type: "string",
  										"default": "latest",
  										title: "SubflowVersion",
  										description: "The version of the workflow to run. Defaults to latest."
  									},
  									input: {
  										type: "object",
  										title: "SubflowInput",
  										description: "The data, if any, to pass as input to the workflow to execute. The value should be validated against the target workflow's input schema, if specified.",
  										additionalProperties: true
  									}
  								},
  								required: [
  									"namespace",
  									"name",
  									"version"
  								]
  							}
  						},
  						required: [
  							"workflow"
  						]
  					}
  				]
  			}
  		}
  	},
  	setTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "SetTask",
  		description: "A task used to set data.",
  		required: [
  			"set"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			set: {
  				type: "object",
  				title: "SetTaskConfiguration",
  				description: "The data to set.",
  				minProperties: 1,
  				additionalProperties: true
  			}
  		}
  	},
  	switchTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "SwitchTask",
  		description: "Enables conditional branching within workflows, allowing them to dynamically select different paths based on specified conditions or criteria.",
  		required: [
  			"switch"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			"switch": {
  				type: "array",
  				title: "SwitchTaskConfiguration",
  				description: "The definition of the switch to use.",
  				minItems: 1,
  				items: {
  					type: "object",
  					title: "SwitchItem",
  					minProperties: 1,
  					maxProperties: 1,
  					additionalProperties: {
  						type: "object",
  						title: "SwitchCase",
  						description: "The definition of a case within a switch task, defining a condition and corresponding tasks to execute if the condition is met.",
  						unevaluatedProperties: false,
  						required: [
  							"then"
  						],
  						properties: {
  							when: {
  								type: "string",
  								title: "SwitchCaseCondition",
  								description: "A runtime expression used to determine whether or not the case matches."
  							},
  							then: {
  								$ref: "#/$defs/flowDirective",
  								title: "SwitchCaseOutcome",
  								description: "The flow directive to execute when the case matches."
  							}
  						}
  					}
  				}
  			}
  		}
  	},
  	tryTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "TryTask",
  		description: "Serves as a mechanism within workflows to handle errors gracefully, potentially retrying failed tasks before proceeding with alternate ones.",
  		required: [
  			"try",
  			"catch"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			"try": {
  				$ref: "#/$defs/taskList",
  				title: "TryTaskConfiguration",
  				description: "The task(s) to perform."
  			},
  			"catch": {
  				type: "object",
  				title: "TryTaskCatch",
  				description: "The object used to define the errors to catch.",
  				unevaluatedProperties: false,
  				properties: {
  					errors: {
  						type: "object",
  						title: "CatchErrors",
  						properties: {
  							"with": {
  								$ref: "#/$defs/errorFilter"
  							}
  						},
  						description: "static error filter"
  					},
  					as: {
  						type: "string",
  						title: "CatchAs",
  						description: "The name of the runtime expression variable to save the error as. Defaults to 'error'."
  					},
  					when: {
  						type: "string",
  						title: "CatchWhen",
  						description: "A runtime expression used to determine whether to catch the filtered error."
  					},
  					exceptWhen: {
  						type: "string",
  						title: "CatchExceptWhen",
  						description: "A runtime expression used to determine whether not to catch the filtered error."
  					},
  					retry: {
  						oneOf: [
  							{
  								$ref: "#/$defs/retryPolicy",
  								title: "RetryPolicyDefinition",
  								description: "The retry policy to use, if any, when catching errors."
  							},
  							{
  								type: "string",
  								title: "RetryPolicyReference",
  								description: "The name of the retry policy to use, if any, when catching errors."
  							}
  						]
  					},
  					"do": {
  						$ref: "#/$defs/taskList",
  						title: "TryTaskCatchDo",
  						description: "The definition of the task(s) to run when catching an error."
  					}
  				}
  			}
  		}
  	},
  	waitTask: {
  		type: "object",
  		$ref: "#/$defs/taskBase",
  		title: "WaitTask",
  		description: "Allows workflows to pause or delay their execution for a specified period of time.",
  		required: [
  			"wait"
  		],
  		unevaluatedProperties: false,
  		properties: {
  			wait: {
  				$ref: "#/$defs/duration",
  				title: "WaitTaskConfiguration",
  				description: "The amount of time to wait."
  			}
  		}
  	},
  	flowDirective: {
  		title: "FlowDirective",
  		description: "Represents different transition options for a workflow.",
  		anyOf: [
  			{
  				title: "FlowDirectiveEnum",
  				type: "string",
  				"enum": [
  					"continue",
  					"exit",
  					"end"
  				],
  				"default": "continue"
  			},
  			{
  				type: "string"
  			}
  		]
  	},
  	referenceableAuthenticationPolicy: {
  		type: "object",
  		title: "ReferenceableAuthenticationPolicy",
  		description: "Represents a referenceable authentication policy.",
  		unevaluatedProperties: false,
  		oneOf: [
  			{
  				title: "AuthenticationPolicyReference",
  				description: "The reference of the authentication policy to use.",
  				properties: {
  					use: {
  						type: "string",
  						minLength: 1,
  						title: "ReferenceableAuthenticationPolicyName",
  						description: "The name of the authentication policy to use."
  					}
  				},
  				required: [
  					"use"
  				]
  			},
  			{
  				$ref: "#/$defs/authenticationPolicy"
  			}
  		]
  	},
  	secretBasedAuthenticationPolicy: {
  		type: "object",
  		title: "SecretBasedAuthenticationPolicy",
  		description: "Represents an authentication policy based on secrets.",
  		unevaluatedProperties: false,
  		properties: {
  			use: {
  				type: "string",
  				minLength: 1,
  				title: "SecretBasedAuthenticationPolicyName",
  				description: "The name of the authentication policy to use."
  			}
  		},
  		required: [
  			"use"
  		]
  	},
  	authenticationPolicy: {
  		type: "object",
  		title: "AuthenticationPolicy",
  		description: "Defines an authentication policy.",
  		oneOf: [
  			{
  				title: "BasicAuthenticationPolicy",
  				description: "Use basic authentication.",
  				properties: {
  					basic: {
  						type: "object",
  						title: "BasicAuthenticationPolicyConfiguration",
  						description: "The configuration of the basic authentication policy.",
  						unevaluatedProperties: false,
  						oneOf: [
  							{
  								title: "BasicAuthenticationProperties",
  								description: "Inline configuration of the basic authentication policy.",
  								properties: {
  									username: {
  										type: "string",
  										description: "The username to use."
  									},
  									password: {
  										type: "string",
  										description: "The password to use."
  									}
  								},
  								required: [
  									"username",
  									"password"
  								]
  							},
  							{
  								$ref: "#/$defs/secretBasedAuthenticationPolicy",
  								title: "BasicAuthenticationPolicySecret",
  								description: "Secret based configuration of the basic authentication policy."
  							}
  						]
  					}
  				},
  				required: [
  					"basic"
  				]
  			},
  			{
  				title: "BearerAuthenticationPolicy",
  				description: "Use bearer authentication.",
  				properties: {
  					bearer: {
  						type: "object",
  						title: "BearerAuthenticationPolicyConfiguration",
  						description: "The configuration of the bearer authentication policy.",
  						unevaluatedProperties: false,
  						oneOf: [
  							{
  								title: "BearerAuthenticationProperties",
  								description: "Inline configuration of the bearer authentication policy.",
  								properties: {
  									token: {
  										type: "string",
  										description: "The bearer token to use."
  									}
  								},
  								required: [
  									"token"
  								]
  							},
  							{
  								$ref: "#/$defs/secretBasedAuthenticationPolicy",
  								title: "BearerAuthenticationPolicySecret",
  								description: "Secret based configuration of the bearer authentication policy."
  							}
  						]
  					}
  				},
  				required: [
  					"bearer"
  				]
  			},
  			{
  				title: "DigestAuthenticationPolicy",
  				description: "Use digest authentication.",
  				properties: {
  					digest: {
  						type: "object",
  						title: "DigestAuthenticationPolicyConfiguration",
  						description: "The configuration of the digest authentication policy.",
  						unevaluatedProperties: false,
  						oneOf: [
  							{
  								title: "DigestAuthenticationProperties",
  								description: "Inline configuration of the digest authentication policy.",
  								properties: {
  									username: {
  										type: "string",
  										description: "The username to use."
  									},
  									password: {
  										type: "string",
  										description: "The password to use."
  									}
  								},
  								required: [
  									"username",
  									"password"
  								]
  							},
  							{
  								$ref: "#/$defs/secretBasedAuthenticationPolicy",
  								title: "DigestAuthenticationPolicySecret",
  								description: "Secret based configuration of the digest authentication policy."
  							}
  						]
  					}
  				},
  				required: [
  					"digest"
  				]
  			},
  			{
  				title: "OAuth2AuthenticationPolicy",
  				description: "Use OAuth2 authentication.",
  				properties: {
  					oauth2: {
  						type: "object",
  						title: "OAuth2AuthenticationPolicyConfiguration",
  						description: "The configuration of the OAuth2 authentication policy.",
  						unevaluatedProperties: false,
  						oneOf: [
  							{
  								type: "object",
  								title: "OAuth2ConnectAuthenticationProperties",
  								description: "The inline configuration of the OAuth2 authentication policy.",
  								unevaluatedProperties: false,
  								allOf: [
  									{
  										$ref: "#/$defs/oauth2AuthenticationProperties"
  									},
  									{
  										type: "object",
  										properties: {
  											endpoints: {
  												type: "object",
  												title: "OAuth2AuthenticationPropertiesEndpoints",
  												description: "The endpoint configurations for OAuth2.",
  												properties: {
  													token: {
  														type: "string",
  														format: "uri-template",
  														"default": "/oauth2/token",
  														title: "OAuth2TokenEndpoint",
  														description: "The relative path to the token endpoint. Defaults to `/oauth2/token`."
  													},
  													revocation: {
  														type: "string",
  														format: "uri-template",
  														"default": "/oauth2/revoke",
  														title: "OAuth2RevocationEndpoint",
  														description: "The relative path to the revocation endpoint. Defaults to `/oauth2/revoke`."
  													},
  													introspection: {
  														type: "string",
  														format: "uri-template",
  														"default": "/oauth2/introspect",
  														title: "OAuth2IntrospectionEndpoint",
  														description: "The relative path to the introspection endpoint. Defaults to `/oauth2/introspect`."
  													}
  												}
  											}
  										}
  									}
  								]
  							},
  							{
  								$ref: "#/$defs/secretBasedAuthenticationPolicy",
  								title: "OAuth2AuthenticationPolicySecret",
  								description: "Secret based configuration of the OAuth2 authentication policy."
  							}
  						]
  					}
  				},
  				required: [
  					"oauth2"
  				]
  			},
  			{
  				title: "OpenIdConnectAuthenticationPolicy",
  				description: "Use OpenIdConnect authentication.",
  				properties: {
  					oidc: {
  						type: "object",
  						title: "OpenIdConnectAuthenticationPolicyConfiguration",
  						description: "The configuration of the OpenIdConnect authentication policy.",
  						unevaluatedProperties: false,
  						oneOf: [
  							{
  								$ref: "#/$defs/oauth2AuthenticationProperties",
  								title: "OpenIdConnectAuthenticationProperties",
  								description: "The inline configuration of the OpenIdConnect authentication policy.",
  								unevaluatedProperties: false
  							},
  							{
  								$ref: "#/$defs/secretBasedAuthenticationPolicy",
  								title: "OpenIdConnectAuthenticationPolicySecret",
  								description: "Secret based configuration of the OpenIdConnect authentication policy."
  							}
  						]
  					}
  				},
  				required: [
  					"oidc"
  				]
  			}
  		]
  	},
  	oauth2AuthenticationProperties: {
  		type: "object",
  		title: "OAuth2AutenthicationData",
  		description: "Inline configuration of the OAuth2 authentication policy.",
  		properties: {
  			authority: {
  				$ref: "#/$defs/uriTemplate",
  				title: "OAuth2AutenthicationDataAuthority",
  				description: "The URI that references the OAuth2 authority to use."
  			},
  			grant: {
  				type: "string",
  				"enum": [
  					"authorization_code",
  					"client_credentials",
  					"password",
  					"refresh_token",
  					"urn:ietf:params:oauth:grant-type:token-exchange"
  				],
  				title: "OAuth2AutenthicationDataGrant",
  				description: "The grant type to use."
  			},
  			client: {
  				type: "object",
  				title: "OAuth2AutenthicationDataClient",
  				description: "The definition of an OAuth2 client.",
  				unevaluatedProperties: false,
  				properties: {
  					id: {
  						type: "string",
  						title: "ClientId",
  						description: "The client id to use."
  					},
  					secret: {
  						type: "string",
  						title: "ClientSecret",
  						description: "The client secret to use, if any."
  					},
  					assertion: {
  						type: "string",
  						title: "ClientAssertion",
  						description: "A JWT containing a signed assertion with your application credentials."
  					},
  					authentication: {
  						type: "string",
  						"enum": [
  							"client_secret_basic",
  							"client_secret_post",
  							"client_secret_jwt",
  							"private_key_jwt",
  							"none"
  						],
  						"default": "client_secret_post",
  						title: "ClientAuthentication",
  						description: "The authentication method to use to authenticate the client."
  					}
  				}
  			},
  			request: {
  				type: "object",
  				title: "OAuth2TokenRequest",
  				description: "The configuration of an OAuth2 token request",
  				properties: {
  					encoding: {
  						type: "string",
  						"enum": [
  							"application/x-www-form-urlencoded",
  							"application/json"
  						],
  						"default": "application/x-www-form-urlencoded",
  						title: "Oauth2TokenRequestEncoding"
  					}
  				}
  			},
  			issuers: {
  				type: "array",
  				title: "OAuth2Issuers",
  				description: "A list that contains that contains valid issuers that will be used to check against the issuer of generated tokens.",
  				items: {
  					type: "string"
  				}
  			},
  			scopes: {
  				type: "array",
  				title: "OAuth2AutenthicationDataScopes",
  				description: "The scopes, if any, to request the token for.",
  				items: {
  					type: "string"
  				}
  			},
  			audiences: {
  				type: "array",
  				title: "OAuth2AutenthicationDataAudiences",
  				description: "The audiences, if any, to request the token for.",
  				items: {
  					type: "string"
  				}
  			},
  			username: {
  				type: "string",
  				title: "OAuth2AutenthicationDataUsername",
  				description: "The username to use. Used only if the grant type is Password."
  			},
  			password: {
  				type: "string",
  				title: "OAuth2AutenthicationDataPassword",
  				description: "The password to use. Used only if the grant type is Password."
  			},
  			subject: {
  				$ref: "#/$defs/oauth2Token",
  				title: "OAuth2AutenthicationDataSubject",
  				description: "The security token that represents the identity of the party on behalf of whom the request is being made."
  			},
  			actor: {
  				$ref: "#/$defs/oauth2Token",
  				title: "OAuth2AutenthicationDataActor",
  				description: "The security token that represents the identity of the acting party."
  			}
  		}
  	},
  	oauth2Token: {
  		type: "object",
  		title: "OAuth2TokenDefinition",
  		description: "Represents an OAuth2 token.",
  		unevaluatedProperties: false,
  		properties: {
  			token: {
  				type: "string",
  				title: "OAuth2Token",
  				description: "The security token to use."
  			},
  			type: {
  				type: "string",
  				title: "OAuth2TokenType",
  				description: "The type of the security token to use."
  			}
  		},
  		required: [
  			"token",
  			"type"
  		]
  	},
  	duration: {
  		oneOf: [
  			{
  				type: "object",
  				minProperties: 1,
  				unevaluatedProperties: false,
  				properties: {
  					days: {
  						type: "integer",
  						title: "DurationDays",
  						description: "Number of days, if any."
  					},
  					hours: {
  						type: "integer",
  						title: "DurationHours",
  						description: "Number of days, if any."
  					},
  					minutes: {
  						type: "integer",
  						title: "DurationMinutes",
  						description: "Number of minutes, if any."
  					},
  					seconds: {
  						type: "integer",
  						title: "DurationSeconds",
  						description: "Number of seconds, if any."
  					},
  					milliseconds: {
  						type: "integer",
  						title: "DurationMilliseconds",
  						description: "Number of milliseconds, if any."
  					}
  				},
  				title: "DurationInline",
  				description: "The inline definition of a duration."
  			},
  			{
  				type: "string",
  				pattern: "^P(?!$)(\\d+(?:\\.\\d+)?Y)?(\\d+(?:\\.\\d+)?M)?(\\d+(?:\\.\\d+)?W)?(\\d+(?:\\.\\d+)?D)?(T(?=\\d)(\\d+(?:\\.\\d+)?H)?(\\d+(?:\\.\\d+)?M)?(\\d+(?:\\.\\d+)?S)?)?$",
  				title: "DurationExpression",
  				description: "The ISO 8601 expression of a duration."
  			}
  		]
  	},
  	error: {
  		type: "object",
  		title: "Error",
  		description: "Represents an error.",
  		unevaluatedProperties: false,
  		properties: {
  			type: {
  				title: "ErrorType",
  				description: "A URI reference that identifies the error type.",
  				oneOf: [
  					{
  						title: "LiteralErrorType",
  						$ref: "#/$defs/uriTemplate",
  						description: "The literal error type."
  					},
  					{
  						title: "ExpressionErrorType",
  						$ref: "#/$defs/runtimeExpression",
  						description: "An expression based error type."
  					}
  				]
  			},
  			status: {
  				type: "integer",
  				title: "ErrorStatus",
  				description: "The status code generated by the origin for this occurrence of the error."
  			},
  			instance: {
  				title: "ErrorInstance",
  				description: "A JSON Pointer used to reference the component the error originates from.",
  				oneOf: [
  					{
  						title: "LiteralErrorInstance",
  						description: "The literal error instance.",
  						type: "string",
  						format: "json-pointer"
  					},
  					{
  						$ref: "#/$defs/runtimeExpression",
  						title: "ExpressionErrorInstance",
  						description: "An expression based error instance."
  					}
  				]
  			},
  			title: {
  				type: "string",
  				title: "ErrorTitle",
  				description: "A short, human-readable summary of the error."
  			},
  			detail: {
  				type: "string",
  				title: "ErrorDetails",
  				description: "A human-readable explanation specific to this occurrence of the error."
  			}
  		},
  		required: [
  			"type",
  			"status"
  		]
  	},
  	errorFilter: {
  		type: "object",
  		title: "ErrorFilter",
  		description: "Error filtering base on static values. For error filtering on dynamic values, use catch.when property",
  		minProperties: 1,
  		properties: {
  			type: {
  				type: "string",
  				description: "if present, means this value should be used for filtering"
  			},
  			status: {
  				type: "integer",
  				description: "if present, means this value should be used for filtering"
  			},
  			instance: {
  				type: "string",
  				description: "if present, means this value should be used for filtering"
  			},
  			title: {
  				type: "string",
  				description: "if present, means this value should be used for filtering"
  			},
  			details: {
  				type: "string",
  				description: "if present, means this value should be used for filtering"
  			}
  		}
  	},
  	uriTemplate: {
  		title: "UriTemplate",
  		anyOf: [
  			{
  				title: "LiteralUriTemplate",
  				type: "string",
  				format: "uri-template",
  				pattern: "^[A-Za-z][A-Za-z0-9+\\-.]*://.*"
  			},
  			{
  				title: "LiteralUri",
  				type: "string",
  				format: "uri",
  				pattern: "^[A-Za-z][A-Za-z0-9+\\-.]*://.*"
  			}
  		]
  	},
  	endpoint: {
  		title: "Endpoint",
  		description: "Represents an endpoint.",
  		oneOf: [
  			{
  				$ref: "#/$defs/runtimeExpression"
  			},
  			{
  				$ref: "#/$defs/uriTemplate"
  			},
  			{
  				title: "EndpointConfiguration",
  				type: "object",
  				unevaluatedProperties: false,
  				properties: {
  					uri: {
  						title: "EndpointUri",
  						description: "The endpoint's URI.",
  						oneOf: [
  							{
  								title: "LiteralEndpointURI",
  								description: "The literal endpoint's URI.",
  								$ref: "#/$defs/uriTemplate"
  							},
  							{
  								title: "ExpressionEndpointURI",
  								$ref: "#/$defs/runtimeExpression",
  								description: "An expression based endpoint's URI."
  							}
  						]
  					},
  					authentication: {
  						$ref: "#/$defs/referenceableAuthenticationPolicy",
  						title: "EndpointAuthentication",
  						description: "The authentication policy to use."
  					}
  				},
  				required: [
  					"uri"
  				]
  			}
  		]
  	},
  	eventProperties: {
  		type: "object",
  		title: "EventProperties",
  		description: "Describes the properties of an event.",
  		properties: {
  			id: {
  				type: "string",
  				title: "EventId",
  				description: "The event's unique identifier."
  			},
  			source: {
  				title: "EventSource",
  				description: "Identifies the context in which an event happened.",
  				oneOf: [
  					{
  						$ref: "#/$defs/uriTemplate"
  					},
  					{
  						$ref: "#/$defs/runtimeExpression"
  					}
  				]
  			},
  			type: {
  				type: "string",
  				title: "EventType",
  				description: "This attribute contains a value describing the type of event related to the originating occurrence."
  			},
  			time: {
  				title: "EventTime",
  				description: "When the event occured.",
  				oneOf: [
  					{
  						title: "LiteralTime",
  						type: "string",
  						format: "date-time"
  					},
  					{
  						$ref: "#/$defs/runtimeExpression"
  					}
  				]
  			},
  			subject: {
  				type: "string",
  				title: "EventSubject",
  				description: "The subject of the event."
  			},
  			datacontenttype: {
  				type: "string",
  				title: "EventDataContentType",
  				description: "Content type of data value. This attribute enables data to carry any type of content, whereby format and encoding might differ from that of the chosen event format."
  			},
  			dataschema: {
  				title: "EventDataschema",
  				description: "The schema describing the event format.",
  				oneOf: [
  					{
  						title: "LiteralDataSchema",
  						$ref: "#/$defs/uriTemplate",
  						description: "The literal event data schema."
  					},
  					{
  						title: "ExpressionDataSchema",
  						$ref: "#/$defs/runtimeExpression",
  						description: "An expression based event data schema."
  					}
  				]
  			},
  			data: {
  				title: "EventData",
  				description: "The event's payload data",
  				anyOf: [
  					{
  						$ref: "#/$defs/runtimeExpression"
  					},
  					{
  					}
  				]
  			}
  		},
  		additionalProperties: true
  	},
  	eventConsumptionStrategy: {
  		type: "object",
  		title: "EventConsumptionStrategy",
  		description: "Describe the event consumption strategy to adopt.",
  		unevaluatedProperties: false,
  		oneOf: [
  			{
  				title: "AllEventConsumptionStrategy",
  				properties: {
  					all: {
  						type: "array",
  						title: "AllEventConsumptionStrategyConfiguration",
  						description: "A list containing all the events that must be consumed.",
  						items: {
  							$ref: "#/$defs/eventFilter"
  						}
  					}
  				},
  				required: [
  					"all"
  				]
  			},
  			{
  				title: "AnyEventConsumptionStrategy",
  				properties: {
  					any: {
  						type: "array",
  						title: "AnyEventConsumptionStrategyConfiguration",
  						description: "A list containing any of the events to consume.",
  						items: {
  							$ref: "#/$defs/eventFilter"
  						}
  					},
  					until: {
  						oneOf: [
  							{
  								type: "string",
  								title: "AnyEventUntilCondition",
  								description: "A runtime expression condition evaluated after consuming an event and which determines whether or not to continue listening."
  							},
  							{
  								allOf: [
  									{
  										$ref: "#/$defs/eventConsumptionStrategy",
  										description: "The strategy that defines the event(s) to consume to stop listening."
  									},
  									{
  										properties: {
  											until: false
  										}
  									}
  								],
  								title: "AnyEventUntilConsumed"
  							}
  						]
  					}
  				},
  				required: [
  					"any"
  				]
  			},
  			{
  				title: "OneEventConsumptionStrategy",
  				properties: {
  					one: {
  						$ref: "#/$defs/eventFilter",
  						title: "OneEventConsumptionStrategyConfiguration",
  						description: "The single event to consume."
  					}
  				},
  				required: [
  					"one"
  				]
  			}
  		]
  	},
  	eventFilter: {
  		type: "object",
  		title: "EventFilter",
  		description: "An event filter is a mechanism used to selectively process or handle events based on predefined criteria, such as event type, source, or specific attributes.",
  		unevaluatedProperties: false,
  		properties: {
  			"with": {
  				$ref: "#/$defs/eventProperties",
  				minProperties: 1,
  				title: "WithEvent",
  				description: "An event filter is a mechanism used to selectively process or handle events based on predefined criteria, such as event type, source, or specific attributes."
  			},
  			correlate: {
  				type: "object",
  				title: "EventFilterCorrelate",
  				description: "A correlation is a link between events and data, established by mapping event attributes to specific data attributes, allowing for coordinated processing or handling based on event characteristics.",
  				additionalProperties: {
  					type: "object",
  					properties: {
  						from: {
  							type: "string",
  							title: "CorrelateFrom",
  							description: "A runtime expression used to extract the correlation value from the filtered event."
  						},
  						expect: {
  							type: "string",
  							title: "CorrelateExpect",
  							description: "A constant or a runtime expression, if any, used to determine whether or not the extracted correlation value matches expectations. If not set, the first extracted value will be used as the correlation's expectation."
  						}
  					},
  					required: [
  						"from"
  					]
  				}
  			}
  		},
  		required: [
  			"with"
  		]
  	},
  	extension: {
  		type: "object",
  		title: "Extension",
  		description: "The definition of an extension.",
  		unevaluatedProperties: false,
  		properties: {
  			extend: {
  				type: "string",
  				"enum": [
  					"call",
  					"composite",
  					"emit",
  					"for",
  					"listen",
  					"raise",
  					"run",
  					"set",
  					"switch",
  					"try",
  					"wait",
  					"all"
  				],
  				title: "ExtensionTarget",
  				description: "The type of task to extend."
  			},
  			when: {
  				type: "string",
  				title: "ExtensionCondition",
  				description: "A runtime expression, if any, used to determine whether or not the extension should apply in the specified context."
  			},
  			before: {
  				$ref: "#/$defs/taskList",
  				title: "ExtensionDoBefore",
  				description: "The task(s) to execute before the extended task, if any."
  			},
  			after: {
  				$ref: "#/$defs/taskList",
  				title: "ExtensionDoAfter",
  				description: "The task(s) to execute after the extended task, if any."
  			}
  		},
  		required: [
  			"extend"
  		]
  	},
  	externalResource: {
  		type: "object",
  		title: "ExternalResource",
  		description: "Represents an external resource.",
  		unevaluatedProperties: false,
  		properties: {
  			name: {
  				type: "string",
  				title: "ExternalResourceName",
  				description: "The name of the external resource, if any."
  			},
  			endpoint: {
  				$ref: "#/$defs/endpoint",
  				title: "ExternalResourceEndpoint",
  				description: "The endpoint of the external resource."
  			}
  		},
  		required: [
  			"endpoint"
  		]
  	},
  	input: {
  		type: "object",
  		title: "Input",
  		description: "Configures the input of a workflow or task.",
  		unevaluatedProperties: false,
  		properties: {
  			schema: {
  				$ref: "#/$defs/schema",
  				title: "InputSchema",
  				description: "The schema used to describe and validate the input of the workflow or task."
  			},
  			from: {
  				title: "InputFrom",
  				description: "A runtime expression, if any, used to mutate and/or filter the input of the workflow or task.",
  				oneOf: [
  					{
  						type: "string"
  					},
  					{
  						type: "object"
  					}
  				]
  			}
  		}
  	},
  	output: {
  		type: "object",
  		title: "Output",
  		description: "Configures the output of a workflow or task.",
  		unevaluatedProperties: false,
  		properties: {
  			schema: {
  				$ref: "#/$defs/schema",
  				title: "OutputSchema",
  				description: "The schema used to describe and validate the output of the workflow or task."
  			},
  			as: {
  				title: "OutputAs",
  				description: "A runtime expression, if any, used to mutate and/or filter the output of the workflow or task.",
  				oneOf: [
  					{
  						type: "string"
  					},
  					{
  						type: "object"
  					}
  				]
  			}
  		}
  	},
  	"export": {
  		type: "object",
  		title: "Export",
  		description: "Set the content of the context. .",
  		unevaluatedProperties: false,
  		properties: {
  			schema: {
  				$ref: "#/$defs/schema",
  				title: "ExportSchema",
  				description: "The schema used to describe and validate the workflow context."
  			},
  			as: {
  				title: "ExportAs",
  				description: "A runtime expression, if any, used to export the output data to the context.",
  				oneOf: [
  					{
  						type: "string"
  					},
  					{
  						type: "object"
  					}
  				]
  			}
  		}
  	},
  	retryPolicy: {
  		type: "object",
  		title: "RetryPolicy",
  		description: "Defines a retry policy.",
  		unevaluatedProperties: false,
  		properties: {
  			when: {
  				type: "string",
  				title: "RetryWhen",
  				description: "A runtime expression, if any, used to determine whether or not to retry running the task, in a given context."
  			},
  			exceptWhen: {
  				type: "string",
  				title: "RetryExcepWhen",
  				description: "A runtime expression used to determine whether or not to retry running the task, in a given context."
  			},
  			delay: {
  				$ref: "#/$defs/duration",
  				title: "RetryDelay",
  				description: "The duration to wait between retry attempts."
  			},
  			backoff: {
  				type: "object",
  				title: "RetryBackoff",
  				description: "The retry duration backoff.",
  				unevaluatedProperties: false,
  				oneOf: [
  					{
  						title: "ConstantBackoff",
  						properties: {
  							constant: {
  								type: "object",
  								description: "The definition of the constant backoff to use, if any."
  							}
  						},
  						required: [
  							"constant"
  						]
  					},
  					{
  						title: "ExponentialBackOff",
  						properties: {
  							exponential: {
  								type: "object",
  								description: "The definition of the exponential backoff to use, if any."
  							}
  						},
  						required: [
  							"exponential"
  						]
  					},
  					{
  						title: "LinearBackoff",
  						properties: {
  							linear: {
  								type: "object",
  								description: "The definition of the linear backoff to use, if any."
  							}
  						},
  						required: [
  							"linear"
  						]
  					}
  				]
  			},
  			limit: {
  				type: "object",
  				title: "RetryLimit",
  				unevaluatedProperties: false,
  				properties: {
  					attempt: {
  						type: "object",
  						title: "RetryLimitAttempt",
  						unevaluatedProperties: false,
  						properties: {
  							count: {
  								type: "integer",
  								title: "RetryLimitAttemptCount",
  								description: "The maximum amount of retry attempts, if any."
  							},
  							duration: {
  								$ref: "#/$defs/duration",
  								title: "RetryLimitAttemptDuration",
  								description: "The maximum duration for each retry attempt."
  							}
  						}
  					},
  					duration: {
  						$ref: "#/$defs/duration",
  						title: "RetryLimitDuration",
  						description: "The duration limit, if any, for all retry attempts."
  					}
  				},
  				description: "The retry limit, if any."
  			},
  			jitter: {
  				type: "object",
  				title: "RetryPolicyJitter",
  				description: "The parameters, if any, that control the randomness or variability of the delay between retry attempts.",
  				unevaluatedProperties: false,
  				properties: {
  					from: {
  						$ref: "#/$defs/duration",
  						title: "RetryPolicyJitterFrom",
  						description: "The minimum duration of the jitter range."
  					},
  					to: {
  						$ref: "#/$defs/duration",
  						title: "RetryPolicyJitterTo",
  						description: "The maximum duration of the jitter range."
  					}
  				},
  				required: [
  					"from",
  					"to"
  				]
  			}
  		}
  	},
  	schema: {
  		type: "object",
  		title: "Schema",
  		description: "Represents the definition of a schema.",
  		unevaluatedProperties: false,
  		properties: {
  			format: {
  				type: "string",
  				"default": "json",
  				title: "SchemaFormat",
  				description: "The schema's format. Defaults to 'json'. The (optional) version of the format can be set using `{format}:{version}`."
  			}
  		},
  		oneOf: [
  			{
  				title: "SchemaInline",
  				properties: {
  					document: {
  						description: "The schema's inline definition."
  					}
  				},
  				required: [
  					"document"
  				]
  			},
  			{
  				title: "SchemaExternal",
  				properties: {
  					resource: {
  						$ref: "#/$defs/externalResource",
  						title: "SchemaExternalResource",
  						description: "The schema's external resource."
  					}
  				},
  				required: [
  					"resource"
  				]
  			}
  		]
  	},
  	timeout: {
  		type: "object",
  		title: "Timeout",
  		description: "The definition of a timeout.",
  		unevaluatedProperties: false,
  		properties: {
  			after: {
  				$ref: "#/$defs/duration",
  				title: "TimeoutAfter",
  				description: "The duration after which to timeout."
  			}
  		},
  		required: [
  			"after"
  		]
  	},
  	catalog: {
  		type: "object",
  		title: "Catalog",
  		description: "The definition of a resource catalog.",
  		unevaluatedProperties: false,
  		properties: {
  			endpoint: {
  				$ref: "#/$defs/endpoint",
  				title: "CatalogEndpoint",
  				description: "The root URL where the catalog is hosted."
  			}
  		},
  		required: [
  			"endpoint"
  		]
  	},
  	runtimeExpression: {
  		type: "string",
  		title: "RuntimeExpression",
  		description: "A runtime expression.",
  		pattern: "^\\s*\\$\\{.+\\}\\s*$"
  	},
  	containerLifetime: {
  		type: "object",
  		title: "ContainerLifetime",
  		description: "The configuration of a container's lifetime",
  		unevaluatedProperties: false,
  		properties: {
  			cleanup: {
  				type: "string",
  				title: "ContainerCleanupPolicy",
  				description: "The container cleanup policy to use",
  				"enum": [
  					"always",
  					"never",
  					"eventually"
  				],
  				"default": "never"
  			},
  			after: {
  				$ref: "#/$defs/duration",
  				title: "ContainerLifetimeDuration",
  				description: "The duration after which to cleanup the container, in case the cleanup policy has been set to 'eventually'"
  			}
  		},
  		required: [
  			"cleanup"
  		],
  		"if": {
  			properties: {
  				cleanup: {
  					"const": "eventually"
  				}
  			}
  		},
  		then: {
  			required: [
  				"after"
  			]
  		},
  		"else": {
  			not: {
  				required: [
  					"after"
  				]
  			}
  		}
  	},
  	processResult: {
  		type: "object",
  		title: "ProcessResult",
  		description: "The object returned by a run task when its return type has been set 'all'.",
  		unevaluatedProperties: false,
  		properties: {
  			code: {
  				type: "integer",
  				title: "ProcessExitCode",
  				description: "The process's exit code."
  			},
  			stdout: {
  				type: "string",
  				title: "ProcessStandardOutput",
  				description: "The content of the process's STDOUT."
  			},
  			stderr: {
  				type: "string",
  				title: "ProcessStandardError",
  				description: "The content of the process's STDERR."
  			}
  		},
  		required: [
  			"code",
  			"stdout",
  			"stderr"
  		]
  	},
  	asyncApiServer: {
  		type: "object",
  		title: "AsyncApiServer",
  		description: "Configures the target server of an AsyncAPI operation.",
  		unevaluatedProperties: false,
  		properties: {
  			name: {
  				type: "string",
  				title: "AsyncApiServerName",
  				description: "The target server's name."
  			},
  			variables: {
  				type: "object",
  				title: "AsyncApiServerVariables",
  				description: "The target server's variables, if any."
  			}
  		},
  		required: [
  			"name"
  		]
  	},
  	asyncApiOutboundMessage: {
  		type: "object",
  		title: "AsyncApiOutboundMessage",
  		description: "An object used to configure the message to publish using the target operation.",
  		unevaluatedProperties: false,
  		properties: {
  			payload: {
  				type: "object",
  				title: "AsyncApiMessagePayload",
  				description: "The message's payload, if any.",
  				additionalProperties: true
  			},
  			headers: {
  				type: "object",
  				title: "AsyncApiMessageHeaders",
  				description: "The message's headers, if any.",
  				additionalProperties: true
  			}
  		}
  	},
  	asyncApiInboundMessage: {
  		type: "object",
  		title: "AsyncApiInboundMessage",
  		description: "Represents a message counsumed by an AsyncAPI subscription.",
  		allOf: [
  			{
  				$ref: "#/$defs/asyncApiOutboundMessage"
  			}
  		],
  		properties: {
  			correlationId: {
  				type: "string",
  				title: "AsyncApiMessageCorrelationId",
  				description: "The message's correlation id, if any."
  			}
  		}
  	},
  	asyncApiSubscription: {
  		type: "object",
  		title: "AsyncApiSubscription",
  		description: "An object used to configure the subscription to messages consumed using the target operation.",
  		unevaluatedProperties: false,
  		properties: {
  			filter: {
  				$ref: "#/$defs/runtimeExpression",
  				title: "AsyncApiSubscriptionCorrelation",
  				description: "A runtime expression, if any, used to filter consumed messages."
  			},
  			consume: {
  				$ref: "#/$defs/asyncApiMessageConsumptionPolicy",
  				title: "AsyncApiMessageConsumptionPolicy",
  				description: "An object used to configure the subscription's message consumption policy."
  			},
  			foreach: {
  				$ref: "#/$defs/subscriptionIterator",
  				title: "AsyncApiSubscriptionIterator",
  				description: "Configures the iterator, if any, for processing consumed messages(s)."
  			}
  		},
  		required: [
  			"consume"
  		]
  	},
  	asyncApiMessageConsumptionPolicy: {
  		type: "object",
  		title: "AsyncApiMessageConsumptionPolicy",
  		description: "An object used to configure a subscription's message consumption policy.",
  		unevaluatedProperties: false,
  		properties: {
  			"for": {
  				$ref: "#/$defs/duration",
  				title: "AsyncApiMessageConsumptionPolicyFor",
  				description: "Specifies the time period over which messages will be consumed."
  			}
  		},
  		oneOf: [
  			{
  				properties: {
  					amount: {
  						type: "integer",
  						description: "The amount of (filtered) messages to consume before disposing of the subscription."
  					}
  				},
  				title: "AsyncApiMessageConsumptionPolicyAmount",
  				required: [
  					"amount"
  				]
  			},
  			{
  				properties: {
  					"while": {
  						$ref: "#/$defs/runtimeExpression",
  						description: "A runtime expression evaluated after each consumed (filtered) message to decide if message consumption should continue."
  					}
  				},
  				title: "AsyncApiMessageConsumptionPolicyWhile",
  				required: [
  					"while"
  				]
  			},
  			{
  				properties: {
  					until: {
  						$ref: "#/$defs/runtimeExpression",
  						description: "A runtime expression evaluated before each consumed (filtered) message to decide if message consumption should continue."
  					}
  				},
  				title: "AsyncApiMessageConsumptionPolicyUntil",
  				required: [
  					"until"
  				]
  			}
  		]
  	},
  	subscriptionIterator: {
  		type: "object",
  		title: "SubscriptionIterator",
  		description: "Configures the iteration over each item (event or message) consumed by a subscription.",
  		unevaluatedProperties: false,
  		properties: {
  			item: {
  				type: "string",
  				title: "SubscriptionIteratorItem",
  				description: "The name of the variable used to store the current item being enumerated.",
  				"default": "item"
  			},
  			at: {
  				type: "string",
  				title: "SubscriptionIteratorIndex",
  				description: "The name of the variable used to store the index of the current item being enumerated.",
  				"default": "index"
  			},
  			"do": {
  				$ref: "#/$defs/taskList",
  				title: "SubscriptionIteratorTasks",
  				description: "The tasks to perform for each consumed item."
  			},
  			output: {
  				$ref: "#/$defs/output",
  				title: "SubscriptionIteratorOutput",
  				description: "An object, if any, used to customize the item's output and to document its schema."
  			},
  			"export": {
  				$ref: "#/$defs/export",
  				title: "SubscriptionIteratorExport",
  				description: "An object, if any, used to customize the content of the workflow context."
  			}
  		}
  	}
  };
  var workflowSchema = {
  	$id: $id,
  	$schema: $schema,
  	description: description,
  	type: type$1,
  	required: required,
  	properties: properties,
  	$defs: $defs
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /*****************************************************************************************
   *
   * /!\ This file is computer generated. Any manual modification can and will be lost. /!\
   *
   *****************************************************************************************/
  /**
   * A map of type names and their corresponding schema
   */
  var validationPointers = {
      Workflow: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#',
      AllEventConsumptionStrategy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/0',
      AllEventConsumptionStrategyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/0/properties/all',
      AnyEventConsumptionStrategy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/1',
      AnyEventConsumptionStrategyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/1/properties/any',
      AnyEventConsumptionStrategyUntil: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/1/properties/until',
      AnyEventUntilConsumed: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/1/properties/until/oneOf/1',
      AsyncApiArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/0/properties/with',
      AuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy',
      AuthenticationPolicyReference: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/referenceableAuthenticationPolicy/oneOf/0',
      BasicAuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/0',
      BasicAuthenticationPolicyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/0/properties/basic',
      BasicAuthenticationProperties: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/0/properties/basic/oneOf/0',
      BearerAuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/1',
      BearerAuthenticationPolicyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/1/properties/bearer',
      BearerAuthenticationProperties: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/1/properties/bearer/oneOf/0',
      CallAsyncAPI: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/0',
      CallFunction: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/4',
      CallGRPC: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/1',
      CallHTTP: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/2',
      CallOpenAPI: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/3',
      CallTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask',
      Catalog: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/catalog',
      CatchErrors: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/tryTask/properties/catch/properties/errors',
      ConstantBackoff: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/backoff/oneOf/0',
      Container: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/0/properties/container',
      ContainerEnvironment: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/0/properties/container/properties/environment',
      ContainerLifetime: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/0/properties/container/properties/lifetime',
      ContainerPorts: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/0/properties/container/properties/ports',
      ContainerVolumes: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/0/properties/container/properties/volumes',
      DigestAuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/2',
      DigestAuthenticationPolicyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/2/properties/digest',
      DigestAuthenticationProperties: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/2/properties/digest/oneOf/0',
      Document: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/document',
      DoTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/doTask',
      Duration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/duration',
      DurationInline: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/duration/oneOf/0',
      EmitEventDefinition: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/emitTask/properties/emit/properties/event',
      EmitEventWith: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/emitTask/properties/emit/properties/event/properties/with',
      EmitTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/emitTask',
      EmitTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/emitTask/properties/emit',
      Endpoint: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/endpoint',
      EndpointConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/endpoint/oneOf/2',
      EndpointUri: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/endpoint/oneOf/2/properties/uri',
      Error: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/error',
      ErrorFilter: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/errorFilter',
      ErrorInstance: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/error/properties/instance',
      ErrorType: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/error/properties/type',
      EventConsumptionStrategy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy',
      EventData: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventProperties/properties/data',
      EventDataschema: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventProperties/properties/dataschema',
      EventFilter: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventFilter',
      EventFilterCorrelate: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventFilter/properties/correlate',
      EventSource: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventProperties/properties/source',
      EventTime: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventProperties/properties/time',
      ExponentialBackOff: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/backoff/oneOf/1',
      Export: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/export',
      ExportAs: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/export/properties/as',
      Extension: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/extension',
      ExtensionItem: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/extensions/items',
      ExternalResource: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/externalResource',
      ExternalScript: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/1/properties/script/oneOf/1',
      FlowDirective: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/flowDirective',
      ForkTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/forkTask',
      ForkTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/forkTask/properties/fork',
      ForTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/forTask',
      ForTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/forTask/properties/for',
      FunctionArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/4/properties/with',
      GRPCArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/1/properties/with',
      HTTPArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/2/properties/with',
      HTTPBody: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/2/properties/with/properties/body',
      HTTPHeaders: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/2/properties/with/properties/headers',
      HTTPQuery: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/2/properties/with/properties/query',
      InlineScript: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/1/properties/script/oneOf/0',
      Input: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/input',
      InputFrom: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/input/properties/from',
      LinearBackoff: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/backoff/oneOf/2',
      ListenTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/listenTask',
      ListenTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/listenTask/properties/listen',
      OAuth2AutenthicationData: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2AuthenticationProperties',
      OAuth2AutenthicationDataAudiences: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2AuthenticationProperties/properties/audiences',
      OAuth2AutenthicationDataClient: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2AuthenticationProperties/properties/client',
      OAuth2AutenthicationDataScopes: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2AuthenticationProperties/properties/scopes',
      OAuth2AuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/3',
      OAuth2AuthenticationPolicyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/3/properties/oauth2',
      OAuth2AuthenticationPropertiesEndpoints: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/3/properties/oauth2/oneOf/0/allOf/1/properties/endpoints',
      OAuth2ConnectAuthenticationProperties: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/3/properties/oauth2/oneOf/0',
      OAuth2Issuers: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2AuthenticationProperties/properties/issuers',
      OAuth2TokenDefinition: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2Token',
      OAuth2TokenRequest: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/oauth2AuthenticationProperties/properties/request',
      OneEventConsumptionStrategy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventConsumptionStrategy/oneOf/2',
      OpenAPIArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/3/properties/with',
      OpenIdConnectAuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/4',
      OpenIdConnectAuthenticationPolicyConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/4/properties/oidc',
      OpenIdConnectAuthenticationProperties: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/authenticationPolicy/oneOf/4/properties/oidc/oneOf/0',
      Output: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/output',
      OutputAs: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/output/properties/as',
      RaiseTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/raiseTask',
      RaiseTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/raiseTask/properties/raise',
      RaiseTaskError: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/raiseTask/properties/raise/properties/error',
      ReferenceableAuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/referenceableAuthenticationPolicy',
      RetryBackoff: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/backoff',
      RetryLimit: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/limit',
      RetryLimitAttempt: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/limit/properties/attempt',
      RetryPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy',
      RetryPolicyJitter: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/retryPolicy/properties/jitter',
      RunContainer: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/0',
      RunScript: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/1',
      RunShell: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/2',
      RunTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask',
      RunTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run',
      RuntimeExpression: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runtimeExpression',
      RunWorkflow: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/3',
      Schedule: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/schedule',
      Schema: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/schema',
      SchemaExternal: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/schema/oneOf/1',
      SchemaInline: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/schema/oneOf/0',
      Script: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/1/properties/script',
      SecretBasedAuthenticationPolicy: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/secretBasedAuthenticationPolicy',
      SetTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/setTask',
      SetTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/setTask/properties/set',
      Shell: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/2/properties/shell',
      ShellArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/2/properties/shell/properties/arguments',
      ShellEnvironment: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/2/properties/shell/properties/environment',
      SubflowConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/3/properties/workflow',
      SubflowInput: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/runTask/properties/run/oneOf/3/properties/workflow/properties/input',
      SubscriptionIterator: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/subscriptionIterator',
      SwitchCase: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/switchTask/properties/switch/items/additionalProperties',
      SwitchItem: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/switchTask/properties/switch/items',
      SwitchTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/switchTask',
      SwitchTaskConfiguration: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/switchTask/properties/switch',
      Task: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/task',
      TaskBase: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/taskBase',
      TaskBaseIf: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/taskBase/properties/if',
      TaskItem: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/taskList/items',
      TaskList: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/taskList',
      TaskMetadata: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/taskBase/properties/metadata',
      TaskTimeout: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/taskBase/properties/timeout',
      Timeout: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/timeout',
      TryTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/tryTask',
      TryTaskCatch: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/tryTask/properties/catch',
      TryTaskCatchRetry: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/tryTask/properties/catch/properties/retry',
      UriTemplate: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/uriTemplate',
      Use: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use',
      UseAuthentications: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/authentications',
      UseCatalogs: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/catalogs',
      UseErrors: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/errors',
      UseExtensions: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/extensions',
      UseFunctions: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/functions',
      UseRetries: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/retries',
      UseSecrets: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/secrets',
      UseTimeouts: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/use/properties/timeouts',
      WaitTask: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/waitTask',
      WithEvent: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/eventFilter/properties/with',
      WithGRPCArguments: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/1/properties/with/properties/arguments',
      WithGRPCService: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/1/properties/with/properties/service',
      WithOpenAPIParameters: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/$defs/callTask/oneOf/3/properties/with/properties/parameters',
      WorkflowMetadata: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/document/properties/metadata',
      WorkflowTags: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/document/properties/tags',
      WorkflowTimeout: 'https://serverlessworkflow.io/schemas/1.0.0/workflow.json#/properties/timeout',
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * oUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   *
   */
  var ajv = new Ajv({
      schemas: [workflowSchema],
      strict: false,
  });
  addFormats(ajv);
  /**
   * A Map of validation functions, where the key is the name of the schema to validate with
   */
  var validators = new Map(Object.entries(validationPointers).map(function (_a) {
      var typeName = _a[0], jsonPointer = _a[1];
      if (!jsonPointer)
          throw "No JSON pointer provided for type '".concat(typeName, "'");
      var validate = ajv.getSchema(jsonPointer);
      if (!validate)
          throw "Unable to find schema '".concat(jsonPointer, "' for type '").concat(typeName, "'");
      return [typeName, validate];
  }));
  /**
   * Validates the provided data or throws an error
   * @param typeName The data type to validate
   * @param data The data to validate
   * @param workflow A workflow instance, used for DSL level validation
   * @returns Throws if invalid
   */
  var validate = function (typeName, data, workflow) {
      var _a, _b, _c, _d, _e, _f;
      (_b = (_a = getLifecycleHooks(typeName)) === null || _a === void 0 ? void 0 : _a.preValidation) === null || _b === void 0 ? void 0 : _b.call(_a, data, workflow);
      var validateFn = validators.get(typeName);
      if (!validateFn) {
          throw Error("Unable to find a validation function for '".concat(typeName, "'"));
      }
      // prevents possible data mutation and invalid "additional properties" from the classes like constructor/validate/normalize
      if (!validateFn(deepCopy(data))) {
          throw new Error("'".concat(typeName, "' is invalid:\n").concat((_d = (_c = validateFn.errors) === null || _c === void 0 ? void 0 : _c.reduce(function (acc, error) { return acc + "- ".concat(error.instancePath, " | ").concat(error.schemaPath, " | ").concat(error.message, " | ").concat(JSON.stringify(error.params), "\n"); }, '')) !== null && _d !== void 0 ? _d : '', "\n\ndata: ").concat(JSON.stringify(data, null, 4)));
      }
      (_f = (_e = getLifecycleHooks(typeName)) === null || _e === void 0 ? void 0 : _e.postValidation) === null || _f === void 0 ? void 0 : _f.call(_e, data, workflow);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WithEvent with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WithEvent = /** @class */ (function (_super) {
      __extends(WithEvent, _super);
      /**
       * Instanciates a new instance of the WithEvent class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WithEvent.
       */
      function WithEvent(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('WithEvent')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WithEvent.
       * Throws if invalid.
       */
      WithEvent.prototype.validate = function (workflow) {
          var copy = new WithEvent(this);
          validate('WithEvent', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WithEvent.
       * Creates a copy of the WithEvent, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WithEvent instance.
       */
      WithEvent.prototype.normalize = function () {
          var _a, _b;
          var copy = new WithEvent(this);
          return ((_b = (_a = getLifecycleHooks('WithEvent')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WithEvent;
  }(ObjectHydrator));
  var _WithEvent = WithEvent;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventFilterCorrelate with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventFilterCorrelate = /** @class */ (function (_super) {
      __extends(EventFilterCorrelate, _super);
      /**
       * Instanciates a new instance of the EventFilterCorrelate class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventFilterCorrelate.
       */
      function EventFilterCorrelate(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EventFilterCorrelate')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventFilterCorrelate.
       * Throws if invalid.
       */
      EventFilterCorrelate.prototype.validate = function (workflow) {
          var copy = new EventFilterCorrelate(this);
          validate('EventFilterCorrelate', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventFilterCorrelate.
       * Creates a copy of the EventFilterCorrelate, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventFilterCorrelate instance.
       */
      EventFilterCorrelate.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventFilterCorrelate(this);
          return ((_b = (_a = getLifecycleHooks('EventFilterCorrelate')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventFilterCorrelate;
  }(ObjectHydrator));
  var _EventFilterCorrelate = EventFilterCorrelate;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventFilter with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventFilter = /** @class */ (function (_super) {
      __extends(EventFilter, _super);
      /**
       * Instanciates a new instance of the EventFilter class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventFilter.
       */
      function EventFilter(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.with === 'object')
                  self.with = new _WithEvent(model.with);
              if (typeof model.correlate === 'object')
                  self.correlate = new _EventFilterCorrelate(model.correlate);
          }
          (_b = (_a = getLifecycleHooks('EventFilter')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventFilter.
       * Throws if invalid.
       */
      EventFilter.prototype.validate = function (workflow) {
          var copy = new EventFilter(this);
          validate('EventFilter', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventFilter.
       * Creates a copy of the EventFilter, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventFilter instance.
       */
      EventFilter.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventFilter(this);
          return ((_b = (_a = getLifecycleHooks('EventFilter')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventFilter;
  }(ObjectHydrator));
  var _EventFilter = EventFilter;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of Specification.EventFilter.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var AllEventConsumptionStrategyConfiguration = /** @class */ (function (_super) {
      __extends(AllEventConsumptionStrategyConfiguration, _super);
      /**
       * Constructs a new instance of the AllEventConsumptionStrategyConfiguration class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function AllEventConsumptionStrategyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(new _EventFilter(item)); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(AllEventConsumptionStrategyConfiguration.prototype));
          (_b = (_a = getLifecycleHooks('AllEventConsumptionStrategyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AllEventConsumptionStrategyConfiguration.
       * Throws if invalid.
       */
      AllEventConsumptionStrategyConfiguration.prototype.validate = function (workflow) {
          var copy = new AllEventConsumptionStrategyConfiguration(this);
          validate('AllEventConsumptionStrategyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AllEventConsumptionStrategyConfiguration.
       * Creates a copy of the AllEventConsumptionStrategyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AllEventConsumptionStrategyConfiguration instance.
       */
      AllEventConsumptionStrategyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new AllEventConsumptionStrategyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('AllEventConsumptionStrategyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AllEventConsumptionStrategyConfiguration;
  }(ArrayHydrator));
  var _AllEventConsumptionStrategyConfiguration = AllEventConsumptionStrategyConfiguration;
  //export const _AllEventConsumptionStrategyConfiguration = AllEventConsumptionStrategyConfiguration; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AllEventConsumptionStrategy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AllEventConsumptionStrategy = /** @class */ (function (_super) {
      __extends(AllEventConsumptionStrategy, _super);
      /**
       * Instanciates a new instance of the AllEventConsumptionStrategy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AllEventConsumptionStrategy.
       */
      function AllEventConsumptionStrategy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.all === 'object')
                  self.all = new _AllEventConsumptionStrategyConfiguration(model.all);
          }
          (_b = (_a = getLifecycleHooks('AllEventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AllEventConsumptionStrategy.
       * Throws if invalid.
       */
      AllEventConsumptionStrategy.prototype.validate = function (workflow) {
          var copy = new AllEventConsumptionStrategy(this);
          validate('AllEventConsumptionStrategy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AllEventConsumptionStrategy.
       * Creates a copy of the AllEventConsumptionStrategy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AllEventConsumptionStrategy instance.
       */
      AllEventConsumptionStrategy.prototype.normalize = function () {
          var _a, _b;
          var copy = new AllEventConsumptionStrategy(this);
          return ((_b = (_a = getLifecycleHooks('AllEventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AllEventConsumptionStrategy;
  }(ObjectHydrator));
  var _AllEventConsumptionStrategy = AllEventConsumptionStrategy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of Specification.EventFilter.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var AnyEventConsumptionStrategyConfiguration = /** @class */ (function (_super) {
      __extends(AnyEventConsumptionStrategyConfiguration, _super);
      /**
       * Constructs a new instance of the AnyEventConsumptionStrategyConfiguration class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function AnyEventConsumptionStrategyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(new _EventFilter(item)); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(AnyEventConsumptionStrategyConfiguration.prototype));
          (_b = (_a = getLifecycleHooks('AnyEventConsumptionStrategyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AnyEventConsumptionStrategyConfiguration.
       * Throws if invalid.
       */
      AnyEventConsumptionStrategyConfiguration.prototype.validate = function (workflow) {
          var copy = new AnyEventConsumptionStrategyConfiguration(this);
          validate('AnyEventConsumptionStrategyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AnyEventConsumptionStrategyConfiguration.
       * Creates a copy of the AnyEventConsumptionStrategyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AnyEventConsumptionStrategyConfiguration instance.
       */
      AnyEventConsumptionStrategyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new AnyEventConsumptionStrategyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('AnyEventConsumptionStrategyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AnyEventConsumptionStrategyConfiguration;
  }(ArrayHydrator));
  var _AnyEventConsumptionStrategyConfiguration = AnyEventConsumptionStrategyConfiguration;
  //export const _AnyEventConsumptionStrategyConfiguration = AnyEventConsumptionStrategyConfiguration; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AnyEventConsumptionStrategyUntil with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AnyEventConsumptionStrategyUntil = /** @class */ (function (_super) {
      __extends(AnyEventConsumptionStrategyUntil, _super);
      /**
       * Instanciates a new instance of the AnyEventConsumptionStrategyUntil class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AnyEventConsumptionStrategyUntil.
       */
      function AnyEventConsumptionStrategyUntil(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.all === 'object')
                  self.all = new _AllEventConsumptionStrategyConfiguration(model
                      .all);
              if (typeof model.any === 'object')
                  self.any = new _AnyEventConsumptionStrategyConfiguration(model
                      .any);
              if (typeof model.one === 'object')
                  self.one = new _EventFilter(model.one);
          }
          (_b = (_a = getLifecycleHooks('AnyEventConsumptionStrategyUntil')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AnyEventConsumptionStrategyUntil.
       * Throws if invalid.
       */
      AnyEventConsumptionStrategyUntil.prototype.validate = function (workflow) {
          var copy = new AnyEventConsumptionStrategyUntil(this);
          validate('AnyEventConsumptionStrategyUntil', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AnyEventConsumptionStrategyUntil.
       * Creates a copy of the AnyEventConsumptionStrategyUntil, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AnyEventConsumptionStrategyUntil instance.
       */
      AnyEventConsumptionStrategyUntil.prototype.normalize = function () {
          var _a, _b;
          var copy = new AnyEventConsumptionStrategyUntil(this);
          return ((_b = (_a = getLifecycleHooks('AnyEventConsumptionStrategyUntil')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AnyEventConsumptionStrategyUntil;
  }(ObjectHydrator));
  var _AnyEventConsumptionStrategyUntil = AnyEventConsumptionStrategyUntil;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AnyEventConsumptionStrategy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AnyEventConsumptionStrategy = /** @class */ (function (_super) {
      __extends(AnyEventConsumptionStrategy, _super);
      /**
       * Instanciates a new instance of the AnyEventConsumptionStrategy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AnyEventConsumptionStrategy.
       */
      function AnyEventConsumptionStrategy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.any === 'object')
                  self.any = new _AnyEventConsumptionStrategyConfiguration(model.any);
              if (typeof model.until === 'object')
                  self.until = new _AnyEventConsumptionStrategyUntil(model.until);
          }
          (_b = (_a = getLifecycleHooks('AnyEventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AnyEventConsumptionStrategy.
       * Throws if invalid.
       */
      AnyEventConsumptionStrategy.prototype.validate = function (workflow) {
          var copy = new AnyEventConsumptionStrategy(this);
          validate('AnyEventConsumptionStrategy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AnyEventConsumptionStrategy.
       * Creates a copy of the AnyEventConsumptionStrategy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AnyEventConsumptionStrategy instance.
       */
      AnyEventConsumptionStrategy.prototype.normalize = function () {
          var _a, _b;
          var copy = new AnyEventConsumptionStrategy(this);
          return ((_b = (_a = getLifecycleHooks('AnyEventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AnyEventConsumptionStrategy;
  }(ObjectHydrator));
  var _AnyEventConsumptionStrategy = AnyEventConsumptionStrategy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AnyEventUntilConsumed with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AnyEventUntilConsumed = /** @class */ (function (_super) {
      __extends(AnyEventUntilConsumed, _super);
      /**
       * Instanciates a new instance of the AnyEventUntilConsumed class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AnyEventUntilConsumed.
       */
      function AnyEventUntilConsumed(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.all === 'object')
                  self.all = new _AllEventConsumptionStrategyConfiguration(model
                      .all);
              if (typeof model.any === 'object')
                  self.any = new _AnyEventConsumptionStrategyConfiguration(model
                      .any);
              if (typeof model.one === 'object')
                  self.one = new _EventFilter(model.one);
          }
          (_b = (_a = getLifecycleHooks('AnyEventUntilConsumed')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AnyEventUntilConsumed.
       * Throws if invalid.
       */
      AnyEventUntilConsumed.prototype.validate = function (workflow) {
          var copy = new AnyEventUntilConsumed(this);
          validate('AnyEventUntilConsumed', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AnyEventUntilConsumed.
       * Creates a copy of the AnyEventUntilConsumed, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AnyEventUntilConsumed instance.
       */
      AnyEventUntilConsumed.prototype.normalize = function () {
          var _a, _b;
          var copy = new AnyEventUntilConsumed(this);
          return ((_b = (_a = getLifecycleHooks('AnyEventUntilConsumed')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AnyEventUntilConsumed;
  }(ObjectHydrator));
  var _AnyEventUntilConsumed = AnyEventUntilConsumed;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AsyncApiArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AsyncApiArguments = /** @class */ (function (_super) {
      __extends(AsyncApiArguments, _super);
      /**
       * Instanciates a new instance of the AsyncApiArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AsyncApiArguments.
       */
      function AsyncApiArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('AsyncApiArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AsyncApiArguments.
       * Throws if invalid.
       */
      AsyncApiArguments.prototype.validate = function (workflow) {
          var copy = new AsyncApiArguments(this);
          validate('AsyncApiArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AsyncApiArguments.
       * Creates a copy of the AsyncApiArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AsyncApiArguments instance.
       */
      AsyncApiArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new AsyncApiArguments(this);
          return ((_b = (_a = getLifecycleHooks('AsyncApiArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AsyncApiArguments;
  }(ObjectHydrator));
  var _AsyncApiArguments = AsyncApiArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a BasicAuthenticationPolicyConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var BasicAuthenticationPolicyConfiguration = /** @class */ (function (_super) {
      __extends(BasicAuthenticationPolicyConfiguration, _super);
      /**
       * Instanciates a new instance of the BasicAuthenticationPolicyConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the BasicAuthenticationPolicyConfiguration.
       */
      function BasicAuthenticationPolicyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('BasicAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the BasicAuthenticationPolicyConfiguration.
       * Throws if invalid.
       */
      BasicAuthenticationPolicyConfiguration.prototype.validate = function (workflow) {
          var copy = new BasicAuthenticationPolicyConfiguration(this);
          validate('BasicAuthenticationPolicyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the BasicAuthenticationPolicyConfiguration.
       * Creates a copy of the BasicAuthenticationPolicyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the BasicAuthenticationPolicyConfiguration instance.
       */
      BasicAuthenticationPolicyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new BasicAuthenticationPolicyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('BasicAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return BasicAuthenticationPolicyConfiguration;
  }(ObjectHydrator));
  var _BasicAuthenticationPolicyConfiguration = BasicAuthenticationPolicyConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a BearerAuthenticationPolicyConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var BearerAuthenticationPolicyConfiguration = /** @class */ (function (_super) {
      __extends(BearerAuthenticationPolicyConfiguration, _super);
      /**
       * Instanciates a new instance of the BearerAuthenticationPolicyConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the BearerAuthenticationPolicyConfiguration.
       */
      function BearerAuthenticationPolicyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('BearerAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the BearerAuthenticationPolicyConfiguration.
       * Throws if invalid.
       */
      BearerAuthenticationPolicyConfiguration.prototype.validate = function (workflow) {
          var copy = new BearerAuthenticationPolicyConfiguration(this);
          validate('BearerAuthenticationPolicyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the BearerAuthenticationPolicyConfiguration.
       * Creates a copy of the BearerAuthenticationPolicyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the BearerAuthenticationPolicyConfiguration instance.
       */
      BearerAuthenticationPolicyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new BearerAuthenticationPolicyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('BearerAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return BearerAuthenticationPolicyConfiguration;
  }(ObjectHydrator));
  var _BearerAuthenticationPolicyConfiguration = BearerAuthenticationPolicyConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a DigestAuthenticationPolicyConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var DigestAuthenticationPolicyConfiguration = /** @class */ (function (_super) {
      __extends(DigestAuthenticationPolicyConfiguration, _super);
      /**
       * Instanciates a new instance of the DigestAuthenticationPolicyConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the DigestAuthenticationPolicyConfiguration.
       */
      function DigestAuthenticationPolicyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('DigestAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the DigestAuthenticationPolicyConfiguration.
       * Throws if invalid.
       */
      DigestAuthenticationPolicyConfiguration.prototype.validate = function (workflow) {
          var copy = new DigestAuthenticationPolicyConfiguration(this);
          validate('DigestAuthenticationPolicyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the DigestAuthenticationPolicyConfiguration.
       * Creates a copy of the DigestAuthenticationPolicyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the DigestAuthenticationPolicyConfiguration instance.
       */
      DigestAuthenticationPolicyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new DigestAuthenticationPolicyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('DigestAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return DigestAuthenticationPolicyConfiguration;
  }(ObjectHydrator));
  var _DigestAuthenticationPolicyConfiguration = DigestAuthenticationPolicyConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2AutenthicationDataClient with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2AutenthicationDataClient = /** @class */ (function (_super) {
      __extends(OAuth2AutenthicationDataClient, _super);
      /**
       * Instanciates a new instance of the OAuth2AutenthicationDataClient class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2AutenthicationDataClient.
       */
      function OAuth2AutenthicationDataClient(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('OAuth2AutenthicationDataClient')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AutenthicationDataClient.
       * Throws if invalid.
       */
      OAuth2AutenthicationDataClient.prototype.validate = function (workflow) {
          var copy = new OAuth2AutenthicationDataClient(this);
          validate('OAuth2AutenthicationDataClient', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AutenthicationDataClient.
       * Creates a copy of the OAuth2AutenthicationDataClient, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AutenthicationDataClient instance.
       */
      OAuth2AutenthicationDataClient.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AutenthicationDataClient(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AutenthicationDataClient')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AutenthicationDataClient;
  }(ObjectHydrator));
  var _OAuth2AutenthicationDataClient = OAuth2AutenthicationDataClient;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2TokenRequest with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2TokenRequest = /** @class */ (function (_super) {
      __extends(OAuth2TokenRequest, _super);
      /**
       * Instanciates a new instance of the OAuth2TokenRequest class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2TokenRequest.
       */
      function OAuth2TokenRequest(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('OAuth2TokenRequest')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2TokenRequest.
       * Throws if invalid.
       */
      OAuth2TokenRequest.prototype.validate = function (workflow) {
          var copy = new OAuth2TokenRequest(this);
          validate('OAuth2TokenRequest', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2TokenRequest.
       * Creates a copy of the OAuth2TokenRequest, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2TokenRequest instance.
       */
      OAuth2TokenRequest.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2TokenRequest(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2TokenRequest')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2TokenRequest;
  }(ObjectHydrator));
  var _OAuth2TokenRequest = OAuth2TokenRequest;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2TokenDefinition with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2TokenDefinition = /** @class */ (function (_super) {
      __extends(OAuth2TokenDefinition, _super);
      /**
       * Instanciates a new instance of the OAuth2TokenDefinition class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2TokenDefinition.
       */
      function OAuth2TokenDefinition(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('OAuth2TokenDefinition')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2TokenDefinition.
       * Throws if invalid.
       */
      OAuth2TokenDefinition.prototype.validate = function (workflow) {
          var copy = new OAuth2TokenDefinition(this);
          validate('OAuth2TokenDefinition', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2TokenDefinition.
       * Creates a copy of the OAuth2TokenDefinition, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2TokenDefinition instance.
       */
      OAuth2TokenDefinition.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2TokenDefinition(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2TokenDefinition')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2TokenDefinition;
  }(ObjectHydrator));
  var _OAuth2TokenDefinition = OAuth2TokenDefinition;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2AuthenticationPropertiesEndpoints with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2AuthenticationPropertiesEndpoints = /** @class */ (function (_super) {
      __extends(OAuth2AuthenticationPropertiesEndpoints, _super);
      /**
       * Instanciates a new instance of the OAuth2AuthenticationPropertiesEndpoints class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2AuthenticationPropertiesEndpoints.
       */
      function OAuth2AuthenticationPropertiesEndpoints(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('OAuth2AuthenticationPropertiesEndpoints')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AuthenticationPropertiesEndpoints.
       * Throws if invalid.
       */
      OAuth2AuthenticationPropertiesEndpoints.prototype.validate = function (workflow) {
          var copy = new OAuth2AuthenticationPropertiesEndpoints(this);
          validate('OAuth2AuthenticationPropertiesEndpoints', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AuthenticationPropertiesEndpoints.
       * Creates a copy of the OAuth2AuthenticationPropertiesEndpoints, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AuthenticationPropertiesEndpoints instance.
       */
      OAuth2AuthenticationPropertiesEndpoints.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AuthenticationPropertiesEndpoints(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AuthenticationPropertiesEndpoints')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AuthenticationPropertiesEndpoints;
  }(ObjectHydrator));
  var _OAuth2AuthenticationPropertiesEndpoints = OAuth2AuthenticationPropertiesEndpoints;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2AuthenticationPolicyConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2AuthenticationPolicyConfiguration = /** @class */ (function (_super) {
      __extends(OAuth2AuthenticationPolicyConfiguration, _super);
      /**
       * Instanciates a new instance of the OAuth2AuthenticationPolicyConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2AuthenticationPolicyConfiguration.
       */
      function OAuth2AuthenticationPolicyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.client === 'object')
                  self.client = new _OAuth2AutenthicationDataClient(model.client);
              if (typeof model.request === 'object')
                  self.request = new _OAuth2TokenRequest(model.request);
              if (typeof model.subject === 'object')
                  self.subject = new _OAuth2TokenDefinition(model.subject);
              if (typeof model.actor === 'object')
                  self.actor = new _OAuth2TokenDefinition(model.actor);
              if (typeof model
                  .endpoints === 'object')
                  self.endpoints = new _OAuth2AuthenticationPropertiesEndpoints(model
                      .endpoints);
          }
          (_b = (_a = getLifecycleHooks('OAuth2AuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AuthenticationPolicyConfiguration.
       * Throws if invalid.
       */
      OAuth2AuthenticationPolicyConfiguration.prototype.validate = function (workflow) {
          var copy = new OAuth2AuthenticationPolicyConfiguration(this);
          validate('OAuth2AuthenticationPolicyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AuthenticationPolicyConfiguration.
       * Creates a copy of the OAuth2AuthenticationPolicyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AuthenticationPolicyConfiguration instance.
       */
      OAuth2AuthenticationPolicyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AuthenticationPolicyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AuthenticationPolicyConfiguration;
  }(ObjectHydrator));
  var _OAuth2AuthenticationPolicyConfiguration = OAuth2AuthenticationPolicyConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OpenIdConnectAuthenticationPolicyConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OpenIdConnectAuthenticationPolicyConfiguration = /** @class */ (function (_super) {
      __extends(OpenIdConnectAuthenticationPolicyConfiguration, _super);
      /**
       * Instanciates a new instance of the OpenIdConnectAuthenticationPolicyConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OpenIdConnectAuthenticationPolicyConfiguration.
       */
      function OpenIdConnectAuthenticationPolicyConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.client === 'object')
                  self.client = new _OAuth2AutenthicationDataClient(model
                      .client);
              if (typeof model.request === 'object')
                  self.request = new _OAuth2TokenRequest(model.request);
              if (typeof model.subject === 'object')
                  self.subject = new _OAuth2TokenDefinition(model.subject);
              if (typeof model.actor === 'object')
                  self.actor = new _OAuth2TokenDefinition(model.actor);
          }
          (_b = (_a = getLifecycleHooks('OpenIdConnectAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OpenIdConnectAuthenticationPolicyConfiguration.
       * Throws if invalid.
       */
      OpenIdConnectAuthenticationPolicyConfiguration.prototype.validate = function (workflow) {
          var copy = new OpenIdConnectAuthenticationPolicyConfiguration(this);
          validate('OpenIdConnectAuthenticationPolicyConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OpenIdConnectAuthenticationPolicyConfiguration.
       * Creates a copy of the OpenIdConnectAuthenticationPolicyConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OpenIdConnectAuthenticationPolicyConfiguration instance.
       */
      OpenIdConnectAuthenticationPolicyConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new OpenIdConnectAuthenticationPolicyConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('OpenIdConnectAuthenticationPolicyConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OpenIdConnectAuthenticationPolicyConfiguration;
  }(ObjectHydrator));
  var _OpenIdConnectAuthenticationPolicyConfiguration = OpenIdConnectAuthenticationPolicyConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AuthenticationPolicy = /** @class */ (function (_super) {
      __extends(AuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the AuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AuthenticationPolicy.
       */
      function AuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.basic === 'object')
                  self.basic = new _BasicAuthenticationPolicyConfiguration(model
                      .basic);
              if (typeof model.bearer === 'object')
                  self.bearer = new _BearerAuthenticationPolicyConfiguration(model
                      .bearer);
              if (typeof model.digest === 'object')
                  self.digest = new _DigestAuthenticationPolicyConfiguration(model
                      .digest);
              if (typeof model.oauth2 === 'object')
                  self.oauth2 = new _OAuth2AuthenticationPolicyConfiguration(model
                      .oauth2);
              if (typeof model.oidc === 'object')
                  self.oidc =
                      new _OpenIdConnectAuthenticationPolicyConfiguration(model
                          .oidc);
          }
          (_b = (_a = getLifecycleHooks('AuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AuthenticationPolicy.
       * Throws if invalid.
       */
      AuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new AuthenticationPolicy(this);
          validate('AuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AuthenticationPolicy.
       * Creates a copy of the AuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AuthenticationPolicy instance.
       */
      AuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new AuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('AuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AuthenticationPolicy;
  }(ObjectHydrator));
  var _AuthenticationPolicy = AuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a AuthenticationPolicyReference with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var AuthenticationPolicyReference = /** @class */ (function (_super) {
      __extends(AuthenticationPolicyReference, _super);
      /**
       * Instanciates a new instance of the AuthenticationPolicyReference class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the AuthenticationPolicyReference.
       */
      function AuthenticationPolicyReference(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('AuthenticationPolicyReference')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the AuthenticationPolicyReference.
       * Throws if invalid.
       */
      AuthenticationPolicyReference.prototype.validate = function (workflow) {
          var copy = new AuthenticationPolicyReference(this);
          validate('AuthenticationPolicyReference', copy, workflow);
      };
      /**
       * Normalizes the current instance of the AuthenticationPolicyReference.
       * Creates a copy of the AuthenticationPolicyReference, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the AuthenticationPolicyReference instance.
       */
      AuthenticationPolicyReference.prototype.normalize = function () {
          var _a, _b;
          var copy = new AuthenticationPolicyReference(this);
          return ((_b = (_a = getLifecycleHooks('AuthenticationPolicyReference')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return AuthenticationPolicyReference;
  }(ObjectHydrator));
  var _AuthenticationPolicyReference = AuthenticationPolicyReference;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a BasicAuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var BasicAuthenticationPolicy = /** @class */ (function (_super) {
      __extends(BasicAuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the BasicAuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the BasicAuthenticationPolicy.
       */
      function BasicAuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.basic === 'object')
                  self.basic = new _BasicAuthenticationPolicyConfiguration(model.basic);
          }
          (_b = (_a = getLifecycleHooks('BasicAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the BasicAuthenticationPolicy.
       * Throws if invalid.
       */
      BasicAuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new BasicAuthenticationPolicy(this);
          validate('BasicAuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the BasicAuthenticationPolicy.
       * Creates a copy of the BasicAuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the BasicAuthenticationPolicy instance.
       */
      BasicAuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new BasicAuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('BasicAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return BasicAuthenticationPolicy;
  }(ObjectHydrator));
  var _BasicAuthenticationPolicy = BasicAuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a BasicAuthenticationProperties with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var BasicAuthenticationProperties = /** @class */ (function (_super) {
      __extends(BasicAuthenticationProperties, _super);
      /**
       * Instanciates a new instance of the BasicAuthenticationProperties class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the BasicAuthenticationProperties.
       */
      function BasicAuthenticationProperties(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('BasicAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the BasicAuthenticationProperties.
       * Throws if invalid.
       */
      BasicAuthenticationProperties.prototype.validate = function (workflow) {
          var copy = new BasicAuthenticationProperties(this);
          validate('BasicAuthenticationProperties', copy, workflow);
      };
      /**
       * Normalizes the current instance of the BasicAuthenticationProperties.
       * Creates a copy of the BasicAuthenticationProperties, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the BasicAuthenticationProperties instance.
       */
      BasicAuthenticationProperties.prototype.normalize = function () {
          var _a, _b;
          var copy = new BasicAuthenticationProperties(this);
          return ((_b = (_a = getLifecycleHooks('BasicAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return BasicAuthenticationProperties;
  }(ObjectHydrator));
  var _BasicAuthenticationProperties = BasicAuthenticationProperties;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a BearerAuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var BearerAuthenticationPolicy = /** @class */ (function (_super) {
      __extends(BearerAuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the BearerAuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the BearerAuthenticationPolicy.
       */
      function BearerAuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.bearer === 'object')
                  self.bearer = new _BearerAuthenticationPolicyConfiguration(model.bearer);
          }
          (_b = (_a = getLifecycleHooks('BearerAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the BearerAuthenticationPolicy.
       * Throws if invalid.
       */
      BearerAuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new BearerAuthenticationPolicy(this);
          validate('BearerAuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the BearerAuthenticationPolicy.
       * Creates a copy of the BearerAuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the BearerAuthenticationPolicy instance.
       */
      BearerAuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new BearerAuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('BearerAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return BearerAuthenticationPolicy;
  }(ObjectHydrator));
  var _BearerAuthenticationPolicy = BearerAuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a BearerAuthenticationProperties with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var BearerAuthenticationProperties = /** @class */ (function (_super) {
      __extends(BearerAuthenticationProperties, _super);
      /**
       * Instanciates a new instance of the BearerAuthenticationProperties class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the BearerAuthenticationProperties.
       */
      function BearerAuthenticationProperties(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('BearerAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the BearerAuthenticationProperties.
       * Throws if invalid.
       */
      BearerAuthenticationProperties.prototype.validate = function (workflow) {
          var copy = new BearerAuthenticationProperties(this);
          validate('BearerAuthenticationProperties', copy, workflow);
      };
      /**
       * Normalizes the current instance of the BearerAuthenticationProperties.
       * Creates a copy of the BearerAuthenticationProperties, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the BearerAuthenticationProperties instance.
       */
      BearerAuthenticationProperties.prototype.normalize = function () {
          var _a, _b;
          var copy = new BearerAuthenticationProperties(this);
          return ((_b = (_a = getLifecycleHooks('BearerAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return BearerAuthenticationProperties;
  }(ObjectHydrator));
  var _BearerAuthenticationProperties = BearerAuthenticationProperties;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ReferenceableAuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ReferenceableAuthenticationPolicy = /** @class */ (function (_super) {
      __extends(ReferenceableAuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the ReferenceableAuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ReferenceableAuthenticationPolicy.
       */
      function ReferenceableAuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.basic === 'object')
                  self.basic = new _BasicAuthenticationPolicyConfiguration(model
                      .basic);
              if (typeof model.bearer === 'object')
                  self.bearer = new _BearerAuthenticationPolicyConfiguration(model
                      .bearer);
              if (typeof model.digest === 'object')
                  self.digest = new _DigestAuthenticationPolicyConfiguration(model
                      .digest);
              if (typeof model.oauth2 === 'object')
                  self.oauth2 = new _OAuth2AuthenticationPolicyConfiguration(model
                      .oauth2);
              if (typeof model.oidc === 'object')
                  self.oidc =
                      new _OpenIdConnectAuthenticationPolicyConfiguration(model
                          .oidc);
          }
          (_b = (_a = getLifecycleHooks('ReferenceableAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ReferenceableAuthenticationPolicy.
       * Throws if invalid.
       */
      ReferenceableAuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new ReferenceableAuthenticationPolicy(this);
          validate('ReferenceableAuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ReferenceableAuthenticationPolicy.
       * Creates a copy of the ReferenceableAuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ReferenceableAuthenticationPolicy instance.
       */
      ReferenceableAuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new ReferenceableAuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('ReferenceableAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ReferenceableAuthenticationPolicy;
  }(ObjectHydrator));
  var _ReferenceableAuthenticationPolicy = ReferenceableAuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Endpoint with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Endpoint = /** @class */ (function (_super) {
      __extends(Endpoint, _super);
      /**
       * Instanciates a new instance of the Endpoint class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Endpoint.
       */
      function Endpoint(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.authentication === 'object')
                  self.authentication = new _ReferenceableAuthenticationPolicy(model
                      .authentication);
          }
          (_b = (_a = getLifecycleHooks('Endpoint')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Endpoint.
       * Throws if invalid.
       */
      Endpoint.prototype.validate = function (workflow) {
          var copy = new Endpoint(this);
          validate('Endpoint', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Endpoint.
       * Creates a copy of the Endpoint, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Endpoint instance.
       */
      Endpoint.prototype.normalize = function () {
          var _a, _b;
          var copy = new Endpoint(this);
          return ((_b = (_a = getLifecycleHooks('Endpoint')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Endpoint;
  }(ObjectHydrator));
  var _Endpoint = Endpoint;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ExternalResource with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ExternalResource = /** @class */ (function (_super) {
      __extends(ExternalResource, _super);
      /**
       * Instanciates a new instance of the ExternalResource class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ExternalResource.
       */
      function ExternalResource(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.endpoint === 'object')
                  self.endpoint = new _Endpoint(model.endpoint);
          }
          (_b = (_a = getLifecycleHooks('ExternalResource')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ExternalResource.
       * Throws if invalid.
       */
      ExternalResource.prototype.validate = function (workflow) {
          var copy = new ExternalResource(this);
          validate('ExternalResource', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ExternalResource.
       * Creates a copy of the ExternalResource, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ExternalResource instance.
       */
      ExternalResource.prototype.normalize = function () {
          var _a, _b;
          var copy = new ExternalResource(this);
          return ((_b = (_a = getLifecycleHooks('ExternalResource')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ExternalResource;
  }(ObjectHydrator));
  var _ExternalResource = ExternalResource;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Schema with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Schema = /** @class */ (function (_super) {
      __extends(Schema, _super);
      /**
       * Instanciates a new instance of the Schema class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Schema.
       */
      function Schema(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.resource === 'object')
                  self.resource = new _ExternalResource(model.resource);
          }
          (_b = (_a = getLifecycleHooks('Schema')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Schema.
       * Throws if invalid.
       */
      Schema.prototype.validate = function (workflow) {
          var copy = new Schema(this);
          validate('Schema', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Schema.
       * Creates a copy of the Schema, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Schema instance.
       */
      Schema.prototype.normalize = function () {
          var _a, _b;
          var copy = new Schema(this);
          return ((_b = (_a = getLifecycleHooks('Schema')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Schema;
  }(ObjectHydrator));
  var _Schema = Schema;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Input with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Input = /** @class */ (function (_super) {
      __extends(Input, _super);
      /**
       * Instanciates a new instance of the Input class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Input.
       */
      function Input(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.schema === 'object')
                  self.schema = new _Schema(model.schema);
          }
          (_b = (_a = getLifecycleHooks('Input')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Input.
       * Throws if invalid.
       */
      Input.prototype.validate = function (workflow) {
          var copy = new Input(this);
          validate('Input', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Input.
       * Creates a copy of the Input, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Input instance.
       */
      Input.prototype.normalize = function () {
          var _a, _b;
          var copy = new Input(this);
          return ((_b = (_a = getLifecycleHooks('Input')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Input;
  }(ObjectHydrator));
  var _Input = Input;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Output with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Output = /** @class */ (function (_super) {
      __extends(Output, _super);
      /**
       * Instanciates a new instance of the Output class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Output.
       */
      function Output(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.schema === 'object')
                  self.schema = new _Schema(model.schema);
          }
          (_b = (_a = getLifecycleHooks('Output')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Output.
       * Throws if invalid.
       */
      Output.prototype.validate = function (workflow) {
          var copy = new Output(this);
          validate('Output', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Output.
       * Creates a copy of the Output, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Output instance.
       */
      Output.prototype.normalize = function () {
          var _a, _b;
          var copy = new Output(this);
          return ((_b = (_a = getLifecycleHooks('Output')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Output;
  }(ObjectHydrator));
  var _Output = Output;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Export with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Export = /** @class */ (function (_super) {
      __extends(Export, _super);
      /**
       * Instanciates a new instance of the Export class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Export.
       */
      function Export(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.schema === 'object')
                  self.schema = new _Schema(model.schema);
          }
          (_b = (_a = getLifecycleHooks('Export')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Export.
       * Throws if invalid.
       */
      Export.prototype.validate = function (workflow) {
          var copy = new Export(this);
          validate('Export', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Export.
       * Creates a copy of the Export, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Export instance.
       */
      Export.prototype.normalize = function () {
          var _a, _b;
          var copy = new Export(this);
          return ((_b = (_a = getLifecycleHooks('Export')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Export;
  }(ObjectHydrator));
  var _Export = Export;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Duration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Duration = /** @class */ (function (_super) {
      __extends(Duration, _super);
      /**
       * Instanciates a new instance of the Duration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Duration.
       */
      function Duration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('Duration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Duration.
       * Throws if invalid.
       */
      Duration.prototype.validate = function (workflow) {
          var copy = new Duration(this);
          validate('Duration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Duration.
       * Creates a copy of the Duration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Duration instance.
       */
      Duration.prototype.normalize = function () {
          var _a, _b;
          var copy = new Duration(this);
          return ((_b = (_a = getLifecycleHooks('Duration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Duration;
  }(ObjectHydrator));
  var _Duration = Duration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TaskTimeout with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TaskTimeout = /** @class */ (function (_super) {
      __extends(TaskTimeout, _super);
      /**
       * Instanciates a new instance of the TaskTimeout class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TaskTimeout.
       */
      function TaskTimeout(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.after === 'object')
                  self.after = new _Duration(model.after);
          }
          (_b = (_a = getLifecycleHooks('TaskTimeout')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TaskTimeout.
       * Throws if invalid.
       */
      TaskTimeout.prototype.validate = function (workflow) {
          var copy = new TaskTimeout(this);
          validate('TaskTimeout', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TaskTimeout.
       * Creates a copy of the TaskTimeout, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TaskTimeout instance.
       */
      TaskTimeout.prototype.normalize = function () {
          var _a, _b;
          var copy = new TaskTimeout(this);
          return ((_b = (_a = getLifecycleHooks('TaskTimeout')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TaskTimeout;
  }(ObjectHydrator));
  var _TaskTimeout = TaskTimeout;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TaskMetadata with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TaskMetadata = /** @class */ (function (_super) {
      __extends(TaskMetadata, _super);
      /**
       * Instanciates a new instance of the TaskMetadata class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TaskMetadata.
       */
      function TaskMetadata(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('TaskMetadata')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TaskMetadata.
       * Throws if invalid.
       */
      TaskMetadata.prototype.validate = function (workflow) {
          var copy = new TaskMetadata(this);
          validate('TaskMetadata', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TaskMetadata.
       * Creates a copy of the TaskMetadata, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TaskMetadata instance.
       */
      TaskMetadata.prototype.normalize = function () {
          var _a, _b;
          var copy = new TaskMetadata(this);
          return ((_b = (_a = getLifecycleHooks('TaskMetadata')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TaskMetadata;
  }(ObjectHydrator));
  var _TaskMetadata = TaskMetadata;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TaskBase with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TaskBase = /** @class */ (function (_super) {
      __extends(TaskBase, _super);
      /**
       * Instanciates a new instance of the TaskBase class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TaskBase.
       */
      function TaskBase(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
          }
          (_b = (_a = getLifecycleHooks('TaskBase')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TaskBase.
       * Throws if invalid.
       */
      TaskBase.prototype.validate = function (workflow) {
          var copy = new TaskBase(this);
          validate('TaskBase', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TaskBase.
       * Creates a copy of the TaskBase, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TaskBase instance.
       */
      TaskBase.prototype.normalize = function () {
          var _a, _b;
          var copy = new TaskBase(this);
          return ((_b = (_a = getLifecycleHooks('TaskBase')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TaskBase;
  }(ObjectHydrator));
  var _TaskBase = TaskBase;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CallAsyncAPI with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CallAsyncAPI = /** @class */ (function (_super) {
      __extends(CallAsyncAPI, _super);
      /**
       * Instanciates a new instance of the CallAsyncAPI class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CallAsyncAPI.
       */
      function CallAsyncAPI(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              self.call = 'asyncapi';
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
          }
          (_b = (_a = getLifecycleHooks('CallAsyncAPI')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CallAsyncAPI.
       * Throws if invalid.
       */
      CallAsyncAPI.prototype.validate = function (workflow) {
          var copy = new CallAsyncAPI(this);
          validate('CallAsyncAPI', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CallAsyncAPI.
       * Creates a copy of the CallAsyncAPI, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CallAsyncAPI instance.
       */
      CallAsyncAPI.prototype.normalize = function () {
          var _a, _b;
          var copy = new CallAsyncAPI(this);
          return ((_b = (_a = getLifecycleHooks('CallAsyncAPI')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CallAsyncAPI;
  }(_TaskBase));
  var _CallAsyncAPI = CallAsyncAPI;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a FunctionArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var FunctionArguments = /** @class */ (function (_super) {
      __extends(FunctionArguments, _super);
      /**
       * Instanciates a new instance of the FunctionArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the FunctionArguments.
       */
      function FunctionArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('FunctionArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the FunctionArguments.
       * Throws if invalid.
       */
      FunctionArguments.prototype.validate = function (workflow) {
          var copy = new FunctionArguments(this);
          validate('FunctionArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the FunctionArguments.
       * Creates a copy of the FunctionArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the FunctionArguments instance.
       */
      FunctionArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new FunctionArguments(this);
          return ((_b = (_a = getLifecycleHooks('FunctionArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return FunctionArguments;
  }(ObjectHydrator));
  var _FunctionArguments = FunctionArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CallFunction with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CallFunction = /** @class */ (function (_super) {
      __extends(CallFunction, _super);
      /**
       * Instanciates a new instance of the CallFunction class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CallFunction.
       */
      function CallFunction(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.with === 'object')
                  self.with = new _FunctionArguments(model.with);
          }
          (_b = (_a = getLifecycleHooks('CallFunction')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CallFunction.
       * Throws if invalid.
       */
      CallFunction.prototype.validate = function (workflow) {
          var copy = new CallFunction(this);
          validate('CallFunction', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CallFunction.
       * Creates a copy of the CallFunction, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CallFunction instance.
       */
      CallFunction.prototype.normalize = function () {
          var _a, _b;
          var copy = new CallFunction(this);
          return ((_b = (_a = getLifecycleHooks('CallFunction')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CallFunction;
  }(_TaskBase));
  var _CallFunction = CallFunction;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WithGRPCService with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WithGRPCService = /** @class */ (function (_super) {
      __extends(WithGRPCService, _super);
      /**
       * Instanciates a new instance of the WithGRPCService class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WithGRPCService.
       */
      function WithGRPCService(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.authentication === 'object')
                  self.authentication = new _ReferenceableAuthenticationPolicy(model.authentication);
          }
          (_b = (_a = getLifecycleHooks('WithGRPCService')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WithGRPCService.
       * Throws if invalid.
       */
      WithGRPCService.prototype.validate = function (workflow) {
          var copy = new WithGRPCService(this);
          validate('WithGRPCService', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WithGRPCService.
       * Creates a copy of the WithGRPCService, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WithGRPCService instance.
       */
      WithGRPCService.prototype.normalize = function () {
          var _a, _b;
          var copy = new WithGRPCService(this);
          return ((_b = (_a = getLifecycleHooks('WithGRPCService')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WithGRPCService;
  }(ObjectHydrator));
  var _WithGRPCService = WithGRPCService;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WithGRPCArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WithGRPCArguments = /** @class */ (function (_super) {
      __extends(WithGRPCArguments, _super);
      /**
       * Instanciates a new instance of the WithGRPCArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WithGRPCArguments.
       */
      function WithGRPCArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('WithGRPCArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WithGRPCArguments.
       * Throws if invalid.
       */
      WithGRPCArguments.prototype.validate = function (workflow) {
          var copy = new WithGRPCArguments(this);
          validate('WithGRPCArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WithGRPCArguments.
       * Creates a copy of the WithGRPCArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WithGRPCArguments instance.
       */
      WithGRPCArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new WithGRPCArguments(this);
          return ((_b = (_a = getLifecycleHooks('WithGRPCArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WithGRPCArguments;
  }(ObjectHydrator));
  var _WithGRPCArguments = WithGRPCArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a GRPCArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var GRPCArguments = /** @class */ (function (_super) {
      __extends(GRPCArguments, _super);
      /**
       * Instanciates a new instance of the GRPCArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the GRPCArguments.
       */
      function GRPCArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.proto === 'object')
                  self.proto = new _ExternalResource(model.proto);
              if (typeof model.service === 'object')
                  self.service = new _WithGRPCService(model.service);
              if (typeof model.arguments === 'object')
                  self.arguments = new _WithGRPCArguments(model.arguments);
          }
          (_b = (_a = getLifecycleHooks('GRPCArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the GRPCArguments.
       * Throws if invalid.
       */
      GRPCArguments.prototype.validate = function (workflow) {
          var copy = new GRPCArguments(this);
          validate('GRPCArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the GRPCArguments.
       * Creates a copy of the GRPCArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the GRPCArguments instance.
       */
      GRPCArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new GRPCArguments(this);
          return ((_b = (_a = getLifecycleHooks('GRPCArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return GRPCArguments;
  }(ObjectHydrator));
  var _GRPCArguments = GRPCArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CallGRPC with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CallGRPC = /** @class */ (function (_super) {
      __extends(CallGRPC, _super);
      /**
       * Instanciates a new instance of the CallGRPC class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CallGRPC.
       */
      function CallGRPC(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              self.call = 'grpc';
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.with === 'object')
                  self.with = new _GRPCArguments(model.with);
          }
          (_b = (_a = getLifecycleHooks('CallGRPC')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CallGRPC.
       * Throws if invalid.
       */
      CallGRPC.prototype.validate = function (workflow) {
          var copy = new CallGRPC(this);
          validate('CallGRPC', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CallGRPC.
       * Creates a copy of the CallGRPC, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CallGRPC instance.
       */
      CallGRPC.prototype.normalize = function () {
          var _a, _b;
          var copy = new CallGRPC(this);
          return ((_b = (_a = getLifecycleHooks('CallGRPC')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CallGRPC;
  }(_TaskBase));
  var _CallGRPC = CallGRPC;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a HTTPHeaders with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var HTTPHeaders = /** @class */ (function (_super) {
      __extends(HTTPHeaders, _super);
      /**
       * Instanciates a new instance of the HTTPHeaders class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the HTTPHeaders.
       */
      function HTTPHeaders(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('HTTPHeaders')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the HTTPHeaders.
       * Throws if invalid.
       */
      HTTPHeaders.prototype.validate = function (workflow) {
          var copy = new HTTPHeaders(this);
          validate('HTTPHeaders', copy, workflow);
      };
      /**
       * Normalizes the current instance of the HTTPHeaders.
       * Creates a copy of the HTTPHeaders, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the HTTPHeaders instance.
       */
      HTTPHeaders.prototype.normalize = function () {
          var _a, _b;
          var copy = new HTTPHeaders(this);
          return ((_b = (_a = getLifecycleHooks('HTTPHeaders')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return HTTPHeaders;
  }(ObjectHydrator));
  var _HTTPHeaders = HTTPHeaders;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a HTTPBody with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var HTTPBody = /** @class */ (function (_super) {
      __extends(HTTPBody, _super);
      /**
       * Instanciates a new instance of the HTTPBody class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the HTTPBody.
       */
      function HTTPBody(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('HTTPBody')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the HTTPBody.
       * Throws if invalid.
       */
      HTTPBody.prototype.validate = function (workflow) {
          var copy = new HTTPBody(this);
          validate('HTTPBody', copy, workflow);
      };
      /**
       * Normalizes the current instance of the HTTPBody.
       * Creates a copy of the HTTPBody, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the HTTPBody instance.
       */
      HTTPBody.prototype.normalize = function () {
          var _a, _b;
          var copy = new HTTPBody(this);
          return ((_b = (_a = getLifecycleHooks('HTTPBody')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return HTTPBody;
  }(ObjectHydrator));
  var _HTTPBody = HTTPBody;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a HTTPQuery with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var HTTPQuery = /** @class */ (function (_super) {
      __extends(HTTPQuery, _super);
      /**
       * Instanciates a new instance of the HTTPQuery class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the HTTPQuery.
       */
      function HTTPQuery(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('HTTPQuery')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the HTTPQuery.
       * Throws if invalid.
       */
      HTTPQuery.prototype.validate = function (workflow) {
          var copy = new HTTPQuery(this);
          validate('HTTPQuery', copy, workflow);
      };
      /**
       * Normalizes the current instance of the HTTPQuery.
       * Creates a copy of the HTTPQuery, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the HTTPQuery instance.
       */
      HTTPQuery.prototype.normalize = function () {
          var _a, _b;
          var copy = new HTTPQuery(this);
          return ((_b = (_a = getLifecycleHooks('HTTPQuery')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return HTTPQuery;
  }(ObjectHydrator));
  var _HTTPQuery = HTTPQuery;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a HTTPArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var HTTPArguments = /** @class */ (function (_super) {
      __extends(HTTPArguments, _super);
      /**
       * Instanciates a new instance of the HTTPArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the HTTPArguments.
       */
      function HTTPArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.endpoint === 'object')
                  self.endpoint = new _Endpoint(model.endpoint);
              if (typeof model.headers === 'object')
                  self.headers = new _HTTPHeaders(model.headers);
              if (typeof model.body === 'object')
                  self.body = new _HTTPBody(model.body);
              if (typeof model.query === 'object')
                  self.query = new _HTTPQuery(model.query);
          }
          (_b = (_a = getLifecycleHooks('HTTPArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the HTTPArguments.
       * Throws if invalid.
       */
      HTTPArguments.prototype.validate = function (workflow) {
          var copy = new HTTPArguments(this);
          validate('HTTPArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the HTTPArguments.
       * Creates a copy of the HTTPArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the HTTPArguments instance.
       */
      HTTPArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new HTTPArguments(this);
          return ((_b = (_a = getLifecycleHooks('HTTPArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return HTTPArguments;
  }(ObjectHydrator));
  var _HTTPArguments = HTTPArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CallHTTP with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CallHTTP = /** @class */ (function (_super) {
      __extends(CallHTTP, _super);
      /**
       * Instanciates a new instance of the CallHTTP class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CallHTTP.
       */
      function CallHTTP(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              self.call = 'http';
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.with === 'object')
                  self.with = new _HTTPArguments(model.with);
          }
          (_b = (_a = getLifecycleHooks('CallHTTP')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CallHTTP.
       * Throws if invalid.
       */
      CallHTTP.prototype.validate = function (workflow) {
          var copy = new CallHTTP(this);
          validate('CallHTTP', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CallHTTP.
       * Creates a copy of the CallHTTP, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CallHTTP instance.
       */
      CallHTTP.prototype.normalize = function () {
          var _a, _b;
          var copy = new CallHTTP(this);
          return ((_b = (_a = getLifecycleHooks('CallHTTP')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CallHTTP;
  }(_TaskBase));
  var _CallHTTP = CallHTTP;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WithOpenAPIParameters with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WithOpenAPIParameters = /** @class */ (function (_super) {
      __extends(WithOpenAPIParameters, _super);
      /**
       * Instanciates a new instance of the WithOpenAPIParameters class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WithOpenAPIParameters.
       */
      function WithOpenAPIParameters(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('WithOpenAPIParameters')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WithOpenAPIParameters.
       * Throws if invalid.
       */
      WithOpenAPIParameters.prototype.validate = function (workflow) {
          var copy = new WithOpenAPIParameters(this);
          validate('WithOpenAPIParameters', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WithOpenAPIParameters.
       * Creates a copy of the WithOpenAPIParameters, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WithOpenAPIParameters instance.
       */
      WithOpenAPIParameters.prototype.normalize = function () {
          var _a, _b;
          var copy = new WithOpenAPIParameters(this);
          return ((_b = (_a = getLifecycleHooks('WithOpenAPIParameters')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WithOpenAPIParameters;
  }(ObjectHydrator));
  var _WithOpenAPIParameters = WithOpenAPIParameters;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OpenAPIArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OpenAPIArguments = /** @class */ (function (_super) {
      __extends(OpenAPIArguments, _super);
      /**
       * Instanciates a new instance of the OpenAPIArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OpenAPIArguments.
       */
      function OpenAPIArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.document === 'object')
                  self.document = new _ExternalResource(model.document);
              if (typeof model.parameters === 'object')
                  self.parameters = new _WithOpenAPIParameters(model.parameters);
              if (typeof model.authentication === 'object')
                  self.authentication = new _ReferenceableAuthenticationPolicy(model.authentication);
          }
          (_b = (_a = getLifecycleHooks('OpenAPIArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OpenAPIArguments.
       * Throws if invalid.
       */
      OpenAPIArguments.prototype.validate = function (workflow) {
          var copy = new OpenAPIArguments(this);
          validate('OpenAPIArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OpenAPIArguments.
       * Creates a copy of the OpenAPIArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OpenAPIArguments instance.
       */
      OpenAPIArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new OpenAPIArguments(this);
          return ((_b = (_a = getLifecycleHooks('OpenAPIArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OpenAPIArguments;
  }(ObjectHydrator));
  var _OpenAPIArguments = OpenAPIArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CallOpenAPI with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CallOpenAPI = /** @class */ (function (_super) {
      __extends(CallOpenAPI, _super);
      /**
       * Instanciates a new instance of the CallOpenAPI class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CallOpenAPI.
       */
      function CallOpenAPI(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              self.call = 'openapi';
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.with === 'object')
                  self.with = new _OpenAPIArguments(model.with);
          }
          (_b = (_a = getLifecycleHooks('CallOpenAPI')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CallOpenAPI.
       * Throws if invalid.
       */
      CallOpenAPI.prototype.validate = function (workflow) {
          var copy = new CallOpenAPI(this);
          validate('CallOpenAPI', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CallOpenAPI.
       * Creates a copy of the CallOpenAPI, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CallOpenAPI instance.
       */
      CallOpenAPI.prototype.normalize = function () {
          var _a, _b;
          var copy = new CallOpenAPI(this);
          return ((_b = (_a = getLifecycleHooks('CallOpenAPI')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CallOpenAPI;
  }(_TaskBase));
  var _CallOpenAPI = CallOpenAPI;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CallTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CallTask = /** @class */ (function (_super) {
      __extends(CallTask, _super);
      /**
       * Instanciates a new instance of the CallTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CallTask.
       */
      function CallTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
          }
          (_b = (_a = getLifecycleHooks('CallTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CallTask.
       * Throws if invalid.
       */
      CallTask.prototype.validate = function (workflow) {
          var copy = new CallTask(this);
          validate('CallTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CallTask.
       * Creates a copy of the CallTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CallTask instance.
       */
      CallTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new CallTask(this);
          return ((_b = (_a = getLifecycleHooks('CallTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CallTask;
  }(ObjectHydrator));
  var _CallTask = CallTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Catalog with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Catalog = /** @class */ (function (_super) {
      __extends(Catalog, _super);
      /**
       * Instanciates a new instance of the Catalog class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Catalog.
       */
      function Catalog(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.endpoint === 'object')
                  self.endpoint = new _Endpoint(model.endpoint);
          }
          (_b = (_a = getLifecycleHooks('Catalog')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Catalog.
       * Throws if invalid.
       */
      Catalog.prototype.validate = function (workflow) {
          var copy = new Catalog(this);
          validate('Catalog', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Catalog.
       * Creates a copy of the Catalog, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Catalog instance.
       */
      Catalog.prototype.normalize = function () {
          var _a, _b;
          var copy = new Catalog(this);
          return ((_b = (_a = getLifecycleHooks('Catalog')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Catalog;
  }(ObjectHydrator));
  var _Catalog = Catalog;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ErrorFilter with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ErrorFilter = /** @class */ (function (_super) {
      __extends(ErrorFilter, _super);
      /**
       * Instanciates a new instance of the ErrorFilter class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ErrorFilter.
       */
      function ErrorFilter(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ErrorFilter')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ErrorFilter.
       * Throws if invalid.
       */
      ErrorFilter.prototype.validate = function (workflow) {
          var copy = new ErrorFilter(this);
          validate('ErrorFilter', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ErrorFilter.
       * Creates a copy of the ErrorFilter, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ErrorFilter instance.
       */
      ErrorFilter.prototype.normalize = function () {
          var _a, _b;
          var copy = new ErrorFilter(this);
          return ((_b = (_a = getLifecycleHooks('ErrorFilter')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ErrorFilter;
  }(ObjectHydrator));
  var _ErrorFilter = ErrorFilter;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a CatchErrors with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var CatchErrors = /** @class */ (function (_super) {
      __extends(CatchErrors, _super);
      /**
       * Instanciates a new instance of the CatchErrors class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the CatchErrors.
       */
      function CatchErrors(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.with === 'object')
                  self.with = new _ErrorFilter(model.with);
          }
          (_b = (_a = getLifecycleHooks('CatchErrors')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the CatchErrors.
       * Throws if invalid.
       */
      CatchErrors.prototype.validate = function (workflow) {
          var copy = new CatchErrors(this);
          validate('CatchErrors', copy, workflow);
      };
      /**
       * Normalizes the current instance of the CatchErrors.
       * Creates a copy of the CatchErrors, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the CatchErrors instance.
       */
      CatchErrors.prototype.normalize = function () {
          var _a, _b;
          var copy = new CatchErrors(this);
          return ((_b = (_a = getLifecycleHooks('CatchErrors')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return CatchErrors;
  }(ObjectHydrator));
  var _CatchErrors = CatchErrors;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ConstantBackoff with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ConstantBackoff = /** @class */ (function (_super) {
      __extends(ConstantBackoff, _super);
      /**
       * Instanciates a new instance of the ConstantBackoff class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ConstantBackoff.
       */
      function ConstantBackoff(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ConstantBackoff')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ConstantBackoff.
       * Throws if invalid.
       */
      ConstantBackoff.prototype.validate = function (workflow) {
          var copy = new ConstantBackoff(this);
          validate('ConstantBackoff', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ConstantBackoff.
       * Creates a copy of the ConstantBackoff, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ConstantBackoff instance.
       */
      ConstantBackoff.prototype.normalize = function () {
          var _a, _b;
          var copy = new ConstantBackoff(this);
          return ((_b = (_a = getLifecycleHooks('ConstantBackoff')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ConstantBackoff;
  }(ObjectHydrator));
  var _ConstantBackoff = ConstantBackoff;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ContainerPorts with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ContainerPorts = /** @class */ (function (_super) {
      __extends(ContainerPorts, _super);
      /**
       * Instanciates a new instance of the ContainerPorts class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ContainerPorts.
       */
      function ContainerPorts(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ContainerPorts')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ContainerPorts.
       * Throws if invalid.
       */
      ContainerPorts.prototype.validate = function (workflow) {
          var copy = new ContainerPorts(this);
          validate('ContainerPorts', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ContainerPorts.
       * Creates a copy of the ContainerPorts, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ContainerPorts instance.
       */
      ContainerPorts.prototype.normalize = function () {
          var _a, _b;
          var copy = new ContainerPorts(this);
          return ((_b = (_a = getLifecycleHooks('ContainerPorts')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ContainerPorts;
  }(ObjectHydrator));
  var _ContainerPorts = ContainerPorts;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ContainerVolumes with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ContainerVolumes = /** @class */ (function (_super) {
      __extends(ContainerVolumes, _super);
      /**
       * Instanciates a new instance of the ContainerVolumes class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ContainerVolumes.
       */
      function ContainerVolumes(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ContainerVolumes')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ContainerVolumes.
       * Throws if invalid.
       */
      ContainerVolumes.prototype.validate = function (workflow) {
          var copy = new ContainerVolumes(this);
          validate('ContainerVolumes', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ContainerVolumes.
       * Creates a copy of the ContainerVolumes, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ContainerVolumes instance.
       */
      ContainerVolumes.prototype.normalize = function () {
          var _a, _b;
          var copy = new ContainerVolumes(this);
          return ((_b = (_a = getLifecycleHooks('ContainerVolumes')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ContainerVolumes;
  }(ObjectHydrator));
  var _ContainerVolumes = ContainerVolumes;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ContainerEnvironment with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ContainerEnvironment = /** @class */ (function (_super) {
      __extends(ContainerEnvironment, _super);
      /**
       * Instanciates a new instance of the ContainerEnvironment class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ContainerEnvironment.
       */
      function ContainerEnvironment(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ContainerEnvironment')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ContainerEnvironment.
       * Throws if invalid.
       */
      ContainerEnvironment.prototype.validate = function (workflow) {
          var copy = new ContainerEnvironment(this);
          validate('ContainerEnvironment', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ContainerEnvironment.
       * Creates a copy of the ContainerEnvironment, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ContainerEnvironment instance.
       */
      ContainerEnvironment.prototype.normalize = function () {
          var _a, _b;
          var copy = new ContainerEnvironment(this);
          return ((_b = (_a = getLifecycleHooks('ContainerEnvironment')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ContainerEnvironment;
  }(ObjectHydrator));
  var _ContainerEnvironment = ContainerEnvironment;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ContainerLifetime with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ContainerLifetime = /** @class */ (function (_super) {
      __extends(ContainerLifetime, _super);
      /**
       * Instanciates a new instance of the ContainerLifetime class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ContainerLifetime.
       */
      function ContainerLifetime(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.after === 'object')
                  self.after = new _Duration(model.after);
          }
          (_b = (_a = getLifecycleHooks('ContainerLifetime')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ContainerLifetime.
       * Throws if invalid.
       */
      ContainerLifetime.prototype.validate = function (workflow) {
          var copy = new ContainerLifetime(this);
          validate('ContainerLifetime', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ContainerLifetime.
       * Creates a copy of the ContainerLifetime, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ContainerLifetime instance.
       */
      ContainerLifetime.prototype.normalize = function () {
          var _a, _b;
          var copy = new ContainerLifetime(this);
          return ((_b = (_a = getLifecycleHooks('ContainerLifetime')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ContainerLifetime;
  }(ObjectHydrator));
  var _ContainerLifetime = ContainerLifetime;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Container with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Container = /** @class */ (function (_super) {
      __extends(Container, _super);
      /**
       * Instanciates a new instance of the Container class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Container.
       */
      function Container(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.ports === 'object')
                  self.ports = new _ContainerPorts(model.ports);
              if (typeof model.volumes === 'object')
                  self.volumes = new _ContainerVolumes(model.volumes);
              if (typeof model.environment === 'object')
                  self.environment = new _ContainerEnvironment(model.environment);
              if (typeof model.lifetime === 'object')
                  self.lifetime = new _ContainerLifetime(model.lifetime);
          }
          (_b = (_a = getLifecycleHooks('Container')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Container.
       * Throws if invalid.
       */
      Container.prototype.validate = function (workflow) {
          var copy = new Container(this);
          validate('Container', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Container.
       * Creates a copy of the Container, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Container instance.
       */
      Container.prototype.normalize = function () {
          var _a, _b;
          var copy = new Container(this);
          return ((_b = (_a = getLifecycleHooks('Container')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Container;
  }(ObjectHydrator));
  var _Container = Container;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a DigestAuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var DigestAuthenticationPolicy = /** @class */ (function (_super) {
      __extends(DigestAuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the DigestAuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the DigestAuthenticationPolicy.
       */
      function DigestAuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.digest === 'object')
                  self.digest = new _DigestAuthenticationPolicyConfiguration(model.digest);
          }
          (_b = (_a = getLifecycleHooks('DigestAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the DigestAuthenticationPolicy.
       * Throws if invalid.
       */
      DigestAuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new DigestAuthenticationPolicy(this);
          validate('DigestAuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the DigestAuthenticationPolicy.
       * Creates a copy of the DigestAuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the DigestAuthenticationPolicy instance.
       */
      DigestAuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new DigestAuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('DigestAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return DigestAuthenticationPolicy;
  }(ObjectHydrator));
  var _DigestAuthenticationPolicy = DigestAuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a DigestAuthenticationProperties with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var DigestAuthenticationProperties = /** @class */ (function (_super) {
      __extends(DigestAuthenticationProperties, _super);
      /**
       * Instanciates a new instance of the DigestAuthenticationProperties class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the DigestAuthenticationProperties.
       */
      function DigestAuthenticationProperties(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('DigestAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the DigestAuthenticationProperties.
       * Throws if invalid.
       */
      DigestAuthenticationProperties.prototype.validate = function (workflow) {
          var copy = new DigestAuthenticationProperties(this);
          validate('DigestAuthenticationProperties', copy, workflow);
      };
      /**
       * Normalizes the current instance of the DigestAuthenticationProperties.
       * Creates a copy of the DigestAuthenticationProperties, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the DigestAuthenticationProperties instance.
       */
      DigestAuthenticationProperties.prototype.normalize = function () {
          var _a, _b;
          var copy = new DigestAuthenticationProperties(this);
          return ((_b = (_a = getLifecycleHooks('DigestAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return DigestAuthenticationProperties;
  }(ObjectHydrator));
  var _DigestAuthenticationProperties = DigestAuthenticationProperties;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WorkflowTags with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WorkflowTags = /** @class */ (function (_super) {
      __extends(WorkflowTags, _super);
      /**
       * Instanciates a new instance of the WorkflowTags class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WorkflowTags.
       */
      function WorkflowTags(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('WorkflowTags')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WorkflowTags.
       * Throws if invalid.
       */
      WorkflowTags.prototype.validate = function (workflow) {
          var copy = new WorkflowTags(this);
          validate('WorkflowTags', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WorkflowTags.
       * Creates a copy of the WorkflowTags, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WorkflowTags instance.
       */
      WorkflowTags.prototype.normalize = function () {
          var _a, _b;
          var copy = new WorkflowTags(this);
          return ((_b = (_a = getLifecycleHooks('WorkflowTags')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WorkflowTags;
  }(ObjectHydrator));
  var _WorkflowTags = WorkflowTags;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WorkflowMetadata with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WorkflowMetadata = /** @class */ (function (_super) {
      __extends(WorkflowMetadata, _super);
      /**
       * Instanciates a new instance of the WorkflowMetadata class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WorkflowMetadata.
       */
      function WorkflowMetadata(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('WorkflowMetadata')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WorkflowMetadata.
       * Throws if invalid.
       */
      WorkflowMetadata.prototype.validate = function (workflow) {
          var copy = new WorkflowMetadata(this);
          validate('WorkflowMetadata', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WorkflowMetadata.
       * Creates a copy of the WorkflowMetadata, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WorkflowMetadata instance.
       */
      WorkflowMetadata.prototype.normalize = function () {
          var _a, _b;
          var copy = new WorkflowMetadata(this);
          return ((_b = (_a = getLifecycleHooks('WorkflowMetadata')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WorkflowMetadata;
  }(ObjectHydrator));
  var _WorkflowMetadata = WorkflowMetadata;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Document with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Document = /** @class */ (function (_super) {
      __extends(Document, _super);
      /**
       * Instanciates a new instance of the Document class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Document.
       */
      function Document(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.tags === 'object')
                  self.tags = new _WorkflowTags(model.tags);
              if (typeof model.metadata === 'object')
                  self.metadata = new _WorkflowMetadata(model.metadata);
          }
          (_b = (_a = getLifecycleHooks('Document')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Document.
       * Throws if invalid.
       */
      Document.prototype.validate = function (workflow) {
          var copy = new Document(this);
          validate('Document', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Document.
       * Creates a copy of the Document, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Document instance.
       */
      Document.prototype.normalize = function () {
          var _a, _b;
          var copy = new Document(this);
          return ((_b = (_a = getLifecycleHooks('Document')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Document;
  }(ObjectHydrator));
  var _Document = Document;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ForkTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ForkTaskConfiguration = /** @class */ (function (_super) {
      __extends(ForkTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the ForkTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ForkTaskConfiguration.
       */
      function ForkTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.branches === 'object')
                  self.branches = new _TaskList(model.branches);
          }
          (_b = (_a = getLifecycleHooks('ForkTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ForkTaskConfiguration.
       * Throws if invalid.
       */
      ForkTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new ForkTaskConfiguration(this);
          validate('ForkTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ForkTaskConfiguration.
       * Creates a copy of the ForkTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ForkTaskConfiguration instance.
       */
      ForkTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new ForkTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('ForkTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ForkTaskConfiguration;
  }(ObjectHydrator));
  var _ForkTaskConfiguration = ForkTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EmitEventWith with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EmitEventWith = /** @class */ (function (_super) {
      __extends(EmitEventWith, _super);
      /**
       * Instanciates a new instance of the EmitEventWith class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EmitEventWith.
       */
      function EmitEventWith(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EmitEventWith')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EmitEventWith.
       * Throws if invalid.
       */
      EmitEventWith.prototype.validate = function (workflow) {
          var copy = new EmitEventWith(this);
          validate('EmitEventWith', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EmitEventWith.
       * Creates a copy of the EmitEventWith, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EmitEventWith instance.
       */
      EmitEventWith.prototype.normalize = function () {
          var _a, _b;
          var copy = new EmitEventWith(this);
          return ((_b = (_a = getLifecycleHooks('EmitEventWith')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EmitEventWith;
  }(ObjectHydrator));
  var _EmitEventWith = EmitEventWith;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EmitEventDefinition with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EmitEventDefinition = /** @class */ (function (_super) {
      __extends(EmitEventDefinition, _super);
      /**
       * Instanciates a new instance of the EmitEventDefinition class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EmitEventDefinition.
       */
      function EmitEventDefinition(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.with === 'object')
                  self.with = new _EmitEventWith(model.with);
          }
          (_b = (_a = getLifecycleHooks('EmitEventDefinition')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EmitEventDefinition.
       * Throws if invalid.
       */
      EmitEventDefinition.prototype.validate = function (workflow) {
          var copy = new EmitEventDefinition(this);
          validate('EmitEventDefinition', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EmitEventDefinition.
       * Creates a copy of the EmitEventDefinition, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EmitEventDefinition instance.
       */
      EmitEventDefinition.prototype.normalize = function () {
          var _a, _b;
          var copy = new EmitEventDefinition(this);
          return ((_b = (_a = getLifecycleHooks('EmitEventDefinition')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EmitEventDefinition;
  }(ObjectHydrator));
  var _EmitEventDefinition = EmitEventDefinition;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EmitTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EmitTaskConfiguration = /** @class */ (function (_super) {
      __extends(EmitTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the EmitTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EmitTaskConfiguration.
       */
      function EmitTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.event === 'object')
                  self.event = new _EmitEventDefinition(model.event);
          }
          (_b = (_a = getLifecycleHooks('EmitTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EmitTaskConfiguration.
       * Throws if invalid.
       */
      EmitTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new EmitTaskConfiguration(this);
          validate('EmitTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EmitTaskConfiguration.
       * Creates a copy of the EmitTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EmitTaskConfiguration instance.
       */
      EmitTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new EmitTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('EmitTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EmitTaskConfiguration;
  }(ObjectHydrator));
  var _EmitTaskConfiguration = EmitTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ForTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ForTaskConfiguration = /** @class */ (function (_super) {
      __extends(ForTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the ForTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ForTaskConfiguration.
       */
      function ForTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ForTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ForTaskConfiguration.
       * Throws if invalid.
       */
      ForTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new ForTaskConfiguration(this);
          validate('ForTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ForTaskConfiguration.
       * Creates a copy of the ForTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ForTaskConfiguration instance.
       */
      ForTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new ForTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('ForTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ForTaskConfiguration;
  }(ObjectHydrator));
  var _ForTaskConfiguration = ForTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventConsumptionStrategy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventConsumptionStrategy = /** @class */ (function (_super) {
      __extends(EventConsumptionStrategy, _super);
      /**
       * Instanciates a new instance of the EventConsumptionStrategy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventConsumptionStrategy.
       */
      function EventConsumptionStrategy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.all === 'object')
                  self.all = new _AllEventConsumptionStrategyConfiguration(model
                      .all);
              if (typeof model.any === 'object')
                  self.any = new _AnyEventConsumptionStrategyConfiguration(model
                      .any);
              if (typeof model.until === 'object')
                  self.until = new _AnyEventConsumptionStrategyUntil(model.until);
              if (typeof model.one === 'object')
                  self.one = new _EventFilter(model.one);
          }
          (_b = (_a = getLifecycleHooks('EventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventConsumptionStrategy.
       * Throws if invalid.
       */
      EventConsumptionStrategy.prototype.validate = function (workflow) {
          var copy = new EventConsumptionStrategy(this);
          validate('EventConsumptionStrategy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventConsumptionStrategy.
       * Creates a copy of the EventConsumptionStrategy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventConsumptionStrategy instance.
       */
      EventConsumptionStrategy.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventConsumptionStrategy(this);
          return ((_b = (_a = getLifecycleHooks('EventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventConsumptionStrategy;
  }(ObjectHydrator));
  var _EventConsumptionStrategy = EventConsumptionStrategy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ListenTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ListenTaskConfiguration = /** @class */ (function (_super) {
      __extends(ListenTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the ListenTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ListenTaskConfiguration.
       */
      function ListenTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.to === 'object')
                  self.to = new _EventConsumptionStrategy(model.to);
          }
          (_b = (_a = getLifecycleHooks('ListenTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ListenTaskConfiguration.
       * Throws if invalid.
       */
      ListenTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new ListenTaskConfiguration(this);
          validate('ListenTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ListenTaskConfiguration.
       * Creates a copy of the ListenTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ListenTaskConfiguration instance.
       */
      ListenTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new ListenTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('ListenTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ListenTaskConfiguration;
  }(ObjectHydrator));
  var _ListenTaskConfiguration = ListenTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SubscriptionIterator with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SubscriptionIterator = /** @class */ (function (_super) {
      __extends(SubscriptionIterator, _super);
      /**
       * Instanciates a new instance of the SubscriptionIterator class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SubscriptionIterator.
       */
      function SubscriptionIterator(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.do === 'object')
                  self.do = new _TaskList(model.do);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
          }
          (_b = (_a = getLifecycleHooks('SubscriptionIterator')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SubscriptionIterator.
       * Throws if invalid.
       */
      SubscriptionIterator.prototype.validate = function (workflow) {
          var copy = new SubscriptionIterator(this);
          validate('SubscriptionIterator', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SubscriptionIterator.
       * Creates a copy of the SubscriptionIterator, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SubscriptionIterator instance.
       */
      SubscriptionIterator.prototype.normalize = function () {
          var _a, _b;
          var copy = new SubscriptionIterator(this);
          return ((_b = (_a = getLifecycleHooks('SubscriptionIterator')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SubscriptionIterator;
  }(ObjectHydrator));
  var _SubscriptionIterator = SubscriptionIterator;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RaiseTaskError with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RaiseTaskError = /** @class */ (function (_super) {
      __extends(RaiseTaskError, _super);
      /**
       * Instanciates a new instance of the RaiseTaskError class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RaiseTaskError.
       */
      function RaiseTaskError(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('RaiseTaskError')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RaiseTaskError.
       * Throws if invalid.
       */
      RaiseTaskError.prototype.validate = function (workflow) {
          var copy = new RaiseTaskError(this);
          validate('RaiseTaskError', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RaiseTaskError.
       * Creates a copy of the RaiseTaskError, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RaiseTaskError instance.
       */
      RaiseTaskError.prototype.normalize = function () {
          var _a, _b;
          var copy = new RaiseTaskError(this);
          return ((_b = (_a = getLifecycleHooks('RaiseTaskError')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RaiseTaskError;
  }(ObjectHydrator));
  var _RaiseTaskError = RaiseTaskError;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RaiseTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RaiseTaskConfiguration = /** @class */ (function (_super) {
      __extends(RaiseTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the RaiseTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RaiseTaskConfiguration.
       */
      function RaiseTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.error === 'object')
                  self.error = new _RaiseTaskError(model.error);
          }
          (_b = (_a = getLifecycleHooks('RaiseTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RaiseTaskConfiguration.
       * Throws if invalid.
       */
      RaiseTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new RaiseTaskConfiguration(this);
          validate('RaiseTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RaiseTaskConfiguration.
       * Creates a copy of the RaiseTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RaiseTaskConfiguration instance.
       */
      RaiseTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new RaiseTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('RaiseTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RaiseTaskConfiguration;
  }(ObjectHydrator));
  var _RaiseTaskConfiguration = RaiseTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Script with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Script = /** @class */ (function (_super) {
      __extends(Script, _super);
      /**
       * Instanciates a new instance of the Script class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Script.
       */
      function Script(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.source === 'object')
                  self.source = new _ExternalResource(model.source);
          }
          (_b = (_a = getLifecycleHooks('Script')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Script.
       * Throws if invalid.
       */
      Script.prototype.validate = function (workflow) {
          var copy = new Script(this);
          validate('Script', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Script.
       * Creates a copy of the Script, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Script instance.
       */
      Script.prototype.normalize = function () {
          var _a, _b;
          var copy = new Script(this);
          return ((_b = (_a = getLifecycleHooks('Script')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Script;
  }(ObjectHydrator));
  var _Script = Script;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ShellArguments with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ShellArguments = /** @class */ (function (_super) {
      __extends(ShellArguments, _super);
      /**
       * Instanciates a new instance of the ShellArguments class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ShellArguments.
       */
      function ShellArguments(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ShellArguments')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ShellArguments.
       * Throws if invalid.
       */
      ShellArguments.prototype.validate = function (workflow) {
          var copy = new ShellArguments(this);
          validate('ShellArguments', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ShellArguments.
       * Creates a copy of the ShellArguments, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ShellArguments instance.
       */
      ShellArguments.prototype.normalize = function () {
          var _a, _b;
          var copy = new ShellArguments(this);
          return ((_b = (_a = getLifecycleHooks('ShellArguments')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ShellArguments;
  }(ObjectHydrator));
  var _ShellArguments = ShellArguments;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ShellEnvironment with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ShellEnvironment = /** @class */ (function (_super) {
      __extends(ShellEnvironment, _super);
      /**
       * Instanciates a new instance of the ShellEnvironment class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ShellEnvironment.
       */
      function ShellEnvironment(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ShellEnvironment')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ShellEnvironment.
       * Throws if invalid.
       */
      ShellEnvironment.prototype.validate = function (workflow) {
          var copy = new ShellEnvironment(this);
          validate('ShellEnvironment', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ShellEnvironment.
       * Creates a copy of the ShellEnvironment, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ShellEnvironment instance.
       */
      ShellEnvironment.prototype.normalize = function () {
          var _a, _b;
          var copy = new ShellEnvironment(this);
          return ((_b = (_a = getLifecycleHooks('ShellEnvironment')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ShellEnvironment;
  }(ObjectHydrator));
  var _ShellEnvironment = ShellEnvironment;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Shell with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Shell = /** @class */ (function (_super) {
      __extends(Shell, _super);
      /**
       * Instanciates a new instance of the Shell class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Shell.
       */
      function Shell(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.arguments === 'object')
                  self.arguments = new _ShellArguments(model.arguments);
              if (typeof model.environment === 'object')
                  self.environment = new _ShellEnvironment(model.environment);
          }
          (_b = (_a = getLifecycleHooks('Shell')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Shell.
       * Throws if invalid.
       */
      Shell.prototype.validate = function (workflow) {
          var copy = new Shell(this);
          validate('Shell', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Shell.
       * Creates a copy of the Shell, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Shell instance.
       */
      Shell.prototype.normalize = function () {
          var _a, _b;
          var copy = new Shell(this);
          return ((_b = (_a = getLifecycleHooks('Shell')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Shell;
  }(ObjectHydrator));
  var _Shell = Shell;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SubflowInput with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SubflowInput = /** @class */ (function (_super) {
      __extends(SubflowInput, _super);
      /**
       * Instanciates a new instance of the SubflowInput class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SubflowInput.
       */
      function SubflowInput(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('SubflowInput')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SubflowInput.
       * Throws if invalid.
       */
      SubflowInput.prototype.validate = function (workflow) {
          var copy = new SubflowInput(this);
          validate('SubflowInput', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SubflowInput.
       * Creates a copy of the SubflowInput, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SubflowInput instance.
       */
      SubflowInput.prototype.normalize = function () {
          var _a, _b;
          var copy = new SubflowInput(this);
          return ((_b = (_a = getLifecycleHooks('SubflowInput')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SubflowInput;
  }(ObjectHydrator));
  var _SubflowInput = SubflowInput;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SubflowConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SubflowConfiguration = /** @class */ (function (_super) {
      __extends(SubflowConfiguration, _super);
      /**
       * Instanciates a new instance of the SubflowConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SubflowConfiguration.
       */
      function SubflowConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _SubflowInput(model.input);
          }
          (_b = (_a = getLifecycleHooks('SubflowConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SubflowConfiguration.
       * Throws if invalid.
       */
      SubflowConfiguration.prototype.validate = function (workflow) {
          var copy = new SubflowConfiguration(this);
          validate('SubflowConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SubflowConfiguration.
       * Creates a copy of the SubflowConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SubflowConfiguration instance.
       */
      SubflowConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new SubflowConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('SubflowConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SubflowConfiguration;
  }(ObjectHydrator));
  var _SubflowConfiguration = SubflowConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RunTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RunTaskConfiguration = /** @class */ (function (_super) {
      __extends(RunTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the RunTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RunTaskConfiguration.
       */
      function RunTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.container === 'object')
                  self.container = new _Container(model.container);
              if (typeof model.script === 'object')
                  self.script = new _Script(model.script);
              if (typeof model.shell === 'object')
                  self.shell = new _Shell(model.shell);
              if (typeof model.workflow === 'object')
                  self.workflow = new _SubflowConfiguration(model.workflow);
          }
          (_b = (_a = getLifecycleHooks('RunTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RunTaskConfiguration.
       * Throws if invalid.
       */
      RunTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new RunTaskConfiguration(this);
          validate('RunTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RunTaskConfiguration.
       * Creates a copy of the RunTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RunTaskConfiguration instance.
       */
      RunTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new RunTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('RunTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RunTaskConfiguration;
  }(ObjectHydrator));
  var _RunTaskConfiguration = RunTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SetTaskConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SetTaskConfiguration = /** @class */ (function (_super) {
      __extends(SetTaskConfiguration, _super);
      /**
       * Instanciates a new instance of the SetTaskConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SetTaskConfiguration.
       */
      function SetTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('SetTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SetTaskConfiguration.
       * Throws if invalid.
       */
      SetTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new SetTaskConfiguration(this);
          validate('SetTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SetTaskConfiguration.
       * Creates a copy of the SetTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SetTaskConfiguration instance.
       */
      SetTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new SetTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('SetTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SetTaskConfiguration;
  }(ObjectHydrator));
  var _SetTaskConfiguration = SetTaskConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SwitchCase with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SwitchCase = /** @class */ (function (_super) {
      __extends(SwitchCase, _super);
      /**
       * Instanciates a new instance of the SwitchCase class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SwitchCase.
       */
      function SwitchCase(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('SwitchCase')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SwitchCase.
       * Throws if invalid.
       */
      SwitchCase.prototype.validate = function (workflow) {
          var copy = new SwitchCase(this);
          validate('SwitchCase', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SwitchCase.
       * Creates a copy of the SwitchCase, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SwitchCase instance.
       */
      SwitchCase.prototype.normalize = function () {
          var _a, _b;
          var copy = new SwitchCase(this);
          return ((_b = (_a = getLifecycleHooks('SwitchCase')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SwitchCase;
  }(ObjectHydrator));
  var _SwitchCase = SwitchCase;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SwitchItem with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SwitchItem = /** @class */ (function (_super) {
      __extends(SwitchItem, _super);
      /**
       * Instanciates a new instance of the SwitchItem class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SwitchItem.
       */
      function SwitchItem(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _SwitchCase(value);
              });
          }
          (_b = (_a = getLifecycleHooks('SwitchItem')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SwitchItem.
       * Throws if invalid.
       */
      SwitchItem.prototype.validate = function (workflow) {
          var copy = new SwitchItem(this);
          validate('SwitchItem', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SwitchItem.
       * Creates a copy of the SwitchItem, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SwitchItem instance.
       */
      SwitchItem.prototype.normalize = function () {
          var _a, _b;
          var copy = new SwitchItem(this);
          return ((_b = (_a = getLifecycleHooks('SwitchItem')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SwitchItem;
  }(ObjectHydrator));
  var _SwitchItem = SwitchItem;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of Specification.SwitchItem.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var SwitchTaskConfiguration = /** @class */ (function (_super) {
      __extends(SwitchTaskConfiguration, _super);
      /**
       * Constructs a new instance of the SwitchTaskConfiguration class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function SwitchTaskConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(new _SwitchItem(item)); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(SwitchTaskConfiguration.prototype));
          (_b = (_a = getLifecycleHooks('SwitchTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SwitchTaskConfiguration.
       * Throws if invalid.
       */
      SwitchTaskConfiguration.prototype.validate = function (workflow) {
          var copy = new SwitchTaskConfiguration(this);
          validate('SwitchTaskConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SwitchTaskConfiguration.
       * Creates a copy of the SwitchTaskConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SwitchTaskConfiguration instance.
       */
      SwitchTaskConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new SwitchTaskConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('SwitchTaskConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SwitchTaskConfiguration;
  }(ArrayHydrator));
  var _SwitchTaskConfiguration = SwitchTaskConfiguration;
  //export const _SwitchTaskConfiguration = SwitchTaskConfiguration; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RetryBackoff with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RetryBackoff = /** @class */ (function (_super) {
      __extends(RetryBackoff, _super);
      /**
       * Instanciates a new instance of the RetryBackoff class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RetryBackoff.
       */
      function RetryBackoff(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('RetryBackoff')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RetryBackoff.
       * Throws if invalid.
       */
      RetryBackoff.prototype.validate = function (workflow) {
          var copy = new RetryBackoff(this);
          validate('RetryBackoff', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RetryBackoff.
       * Creates a copy of the RetryBackoff, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RetryBackoff instance.
       */
      RetryBackoff.prototype.normalize = function () {
          var _a, _b;
          var copy = new RetryBackoff(this);
          return ((_b = (_a = getLifecycleHooks('RetryBackoff')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RetryBackoff;
  }(ObjectHydrator));
  var _RetryBackoff = RetryBackoff;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RetryLimitAttempt with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RetryLimitAttempt = /** @class */ (function (_super) {
      __extends(RetryLimitAttempt, _super);
      /**
       * Instanciates a new instance of the RetryLimitAttempt class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RetryLimitAttempt.
       */
      function RetryLimitAttempt(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.duration === 'object')
                  self.duration = new _Duration(model.duration);
          }
          (_b = (_a = getLifecycleHooks('RetryLimitAttempt')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RetryLimitAttempt.
       * Throws if invalid.
       */
      RetryLimitAttempt.prototype.validate = function (workflow) {
          var copy = new RetryLimitAttempt(this);
          validate('RetryLimitAttempt', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RetryLimitAttempt.
       * Creates a copy of the RetryLimitAttempt, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RetryLimitAttempt instance.
       */
      RetryLimitAttempt.prototype.normalize = function () {
          var _a, _b;
          var copy = new RetryLimitAttempt(this);
          return ((_b = (_a = getLifecycleHooks('RetryLimitAttempt')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RetryLimitAttempt;
  }(ObjectHydrator));
  var _RetryLimitAttempt = RetryLimitAttempt;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RetryLimit with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RetryLimit = /** @class */ (function (_super) {
      __extends(RetryLimit, _super);
      /**
       * Instanciates a new instance of the RetryLimit class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RetryLimit.
       */
      function RetryLimit(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.attempt === 'object')
                  self.attempt = new _RetryLimitAttempt(model.attempt);
              if (typeof model.duration === 'object')
                  self.duration = new _Duration(model.duration);
          }
          (_b = (_a = getLifecycleHooks('RetryLimit')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RetryLimit.
       * Throws if invalid.
       */
      RetryLimit.prototype.validate = function (workflow) {
          var copy = new RetryLimit(this);
          validate('RetryLimit', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RetryLimit.
       * Creates a copy of the RetryLimit, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RetryLimit instance.
       */
      RetryLimit.prototype.normalize = function () {
          var _a, _b;
          var copy = new RetryLimit(this);
          return ((_b = (_a = getLifecycleHooks('RetryLimit')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RetryLimit;
  }(ObjectHydrator));
  var _RetryLimit = RetryLimit;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RetryPolicyJitter with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RetryPolicyJitter = /** @class */ (function (_super) {
      __extends(RetryPolicyJitter, _super);
      /**
       * Instanciates a new instance of the RetryPolicyJitter class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RetryPolicyJitter.
       */
      function RetryPolicyJitter(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.from === 'object')
                  self.from = new _Duration(model.from);
              if (typeof model.to === 'object')
                  self.to = new _Duration(model.to);
          }
          (_b = (_a = getLifecycleHooks('RetryPolicyJitter')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RetryPolicyJitter.
       * Throws if invalid.
       */
      RetryPolicyJitter.prototype.validate = function (workflow) {
          var copy = new RetryPolicyJitter(this);
          validate('RetryPolicyJitter', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RetryPolicyJitter.
       * Creates a copy of the RetryPolicyJitter, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RetryPolicyJitter instance.
       */
      RetryPolicyJitter.prototype.normalize = function () {
          var _a, _b;
          var copy = new RetryPolicyJitter(this);
          return ((_b = (_a = getLifecycleHooks('RetryPolicyJitter')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RetryPolicyJitter;
  }(ObjectHydrator));
  var _RetryPolicyJitter = RetryPolicyJitter;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TryTaskCatchRetry with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TryTaskCatchRetry = /** @class */ (function (_super) {
      __extends(TryTaskCatchRetry, _super);
      /**
       * Instanciates a new instance of the TryTaskCatchRetry class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TryTaskCatchRetry.
       */
      function TryTaskCatchRetry(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.delay === 'object')
                  self.delay = new _Duration(model.delay);
              if (typeof model.backoff === 'object')
                  self.backoff = new _RetryBackoff(model.backoff);
              if (typeof model.limit === 'object')
                  self.limit = new _RetryLimit(model.limit);
              if (typeof model.jitter === 'object')
                  self.jitter = new _RetryPolicyJitter(model.jitter);
          }
          (_b = (_a = getLifecycleHooks('TryTaskCatchRetry')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TryTaskCatchRetry.
       * Throws if invalid.
       */
      TryTaskCatchRetry.prototype.validate = function (workflow) {
          var copy = new TryTaskCatchRetry(this);
          validate('TryTaskCatchRetry', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TryTaskCatchRetry.
       * Creates a copy of the TryTaskCatchRetry, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TryTaskCatchRetry instance.
       */
      TryTaskCatchRetry.prototype.normalize = function () {
          var _a, _b;
          var copy = new TryTaskCatchRetry(this);
          return ((_b = (_a = getLifecycleHooks('TryTaskCatchRetry')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TryTaskCatchRetry;
  }(ObjectHydrator));
  var _TryTaskCatchRetry = TryTaskCatchRetry;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TryTaskCatch with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TryTaskCatch = /** @class */ (function (_super) {
      __extends(TryTaskCatch, _super);
      /**
       * Instanciates a new instance of the TryTaskCatch class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TryTaskCatch.
       */
      function TryTaskCatch(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.errors === 'object')
                  self.errors = new _CatchErrors(model.errors);
              if (typeof model.retry === 'object')
                  self.retry = new _TryTaskCatchRetry(model.retry);
              if (typeof model.do === 'object')
                  self.do = new _TaskList(model.do);
          }
          (_b = (_a = getLifecycleHooks('TryTaskCatch')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TryTaskCatch.
       * Throws if invalid.
       */
      TryTaskCatch.prototype.validate = function (workflow) {
          var copy = new TryTaskCatch(this);
          validate('TryTaskCatch', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TryTaskCatch.
       * Creates a copy of the TryTaskCatch, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TryTaskCatch instance.
       */
      TryTaskCatch.prototype.normalize = function () {
          var _a, _b;
          var copy = new TryTaskCatch(this);
          return ((_b = (_a = getLifecycleHooks('TryTaskCatch')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TryTaskCatch;
  }(ObjectHydrator));
  var _TryTaskCatch = TryTaskCatch;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Task with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Task = /** @class */ (function (_super) {
      __extends(Task, _super);
      /**
       * Instanciates a new instance of the Task class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Task.
       */
      function Task(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.fork === 'object')
                  self.fork =
                      new _ForkTaskConfiguration(model
                          .fork);
              if (typeof model.emit === 'object')
                  self.emit =
                      new _EmitTaskConfiguration(model
                          .emit);
              if (typeof model.for === 'object')
                  self.for = new _ForTaskConfiguration(model.for);
              if (typeof model.listen === 'object')
                  self.listen = new _ListenTaskConfiguration(model.listen);
              if (typeof model.foreach === 'object')
                  self.foreach = new _SubscriptionIterator(model.foreach);
              if (typeof model.raise === 'object')
                  self.raise =
                      new _RaiseTaskConfiguration(model
                          .raise);
              if (typeof model.run === 'object')
                  self.run = new _RunTaskConfiguration(model
                      .run);
              if (typeof model.set === 'object')
                  self.set = new _SetTaskConfiguration(model
                      .set);
              if (typeof model.switch === 'object')
                  self.switch =
                      new _SwitchTaskConfiguration(model
                          .switch);
              if (typeof model
                  .try === 'object')
                  self.try =
                      new _TaskList(model
                          .try);
              if (typeof model
                  .catch === 'object')
                  self.catch =
                      new _TryTaskCatch(model
                          .catch);
              if (typeof model.wait === 'object')
                  self.wait = new _Duration(model.wait);
          }
          (_b = (_a = getLifecycleHooks('Task')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Task.
       * Throws if invalid.
       */
      Task.prototype.validate = function (workflow) {
          var copy = new Task(this);
          validate('Task', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Task.
       * Creates a copy of the Task, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Task instance.
       */
      Task.prototype.normalize = function () {
          var _a, _b;
          var copy = new Task(this);
          return ((_b = (_a = getLifecycleHooks('Task')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Task;
  }(ObjectHydrator));
  var _Task = Task;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TaskItem with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TaskItem = /** @class */ (function (_super) {
      __extends(TaskItem, _super);
      /**
       * Instanciates a new instance of the TaskItem class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TaskItem.
       */
      function TaskItem(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _Task(value);
              });
          }
          (_b = (_a = getLifecycleHooks('TaskItem')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TaskItem.
       * Throws if invalid.
       */
      TaskItem.prototype.validate = function (workflow) {
          var copy = new TaskItem(this);
          validate('TaskItem', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TaskItem.
       * Creates a copy of the TaskItem, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TaskItem instance.
       */
      TaskItem.prototype.normalize = function () {
          var _a, _b;
          var copy = new TaskItem(this);
          return ((_b = (_a = getLifecycleHooks('TaskItem')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TaskItem;
  }(ObjectHydrator));
  var _TaskItem = TaskItem;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of Specification.TaskItem.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var TaskList = /** @class */ (function (_super) {
      __extends(TaskList, _super);
      /**
       * Constructs a new instance of the TaskList class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function TaskList(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(new _TaskItem(item)); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(TaskList.prototype));
          (_b = (_a = getLifecycleHooks('TaskList')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TaskList.
       * Throws if invalid.
       */
      TaskList.prototype.validate = function (workflow) {
          var copy = new TaskList(this);
          validate('TaskList', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TaskList.
       * Creates a copy of the TaskList, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TaskList instance.
       */
      TaskList.prototype.normalize = function () {
          var _a, _b;
          var copy = new TaskList(this);
          return ((_b = (_a = getLifecycleHooks('TaskList')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TaskList;
  }(ArrayHydrator));
  var _TaskList = TaskList;
  //export const _TaskList = TaskList; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a DoTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var DoTask = /** @class */ (function (_super) {
      __extends(DoTask, _super);
      /**
       * Instanciates a new instance of the DoTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the DoTask.
       */
      function DoTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.do === 'object')
                  self.do = new _TaskList(model.do);
          }
          (_b = (_a = getLifecycleHooks('DoTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the DoTask.
       * Throws if invalid.
       */
      DoTask.prototype.validate = function (workflow) {
          var copy = new DoTask(this);
          validate('DoTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the DoTask.
       * Creates a copy of the DoTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the DoTask instance.
       */
      DoTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new DoTask(this);
          return ((_b = (_a = getLifecycleHooks('DoTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return DoTask;
  }(_TaskBase));
  var _DoTask = DoTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a DurationInline with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var DurationInline = /** @class */ (function (_super) {
      __extends(DurationInline, _super);
      /**
       * Instanciates a new instance of the DurationInline class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the DurationInline.
       */
      function DurationInline(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('DurationInline')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the DurationInline.
       * Throws if invalid.
       */
      DurationInline.prototype.validate = function (workflow) {
          var copy = new DurationInline(this);
          validate('DurationInline', copy, workflow);
      };
      /**
       * Normalizes the current instance of the DurationInline.
       * Creates a copy of the DurationInline, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the DurationInline instance.
       */
      DurationInline.prototype.normalize = function () {
          var _a, _b;
          var copy = new DurationInline(this);
          return ((_b = (_a = getLifecycleHooks('DurationInline')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return DurationInline;
  }(ObjectHydrator));
  var _DurationInline = DurationInline;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EmitTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EmitTask = /** @class */ (function (_super) {
      __extends(EmitTask, _super);
      /**
       * Instanciates a new instance of the EmitTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EmitTask.
       */
      function EmitTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.emit === 'object')
                  self.emit = new _EmitTaskConfiguration(model.emit);
          }
          (_b = (_a = getLifecycleHooks('EmitTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EmitTask.
       * Throws if invalid.
       */
      EmitTask.prototype.validate = function (workflow) {
          var copy = new EmitTask(this);
          validate('EmitTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EmitTask.
       * Creates a copy of the EmitTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EmitTask instance.
       */
      EmitTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new EmitTask(this);
          return ((_b = (_a = getLifecycleHooks('EmitTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EmitTask;
  }(_TaskBase));
  var _EmitTask = EmitTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EndpointConfiguration with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EndpointConfiguration = /** @class */ (function (_super) {
      __extends(EndpointConfiguration, _super);
      /**
       * Instanciates a new instance of the EndpointConfiguration class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EndpointConfiguration.
       */
      function EndpointConfiguration(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.authentication === 'object')
                  self.authentication = new _ReferenceableAuthenticationPolicy(model.authentication);
          }
          (_b = (_a = getLifecycleHooks('EndpointConfiguration')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EndpointConfiguration.
       * Throws if invalid.
       */
      EndpointConfiguration.prototype.validate = function (workflow) {
          var copy = new EndpointConfiguration(this);
          validate('EndpointConfiguration', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EndpointConfiguration.
       * Creates a copy of the EndpointConfiguration, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EndpointConfiguration instance.
       */
      EndpointConfiguration.prototype.normalize = function () {
          var _a, _b;
          var copy = new EndpointConfiguration(this);
          return ((_b = (_a = getLifecycleHooks('EndpointConfiguration')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EndpointConfiguration;
  }(ObjectHydrator));
  var _EndpointConfiguration = EndpointConfiguration;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EndpointUri with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EndpointUri = /** @class */ (function (_super) {
      __extends(EndpointUri, _super);
      /**
       * Instanciates a new instance of the EndpointUri class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EndpointUri.
       */
      function EndpointUri(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EndpointUri')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EndpointUri.
       * Throws if invalid.
       */
      EndpointUri.prototype.validate = function (workflow) {
          var copy = new EndpointUri(this);
          validate('EndpointUri', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EndpointUri.
       * Creates a copy of the EndpointUri, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EndpointUri instance.
       */
      EndpointUri.prototype.normalize = function () {
          var _a, _b;
          var copy = new EndpointUri(this);
          return ((_b = (_a = getLifecycleHooks('EndpointUri')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EndpointUri;
  }(ObjectHydrator));
  var _EndpointUri = EndpointUri;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Error with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Error$1 = /** @class */ (function (_super) {
      __extends(Error, _super);
      /**
       * Instanciates a new instance of the Error class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Error.
       */
      function Error(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('Error')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Error.
       * Throws if invalid.
       */
      Error.prototype.validate = function (workflow) {
          var copy = new Error(this);
          validate('Error', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Error.
       * Creates a copy of the Error, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Error instance.
       */
      Error.prototype.normalize = function () {
          var _a, _b;
          var copy = new Error(this);
          return ((_b = (_a = getLifecycleHooks('Error')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Error;
  }(ObjectHydrator));
  var _Error = Error$1;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ErrorInstance with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ErrorInstance = /** @class */ (function (_super) {
      __extends(ErrorInstance, _super);
      /**
       * Instanciates a new instance of the ErrorInstance class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ErrorInstance.
       */
      function ErrorInstance(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ErrorInstance')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ErrorInstance.
       * Throws if invalid.
       */
      ErrorInstance.prototype.validate = function (workflow) {
          var copy = new ErrorInstance(this);
          validate('ErrorInstance', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ErrorInstance.
       * Creates a copy of the ErrorInstance, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ErrorInstance instance.
       */
      ErrorInstance.prototype.normalize = function () {
          var _a, _b;
          var copy = new ErrorInstance(this);
          return ((_b = (_a = getLifecycleHooks('ErrorInstance')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ErrorInstance;
  }(ObjectHydrator));
  var _ErrorInstance = ErrorInstance;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ErrorType with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ErrorType = /** @class */ (function (_super) {
      __extends(ErrorType, _super);
      /**
       * Instanciates a new instance of the ErrorType class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ErrorType.
       */
      function ErrorType(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ErrorType')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ErrorType.
       * Throws if invalid.
       */
      ErrorType.prototype.validate = function (workflow) {
          var copy = new ErrorType(this);
          validate('ErrorType', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ErrorType.
       * Creates a copy of the ErrorType, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ErrorType instance.
       */
      ErrorType.prototype.normalize = function () {
          var _a, _b;
          var copy = new ErrorType(this);
          return ((_b = (_a = getLifecycleHooks('ErrorType')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ErrorType;
  }(ObjectHydrator));
  var _ErrorType = ErrorType;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventData with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventData = /** @class */ (function (_super) {
      __extends(EventData, _super);
      /**
       * Instanciates a new instance of the EventData class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventData.
       */
      function EventData(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EventData')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventData.
       * Throws if invalid.
       */
      EventData.prototype.validate = function (workflow) {
          var copy = new EventData(this);
          validate('EventData', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventData.
       * Creates a copy of the EventData, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventData instance.
       */
      EventData.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventData(this);
          return ((_b = (_a = getLifecycleHooks('EventData')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventData;
  }(ObjectHydrator));
  var _EventData = EventData;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventDataschema with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventDataschema = /** @class */ (function (_super) {
      __extends(EventDataschema, _super);
      /**
       * Instanciates a new instance of the EventDataschema class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventDataschema.
       */
      function EventDataschema(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EventDataschema')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventDataschema.
       * Throws if invalid.
       */
      EventDataschema.prototype.validate = function (workflow) {
          var copy = new EventDataschema(this);
          validate('EventDataschema', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventDataschema.
       * Creates a copy of the EventDataschema, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventDataschema instance.
       */
      EventDataschema.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventDataschema(this);
          return ((_b = (_a = getLifecycleHooks('EventDataschema')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventDataschema;
  }(ObjectHydrator));
  var _EventDataschema = EventDataschema;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventSource with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventSource = /** @class */ (function (_super) {
      __extends(EventSource, _super);
      /**
       * Instanciates a new instance of the EventSource class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventSource.
       */
      function EventSource(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EventSource')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventSource.
       * Throws if invalid.
       */
      EventSource.prototype.validate = function (workflow) {
          var copy = new EventSource(this);
          validate('EventSource', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventSource.
       * Creates a copy of the EventSource, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventSource instance.
       */
      EventSource.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventSource(this);
          return ((_b = (_a = getLifecycleHooks('EventSource')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventSource;
  }(ObjectHydrator));
  var _EventSource = EventSource;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a EventTime with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var EventTime = /** @class */ (function (_super) {
      __extends(EventTime, _super);
      /**
       * Instanciates a new instance of the EventTime class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the EventTime.
       */
      function EventTime(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('EventTime')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the EventTime.
       * Throws if invalid.
       */
      EventTime.prototype.validate = function (workflow) {
          var copy = new EventTime(this);
          validate('EventTime', copy, workflow);
      };
      /**
       * Normalizes the current instance of the EventTime.
       * Creates a copy of the EventTime, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the EventTime instance.
       */
      EventTime.prototype.normalize = function () {
          var _a, _b;
          var copy = new EventTime(this);
          return ((_b = (_a = getLifecycleHooks('EventTime')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return EventTime;
  }(ObjectHydrator));
  var _EventTime = EventTime;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ExponentialBackOff with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ExponentialBackOff = /** @class */ (function (_super) {
      __extends(ExponentialBackOff, _super);
      /**
       * Instanciates a new instance of the ExponentialBackOff class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ExponentialBackOff.
       */
      function ExponentialBackOff(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ExponentialBackOff')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ExponentialBackOff.
       * Throws if invalid.
       */
      ExponentialBackOff.prototype.validate = function (workflow) {
          var copy = new ExponentialBackOff(this);
          validate('ExponentialBackOff', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ExponentialBackOff.
       * Creates a copy of the ExponentialBackOff, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ExponentialBackOff instance.
       */
      ExponentialBackOff.prototype.normalize = function () {
          var _a, _b;
          var copy = new ExponentialBackOff(this);
          return ((_b = (_a = getLifecycleHooks('ExponentialBackOff')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ExponentialBackOff;
  }(ObjectHydrator));
  var _ExponentialBackOff = ExponentialBackOff;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ExportAs with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ExportAs = /** @class */ (function (_super) {
      __extends(ExportAs, _super);
      /**
       * Instanciates a new instance of the ExportAs class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ExportAs.
       */
      function ExportAs(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('ExportAs')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ExportAs.
       * Throws if invalid.
       */
      ExportAs.prototype.validate = function (workflow) {
          var copy = new ExportAs(this);
          validate('ExportAs', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ExportAs.
       * Creates a copy of the ExportAs, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ExportAs instance.
       */
      ExportAs.prototype.normalize = function () {
          var _a, _b;
          var copy = new ExportAs(this);
          return ((_b = (_a = getLifecycleHooks('ExportAs')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ExportAs;
  }(ObjectHydrator));
  var _ExportAs = ExportAs;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Extension with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Extension = /** @class */ (function (_super) {
      __extends(Extension, _super);
      /**
       * Instanciates a new instance of the Extension class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Extension.
       */
      function Extension(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.before === 'object')
                  self.before = new _TaskList(model.before);
              if (typeof model.after === 'object')
                  self.after = new _TaskList(model.after);
          }
          (_b = (_a = getLifecycleHooks('Extension')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Extension.
       * Throws if invalid.
       */
      Extension.prototype.validate = function (workflow) {
          var copy = new Extension(this);
          validate('Extension', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Extension.
       * Creates a copy of the Extension, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Extension instance.
       */
      Extension.prototype.normalize = function () {
          var _a, _b;
          var copy = new Extension(this);
          return ((_b = (_a = getLifecycleHooks('Extension')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Extension;
  }(ObjectHydrator));
  var _Extension = Extension;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ExtensionItem with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ExtensionItem = /** @class */ (function (_super) {
      __extends(ExtensionItem, _super);
      /**
       * Instanciates a new instance of the ExtensionItem class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ExtensionItem.
       */
      function ExtensionItem(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _Extension(value);
              });
          }
          (_b = (_a = getLifecycleHooks('ExtensionItem')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ExtensionItem.
       * Throws if invalid.
       */
      ExtensionItem.prototype.validate = function (workflow) {
          var copy = new ExtensionItem(this);
          validate('ExtensionItem', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ExtensionItem.
       * Creates a copy of the ExtensionItem, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ExtensionItem instance.
       */
      ExtensionItem.prototype.normalize = function () {
          var _a, _b;
          var copy = new ExtensionItem(this);
          return ((_b = (_a = getLifecycleHooks('ExtensionItem')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ExtensionItem;
  }(ObjectHydrator));
  var _ExtensionItem = ExtensionItem;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ExternalScript with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ExternalScript = /** @class */ (function (_super) {
      __extends(ExternalScript, _super);
      /**
       * Instanciates a new instance of the ExternalScript class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ExternalScript.
       */
      function ExternalScript(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.source === 'object')
                  self.source = new _ExternalResource(model.source);
          }
          (_b = (_a = getLifecycleHooks('ExternalScript')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ExternalScript.
       * Throws if invalid.
       */
      ExternalScript.prototype.validate = function (workflow) {
          var copy = new ExternalScript(this);
          validate('ExternalScript', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ExternalScript.
       * Creates a copy of the ExternalScript, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ExternalScript instance.
       */
      ExternalScript.prototype.normalize = function () {
          var _a, _b;
          var copy = new ExternalScript(this);
          return ((_b = (_a = getLifecycleHooks('ExternalScript')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ExternalScript;
  }(ObjectHydrator));
  var _ExternalScript = ExternalScript;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a FlowDirective with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var FlowDirective$1 = /** @class */ (function (_super) {
      __extends(FlowDirective, _super);
      /**
       * Instanciates a new instance of the FlowDirective class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the FlowDirective.
       */
      function FlowDirective(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('FlowDirective')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the FlowDirective.
       * Throws if invalid.
       */
      FlowDirective.prototype.validate = function (workflow) {
          var copy = new FlowDirective(this);
          validate('FlowDirective', copy, workflow);
      };
      /**
       * Normalizes the current instance of the FlowDirective.
       * Creates a copy of the FlowDirective, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the FlowDirective instance.
       */
      FlowDirective.prototype.normalize = function () {
          var _a, _b;
          var copy = new FlowDirective(this);
          return ((_b = (_a = getLifecycleHooks('FlowDirective')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return FlowDirective;
  }(ObjectHydrator));
  var _FlowDirective = FlowDirective$1;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ForkTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ForkTask = /** @class */ (function (_super) {
      __extends(ForkTask, _super);
      /**
       * Instanciates a new instance of the ForkTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ForkTask.
       */
      function ForkTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.fork === 'object')
                  self.fork = new _ForkTaskConfiguration(model.fork);
          }
          (_b = (_a = getLifecycleHooks('ForkTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ForkTask.
       * Throws if invalid.
       */
      ForkTask.prototype.validate = function (workflow) {
          var copy = new ForkTask(this);
          validate('ForkTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ForkTask.
       * Creates a copy of the ForkTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ForkTask instance.
       */
      ForkTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new ForkTask(this);
          return ((_b = (_a = getLifecycleHooks('ForkTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ForkTask;
  }(_TaskBase));
  var _ForkTask = ForkTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ForTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ForTask = /** @class */ (function (_super) {
      __extends(ForTask, _super);
      /**
       * Instanciates a new instance of the ForTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ForTask.
       */
      function ForTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.for === 'object')
                  self.for = new _ForTaskConfiguration(model.for);
              if (typeof model.do === 'object')
                  self.do = new _TaskList(model.do);
          }
          (_b = (_a = getLifecycleHooks('ForTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ForTask.
       * Throws if invalid.
       */
      ForTask.prototype.validate = function (workflow) {
          var copy = new ForTask(this);
          validate('ForTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ForTask.
       * Creates a copy of the ForTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ForTask instance.
       */
      ForTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new ForTask(this);
          return ((_b = (_a = getLifecycleHooks('ForTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ForTask;
  }(_TaskBase));
  var _ForTask = ForTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a InlineScript with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var InlineScript = /** @class */ (function (_super) {
      __extends(InlineScript, _super);
      /**
       * Instanciates a new instance of the InlineScript class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the InlineScript.
       */
      function InlineScript(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('InlineScript')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the InlineScript.
       * Throws if invalid.
       */
      InlineScript.prototype.validate = function (workflow) {
          var copy = new InlineScript(this);
          validate('InlineScript', copy, workflow);
      };
      /**
       * Normalizes the current instance of the InlineScript.
       * Creates a copy of the InlineScript, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the InlineScript instance.
       */
      InlineScript.prototype.normalize = function () {
          var _a, _b;
          var copy = new InlineScript(this);
          return ((_b = (_a = getLifecycleHooks('InlineScript')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return InlineScript;
  }(ObjectHydrator));
  var _InlineScript = InlineScript;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a InputFrom with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var InputFrom = /** @class */ (function (_super) {
      __extends(InputFrom, _super);
      /**
       * Instanciates a new instance of the InputFrom class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the InputFrom.
       */
      function InputFrom(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('InputFrom')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the InputFrom.
       * Throws if invalid.
       */
      InputFrom.prototype.validate = function (workflow) {
          var copy = new InputFrom(this);
          validate('InputFrom', copy, workflow);
      };
      /**
       * Normalizes the current instance of the InputFrom.
       * Creates a copy of the InputFrom, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the InputFrom instance.
       */
      InputFrom.prototype.normalize = function () {
          var _a, _b;
          var copy = new InputFrom(this);
          return ((_b = (_a = getLifecycleHooks('InputFrom')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return InputFrom;
  }(ObjectHydrator));
  var _InputFrom = InputFrom;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a LinearBackoff with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var LinearBackoff = /** @class */ (function (_super) {
      __extends(LinearBackoff, _super);
      /**
       * Instanciates a new instance of the LinearBackoff class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the LinearBackoff.
       */
      function LinearBackoff(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('LinearBackoff')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the LinearBackoff.
       * Throws if invalid.
       */
      LinearBackoff.prototype.validate = function (workflow) {
          var copy = new LinearBackoff(this);
          validate('LinearBackoff', copy, workflow);
      };
      /**
       * Normalizes the current instance of the LinearBackoff.
       * Creates a copy of the LinearBackoff, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the LinearBackoff instance.
       */
      LinearBackoff.prototype.normalize = function () {
          var _a, _b;
          var copy = new LinearBackoff(this);
          return ((_b = (_a = getLifecycleHooks('LinearBackoff')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return LinearBackoff;
  }(ObjectHydrator));
  var _LinearBackoff = LinearBackoff;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a ListenTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var ListenTask = /** @class */ (function (_super) {
      __extends(ListenTask, _super);
      /**
       * Instanciates a new instance of the ListenTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the ListenTask.
       */
      function ListenTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.listen === 'object')
                  self.listen = new _ListenTaskConfiguration(model.listen);
              if (typeof model.foreach === 'object')
                  self.foreach = new _SubscriptionIterator(model.foreach);
          }
          (_b = (_a = getLifecycleHooks('ListenTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the ListenTask.
       * Throws if invalid.
       */
      ListenTask.prototype.validate = function (workflow) {
          var copy = new ListenTask(this);
          validate('ListenTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the ListenTask.
       * Creates a copy of the ListenTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the ListenTask instance.
       */
      ListenTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new ListenTask(this);
          return ((_b = (_a = getLifecycleHooks('ListenTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return ListenTask;
  }(_TaskBase));
  var _ListenTask = ListenTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2AutenthicationData with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2AutenthicationData = /** @class */ (function (_super) {
      __extends(OAuth2AutenthicationData, _super);
      /**
       * Instanciates a new instance of the OAuth2AutenthicationData class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2AutenthicationData.
       */
      function OAuth2AutenthicationData(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.client === 'object')
                  self.client = new _OAuth2AutenthicationDataClient(model.client);
              if (typeof model.request === 'object')
                  self.request = new _OAuth2TokenRequest(model.request);
              if (typeof model.subject === 'object')
                  self.subject = new _OAuth2TokenDefinition(model.subject);
              if (typeof model.actor === 'object')
                  self.actor = new _OAuth2TokenDefinition(model.actor);
          }
          (_b = (_a = getLifecycleHooks('OAuth2AutenthicationData')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AutenthicationData.
       * Throws if invalid.
       */
      OAuth2AutenthicationData.prototype.validate = function (workflow) {
          var copy = new OAuth2AutenthicationData(this);
          validate('OAuth2AutenthicationData', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AutenthicationData.
       * Creates a copy of the OAuth2AutenthicationData, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AutenthicationData instance.
       */
      OAuth2AutenthicationData.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AutenthicationData(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AutenthicationData')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AutenthicationData;
  }(ObjectHydrator));
  var _OAuth2AutenthicationData = OAuth2AutenthicationData;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of string.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var OAuth2AutenthicationDataAudiences = /** @class */ (function (_super) {
      __extends(OAuth2AutenthicationDataAudiences, _super);
      /**
       * Constructs a new instance of the OAuth2AutenthicationDataAudiences class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function OAuth2AutenthicationDataAudiences(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(item); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(OAuth2AutenthicationDataAudiences.prototype));
          (_b = (_a = getLifecycleHooks('OAuth2AutenthicationDataAudiences')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AutenthicationDataAudiences.
       * Throws if invalid.
       */
      OAuth2AutenthicationDataAudiences.prototype.validate = function (workflow) {
          var copy = new OAuth2AutenthicationDataAudiences(this);
          validate('OAuth2AutenthicationDataAudiences', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AutenthicationDataAudiences.
       * Creates a copy of the OAuth2AutenthicationDataAudiences, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AutenthicationDataAudiences instance.
       */
      OAuth2AutenthicationDataAudiences.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AutenthicationDataAudiences(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AutenthicationDataAudiences')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AutenthicationDataAudiences;
  }(ArrayHydrator));
  var _OAuth2AutenthicationDataAudiences = OAuth2AutenthicationDataAudiences;
  //export const _OAuth2AutenthicationDataAudiences = OAuth2AutenthicationDataAudiences; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of string.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var OAuth2AutenthicationDataScopes = /** @class */ (function (_super) {
      __extends(OAuth2AutenthicationDataScopes, _super);
      /**
       * Constructs a new instance of the OAuth2AutenthicationDataScopes class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function OAuth2AutenthicationDataScopes(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(item); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(OAuth2AutenthicationDataScopes.prototype));
          (_b = (_a = getLifecycleHooks('OAuth2AutenthicationDataScopes')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AutenthicationDataScopes.
       * Throws if invalid.
       */
      OAuth2AutenthicationDataScopes.prototype.validate = function (workflow) {
          var copy = new OAuth2AutenthicationDataScopes(this);
          validate('OAuth2AutenthicationDataScopes', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AutenthicationDataScopes.
       * Creates a copy of the OAuth2AutenthicationDataScopes, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AutenthicationDataScopes instance.
       */
      OAuth2AutenthicationDataScopes.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AutenthicationDataScopes(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AutenthicationDataScopes')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AutenthicationDataScopes;
  }(ArrayHydrator));
  var _OAuth2AutenthicationDataScopes = OAuth2AutenthicationDataScopes;
  //export const _OAuth2AutenthicationDataScopes = OAuth2AutenthicationDataScopes; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2AuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2AuthenticationPolicy = /** @class */ (function (_super) {
      __extends(OAuth2AuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the OAuth2AuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2AuthenticationPolicy.
       */
      function OAuth2AuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.oauth2 === 'object')
                  self.oauth2 = new _OAuth2AuthenticationPolicyConfiguration(model.oauth2);
          }
          (_b = (_a = getLifecycleHooks('OAuth2AuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2AuthenticationPolicy.
       * Throws if invalid.
       */
      OAuth2AuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new OAuth2AuthenticationPolicy(this);
          validate('OAuth2AuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2AuthenticationPolicy.
       * Creates a copy of the OAuth2AuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2AuthenticationPolicy instance.
       */
      OAuth2AuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2AuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2AuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2AuthenticationPolicy;
  }(ObjectHydrator));
  var _OAuth2AuthenticationPolicy = OAuth2AuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OAuth2ConnectAuthenticationProperties with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OAuth2ConnectAuthenticationProperties = /** @class */ (function (_super) {
      __extends(OAuth2ConnectAuthenticationProperties, _super);
      /**
       * Instanciates a new instance of the OAuth2ConnectAuthenticationProperties class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OAuth2ConnectAuthenticationProperties.
       */
      function OAuth2ConnectAuthenticationProperties(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.client === 'object')
                  self.client = new _OAuth2AutenthicationDataClient(model.client);
              if (typeof model.request === 'object')
                  self.request = new _OAuth2TokenRequest(model.request);
              if (typeof model.subject === 'object')
                  self.subject = new _OAuth2TokenDefinition(model.subject);
              if (typeof model.actor === 'object')
                  self.actor = new _OAuth2TokenDefinition(model.actor);
              if (typeof model.endpoints === 'object')
                  self.endpoints = new _OAuth2AuthenticationPropertiesEndpoints(model.endpoints);
          }
          (_b = (_a = getLifecycleHooks('OAuth2ConnectAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2ConnectAuthenticationProperties.
       * Throws if invalid.
       */
      OAuth2ConnectAuthenticationProperties.prototype.validate = function (workflow) {
          var copy = new OAuth2ConnectAuthenticationProperties(this);
          validate('OAuth2ConnectAuthenticationProperties', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2ConnectAuthenticationProperties.
       * Creates a copy of the OAuth2ConnectAuthenticationProperties, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2ConnectAuthenticationProperties instance.
       */
      OAuth2ConnectAuthenticationProperties.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2ConnectAuthenticationProperties(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2ConnectAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2ConnectAuthenticationProperties;
  }(_OAuth2AutenthicationData));
  var _OAuth2ConnectAuthenticationProperties = OAuth2ConnectAuthenticationProperties;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of string.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var OAuth2Issuers = /** @class */ (function (_super) {
      __extends(OAuth2Issuers, _super);
      /**
       * Constructs a new instance of the OAuth2Issuers class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function OAuth2Issuers(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(item); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(OAuth2Issuers.prototype));
          (_b = (_a = getLifecycleHooks('OAuth2Issuers')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OAuth2Issuers.
       * Throws if invalid.
       */
      OAuth2Issuers.prototype.validate = function (workflow) {
          var copy = new OAuth2Issuers(this);
          validate('OAuth2Issuers', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OAuth2Issuers.
       * Creates a copy of the OAuth2Issuers, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OAuth2Issuers instance.
       */
      OAuth2Issuers.prototype.normalize = function () {
          var _a, _b;
          var copy = new OAuth2Issuers(this);
          return ((_b = (_a = getLifecycleHooks('OAuth2Issuers')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OAuth2Issuers;
  }(ArrayHydrator));
  var _OAuth2Issuers = OAuth2Issuers;
  //export const _OAuth2Issuers = OAuth2Issuers; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OneEventConsumptionStrategy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OneEventConsumptionStrategy = /** @class */ (function (_super) {
      __extends(OneEventConsumptionStrategy, _super);
      /**
       * Instanciates a new instance of the OneEventConsumptionStrategy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OneEventConsumptionStrategy.
       */
      function OneEventConsumptionStrategy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.one === 'object')
                  self.one = new _EventFilter(model.one);
          }
          (_b = (_a = getLifecycleHooks('OneEventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OneEventConsumptionStrategy.
       * Throws if invalid.
       */
      OneEventConsumptionStrategy.prototype.validate = function (workflow) {
          var copy = new OneEventConsumptionStrategy(this);
          validate('OneEventConsumptionStrategy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OneEventConsumptionStrategy.
       * Creates a copy of the OneEventConsumptionStrategy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OneEventConsumptionStrategy instance.
       */
      OneEventConsumptionStrategy.prototype.normalize = function () {
          var _a, _b;
          var copy = new OneEventConsumptionStrategy(this);
          return ((_b = (_a = getLifecycleHooks('OneEventConsumptionStrategy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OneEventConsumptionStrategy;
  }(ObjectHydrator));
  var _OneEventConsumptionStrategy = OneEventConsumptionStrategy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OpenIdConnectAuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OpenIdConnectAuthenticationPolicy = /** @class */ (function (_super) {
      __extends(OpenIdConnectAuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the OpenIdConnectAuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OpenIdConnectAuthenticationPolicy.
       */
      function OpenIdConnectAuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.oidc === 'object')
                  self.oidc = new _OpenIdConnectAuthenticationPolicyConfiguration(model.oidc);
          }
          (_b = (_a = getLifecycleHooks('OpenIdConnectAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OpenIdConnectAuthenticationPolicy.
       * Throws if invalid.
       */
      OpenIdConnectAuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new OpenIdConnectAuthenticationPolicy(this);
          validate('OpenIdConnectAuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OpenIdConnectAuthenticationPolicy.
       * Creates a copy of the OpenIdConnectAuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OpenIdConnectAuthenticationPolicy instance.
       */
      OpenIdConnectAuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new OpenIdConnectAuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('OpenIdConnectAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OpenIdConnectAuthenticationPolicy;
  }(ObjectHydrator));
  var _OpenIdConnectAuthenticationPolicy = OpenIdConnectAuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OpenIdConnectAuthenticationProperties with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OpenIdConnectAuthenticationProperties = /** @class */ (function (_super) {
      __extends(OpenIdConnectAuthenticationProperties, _super);
      /**
       * Instanciates a new instance of the OpenIdConnectAuthenticationProperties class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OpenIdConnectAuthenticationProperties.
       */
      function OpenIdConnectAuthenticationProperties(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.client === 'object')
                  self.client = new _OAuth2AutenthicationDataClient(model.client);
              if (typeof model.request === 'object')
                  self.request = new _OAuth2TokenRequest(model.request);
              if (typeof model.subject === 'object')
                  self.subject = new _OAuth2TokenDefinition(model.subject);
              if (typeof model.actor === 'object')
                  self.actor = new _OAuth2TokenDefinition(model.actor);
          }
          (_b = (_a = getLifecycleHooks('OpenIdConnectAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OpenIdConnectAuthenticationProperties.
       * Throws if invalid.
       */
      OpenIdConnectAuthenticationProperties.prototype.validate = function (workflow) {
          var copy = new OpenIdConnectAuthenticationProperties(this);
          validate('OpenIdConnectAuthenticationProperties', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OpenIdConnectAuthenticationProperties.
       * Creates a copy of the OpenIdConnectAuthenticationProperties, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OpenIdConnectAuthenticationProperties instance.
       */
      OpenIdConnectAuthenticationProperties.prototype.normalize = function () {
          var _a, _b;
          var copy = new OpenIdConnectAuthenticationProperties(this);
          return ((_b = (_a = getLifecycleHooks('OpenIdConnectAuthenticationProperties')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OpenIdConnectAuthenticationProperties;
  }(ObjectHydrator));
  var _OpenIdConnectAuthenticationProperties = OpenIdConnectAuthenticationProperties;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a OutputAs with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var OutputAs = /** @class */ (function (_super) {
      __extends(OutputAs, _super);
      /**
       * Instanciates a new instance of the OutputAs class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the OutputAs.
       */
      function OutputAs(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('OutputAs')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the OutputAs.
       * Throws if invalid.
       */
      OutputAs.prototype.validate = function (workflow) {
          var copy = new OutputAs(this);
          validate('OutputAs', copy, workflow);
      };
      /**
       * Normalizes the current instance of the OutputAs.
       * Creates a copy of the OutputAs, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the OutputAs instance.
       */
      OutputAs.prototype.normalize = function () {
          var _a, _b;
          var copy = new OutputAs(this);
          return ((_b = (_a = getLifecycleHooks('OutputAs')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return OutputAs;
  }(ObjectHydrator));
  var _OutputAs = OutputAs;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RaiseTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RaiseTask = /** @class */ (function (_super) {
      __extends(RaiseTask, _super);
      /**
       * Instanciates a new instance of the RaiseTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RaiseTask.
       */
      function RaiseTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.raise === 'object')
                  self.raise = new _RaiseTaskConfiguration(model.raise);
          }
          (_b = (_a = getLifecycleHooks('RaiseTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RaiseTask.
       * Throws if invalid.
       */
      RaiseTask.prototype.validate = function (workflow) {
          var copy = new RaiseTask(this);
          validate('RaiseTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RaiseTask.
       * Creates a copy of the RaiseTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RaiseTask instance.
       */
      RaiseTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new RaiseTask(this);
          return ((_b = (_a = getLifecycleHooks('RaiseTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RaiseTask;
  }(_TaskBase));
  var _RaiseTask = RaiseTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RetryPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RetryPolicy = /** @class */ (function (_super) {
      __extends(RetryPolicy, _super);
      /**
       * Instanciates a new instance of the RetryPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RetryPolicy.
       */
      function RetryPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.delay === 'object')
                  self.delay = new _Duration(model.delay);
              if (typeof model.backoff === 'object')
                  self.backoff = new _RetryBackoff(model.backoff);
              if (typeof model.limit === 'object')
                  self.limit = new _RetryLimit(model.limit);
              if (typeof model.jitter === 'object')
                  self.jitter = new _RetryPolicyJitter(model.jitter);
          }
          (_b = (_a = getLifecycleHooks('RetryPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RetryPolicy.
       * Throws if invalid.
       */
      RetryPolicy.prototype.validate = function (workflow) {
          var copy = new RetryPolicy(this);
          validate('RetryPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RetryPolicy.
       * Creates a copy of the RetryPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RetryPolicy instance.
       */
      RetryPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new RetryPolicy(this);
          return ((_b = (_a = getLifecycleHooks('RetryPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RetryPolicy;
  }(ObjectHydrator));
  var _RetryPolicy = RetryPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RunContainer with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RunContainer = /** @class */ (function (_super) {
      __extends(RunContainer, _super);
      /**
       * Instanciates a new instance of the RunContainer class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RunContainer.
       */
      function RunContainer(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.container === 'object')
                  self.container = new _Container(model.container);
          }
          (_b = (_a = getLifecycleHooks('RunContainer')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RunContainer.
       * Throws if invalid.
       */
      RunContainer.prototype.validate = function (workflow) {
          var copy = new RunContainer(this);
          validate('RunContainer', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RunContainer.
       * Creates a copy of the RunContainer, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RunContainer instance.
       */
      RunContainer.prototype.normalize = function () {
          var _a, _b;
          var copy = new RunContainer(this);
          return ((_b = (_a = getLifecycleHooks('RunContainer')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RunContainer;
  }(ObjectHydrator));
  var _RunContainer = RunContainer;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RunScript with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RunScript = /** @class */ (function (_super) {
      __extends(RunScript, _super);
      /**
       * Instanciates a new instance of the RunScript class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RunScript.
       */
      function RunScript(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.script === 'object')
                  self.script = new _Script(model.script);
          }
          (_b = (_a = getLifecycleHooks('RunScript')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RunScript.
       * Throws if invalid.
       */
      RunScript.prototype.validate = function (workflow) {
          var copy = new RunScript(this);
          validate('RunScript', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RunScript.
       * Creates a copy of the RunScript, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RunScript instance.
       */
      RunScript.prototype.normalize = function () {
          var _a, _b;
          var copy = new RunScript(this);
          return ((_b = (_a = getLifecycleHooks('RunScript')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RunScript;
  }(ObjectHydrator));
  var _RunScript = RunScript;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RunShell with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RunShell = /** @class */ (function (_super) {
      __extends(RunShell, _super);
      /**
       * Instanciates a new instance of the RunShell class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RunShell.
       */
      function RunShell(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.shell === 'object')
                  self.shell = new _Shell(model.shell);
          }
          (_b = (_a = getLifecycleHooks('RunShell')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RunShell.
       * Throws if invalid.
       */
      RunShell.prototype.validate = function (workflow) {
          var copy = new RunShell(this);
          validate('RunShell', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RunShell.
       * Creates a copy of the RunShell, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RunShell instance.
       */
      RunShell.prototype.normalize = function () {
          var _a, _b;
          var copy = new RunShell(this);
          return ((_b = (_a = getLifecycleHooks('RunShell')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RunShell;
  }(ObjectHydrator));
  var _RunShell = RunShell;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RunTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RunTask = /** @class */ (function (_super) {
      __extends(RunTask, _super);
      /**
       * Instanciates a new instance of the RunTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RunTask.
       */
      function RunTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.run === 'object')
                  self.run = new _RunTaskConfiguration(model.run);
          }
          (_b = (_a = getLifecycleHooks('RunTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RunTask.
       * Throws if invalid.
       */
      RunTask.prototype.validate = function (workflow) {
          var copy = new RunTask(this);
          validate('RunTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RunTask.
       * Creates a copy of the RunTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RunTask instance.
       */
      RunTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new RunTask(this);
          return ((_b = (_a = getLifecycleHooks('RunTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RunTask;
  }(_TaskBase));
  var _RunTask = RunTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RuntimeExpression with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RuntimeExpression = /** @class */ (function (_super) {
      __extends(RuntimeExpression, _super);
      /**
       * Instanciates a new instance of the RuntimeExpression class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RuntimeExpression.
       */
      function RuntimeExpression(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('RuntimeExpression')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RuntimeExpression.
       * Throws if invalid.
       */
      RuntimeExpression.prototype.validate = function (workflow) {
          var copy = new RuntimeExpression(this);
          validate('RuntimeExpression', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RuntimeExpression.
       * Creates a copy of the RuntimeExpression, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RuntimeExpression instance.
       */
      RuntimeExpression.prototype.normalize = function () {
          var _a, _b;
          var copy = new RuntimeExpression(this);
          return ((_b = (_a = getLifecycleHooks('RuntimeExpression')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RuntimeExpression;
  }(ObjectHydrator));
  var _RuntimeExpression = RuntimeExpression;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a RunWorkflow with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var RunWorkflow = /** @class */ (function (_super) {
      __extends(RunWorkflow, _super);
      /**
       * Instanciates a new instance of the RunWorkflow class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the RunWorkflow.
       */
      function RunWorkflow(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.workflow === 'object')
                  self.workflow = new _SubflowConfiguration(model.workflow);
          }
          (_b = (_a = getLifecycleHooks('RunWorkflow')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the RunWorkflow.
       * Throws if invalid.
       */
      RunWorkflow.prototype.validate = function (workflow) {
          var copy = new RunWorkflow(this);
          validate('RunWorkflow', copy, workflow);
      };
      /**
       * Normalizes the current instance of the RunWorkflow.
       * Creates a copy of the RunWorkflow, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the RunWorkflow instance.
       */
      RunWorkflow.prototype.normalize = function () {
          var _a, _b;
          var copy = new RunWorkflow(this);
          return ((_b = (_a = getLifecycleHooks('RunWorkflow')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return RunWorkflow;
  }(ObjectHydrator));
  var _RunWorkflow = RunWorkflow;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Schedule with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Schedule = /** @class */ (function (_super) {
      __extends(Schedule, _super);
      /**
       * Instanciates a new instance of the Schedule class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Schedule.
       */
      function Schedule(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.every === 'object')
                  self.every = new _Duration(model.every);
              if (typeof model.after === 'object')
                  self.after = new _Duration(model.after);
              if (typeof model.on === 'object')
                  self.on = new _EventConsumptionStrategy(model.on);
          }
          (_b = (_a = getLifecycleHooks('Schedule')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Schedule.
       * Throws if invalid.
       */
      Schedule.prototype.validate = function (workflow) {
          var copy = new Schedule(this);
          validate('Schedule', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Schedule.
       * Creates a copy of the Schedule, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Schedule instance.
       */
      Schedule.prototype.normalize = function () {
          var _a, _b;
          var copy = new Schedule(this);
          return ((_b = (_a = getLifecycleHooks('Schedule')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Schedule;
  }(ObjectHydrator));
  var _Schedule = Schedule;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SchemaExternal with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SchemaExternal = /** @class */ (function (_super) {
      __extends(SchemaExternal, _super);
      /**
       * Instanciates a new instance of the SchemaExternal class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SchemaExternal.
       */
      function SchemaExternal(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.resource === 'object')
                  self.resource = new _ExternalResource(model.resource);
          }
          (_b = (_a = getLifecycleHooks('SchemaExternal')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SchemaExternal.
       * Throws if invalid.
       */
      SchemaExternal.prototype.validate = function (workflow) {
          var copy = new SchemaExternal(this);
          validate('SchemaExternal', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SchemaExternal.
       * Creates a copy of the SchemaExternal, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SchemaExternal instance.
       */
      SchemaExternal.prototype.normalize = function () {
          var _a, _b;
          var copy = new SchemaExternal(this);
          return ((_b = (_a = getLifecycleHooks('SchemaExternal')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SchemaExternal;
  }(ObjectHydrator));
  var _SchemaExternal = SchemaExternal;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SchemaInline with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SchemaInline = /** @class */ (function (_super) {
      __extends(SchemaInline, _super);
      /**
       * Instanciates a new instance of the SchemaInline class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SchemaInline.
       */
      function SchemaInline(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('SchemaInline')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SchemaInline.
       * Throws if invalid.
       */
      SchemaInline.prototype.validate = function (workflow) {
          var copy = new SchemaInline(this);
          validate('SchemaInline', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SchemaInline.
       * Creates a copy of the SchemaInline, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SchemaInline instance.
       */
      SchemaInline.prototype.normalize = function () {
          var _a, _b;
          var copy = new SchemaInline(this);
          return ((_b = (_a = getLifecycleHooks('SchemaInline')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SchemaInline;
  }(ObjectHydrator));
  var _SchemaInline = SchemaInline;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SecretBasedAuthenticationPolicy with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SecretBasedAuthenticationPolicy = /** @class */ (function (_super) {
      __extends(SecretBasedAuthenticationPolicy, _super);
      /**
       * Instanciates a new instance of the SecretBasedAuthenticationPolicy class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SecretBasedAuthenticationPolicy.
       */
      function SecretBasedAuthenticationPolicy(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('SecretBasedAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SecretBasedAuthenticationPolicy.
       * Throws if invalid.
       */
      SecretBasedAuthenticationPolicy.prototype.validate = function (workflow) {
          var copy = new SecretBasedAuthenticationPolicy(this);
          validate('SecretBasedAuthenticationPolicy', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SecretBasedAuthenticationPolicy.
       * Creates a copy of the SecretBasedAuthenticationPolicy, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SecretBasedAuthenticationPolicy instance.
       */
      SecretBasedAuthenticationPolicy.prototype.normalize = function () {
          var _a, _b;
          var copy = new SecretBasedAuthenticationPolicy(this);
          return ((_b = (_a = getLifecycleHooks('SecretBasedAuthenticationPolicy')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SecretBasedAuthenticationPolicy;
  }(ObjectHydrator));
  var _SecretBasedAuthenticationPolicy = SecretBasedAuthenticationPolicy;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SetTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SetTask = /** @class */ (function (_super) {
      __extends(SetTask, _super);
      /**
       * Instanciates a new instance of the SetTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SetTask.
       */
      function SetTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.set === 'object')
                  self.set = new _SetTaskConfiguration(model.set);
          }
          (_b = (_a = getLifecycleHooks('SetTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SetTask.
       * Throws if invalid.
       */
      SetTask.prototype.validate = function (workflow) {
          var copy = new SetTask(this);
          validate('SetTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SetTask.
       * Creates a copy of the SetTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SetTask instance.
       */
      SetTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new SetTask(this);
          return ((_b = (_a = getLifecycleHooks('SetTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SetTask;
  }(_TaskBase));
  var _SetTask = SetTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a SwitchTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var SwitchTask = /** @class */ (function (_super) {
      __extends(SwitchTask, _super);
      /**
       * Instanciates a new instance of the SwitchTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the SwitchTask.
       */
      function SwitchTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.switch === 'object')
                  self.switch = new _SwitchTaskConfiguration(model.switch);
          }
          (_b = (_a = getLifecycleHooks('SwitchTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the SwitchTask.
       * Throws if invalid.
       */
      SwitchTask.prototype.validate = function (workflow) {
          var copy = new SwitchTask(this);
          validate('SwitchTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the SwitchTask.
       * Creates a copy of the SwitchTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the SwitchTask instance.
       */
      SwitchTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new SwitchTask(this);
          return ((_b = (_a = getLifecycleHooks('SwitchTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return SwitchTask;
  }(_TaskBase));
  var _SwitchTask = SwitchTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TaskBaseIf with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TaskBaseIf = /** @class */ (function (_super) {
      __extends(TaskBaseIf, _super);
      /**
       * Instanciates a new instance of the TaskBaseIf class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TaskBaseIf.
       */
      function TaskBaseIf(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('TaskBaseIf')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TaskBaseIf.
       * Throws if invalid.
       */
      TaskBaseIf.prototype.validate = function (workflow) {
          var copy = new TaskBaseIf(this);
          validate('TaskBaseIf', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TaskBaseIf.
       * Creates a copy of the TaskBaseIf, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TaskBaseIf instance.
       */
      TaskBaseIf.prototype.normalize = function () {
          var _a, _b;
          var copy = new TaskBaseIf(this);
          return ((_b = (_a = getLifecycleHooks('TaskBaseIf')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TaskBaseIf;
  }(ObjectHydrator));
  var _TaskBaseIf = TaskBaseIf;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Timeout with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Timeout = /** @class */ (function (_super) {
      __extends(Timeout, _super);
      /**
       * Instanciates a new instance of the Timeout class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Timeout.
       */
      function Timeout(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.after === 'object')
                  self.after = new _Duration(model.after);
          }
          (_b = (_a = getLifecycleHooks('Timeout')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Timeout.
       * Throws if invalid.
       */
      Timeout.prototype.validate = function (workflow) {
          var copy = new Timeout(this);
          validate('Timeout', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Timeout.
       * Creates a copy of the Timeout, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Timeout instance.
       */
      Timeout.prototype.normalize = function () {
          var _a, _b;
          var copy = new Timeout(this);
          return ((_b = (_a = getLifecycleHooks('Timeout')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Timeout;
  }(ObjectHydrator));
  var _Timeout = Timeout;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a TryTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var TryTask = /** @class */ (function (_super) {
      __extends(TryTask, _super);
      /**
       * Instanciates a new instance of the TryTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the TryTask.
       */
      function TryTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.try === 'object')
                  self.try = new _TaskList(model.try);
              if (typeof model.catch === 'object')
                  self.catch = new _TryTaskCatch(model.catch);
          }
          (_b = (_a = getLifecycleHooks('TryTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the TryTask.
       * Throws if invalid.
       */
      TryTask.prototype.validate = function (workflow) {
          var copy = new TryTask(this);
          validate('TryTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the TryTask.
       * Creates a copy of the TryTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the TryTask instance.
       */
      TryTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new TryTask(this);
          return ((_b = (_a = getLifecycleHooks('TryTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return TryTask;
  }(_TaskBase));
  var _TryTask = TryTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UriTemplate with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UriTemplate = /** @class */ (function (_super) {
      __extends(UriTemplate, _super);
      /**
       * Instanciates a new instance of the UriTemplate class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UriTemplate.
       */
      function UriTemplate(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          (_b = (_a = getLifecycleHooks('UriTemplate')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UriTemplate.
       * Throws if invalid.
       */
      UriTemplate.prototype.validate = function (workflow) {
          var copy = new UriTemplate(this);
          validate('UriTemplate', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UriTemplate.
       * Creates a copy of the UriTemplate, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UriTemplate instance.
       */
      UriTemplate.prototype.normalize = function () {
          var _a, _b;
          var copy = new UriTemplate(this);
          return ((_b = (_a = getLifecycleHooks('UriTemplate')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UriTemplate;
  }(ObjectHydrator));
  var _UriTemplate = UriTemplate;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UseAuthentications with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UseAuthentications = /** @class */ (function (_super) {
      __extends(UseAuthentications, _super);
      /**
       * Instanciates a new instance of the UseAuthentications class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UseAuthentications.
       */
      function UseAuthentications(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _AuthenticationPolicy(value);
              });
          }
          (_b = (_a = getLifecycleHooks('UseAuthentications')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseAuthentications.
       * Throws if invalid.
       */
      UseAuthentications.prototype.validate = function (workflow) {
          var copy = new UseAuthentications(this);
          validate('UseAuthentications', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseAuthentications.
       * Creates a copy of the UseAuthentications, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseAuthentications instance.
       */
      UseAuthentications.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseAuthentications(this);
          return ((_b = (_a = getLifecycleHooks('UseAuthentications')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseAuthentications;
  }(ObjectHydrator));
  var _UseAuthentications = UseAuthentications;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UseErrors with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UseErrors = /** @class */ (function (_super) {
      __extends(UseErrors, _super);
      /**
       * Instanciates a new instance of the UseErrors class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UseErrors.
       */
      function UseErrors(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _Error(value);
              });
          }
          (_b = (_a = getLifecycleHooks('UseErrors')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseErrors.
       * Throws if invalid.
       */
      UseErrors.prototype.validate = function (workflow) {
          var copy = new UseErrors(this);
          validate('UseErrors', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseErrors.
       * Creates a copy of the UseErrors, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseErrors instance.
       */
      UseErrors.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseErrors(this);
          return ((_b = (_a = getLifecycleHooks('UseErrors')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseErrors;
  }(ObjectHydrator));
  var _UseErrors = UseErrors;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of Specification.ExtensionItem.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var UseExtensions = /** @class */ (function (_super) {
      __extends(UseExtensions, _super);
      /**
       * Constructs a new instance of the UseExtensions class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function UseExtensions(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(new _ExtensionItem(item)); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(UseExtensions.prototype));
          (_b = (_a = getLifecycleHooks('UseExtensions')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseExtensions.
       * Throws if invalid.
       */
      UseExtensions.prototype.validate = function (workflow) {
          var copy = new UseExtensions(this);
          validate('UseExtensions', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseExtensions.
       * Creates a copy of the UseExtensions, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseExtensions instance.
       */
      UseExtensions.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseExtensions(this);
          return ((_b = (_a = getLifecycleHooks('UseExtensions')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseExtensions;
  }(ArrayHydrator));
  var _UseExtensions = UseExtensions;
  //export const _UseExtensions = UseExtensions; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UseFunctions with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UseFunctions = /** @class */ (function (_super) {
      __extends(UseFunctions, _super);
      /**
       * Instanciates a new instance of the UseFunctions class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UseFunctions.
       */
      function UseFunctions(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _Task(value);
              });
          }
          (_b = (_a = getLifecycleHooks('UseFunctions')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseFunctions.
       * Throws if invalid.
       */
      UseFunctions.prototype.validate = function (workflow) {
          var copy = new UseFunctions(this);
          validate('UseFunctions', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseFunctions.
       * Creates a copy of the UseFunctions, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseFunctions instance.
       */
      UseFunctions.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseFunctions(this);
          return ((_b = (_a = getLifecycleHooks('UseFunctions')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseFunctions;
  }(ObjectHydrator));
  var _UseFunctions = UseFunctions;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UseRetries with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UseRetries = /** @class */ (function (_super) {
      __extends(UseRetries, _super);
      /**
       * Instanciates a new instance of the UseRetries class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UseRetries.
       */
      function UseRetries(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _RetryPolicy(value);
              });
          }
          (_b = (_a = getLifecycleHooks('UseRetries')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseRetries.
       * Throws if invalid.
       */
      UseRetries.prototype.validate = function (workflow) {
          var copy = new UseRetries(this);
          validate('UseRetries', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseRetries.
       * Creates a copy of the UseRetries, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseRetries instance.
       */
      UseRetries.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseRetries(this);
          return ((_b = (_a = getLifecycleHooks('UseRetries')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseRetries;
  }(ObjectHydrator));
  var _UseRetries = UseRetries;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UseTimeouts with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UseTimeouts = /** @class */ (function (_super) {
      __extends(UseTimeouts, _super);
      /**
       * Instanciates a new instance of the UseTimeouts class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UseTimeouts.
       */
      function UseTimeouts(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _Timeout(value);
              });
          }
          (_b = (_a = getLifecycleHooks('UseTimeouts')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseTimeouts.
       * Throws if invalid.
       */
      UseTimeouts.prototype.validate = function (workflow) {
          var copy = new UseTimeouts(this);
          validate('UseTimeouts', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseTimeouts.
       * Creates a copy of the UseTimeouts, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseTimeouts instance.
       */
      UseTimeouts.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseTimeouts(this);
          return ((_b = (_a = getLifecycleHooks('UseTimeouts')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseTimeouts;
  }(ObjectHydrator));
  var _UseTimeouts = UseTimeouts;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a UseCatalogs with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var UseCatalogs = /** @class */ (function (_super) {
      __extends(UseCatalogs, _super);
      /**
       * Instanciates a new instance of the UseCatalogs class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the UseCatalogs.
       */
      function UseCatalogs(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              var knownProperties_1 = [];
              Object.entries(model)
                  .filter(function (_a) {
                  var key = _a[0];
                  return !knownProperties_1.includes(key);
              })
                  .forEach(function (_a) {
                  var key = _a[0], value = _a[1];
                  self[key] = new _Catalog(value);
              });
          }
          (_b = (_a = getLifecycleHooks('UseCatalogs')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseCatalogs.
       * Throws if invalid.
       */
      UseCatalogs.prototype.validate = function (workflow) {
          var copy = new UseCatalogs(this);
          validate('UseCatalogs', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseCatalogs.
       * Creates a copy of the UseCatalogs, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseCatalogs instance.
       */
      UseCatalogs.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseCatalogs(this);
          return ((_b = (_a = getLifecycleHooks('UseCatalogs')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseCatalogs;
  }(ObjectHydrator));
  var _UseCatalogs = UseCatalogs;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Use with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Use = /** @class */ (function (_super) {
      __extends(Use, _super);
      /**
       * Instanciates a new instance of the Use class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Use.
       */
      function Use(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.authentications === 'object')
                  self.authentications = new _UseAuthentications(model.authentications);
              if (typeof model.errors === 'object')
                  self.errors = new _UseErrors(model.errors);
              if (typeof model.extensions === 'object')
                  self.extensions = new _UseExtensions(model.extensions);
              if (typeof model.functions === 'object')
                  self.functions = new _UseFunctions(model.functions);
              if (typeof model.retries === 'object')
                  self.retries = new _UseRetries(model.retries);
              if (typeof model.timeouts === 'object')
                  self.timeouts = new _UseTimeouts(model.timeouts);
              if (typeof model.catalogs === 'object')
                  self.catalogs = new _UseCatalogs(model.catalogs);
          }
          (_b = (_a = getLifecycleHooks('Use')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Use.
       * Throws if invalid.
       */
      Use.prototype.validate = function (workflow) {
          var copy = new Use(this);
          validate('Use', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Use.
       * Creates a copy of the Use, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Use instance.
       */
      Use.prototype.normalize = function () {
          var _a, _b;
          var copy = new Use(this);
          return ((_b = (_a = getLifecycleHooks('Use')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return Use;
  }(ObjectHydrator));
  var _Use = Use;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a collection of string.
   * Inherits from ArrayHydrator to handle array-specific hydration.
   */
  var UseSecrets = /** @class */ (function (_super) {
      __extends(UseSecrets, _super);
      /**
       * Constructs a new instance of the UseSecrets class.
       *
       * @param model - Optional parameter which can be an array of objects or a number representing the array length.
       */
      function UseSecrets(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          if (Array.isArray(model)) {
              if (model === null || model === void 0 ? void 0 : model.length) {
                  _this.splice(0, _this.length);
                  model.forEach(function (item) { return _this.push(item); });
              }
          }
          Object.setPrototypeOf(_this, Object.create(UseSecrets.prototype));
          (_b = (_a = getLifecycleHooks('UseSecrets')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the UseSecrets.
       * Throws if invalid.
       */
      UseSecrets.prototype.validate = function (workflow) {
          var copy = new UseSecrets(this);
          validate('UseSecrets', copy, workflow);
      };
      /**
       * Normalizes the current instance of the UseSecrets.
       * Creates a copy of the UseSecrets, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the UseSecrets instance.
       */
      UseSecrets.prototype.normalize = function () {
          var _a, _b;
          var copy = new UseSecrets(this);
          return ((_b = (_a = getLifecycleHooks('UseSecrets')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return UseSecrets;
  }(ArrayHydrator));
  var _UseSecrets = UseSecrets;
  //export const _UseSecrets = UseSecrets; // could be exported directly, but it makes the job of building the index more straightforward as it's consistant with "object" classes

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WaitTask with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WaitTask = /** @class */ (function (_super) {
      __extends(WaitTask, _super);
      /**
       * Instanciates a new instance of the WaitTask class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WaitTask.
       */
      function WaitTask(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.export === 'object')
                  self.export = new _Export(model.export);
              if (typeof model.timeout === 'object')
                  self.timeout = new _TaskTimeout(model.timeout);
              if (typeof model.metadata === 'object')
                  self.metadata = new _TaskMetadata(model.metadata);
              if (typeof model.wait === 'object')
                  self.wait = new _Duration(model.wait);
          }
          (_b = (_a = getLifecycleHooks('WaitTask')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WaitTask.
       * Throws if invalid.
       */
      WaitTask.prototype.validate = function (workflow) {
          var copy = new WaitTask(this);
          validate('WaitTask', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WaitTask.
       * Creates a copy of the WaitTask, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WaitTask instance.
       */
      WaitTask.prototype.normalize = function () {
          var _a, _b;
          var copy = new WaitTask(this);
          return ((_b = (_a = getLifecycleHooks('WaitTask')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WaitTask;
  }(_TaskBase));
  var _WaitTask = WaitTask;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a WorkflowTimeout with methods for validation and normalization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var WorkflowTimeout = /** @class */ (function (_super) {
      __extends(WorkflowTimeout, _super);
      /**
       * Instanciates a new instance of the WorkflowTimeout class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the WorkflowTimeout.
       */
      function WorkflowTimeout(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.after === 'object')
                  self.after = new _Duration(model.after);
          }
          (_b = (_a = getLifecycleHooks('WorkflowTimeout')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the WorkflowTimeout.
       * Throws if invalid.
       */
      WorkflowTimeout.prototype.validate = function (workflow) {
          var copy = new WorkflowTimeout(this);
          validate('WorkflowTimeout', copy, workflow);
      };
      /**
       * Normalizes the current instance of the WorkflowTimeout.
       * Creates a copy of the WorkflowTimeout, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the WorkflowTimeout instance.
       */
      WorkflowTimeout.prototype.normalize = function () {
          var _a, _b;
          var copy = new WorkflowTimeout(this);
          return ((_b = (_a = getLifecycleHooks('WorkflowTimeout')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      return WorkflowTimeout;
  }(ObjectHydrator));
  var _WorkflowTimeout = WorkflowTimeout;

  /*! js-yaml 4.1.0 https://github.com/nodeca/js-yaml @license MIT */
  function isNothing(subject) {
    return (typeof subject === 'undefined') || (subject === null);
  }


  function isObject(subject) {
    return (typeof subject === 'object') && (subject !== null);
  }


  function toArray(sequence) {
    if (Array.isArray(sequence)) return sequence;
    else if (isNothing(sequence)) return [];

    return [ sequence ];
  }


  function extend(target, source) {
    var index, length, key, sourceKeys;

    if (source) {
      sourceKeys = Object.keys(source);

      for (index = 0, length = sourceKeys.length; index < length; index += 1) {
        key = sourceKeys[index];
        target[key] = source[key];
      }
    }

    return target;
  }


  function repeat(string, count) {
    var result = '', cycle;

    for (cycle = 0; cycle < count; cycle += 1) {
      result += string;
    }

    return result;
  }


  function isNegativeZero(number) {
    return (number === 0) && (Number.NEGATIVE_INFINITY === 1 / number);
  }


  var isNothing_1      = isNothing;
  var isObject_1       = isObject;
  var toArray_1        = toArray;
  var repeat_1         = repeat;
  var isNegativeZero_1 = isNegativeZero;
  var extend_1         = extend;

  var common = {
  	isNothing: isNothing_1,
  	isObject: isObject_1,
  	toArray: toArray_1,
  	repeat: repeat_1,
  	isNegativeZero: isNegativeZero_1,
  	extend: extend_1
  };

  // YAML error class. http://stackoverflow.com/questions/8458984


  function formatError(exception, compact) {
    var where = '', message = exception.reason || '(unknown reason)';

    if (!exception.mark) return message;

    if (exception.mark.name) {
      where += 'in "' + exception.mark.name + '" ';
    }

    where += '(' + (exception.mark.line + 1) + ':' + (exception.mark.column + 1) + ')';

    if (!compact && exception.mark.snippet) {
      where += '\n\n' + exception.mark.snippet;
    }

    return message + ' ' + where;
  }


  function YAMLException$1(reason, mark) {
    // Super constructor
    Error.call(this);

    this.name = 'YAMLException';
    this.reason = reason;
    this.mark = mark;
    this.message = formatError(this, false);

    // Include stack trace in error object
    if (Error.captureStackTrace) {
      // Chrome and NodeJS
      Error.captureStackTrace(this, this.constructor);
    } else {
      // FF, IE 10+ and Safari 6+. Fallback for others
      this.stack = (new Error()).stack || '';
    }
  }


  // Inherit from Error
  YAMLException$1.prototype = Object.create(Error.prototype);
  YAMLException$1.prototype.constructor = YAMLException$1;


  YAMLException$1.prototype.toString = function toString(compact) {
    return this.name + ': ' + formatError(this, compact);
  };


  var exception = YAMLException$1;

  // get snippet for a single line, respecting maxLength
  function getLine(buffer, lineStart, lineEnd, position, maxLineLength) {
    var head = '';
    var tail = '';
    var maxHalfLength = Math.floor(maxLineLength / 2) - 1;

    if (position - lineStart > maxHalfLength) {
      head = ' ... ';
      lineStart = position - maxHalfLength + head.length;
    }

    if (lineEnd - position > maxHalfLength) {
      tail = ' ...';
      lineEnd = position + maxHalfLength - tail.length;
    }

    return {
      str: head + buffer.slice(lineStart, lineEnd).replace(/\t/g, '→') + tail,
      pos: position - lineStart + head.length // relative position
    };
  }


  function padStart(string, max) {
    return common.repeat(' ', max - string.length) + string;
  }


  function makeSnippet(mark, options) {
    options = Object.create(options || null);

    if (!mark.buffer) return null;

    if (!options.maxLength) options.maxLength = 79;
    if (typeof options.indent      !== 'number') options.indent      = 1;
    if (typeof options.linesBefore !== 'number') options.linesBefore = 3;
    if (typeof options.linesAfter  !== 'number') options.linesAfter  = 2;

    var re = /\r?\n|\r|\0/g;
    var lineStarts = [ 0 ];
    var lineEnds = [];
    var match;
    var foundLineNo = -1;

    while ((match = re.exec(mark.buffer))) {
      lineEnds.push(match.index);
      lineStarts.push(match.index + match[0].length);

      if (mark.position <= match.index && foundLineNo < 0) {
        foundLineNo = lineStarts.length - 2;
      }
    }

    if (foundLineNo < 0) foundLineNo = lineStarts.length - 1;

    var result = '', i, line;
    var lineNoLength = Math.min(mark.line + options.linesAfter, lineEnds.length).toString().length;
    var maxLineLength = options.maxLength - (options.indent + lineNoLength + 3);

    for (i = 1; i <= options.linesBefore; i++) {
      if (foundLineNo - i < 0) break;
      line = getLine(
        mark.buffer,
        lineStarts[foundLineNo - i],
        lineEnds[foundLineNo - i],
        mark.position - (lineStarts[foundLineNo] - lineStarts[foundLineNo - i]),
        maxLineLength
      );
      result = common.repeat(' ', options.indent) + padStart((mark.line - i + 1).toString(), lineNoLength) +
        ' | ' + line.str + '\n' + result;
    }

    line = getLine(mark.buffer, lineStarts[foundLineNo], lineEnds[foundLineNo], mark.position, maxLineLength);
    result += common.repeat(' ', options.indent) + padStart((mark.line + 1).toString(), lineNoLength) +
      ' | ' + line.str + '\n';
    result += common.repeat('-', options.indent + lineNoLength + 3 + line.pos) + '^' + '\n';

    for (i = 1; i <= options.linesAfter; i++) {
      if (foundLineNo + i >= lineEnds.length) break;
      line = getLine(
        mark.buffer,
        lineStarts[foundLineNo + i],
        lineEnds[foundLineNo + i],
        mark.position - (lineStarts[foundLineNo] - lineStarts[foundLineNo + i]),
        maxLineLength
      );
      result += common.repeat(' ', options.indent) + padStart((mark.line + i + 1).toString(), lineNoLength) +
        ' | ' + line.str + '\n';
    }

    return result.replace(/\n$/, '');
  }


  var snippet = makeSnippet;

  var TYPE_CONSTRUCTOR_OPTIONS = [
    'kind',
    'multi',
    'resolve',
    'construct',
    'instanceOf',
    'predicate',
    'represent',
    'representName',
    'defaultStyle',
    'styleAliases'
  ];

  var YAML_NODE_KINDS = [
    'scalar',
    'sequence',
    'mapping'
  ];

  function compileStyleAliases(map) {
    var result = {};

    if (map !== null) {
      Object.keys(map).forEach(function (style) {
        map[style].forEach(function (alias) {
          result[String(alias)] = style;
        });
      });
    }

    return result;
  }

  function Type$1(tag, options) {
    options = options || {};

    Object.keys(options).forEach(function (name) {
      if (TYPE_CONSTRUCTOR_OPTIONS.indexOf(name) === -1) {
        throw new exception('Unknown option "' + name + '" is met in definition of "' + tag + '" YAML type.');
      }
    });

    // TODO: Add tag format check.
    this.options       = options; // keep original options in case user wants to extend this type later
    this.tag           = tag;
    this.kind          = options['kind']          || null;
    this.resolve       = options['resolve']       || function () { return true; };
    this.construct     = options['construct']     || function (data) { return data; };
    this.instanceOf    = options['instanceOf']    || null;
    this.predicate     = options['predicate']     || null;
    this.represent     = options['represent']     || null;
    this.representName = options['representName'] || null;
    this.defaultStyle  = options['defaultStyle']  || null;
    this.multi         = options['multi']         || false;
    this.styleAliases  = compileStyleAliases(options['styleAliases'] || null);

    if (YAML_NODE_KINDS.indexOf(this.kind) === -1) {
      throw new exception('Unknown kind "' + this.kind + '" is specified for "' + tag + '" YAML type.');
    }
  }

  var type = Type$1;

  /*eslint-disable max-len*/





  function compileList(schema, name) {
    var result = [];

    schema[name].forEach(function (currentType) {
      var newIndex = result.length;

      result.forEach(function (previousType, previousIndex) {
        if (previousType.tag === currentType.tag &&
            previousType.kind === currentType.kind &&
            previousType.multi === currentType.multi) {

          newIndex = previousIndex;
        }
      });

      result[newIndex] = currentType;
    });

    return result;
  }


  function compileMap(/* lists... */) {
    var result = {
          scalar: {},
          sequence: {},
          mapping: {},
          fallback: {},
          multi: {
            scalar: [],
            sequence: [],
            mapping: [],
            fallback: []
          }
        }, index, length;

    function collectType(type) {
      if (type.multi) {
        result.multi[type.kind].push(type);
        result.multi['fallback'].push(type);
      } else {
        result[type.kind][type.tag] = result['fallback'][type.tag] = type;
      }
    }

    for (index = 0, length = arguments.length; index < length; index += 1) {
      arguments[index].forEach(collectType);
    }
    return result;
  }


  function Schema$1(definition) {
    return this.extend(definition);
  }


  Schema$1.prototype.extend = function extend(definition) {
    var implicit = [];
    var explicit = [];

    if (definition instanceof type) {
      // Schema.extend(type)
      explicit.push(definition);

    } else if (Array.isArray(definition)) {
      // Schema.extend([ type1, type2, ... ])
      explicit = explicit.concat(definition);

    } else if (definition && (Array.isArray(definition.implicit) || Array.isArray(definition.explicit))) {
      // Schema.extend({ explicit: [ type1, type2, ... ], implicit: [ type1, type2, ... ] })
      if (definition.implicit) implicit = implicit.concat(definition.implicit);
      if (definition.explicit) explicit = explicit.concat(definition.explicit);

    } else {
      throw new exception('Schema.extend argument should be a Type, [ Type ], ' +
        'or a schema definition ({ implicit: [...], explicit: [...] })');
    }

    implicit.forEach(function (type$1) {
      if (!(type$1 instanceof type)) {
        throw new exception('Specified list of YAML types (or a single Type object) contains a non-Type object.');
      }

      if (type$1.loadKind && type$1.loadKind !== 'scalar') {
        throw new exception('There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.');
      }

      if (type$1.multi) {
        throw new exception('There is a multi type in the implicit list of a schema. Multi tags can only be listed as explicit.');
      }
    });

    explicit.forEach(function (type$1) {
      if (!(type$1 instanceof type)) {
        throw new exception('Specified list of YAML types (or a single Type object) contains a non-Type object.');
      }
    });

    var result = Object.create(Schema$1.prototype);

    result.implicit = (this.implicit || []).concat(implicit);
    result.explicit = (this.explicit || []).concat(explicit);

    result.compiledImplicit = compileList(result, 'implicit');
    result.compiledExplicit = compileList(result, 'explicit');
    result.compiledTypeMap  = compileMap(result.compiledImplicit, result.compiledExplicit);

    return result;
  };


  var schema = Schema$1;

  var str = new type('tag:yaml.org,2002:str', {
    kind: 'scalar',
    construct: function (data) { return data !== null ? data : ''; }
  });

  var seq = new type('tag:yaml.org,2002:seq', {
    kind: 'sequence',
    construct: function (data) { return data !== null ? data : []; }
  });

  var map = new type('tag:yaml.org,2002:map', {
    kind: 'mapping',
    construct: function (data) { return data !== null ? data : {}; }
  });

  var failsafe = new schema({
    explicit: [
      str,
      seq,
      map
    ]
  });

  function resolveYamlNull(data) {
    if (data === null) return true;

    var max = data.length;

    return (max === 1 && data === '~') ||
           (max === 4 && (data === 'null' || data === 'Null' || data === 'NULL'));
  }

  function constructYamlNull() {
    return null;
  }

  function isNull(object) {
    return object === null;
  }

  var _null = new type('tag:yaml.org,2002:null', {
    kind: 'scalar',
    resolve: resolveYamlNull,
    construct: constructYamlNull,
    predicate: isNull,
    represent: {
      canonical: function () { return '~';    },
      lowercase: function () { return 'null'; },
      uppercase: function () { return 'NULL'; },
      camelcase: function () { return 'Null'; },
      empty:     function () { return '';     }
    },
    defaultStyle: 'lowercase'
  });

  function resolveYamlBoolean(data) {
    if (data === null) return false;

    var max = data.length;

    return (max === 4 && (data === 'true' || data === 'True' || data === 'TRUE')) ||
           (max === 5 && (data === 'false' || data === 'False' || data === 'FALSE'));
  }

  function constructYamlBoolean(data) {
    return data === 'true' ||
           data === 'True' ||
           data === 'TRUE';
  }

  function isBoolean(object) {
    return Object.prototype.toString.call(object) === '[object Boolean]';
  }

  var bool = new type('tag:yaml.org,2002:bool', {
    kind: 'scalar',
    resolve: resolveYamlBoolean,
    construct: constructYamlBoolean,
    predicate: isBoolean,
    represent: {
      lowercase: function (object) { return object ? 'true' : 'false'; },
      uppercase: function (object) { return object ? 'TRUE' : 'FALSE'; },
      camelcase: function (object) { return object ? 'True' : 'False'; }
    },
    defaultStyle: 'lowercase'
  });

  function isHexCode(c) {
    return ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */)) ||
           ((0x41/* A */ <= c) && (c <= 0x46/* F */)) ||
           ((0x61/* a */ <= c) && (c <= 0x66/* f */));
  }

  function isOctCode(c) {
    return ((0x30/* 0 */ <= c) && (c <= 0x37/* 7 */));
  }

  function isDecCode(c) {
    return ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */));
  }

  function resolveYamlInteger(data) {
    if (data === null) return false;

    var max = data.length,
        index = 0,
        hasDigits = false,
        ch;

    if (!max) return false;

    ch = data[index];

    // sign
    if (ch === '-' || ch === '+') {
      ch = data[++index];
    }

    if (ch === '0') {
      // 0
      if (index + 1 === max) return true;
      ch = data[++index];

      // base 2, base 8, base 16

      if (ch === 'b') {
        // base 2
        index++;

        for (; index < max; index++) {
          ch = data[index];
          if (ch === '_') continue;
          if (ch !== '0' && ch !== '1') return false;
          hasDigits = true;
        }
        return hasDigits && ch !== '_';
      }


      if (ch === 'x') {
        // base 16
        index++;

        for (; index < max; index++) {
          ch = data[index];
          if (ch === '_') continue;
          if (!isHexCode(data.charCodeAt(index))) return false;
          hasDigits = true;
        }
        return hasDigits && ch !== '_';
      }


      if (ch === 'o') {
        // base 8
        index++;

        for (; index < max; index++) {
          ch = data[index];
          if (ch === '_') continue;
          if (!isOctCode(data.charCodeAt(index))) return false;
          hasDigits = true;
        }
        return hasDigits && ch !== '_';
      }
    }

    // base 10 (except 0)

    // value should not start with `_`;
    if (ch === '_') return false;

    for (; index < max; index++) {
      ch = data[index];
      if (ch === '_') continue;
      if (!isDecCode(data.charCodeAt(index))) {
        return false;
      }
      hasDigits = true;
    }

    // Should have digits and should not end with `_`
    if (!hasDigits || ch === '_') return false;

    return true;
  }

  function constructYamlInteger(data) {
    var value = data, sign = 1, ch;

    if (value.indexOf('_') !== -1) {
      value = value.replace(/_/g, '');
    }

    ch = value[0];

    if (ch === '-' || ch === '+') {
      if (ch === '-') sign = -1;
      value = value.slice(1);
      ch = value[0];
    }

    if (value === '0') return 0;

    if (ch === '0') {
      if (value[1] === 'b') return sign * parseInt(value.slice(2), 2);
      if (value[1] === 'x') return sign * parseInt(value.slice(2), 16);
      if (value[1] === 'o') return sign * parseInt(value.slice(2), 8);
    }

    return sign * parseInt(value, 10);
  }

  function isInteger(object) {
    return (Object.prototype.toString.call(object)) === '[object Number]' &&
           (object % 1 === 0 && !common.isNegativeZero(object));
  }

  var int = new type('tag:yaml.org,2002:int', {
    kind: 'scalar',
    resolve: resolveYamlInteger,
    construct: constructYamlInteger,
    predicate: isInteger,
    represent: {
      binary:      function (obj) { return obj >= 0 ? '0b' + obj.toString(2) : '-0b' + obj.toString(2).slice(1); },
      octal:       function (obj) { return obj >= 0 ? '0o'  + obj.toString(8) : '-0o'  + obj.toString(8).slice(1); },
      decimal:     function (obj) { return obj.toString(10); },
      /* eslint-disable max-len */
      hexadecimal: function (obj) { return obj >= 0 ? '0x' + obj.toString(16).toUpperCase() :  '-0x' + obj.toString(16).toUpperCase().slice(1); }
    },
    defaultStyle: 'decimal',
    styleAliases: {
      binary:      [ 2,  'bin' ],
      octal:       [ 8,  'oct' ],
      decimal:     [ 10, 'dec' ],
      hexadecimal: [ 16, 'hex' ]
    }
  });

  var YAML_FLOAT_PATTERN = new RegExp(
    // 2.5e4, 2.5 and integers
    '^(?:[-+]?(?:[0-9][0-9_]*)(?:\\.[0-9_]*)?(?:[eE][-+]?[0-9]+)?' +
    // .2e4, .2
    // special case, seems not from spec
    '|\\.[0-9_]+(?:[eE][-+]?[0-9]+)?' +
    // .inf
    '|[-+]?\\.(?:inf|Inf|INF)' +
    // .nan
    '|\\.(?:nan|NaN|NAN))$');

  function resolveYamlFloat(data) {
    if (data === null) return false;

    if (!YAML_FLOAT_PATTERN.test(data) ||
        // Quick hack to not allow integers end with `_`
        // Probably should update regexp & check speed
        data[data.length - 1] === '_') {
      return false;
    }

    return true;
  }

  function constructYamlFloat(data) {
    var value, sign;

    value  = data.replace(/_/g, '').toLowerCase();
    sign   = value[0] === '-' ? -1 : 1;

    if ('+-'.indexOf(value[0]) >= 0) {
      value = value.slice(1);
    }

    if (value === '.inf') {
      return (sign === 1) ? Number.POSITIVE_INFINITY : Number.NEGATIVE_INFINITY;

    } else if (value === '.nan') {
      return NaN;
    }
    return sign * parseFloat(value, 10);
  }


  var SCIENTIFIC_WITHOUT_DOT = /^[-+]?[0-9]+e/;

  function representYamlFloat(object, style) {
    var res;

    if (isNaN(object)) {
      switch (style) {
        case 'lowercase': return '.nan';
        case 'uppercase': return '.NAN';
        case 'camelcase': return '.NaN';
      }
    } else if (Number.POSITIVE_INFINITY === object) {
      switch (style) {
        case 'lowercase': return '.inf';
        case 'uppercase': return '.INF';
        case 'camelcase': return '.Inf';
      }
    } else if (Number.NEGATIVE_INFINITY === object) {
      switch (style) {
        case 'lowercase': return '-.inf';
        case 'uppercase': return '-.INF';
        case 'camelcase': return '-.Inf';
      }
    } else if (common.isNegativeZero(object)) {
      return '-0.0';
    }

    res = object.toString(10);

    // JS stringifier can build scientific format without dots: 5e-100,
    // while YAML requres dot: 5.e-100. Fix it with simple hack

    return SCIENTIFIC_WITHOUT_DOT.test(res) ? res.replace('e', '.e') : res;
  }

  function isFloat(object) {
    return (Object.prototype.toString.call(object) === '[object Number]') &&
           (object % 1 !== 0 || common.isNegativeZero(object));
  }

  var float = new type('tag:yaml.org,2002:float', {
    kind: 'scalar',
    resolve: resolveYamlFloat,
    construct: constructYamlFloat,
    predicate: isFloat,
    represent: representYamlFloat,
    defaultStyle: 'lowercase'
  });

  var json = failsafe.extend({
    implicit: [
      _null,
      bool,
      int,
      float
    ]
  });

  var core = json;

  var YAML_DATE_REGEXP = new RegExp(
    '^([0-9][0-9][0-9][0-9])'          + // [1] year
    '-([0-9][0-9])'                    + // [2] month
    '-([0-9][0-9])$');                   // [3] day

  var YAML_TIMESTAMP_REGEXP = new RegExp(
    '^([0-9][0-9][0-9][0-9])'          + // [1] year
    '-([0-9][0-9]?)'                   + // [2] month
    '-([0-9][0-9]?)'                   + // [3] day
    '(?:[Tt]|[ \\t]+)'                 + // ...
    '([0-9][0-9]?)'                    + // [4] hour
    ':([0-9][0-9])'                    + // [5] minute
    ':([0-9][0-9])'                    + // [6] second
    '(?:\\.([0-9]*))?'                 + // [7] fraction
    '(?:[ \\t]*(Z|([-+])([0-9][0-9]?)' + // [8] tz [9] tz_sign [10] tz_hour
    '(?::([0-9][0-9]))?))?$');           // [11] tz_minute

  function resolveYamlTimestamp(data) {
    if (data === null) return false;
    if (YAML_DATE_REGEXP.exec(data) !== null) return true;
    if (YAML_TIMESTAMP_REGEXP.exec(data) !== null) return true;
    return false;
  }

  function constructYamlTimestamp(data) {
    var match, year, month, day, hour, minute, second, fraction = 0,
        delta = null, tz_hour, tz_minute, date;

    match = YAML_DATE_REGEXP.exec(data);
    if (match === null) match = YAML_TIMESTAMP_REGEXP.exec(data);

    if (match === null) throw new Error('Date resolve error');

    // match: [1] year [2] month [3] day

    year = +(match[1]);
    month = +(match[2]) - 1; // JS month starts with 0
    day = +(match[3]);

    if (!match[4]) { // no hour
      return new Date(Date.UTC(year, month, day));
    }

    // match: [4] hour [5] minute [6] second [7] fraction

    hour = +(match[4]);
    minute = +(match[5]);
    second = +(match[6]);

    if (match[7]) {
      fraction = match[7].slice(0, 3);
      while (fraction.length < 3) { // milli-seconds
        fraction += '0';
      }
      fraction = +fraction;
    }

    // match: [8] tz [9] tz_sign [10] tz_hour [11] tz_minute

    if (match[9]) {
      tz_hour = +(match[10]);
      tz_minute = +(match[11] || 0);
      delta = (tz_hour * 60 + tz_minute) * 60000; // delta in mili-seconds
      if (match[9] === '-') delta = -delta;
    }

    date = new Date(Date.UTC(year, month, day, hour, minute, second, fraction));

    if (delta) date.setTime(date.getTime() - delta);

    return date;
  }

  function representYamlTimestamp(object /*, style*/) {
    return object.toISOString();
  }

  var timestamp = new type('tag:yaml.org,2002:timestamp', {
    kind: 'scalar',
    resolve: resolveYamlTimestamp,
    construct: constructYamlTimestamp,
    instanceOf: Date,
    represent: representYamlTimestamp
  });

  function resolveYamlMerge(data) {
    return data === '<<' || data === null;
  }

  var merge = new type('tag:yaml.org,2002:merge', {
    kind: 'scalar',
    resolve: resolveYamlMerge
  });

  /*eslint-disable no-bitwise*/





  // [ 64, 65, 66 ] -> [ padding, CR, LF ]
  var BASE64_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\n\r';


  function resolveYamlBinary(data) {
    if (data === null) return false;

    var code, idx, bitlen = 0, max = data.length, map = BASE64_MAP;

    // Convert one by one.
    for (idx = 0; idx < max; idx++) {
      code = map.indexOf(data.charAt(idx));

      // Skip CR/LF
      if (code > 64) continue;

      // Fail on illegal characters
      if (code < 0) return false;

      bitlen += 6;
    }

    // If there are any bits left, source was corrupted
    return (bitlen % 8) === 0;
  }

  function constructYamlBinary(data) {
    var idx, tailbits,
        input = data.replace(/[\r\n=]/g, ''), // remove CR/LF & padding to simplify scan
        max = input.length,
        map = BASE64_MAP,
        bits = 0,
        result = [];

    // Collect by 6*4 bits (3 bytes)

    for (idx = 0; idx < max; idx++) {
      if ((idx % 4 === 0) && idx) {
        result.push((bits >> 16) & 0xFF);
        result.push((bits >> 8) & 0xFF);
        result.push(bits & 0xFF);
      }

      bits = (bits << 6) | map.indexOf(input.charAt(idx));
    }

    // Dump tail

    tailbits = (max % 4) * 6;

    if (tailbits === 0) {
      result.push((bits >> 16) & 0xFF);
      result.push((bits >> 8) & 0xFF);
      result.push(bits & 0xFF);
    } else if (tailbits === 18) {
      result.push((bits >> 10) & 0xFF);
      result.push((bits >> 2) & 0xFF);
    } else if (tailbits === 12) {
      result.push((bits >> 4) & 0xFF);
    }

    return new Uint8Array(result);
  }

  function representYamlBinary(object /*, style*/) {
    var result = '', bits = 0, idx, tail,
        max = object.length,
        map = BASE64_MAP;

    // Convert every three bytes to 4 ASCII characters.

    for (idx = 0; idx < max; idx++) {
      if ((idx % 3 === 0) && idx) {
        result += map[(bits >> 18) & 0x3F];
        result += map[(bits >> 12) & 0x3F];
        result += map[(bits >> 6) & 0x3F];
        result += map[bits & 0x3F];
      }

      bits = (bits << 8) + object[idx];
    }

    // Dump tail

    tail = max % 3;

    if (tail === 0) {
      result += map[(bits >> 18) & 0x3F];
      result += map[(bits >> 12) & 0x3F];
      result += map[(bits >> 6) & 0x3F];
      result += map[bits & 0x3F];
    } else if (tail === 2) {
      result += map[(bits >> 10) & 0x3F];
      result += map[(bits >> 4) & 0x3F];
      result += map[(bits << 2) & 0x3F];
      result += map[64];
    } else if (tail === 1) {
      result += map[(bits >> 2) & 0x3F];
      result += map[(bits << 4) & 0x3F];
      result += map[64];
      result += map[64];
    }

    return result;
  }

  function isBinary(obj) {
    return Object.prototype.toString.call(obj) ===  '[object Uint8Array]';
  }

  var binary = new type('tag:yaml.org,2002:binary', {
    kind: 'scalar',
    resolve: resolveYamlBinary,
    construct: constructYamlBinary,
    predicate: isBinary,
    represent: representYamlBinary
  });

  var _hasOwnProperty$3 = Object.prototype.hasOwnProperty;
  var _toString$2       = Object.prototype.toString;

  function resolveYamlOmap(data) {
    if (data === null) return true;

    var objectKeys = [], index, length, pair, pairKey, pairHasKey,
        object = data;

    for (index = 0, length = object.length; index < length; index += 1) {
      pair = object[index];
      pairHasKey = false;

      if (_toString$2.call(pair) !== '[object Object]') return false;

      for (pairKey in pair) {
        if (_hasOwnProperty$3.call(pair, pairKey)) {
          if (!pairHasKey) pairHasKey = true;
          else return false;
        }
      }

      if (!pairHasKey) return false;

      if (objectKeys.indexOf(pairKey) === -1) objectKeys.push(pairKey);
      else return false;
    }

    return true;
  }

  function constructYamlOmap(data) {
    return data !== null ? data : [];
  }

  var omap = new type('tag:yaml.org,2002:omap', {
    kind: 'sequence',
    resolve: resolveYamlOmap,
    construct: constructYamlOmap
  });

  var _toString$1 = Object.prototype.toString;

  function resolveYamlPairs(data) {
    if (data === null) return true;

    var index, length, pair, keys, result,
        object = data;

    result = new Array(object.length);

    for (index = 0, length = object.length; index < length; index += 1) {
      pair = object[index];

      if (_toString$1.call(pair) !== '[object Object]') return false;

      keys = Object.keys(pair);

      if (keys.length !== 1) return false;

      result[index] = [ keys[0], pair[keys[0]] ];
    }

    return true;
  }

  function constructYamlPairs(data) {
    if (data === null) return [];

    var index, length, pair, keys, result,
        object = data;

    result = new Array(object.length);

    for (index = 0, length = object.length; index < length; index += 1) {
      pair = object[index];

      keys = Object.keys(pair);

      result[index] = [ keys[0], pair[keys[0]] ];
    }

    return result;
  }

  var pairs = new type('tag:yaml.org,2002:pairs', {
    kind: 'sequence',
    resolve: resolveYamlPairs,
    construct: constructYamlPairs
  });

  var _hasOwnProperty$2 = Object.prototype.hasOwnProperty;

  function resolveYamlSet(data) {
    if (data === null) return true;

    var key, object = data;

    for (key in object) {
      if (_hasOwnProperty$2.call(object, key)) {
        if (object[key] !== null) return false;
      }
    }

    return true;
  }

  function constructYamlSet(data) {
    return data !== null ? data : {};
  }

  var set = new type('tag:yaml.org,2002:set', {
    kind: 'mapping',
    resolve: resolveYamlSet,
    construct: constructYamlSet
  });

  var _default = core.extend({
    implicit: [
      timestamp,
      merge
    ],
    explicit: [
      binary,
      omap,
      pairs,
      set
    ]
  });

  /*eslint-disable max-len,no-use-before-define*/







  var _hasOwnProperty$1 = Object.prototype.hasOwnProperty;


  var CONTEXT_FLOW_IN   = 1;
  var CONTEXT_FLOW_OUT  = 2;
  var CONTEXT_BLOCK_IN  = 3;
  var CONTEXT_BLOCK_OUT = 4;


  var CHOMPING_CLIP  = 1;
  var CHOMPING_STRIP = 2;
  var CHOMPING_KEEP  = 3;


  var PATTERN_NON_PRINTABLE         = /[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/;
  var PATTERN_NON_ASCII_LINE_BREAKS = /[\x85\u2028\u2029]/;
  var PATTERN_FLOW_INDICATORS       = /[,\[\]\{\}]/;
  var PATTERN_TAG_HANDLE            = /^(?:!|!!|![a-z\-]+!)$/i;
  var PATTERN_TAG_URI               = /^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;


  function _class(obj) { return Object.prototype.toString.call(obj); }

  function is_EOL(c) {
    return (c === 0x0A/* LF */) || (c === 0x0D/* CR */);
  }

  function is_WHITE_SPACE(c) {
    return (c === 0x09/* Tab */) || (c === 0x20/* Space */);
  }

  function is_WS_OR_EOL(c) {
    return (c === 0x09/* Tab */) ||
           (c === 0x20/* Space */) ||
           (c === 0x0A/* LF */) ||
           (c === 0x0D/* CR */);
  }

  function is_FLOW_INDICATOR(c) {
    return c === 0x2C/* , */ ||
           c === 0x5B/* [ */ ||
           c === 0x5D/* ] */ ||
           c === 0x7B/* { */ ||
           c === 0x7D/* } */;
  }

  function fromHexCode(c) {
    var lc;

    if ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */)) {
      return c - 0x30;
    }

    /*eslint-disable no-bitwise*/
    lc = c | 0x20;

    if ((0x61/* a */ <= lc) && (lc <= 0x66/* f */)) {
      return lc - 0x61 + 10;
    }

    return -1;
  }

  function escapedHexLen(c) {
    if (c === 0x78/* x */) { return 2; }
    if (c === 0x75/* u */) { return 4; }
    if (c === 0x55/* U */) { return 8; }
    return 0;
  }

  function fromDecimalCode(c) {
    if ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */)) {
      return c - 0x30;
    }

    return -1;
  }

  function simpleEscapeSequence(c) {
    /* eslint-disable indent */
    return (c === 0x30/* 0 */) ? '\x00' :
          (c === 0x61/* a */) ? '\x07' :
          (c === 0x62/* b */) ? '\x08' :
          (c === 0x74/* t */) ? '\x09' :
          (c === 0x09/* Tab */) ? '\x09' :
          (c === 0x6E/* n */) ? '\x0A' :
          (c === 0x76/* v */) ? '\x0B' :
          (c === 0x66/* f */) ? '\x0C' :
          (c === 0x72/* r */) ? '\x0D' :
          (c === 0x65/* e */) ? '\x1B' :
          (c === 0x20/* Space */) ? ' ' :
          (c === 0x22/* " */) ? '\x22' :
          (c === 0x2F/* / */) ? '/' :
          (c === 0x5C/* \ */) ? '\x5C' :
          (c === 0x4E/* N */) ? '\x85' :
          (c === 0x5F/* _ */) ? '\xA0' :
          (c === 0x4C/* L */) ? '\u2028' :
          (c === 0x50/* P */) ? '\u2029' : '';
  }

  function charFromCodepoint(c) {
    if (c <= 0xFFFF) {
      return String.fromCharCode(c);
    }
    // Encode UTF-16 surrogate pair
    // https://en.wikipedia.org/wiki/UTF-16#Code_points_U.2B010000_to_U.2B10FFFF
    return String.fromCharCode(
      ((c - 0x010000) >> 10) + 0xD800,
      ((c - 0x010000) & 0x03FF) + 0xDC00
    );
  }

  var simpleEscapeCheck = new Array(256); // integer, for fast access
  var simpleEscapeMap = new Array(256);
  for (var i = 0; i < 256; i++) {
    simpleEscapeCheck[i] = simpleEscapeSequence(i) ? 1 : 0;
    simpleEscapeMap[i] = simpleEscapeSequence(i);
  }


  function State$1(input, options) {
    this.input = input;

    this.filename  = options['filename']  || null;
    this.schema    = options['schema']    || _default;
    this.onWarning = options['onWarning'] || null;
    // (Hidden) Remove? makes the loader to expect YAML 1.1 documents
    // if such documents have no explicit %YAML directive
    this.legacy    = options['legacy']    || false;

    this.json      = options['json']      || false;
    this.listener  = options['listener']  || null;

    this.implicitTypes = this.schema.compiledImplicit;
    this.typeMap       = this.schema.compiledTypeMap;

    this.length     = input.length;
    this.position   = 0;
    this.line       = 0;
    this.lineStart  = 0;
    this.lineIndent = 0;

    // position of first leading tab in the current line,
    // used to make sure there are no tabs in the indentation
    this.firstTabInLine = -1;

    this.documents = [];

    /*
    this.version;
    this.checkLineBreaks;
    this.tagMap;
    this.anchorMap;
    this.tag;
    this.anchor;
    this.kind;
    this.result;*/

  }


  function generateError(state, message) {
    var mark = {
      name:     state.filename,
      buffer:   state.input.slice(0, -1), // omit trailing \0
      position: state.position,
      line:     state.line,
      column:   state.position - state.lineStart
    };

    mark.snippet = snippet(mark);

    return new exception(message, mark);
  }

  function throwError(state, message) {
    throw generateError(state, message);
  }

  function throwWarning(state, message) {
    if (state.onWarning) {
      state.onWarning.call(null, generateError(state, message));
    }
  }


  var directiveHandlers = {

    YAML: function handleYamlDirective(state, name, args) {

      var match, major, minor;

      if (state.version !== null) {
        throwError(state, 'duplication of %YAML directive');
      }

      if (args.length !== 1) {
        throwError(state, 'YAML directive accepts exactly one argument');
      }

      match = /^([0-9]+)\.([0-9]+)$/.exec(args[0]);

      if (match === null) {
        throwError(state, 'ill-formed argument of the YAML directive');
      }

      major = parseInt(match[1], 10);
      minor = parseInt(match[2], 10);

      if (major !== 1) {
        throwError(state, 'unacceptable YAML version of the document');
      }

      state.version = args[0];
      state.checkLineBreaks = (minor < 2);

      if (minor !== 1 && minor !== 2) {
        throwWarning(state, 'unsupported YAML version of the document');
      }
    },

    TAG: function handleTagDirective(state, name, args) {

      var handle, prefix;

      if (args.length !== 2) {
        throwError(state, 'TAG directive accepts exactly two arguments');
      }

      handle = args[0];
      prefix = args[1];

      if (!PATTERN_TAG_HANDLE.test(handle)) {
        throwError(state, 'ill-formed tag handle (first argument) of the TAG directive');
      }

      if (_hasOwnProperty$1.call(state.tagMap, handle)) {
        throwError(state, 'there is a previously declared suffix for "' + handle + '" tag handle');
      }

      if (!PATTERN_TAG_URI.test(prefix)) {
        throwError(state, 'ill-formed tag prefix (second argument) of the TAG directive');
      }

      try {
        prefix = decodeURIComponent(prefix);
      } catch (err) {
        throwError(state, 'tag prefix is malformed: ' + prefix);
      }

      state.tagMap[handle] = prefix;
    }
  };


  function captureSegment(state, start, end, checkJson) {
    var _position, _length, _character, _result;

    if (start < end) {
      _result = state.input.slice(start, end);

      if (checkJson) {
        for (_position = 0, _length = _result.length; _position < _length; _position += 1) {
          _character = _result.charCodeAt(_position);
          if (!(_character === 0x09 ||
                (0x20 <= _character && _character <= 0x10FFFF))) {
            throwError(state, 'expected valid JSON character');
          }
        }
      } else if (PATTERN_NON_PRINTABLE.test(_result)) {
        throwError(state, 'the stream contains non-printable characters');
      }

      state.result += _result;
    }
  }

  function mergeMappings(state, destination, source, overridableKeys) {
    var sourceKeys, key, index, quantity;

    if (!common.isObject(source)) {
      throwError(state, 'cannot merge mappings; the provided source object is unacceptable');
    }

    sourceKeys = Object.keys(source);

    for (index = 0, quantity = sourceKeys.length; index < quantity; index += 1) {
      key = sourceKeys[index];

      if (!_hasOwnProperty$1.call(destination, key)) {
        destination[key] = source[key];
        overridableKeys[key] = true;
      }
    }
  }

  function storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode,
    startLine, startLineStart, startPos) {

    var index, quantity;

    // The output is a plain object here, so keys can only be strings.
    // We need to convert keyNode to a string, but doing so can hang the process
    // (deeply nested arrays that explode exponentially using aliases).
    if (Array.isArray(keyNode)) {
      keyNode = Array.prototype.slice.call(keyNode);

      for (index = 0, quantity = keyNode.length; index < quantity; index += 1) {
        if (Array.isArray(keyNode[index])) {
          throwError(state, 'nested arrays are not supported inside keys');
        }

        if (typeof keyNode === 'object' && _class(keyNode[index]) === '[object Object]') {
          keyNode[index] = '[object Object]';
        }
      }
    }

    // Avoid code execution in load() via toString property
    // (still use its own toString for arrays, timestamps,
    // and whatever user schema extensions happen to have @@toStringTag)
    if (typeof keyNode === 'object' && _class(keyNode) === '[object Object]') {
      keyNode = '[object Object]';
    }


    keyNode = String(keyNode);

    if (_result === null) {
      _result = {};
    }

    if (keyTag === 'tag:yaml.org,2002:merge') {
      if (Array.isArray(valueNode)) {
        for (index = 0, quantity = valueNode.length; index < quantity; index += 1) {
          mergeMappings(state, _result, valueNode[index], overridableKeys);
        }
      } else {
        mergeMappings(state, _result, valueNode, overridableKeys);
      }
    } else {
      if (!state.json &&
          !_hasOwnProperty$1.call(overridableKeys, keyNode) &&
          _hasOwnProperty$1.call(_result, keyNode)) {
        state.line = startLine || state.line;
        state.lineStart = startLineStart || state.lineStart;
        state.position = startPos || state.position;
        throwError(state, 'duplicated mapping key');
      }

      // used for this specific key only because Object.defineProperty is slow
      if (keyNode === '__proto__') {
        Object.defineProperty(_result, keyNode, {
          configurable: true,
          enumerable: true,
          writable: true,
          value: valueNode
        });
      } else {
        _result[keyNode] = valueNode;
      }
      delete overridableKeys[keyNode];
    }

    return _result;
  }

  function readLineBreak(state) {
    var ch;

    ch = state.input.charCodeAt(state.position);

    if (ch === 0x0A/* LF */) {
      state.position++;
    } else if (ch === 0x0D/* CR */) {
      state.position++;
      if (state.input.charCodeAt(state.position) === 0x0A/* LF */) {
        state.position++;
      }
    } else {
      throwError(state, 'a line break is expected');
    }

    state.line += 1;
    state.lineStart = state.position;
    state.firstTabInLine = -1;
  }

  function skipSeparationSpace(state, allowComments, checkIndent) {
    var lineBreaks = 0,
        ch = state.input.charCodeAt(state.position);

    while (ch !== 0) {
      while (is_WHITE_SPACE(ch)) {
        if (ch === 0x09/* Tab */ && state.firstTabInLine === -1) {
          state.firstTabInLine = state.position;
        }
        ch = state.input.charCodeAt(++state.position);
      }

      if (allowComments && ch === 0x23/* # */) {
        do {
          ch = state.input.charCodeAt(++state.position);
        } while (ch !== 0x0A/* LF */ && ch !== 0x0D/* CR */ && ch !== 0);
      }

      if (is_EOL(ch)) {
        readLineBreak(state);

        ch = state.input.charCodeAt(state.position);
        lineBreaks++;
        state.lineIndent = 0;

        while (ch === 0x20/* Space */) {
          state.lineIndent++;
          ch = state.input.charCodeAt(++state.position);
        }
      } else {
        break;
      }
    }

    if (checkIndent !== -1 && lineBreaks !== 0 && state.lineIndent < checkIndent) {
      throwWarning(state, 'deficient indentation');
    }

    return lineBreaks;
  }

  function testDocumentSeparator(state) {
    var _position = state.position,
        ch;

    ch = state.input.charCodeAt(_position);

    // Condition state.position === state.lineStart is tested
    // in parent on each call, for efficiency. No needs to test here again.
    if ((ch === 0x2D/* - */ || ch === 0x2E/* . */) &&
        ch === state.input.charCodeAt(_position + 1) &&
        ch === state.input.charCodeAt(_position + 2)) {

      _position += 3;

      ch = state.input.charCodeAt(_position);

      if (ch === 0 || is_WS_OR_EOL(ch)) {
        return true;
      }
    }

    return false;
  }

  function writeFoldedLines(state, count) {
    if (count === 1) {
      state.result += ' ';
    } else if (count > 1) {
      state.result += common.repeat('\n', count - 1);
    }
  }


  function readPlainScalar(state, nodeIndent, withinFlowCollection) {
    var preceding,
        following,
        captureStart,
        captureEnd,
        hasPendingContent,
        _line,
        _lineStart,
        _lineIndent,
        _kind = state.kind,
        _result = state.result,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (is_WS_OR_EOL(ch)      ||
        is_FLOW_INDICATOR(ch) ||
        ch === 0x23/* # */    ||
        ch === 0x26/* & */    ||
        ch === 0x2A/* * */    ||
        ch === 0x21/* ! */    ||
        ch === 0x7C/* | */    ||
        ch === 0x3E/* > */    ||
        ch === 0x27/* ' */    ||
        ch === 0x22/* " */    ||
        ch === 0x25/* % */    ||
        ch === 0x40/* @ */    ||
        ch === 0x60/* ` */) {
      return false;
    }

    if (ch === 0x3F/* ? */ || ch === 0x2D/* - */) {
      following = state.input.charCodeAt(state.position + 1);

      if (is_WS_OR_EOL(following) ||
          withinFlowCollection && is_FLOW_INDICATOR(following)) {
        return false;
      }
    }

    state.kind = 'scalar';
    state.result = '';
    captureStart = captureEnd = state.position;
    hasPendingContent = false;

    while (ch !== 0) {
      if (ch === 0x3A/* : */) {
        following = state.input.charCodeAt(state.position + 1);

        if (is_WS_OR_EOL(following) ||
            withinFlowCollection && is_FLOW_INDICATOR(following)) {
          break;
        }

      } else if (ch === 0x23/* # */) {
        preceding = state.input.charCodeAt(state.position - 1);

        if (is_WS_OR_EOL(preceding)) {
          break;
        }

      } else if ((state.position === state.lineStart && testDocumentSeparator(state)) ||
                 withinFlowCollection && is_FLOW_INDICATOR(ch)) {
        break;

      } else if (is_EOL(ch)) {
        _line = state.line;
        _lineStart = state.lineStart;
        _lineIndent = state.lineIndent;
        skipSeparationSpace(state, false, -1);

        if (state.lineIndent >= nodeIndent) {
          hasPendingContent = true;
          ch = state.input.charCodeAt(state.position);
          continue;
        } else {
          state.position = captureEnd;
          state.line = _line;
          state.lineStart = _lineStart;
          state.lineIndent = _lineIndent;
          break;
        }
      }

      if (hasPendingContent) {
        captureSegment(state, captureStart, captureEnd, false);
        writeFoldedLines(state, state.line - _line);
        captureStart = captureEnd = state.position;
        hasPendingContent = false;
      }

      if (!is_WHITE_SPACE(ch)) {
        captureEnd = state.position + 1;
      }

      ch = state.input.charCodeAt(++state.position);
    }

    captureSegment(state, captureStart, captureEnd, false);

    if (state.result) {
      return true;
    }

    state.kind = _kind;
    state.result = _result;
    return false;
  }

  function readSingleQuotedScalar(state, nodeIndent) {
    var ch,
        captureStart, captureEnd;

    ch = state.input.charCodeAt(state.position);

    if (ch !== 0x27/* ' */) {
      return false;
    }

    state.kind = 'scalar';
    state.result = '';
    state.position++;
    captureStart = captureEnd = state.position;

    while ((ch = state.input.charCodeAt(state.position)) !== 0) {
      if (ch === 0x27/* ' */) {
        captureSegment(state, captureStart, state.position, true);
        ch = state.input.charCodeAt(++state.position);

        if (ch === 0x27/* ' */) {
          captureStart = state.position;
          state.position++;
          captureEnd = state.position;
        } else {
          return true;
        }

      } else if (is_EOL(ch)) {
        captureSegment(state, captureStart, captureEnd, true);
        writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
        captureStart = captureEnd = state.position;

      } else if (state.position === state.lineStart && testDocumentSeparator(state)) {
        throwError(state, 'unexpected end of the document within a single quoted scalar');

      } else {
        state.position++;
        captureEnd = state.position;
      }
    }

    throwError(state, 'unexpected end of the stream within a single quoted scalar');
  }

  function readDoubleQuotedScalar(state, nodeIndent) {
    var captureStart,
        captureEnd,
        hexLength,
        hexResult,
        tmp,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (ch !== 0x22/* " */) {
      return false;
    }

    state.kind = 'scalar';
    state.result = '';
    state.position++;
    captureStart = captureEnd = state.position;

    while ((ch = state.input.charCodeAt(state.position)) !== 0) {
      if (ch === 0x22/* " */) {
        captureSegment(state, captureStart, state.position, true);
        state.position++;
        return true;

      } else if (ch === 0x5C/* \ */) {
        captureSegment(state, captureStart, state.position, true);
        ch = state.input.charCodeAt(++state.position);

        if (is_EOL(ch)) {
          skipSeparationSpace(state, false, nodeIndent);

          // TODO: rework to inline fn with no type cast?
        } else if (ch < 256 && simpleEscapeCheck[ch]) {
          state.result += simpleEscapeMap[ch];
          state.position++;

        } else if ((tmp = escapedHexLen(ch)) > 0) {
          hexLength = tmp;
          hexResult = 0;

          for (; hexLength > 0; hexLength--) {
            ch = state.input.charCodeAt(++state.position);

            if ((tmp = fromHexCode(ch)) >= 0) {
              hexResult = (hexResult << 4) + tmp;

            } else {
              throwError(state, 'expected hexadecimal character');
            }
          }

          state.result += charFromCodepoint(hexResult);

          state.position++;

        } else {
          throwError(state, 'unknown escape sequence');
        }

        captureStart = captureEnd = state.position;

      } else if (is_EOL(ch)) {
        captureSegment(state, captureStart, captureEnd, true);
        writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
        captureStart = captureEnd = state.position;

      } else if (state.position === state.lineStart && testDocumentSeparator(state)) {
        throwError(state, 'unexpected end of the document within a double quoted scalar');

      } else {
        state.position++;
        captureEnd = state.position;
      }
    }

    throwError(state, 'unexpected end of the stream within a double quoted scalar');
  }

  function readFlowCollection(state, nodeIndent) {
    var readNext = true,
        _line,
        _lineStart,
        _pos,
        _tag     = state.tag,
        _result,
        _anchor  = state.anchor,
        following,
        terminator,
        isPair,
        isExplicitPair,
        isMapping,
        overridableKeys = Object.create(null),
        keyNode,
        keyTag,
        valueNode,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (ch === 0x5B/* [ */) {
      terminator = 0x5D;/* ] */
      isMapping = false;
      _result = [];
    } else if (ch === 0x7B/* { */) {
      terminator = 0x7D;/* } */
      isMapping = true;
      _result = {};
    } else {
      return false;
    }

    if (state.anchor !== null) {
      state.anchorMap[state.anchor] = _result;
    }

    ch = state.input.charCodeAt(++state.position);

    while (ch !== 0) {
      skipSeparationSpace(state, true, nodeIndent);

      ch = state.input.charCodeAt(state.position);

      if (ch === terminator) {
        state.position++;
        state.tag = _tag;
        state.anchor = _anchor;
        state.kind = isMapping ? 'mapping' : 'sequence';
        state.result = _result;
        return true;
      } else if (!readNext) {
        throwError(state, 'missed comma between flow collection entries');
      } else if (ch === 0x2C/* , */) {
        // "flow collection entries can never be completely empty", as per YAML 1.2, section 7.4
        throwError(state, "expected the node content, but found ','");
      }

      keyTag = keyNode = valueNode = null;
      isPair = isExplicitPair = false;

      if (ch === 0x3F/* ? */) {
        following = state.input.charCodeAt(state.position + 1);

        if (is_WS_OR_EOL(following)) {
          isPair = isExplicitPair = true;
          state.position++;
          skipSeparationSpace(state, true, nodeIndent);
        }
      }

      _line = state.line; // Save the current line.
      _lineStart = state.lineStart;
      _pos = state.position;
      composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
      keyTag = state.tag;
      keyNode = state.result;
      skipSeparationSpace(state, true, nodeIndent);

      ch = state.input.charCodeAt(state.position);

      if ((isExplicitPair || state.line === _line) && ch === 0x3A/* : */) {
        isPair = true;
        ch = state.input.charCodeAt(++state.position);
        skipSeparationSpace(state, true, nodeIndent);
        composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
        valueNode = state.result;
      }

      if (isMapping) {
        storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode, _line, _lineStart, _pos);
      } else if (isPair) {
        _result.push(storeMappingPair(state, null, overridableKeys, keyTag, keyNode, valueNode, _line, _lineStart, _pos));
      } else {
        _result.push(keyNode);
      }

      skipSeparationSpace(state, true, nodeIndent);

      ch = state.input.charCodeAt(state.position);

      if (ch === 0x2C/* , */) {
        readNext = true;
        ch = state.input.charCodeAt(++state.position);
      } else {
        readNext = false;
      }
    }

    throwError(state, 'unexpected end of the stream within a flow collection');
  }

  function readBlockScalar(state, nodeIndent) {
    var captureStart,
        folding,
        chomping       = CHOMPING_CLIP,
        didReadContent = false,
        detectedIndent = false,
        textIndent     = nodeIndent,
        emptyLines     = 0,
        atMoreIndented = false,
        tmp,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (ch === 0x7C/* | */) {
      folding = false;
    } else if (ch === 0x3E/* > */) {
      folding = true;
    } else {
      return false;
    }

    state.kind = 'scalar';
    state.result = '';

    while (ch !== 0) {
      ch = state.input.charCodeAt(++state.position);

      if (ch === 0x2B/* + */ || ch === 0x2D/* - */) {
        if (CHOMPING_CLIP === chomping) {
          chomping = (ch === 0x2B/* + */) ? CHOMPING_KEEP : CHOMPING_STRIP;
        } else {
          throwError(state, 'repeat of a chomping mode identifier');
        }

      } else if ((tmp = fromDecimalCode(ch)) >= 0) {
        if (tmp === 0) {
          throwError(state, 'bad explicit indentation width of a block scalar; it cannot be less than one');
        } else if (!detectedIndent) {
          textIndent = nodeIndent + tmp - 1;
          detectedIndent = true;
        } else {
          throwError(state, 'repeat of an indentation width identifier');
        }

      } else {
        break;
      }
    }

    if (is_WHITE_SPACE(ch)) {
      do { ch = state.input.charCodeAt(++state.position); }
      while (is_WHITE_SPACE(ch));

      if (ch === 0x23/* # */) {
        do { ch = state.input.charCodeAt(++state.position); }
        while (!is_EOL(ch) && (ch !== 0));
      }
    }

    while (ch !== 0) {
      readLineBreak(state);
      state.lineIndent = 0;

      ch = state.input.charCodeAt(state.position);

      while ((!detectedIndent || state.lineIndent < textIndent) &&
             (ch === 0x20/* Space */)) {
        state.lineIndent++;
        ch = state.input.charCodeAt(++state.position);
      }

      if (!detectedIndent && state.lineIndent > textIndent) {
        textIndent = state.lineIndent;
      }

      if (is_EOL(ch)) {
        emptyLines++;
        continue;
      }

      // End of the scalar.
      if (state.lineIndent < textIndent) {

        // Perform the chomping.
        if (chomping === CHOMPING_KEEP) {
          state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
        } else if (chomping === CHOMPING_CLIP) {
          if (didReadContent) { // i.e. only if the scalar is not empty.
            state.result += '\n';
          }
        }

        // Break this `while` cycle and go to the funciton's epilogue.
        break;
      }

      // Folded style: use fancy rules to handle line breaks.
      if (folding) {

        // Lines starting with white space characters (more-indented lines) are not folded.
        if (is_WHITE_SPACE(ch)) {
          atMoreIndented = true;
          // except for the first content line (cf. Example 8.1)
          state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);

        // End of more-indented block.
        } else if (atMoreIndented) {
          atMoreIndented = false;
          state.result += common.repeat('\n', emptyLines + 1);

        // Just one line break - perceive as the same line.
        } else if (emptyLines === 0) {
          if (didReadContent) { // i.e. only if we have already read some scalar content.
            state.result += ' ';
          }

        // Several line breaks - perceive as different lines.
        } else {
          state.result += common.repeat('\n', emptyLines);
        }

      // Literal style: just add exact number of line breaks between content lines.
      } else {
        // Keep all line breaks except the header line break.
        state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
      }

      didReadContent = true;
      detectedIndent = true;
      emptyLines = 0;
      captureStart = state.position;

      while (!is_EOL(ch) && (ch !== 0)) {
        ch = state.input.charCodeAt(++state.position);
      }

      captureSegment(state, captureStart, state.position, false);
    }

    return true;
  }

  function readBlockSequence(state, nodeIndent) {
    var _line,
        _tag      = state.tag,
        _anchor   = state.anchor,
        _result   = [],
        following,
        detected  = false,
        ch;

    // there is a leading tab before this token, so it can't be a block sequence/mapping;
    // it can still be flow sequence/mapping or a scalar
    if (state.firstTabInLine !== -1) return false;

    if (state.anchor !== null) {
      state.anchorMap[state.anchor] = _result;
    }

    ch = state.input.charCodeAt(state.position);

    while (ch !== 0) {
      if (state.firstTabInLine !== -1) {
        state.position = state.firstTabInLine;
        throwError(state, 'tab characters must not be used in indentation');
      }

      if (ch !== 0x2D/* - */) {
        break;
      }

      following = state.input.charCodeAt(state.position + 1);

      if (!is_WS_OR_EOL(following)) {
        break;
      }

      detected = true;
      state.position++;

      if (skipSeparationSpace(state, true, -1)) {
        if (state.lineIndent <= nodeIndent) {
          _result.push(null);
          ch = state.input.charCodeAt(state.position);
          continue;
        }
      }

      _line = state.line;
      composeNode(state, nodeIndent, CONTEXT_BLOCK_IN, false, true);
      _result.push(state.result);
      skipSeparationSpace(state, true, -1);

      ch = state.input.charCodeAt(state.position);

      if ((state.line === _line || state.lineIndent > nodeIndent) && (ch !== 0)) {
        throwError(state, 'bad indentation of a sequence entry');
      } else if (state.lineIndent < nodeIndent) {
        break;
      }
    }

    if (detected) {
      state.tag = _tag;
      state.anchor = _anchor;
      state.kind = 'sequence';
      state.result = _result;
      return true;
    }
    return false;
  }

  function readBlockMapping(state, nodeIndent, flowIndent) {
    var following,
        allowCompact,
        _line,
        _keyLine,
        _keyLineStart,
        _keyPos,
        _tag          = state.tag,
        _anchor       = state.anchor,
        _result       = {},
        overridableKeys = Object.create(null),
        keyTag        = null,
        keyNode       = null,
        valueNode     = null,
        atExplicitKey = false,
        detected      = false,
        ch;

    // there is a leading tab before this token, so it can't be a block sequence/mapping;
    // it can still be flow sequence/mapping or a scalar
    if (state.firstTabInLine !== -1) return false;

    if (state.anchor !== null) {
      state.anchorMap[state.anchor] = _result;
    }

    ch = state.input.charCodeAt(state.position);

    while (ch !== 0) {
      if (!atExplicitKey && state.firstTabInLine !== -1) {
        state.position = state.firstTabInLine;
        throwError(state, 'tab characters must not be used in indentation');
      }

      following = state.input.charCodeAt(state.position + 1);
      _line = state.line; // Save the current line.

      //
      // Explicit notation case. There are two separate blocks:
      // first for the key (denoted by "?") and second for the value (denoted by ":")
      //
      if ((ch === 0x3F/* ? */ || ch === 0x3A/* : */) && is_WS_OR_EOL(following)) {

        if (ch === 0x3F/* ? */) {
          if (atExplicitKey) {
            storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null, _keyLine, _keyLineStart, _keyPos);
            keyTag = keyNode = valueNode = null;
          }

          detected = true;
          atExplicitKey = true;
          allowCompact = true;

        } else if (atExplicitKey) {
          // i.e. 0x3A/* : */ === character after the explicit key.
          atExplicitKey = false;
          allowCompact = true;

        } else {
          throwError(state, 'incomplete explicit mapping pair; a key node is missed; or followed by a non-tabulated empty line');
        }

        state.position += 1;
        ch = following;

      //
      // Implicit notation case. Flow-style node as the key first, then ":", and the value.
      //
      } else {
        _keyLine = state.line;
        _keyLineStart = state.lineStart;
        _keyPos = state.position;

        if (!composeNode(state, flowIndent, CONTEXT_FLOW_OUT, false, true)) {
          // Neither implicit nor explicit notation.
          // Reading is done. Go to the epilogue.
          break;
        }

        if (state.line === _line) {
          ch = state.input.charCodeAt(state.position);

          while (is_WHITE_SPACE(ch)) {
            ch = state.input.charCodeAt(++state.position);
          }

          if (ch === 0x3A/* : */) {
            ch = state.input.charCodeAt(++state.position);

            if (!is_WS_OR_EOL(ch)) {
              throwError(state, 'a whitespace character is expected after the key-value separator within a block mapping');
            }

            if (atExplicitKey) {
              storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null, _keyLine, _keyLineStart, _keyPos);
              keyTag = keyNode = valueNode = null;
            }

            detected = true;
            atExplicitKey = false;
            allowCompact = false;
            keyTag = state.tag;
            keyNode = state.result;

          } else if (detected) {
            throwError(state, 'can not read an implicit mapping pair; a colon is missed');

          } else {
            state.tag = _tag;
            state.anchor = _anchor;
            return true; // Keep the result of `composeNode`.
          }

        } else if (detected) {
          throwError(state, 'can not read a block mapping entry; a multiline key may not be an implicit key');

        } else {
          state.tag = _tag;
          state.anchor = _anchor;
          return true; // Keep the result of `composeNode`.
        }
      }

      //
      // Common reading code for both explicit and implicit notations.
      //
      if (state.line === _line || state.lineIndent > nodeIndent) {
        if (atExplicitKey) {
          _keyLine = state.line;
          _keyLineStart = state.lineStart;
          _keyPos = state.position;
        }

        if (composeNode(state, nodeIndent, CONTEXT_BLOCK_OUT, true, allowCompact)) {
          if (atExplicitKey) {
            keyNode = state.result;
          } else {
            valueNode = state.result;
          }
        }

        if (!atExplicitKey) {
          storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode, _keyLine, _keyLineStart, _keyPos);
          keyTag = keyNode = valueNode = null;
        }

        skipSeparationSpace(state, true, -1);
        ch = state.input.charCodeAt(state.position);
      }

      if ((state.line === _line || state.lineIndent > nodeIndent) && (ch !== 0)) {
        throwError(state, 'bad indentation of a mapping entry');
      } else if (state.lineIndent < nodeIndent) {
        break;
      }
    }

    //
    // Epilogue.
    //

    // Special case: last mapping's node contains only the key in explicit notation.
    if (atExplicitKey) {
      storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null, _keyLine, _keyLineStart, _keyPos);
    }

    // Expose the resulting mapping.
    if (detected) {
      state.tag = _tag;
      state.anchor = _anchor;
      state.kind = 'mapping';
      state.result = _result;
    }

    return detected;
  }

  function readTagProperty(state) {
    var _position,
        isVerbatim = false,
        isNamed    = false,
        tagHandle,
        tagName,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (ch !== 0x21/* ! */) return false;

    if (state.tag !== null) {
      throwError(state, 'duplication of a tag property');
    }

    ch = state.input.charCodeAt(++state.position);

    if (ch === 0x3C/* < */) {
      isVerbatim = true;
      ch = state.input.charCodeAt(++state.position);

    } else if (ch === 0x21/* ! */) {
      isNamed = true;
      tagHandle = '!!';
      ch = state.input.charCodeAt(++state.position);

    } else {
      tagHandle = '!';
    }

    _position = state.position;

    if (isVerbatim) {
      do { ch = state.input.charCodeAt(++state.position); }
      while (ch !== 0 && ch !== 0x3E/* > */);

      if (state.position < state.length) {
        tagName = state.input.slice(_position, state.position);
        ch = state.input.charCodeAt(++state.position);
      } else {
        throwError(state, 'unexpected end of the stream within a verbatim tag');
      }
    } else {
      while (ch !== 0 && !is_WS_OR_EOL(ch)) {

        if (ch === 0x21/* ! */) {
          if (!isNamed) {
            tagHandle = state.input.slice(_position - 1, state.position + 1);

            if (!PATTERN_TAG_HANDLE.test(tagHandle)) {
              throwError(state, 'named tag handle cannot contain such characters');
            }

            isNamed = true;
            _position = state.position + 1;
          } else {
            throwError(state, 'tag suffix cannot contain exclamation marks');
          }
        }

        ch = state.input.charCodeAt(++state.position);
      }

      tagName = state.input.slice(_position, state.position);

      if (PATTERN_FLOW_INDICATORS.test(tagName)) {
        throwError(state, 'tag suffix cannot contain flow indicator characters');
      }
    }

    if (tagName && !PATTERN_TAG_URI.test(tagName)) {
      throwError(state, 'tag name cannot contain such characters: ' + tagName);
    }

    try {
      tagName = decodeURIComponent(tagName);
    } catch (err) {
      throwError(state, 'tag name is malformed: ' + tagName);
    }

    if (isVerbatim) {
      state.tag = tagName;

    } else if (_hasOwnProperty$1.call(state.tagMap, tagHandle)) {
      state.tag = state.tagMap[tagHandle] + tagName;

    } else if (tagHandle === '!') {
      state.tag = '!' + tagName;

    } else if (tagHandle === '!!') {
      state.tag = 'tag:yaml.org,2002:' + tagName;

    } else {
      throwError(state, 'undeclared tag handle "' + tagHandle + '"');
    }

    return true;
  }

  function readAnchorProperty(state) {
    var _position,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (ch !== 0x26/* & */) return false;

    if (state.anchor !== null) {
      throwError(state, 'duplication of an anchor property');
    }

    ch = state.input.charCodeAt(++state.position);
    _position = state.position;

    while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
      ch = state.input.charCodeAt(++state.position);
    }

    if (state.position === _position) {
      throwError(state, 'name of an anchor node must contain at least one character');
    }

    state.anchor = state.input.slice(_position, state.position);
    return true;
  }

  function readAlias(state) {
    var _position, alias,
        ch;

    ch = state.input.charCodeAt(state.position);

    if (ch !== 0x2A/* * */) return false;

    ch = state.input.charCodeAt(++state.position);
    _position = state.position;

    while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
      ch = state.input.charCodeAt(++state.position);
    }

    if (state.position === _position) {
      throwError(state, 'name of an alias node must contain at least one character');
    }

    alias = state.input.slice(_position, state.position);

    if (!_hasOwnProperty$1.call(state.anchorMap, alias)) {
      throwError(state, 'unidentified alias "' + alias + '"');
    }

    state.result = state.anchorMap[alias];
    skipSeparationSpace(state, true, -1);
    return true;
  }

  function composeNode(state, parentIndent, nodeContext, allowToSeek, allowCompact) {
    var allowBlockStyles,
        allowBlockScalars,
        allowBlockCollections,
        indentStatus = 1, // 1: this>parent, 0: this=parent, -1: this<parent
        atNewLine  = false,
        hasContent = false,
        typeIndex,
        typeQuantity,
        typeList,
        type,
        flowIndent,
        blockIndent;

    if (state.listener !== null) {
      state.listener('open', state);
    }

    state.tag    = null;
    state.anchor = null;
    state.kind   = null;
    state.result = null;

    allowBlockStyles = allowBlockScalars = allowBlockCollections =
      CONTEXT_BLOCK_OUT === nodeContext ||
      CONTEXT_BLOCK_IN  === nodeContext;

    if (allowToSeek) {
      if (skipSeparationSpace(state, true, -1)) {
        atNewLine = true;

        if (state.lineIndent > parentIndent) {
          indentStatus = 1;
        } else if (state.lineIndent === parentIndent) {
          indentStatus = 0;
        } else if (state.lineIndent < parentIndent) {
          indentStatus = -1;
        }
      }
    }

    if (indentStatus === 1) {
      while (readTagProperty(state) || readAnchorProperty(state)) {
        if (skipSeparationSpace(state, true, -1)) {
          atNewLine = true;
          allowBlockCollections = allowBlockStyles;

          if (state.lineIndent > parentIndent) {
            indentStatus = 1;
          } else if (state.lineIndent === parentIndent) {
            indentStatus = 0;
          } else if (state.lineIndent < parentIndent) {
            indentStatus = -1;
          }
        } else {
          allowBlockCollections = false;
        }
      }
    }

    if (allowBlockCollections) {
      allowBlockCollections = atNewLine || allowCompact;
    }

    if (indentStatus === 1 || CONTEXT_BLOCK_OUT === nodeContext) {
      if (CONTEXT_FLOW_IN === nodeContext || CONTEXT_FLOW_OUT === nodeContext) {
        flowIndent = parentIndent;
      } else {
        flowIndent = parentIndent + 1;
      }

      blockIndent = state.position - state.lineStart;

      if (indentStatus === 1) {
        if (allowBlockCollections &&
            (readBlockSequence(state, blockIndent) ||
             readBlockMapping(state, blockIndent, flowIndent)) ||
            readFlowCollection(state, flowIndent)) {
          hasContent = true;
        } else {
          if ((allowBlockScalars && readBlockScalar(state, flowIndent)) ||
              readSingleQuotedScalar(state, flowIndent) ||
              readDoubleQuotedScalar(state, flowIndent)) {
            hasContent = true;

          } else if (readAlias(state)) {
            hasContent = true;

            if (state.tag !== null || state.anchor !== null) {
              throwError(state, 'alias node should not have any properties');
            }

          } else if (readPlainScalar(state, flowIndent, CONTEXT_FLOW_IN === nodeContext)) {
            hasContent = true;

            if (state.tag === null) {
              state.tag = '?';
            }
          }

          if (state.anchor !== null) {
            state.anchorMap[state.anchor] = state.result;
          }
        }
      } else if (indentStatus === 0) {
        // Special case: block sequences are allowed to have same indentation level as the parent.
        // http://www.yaml.org/spec/1.2/spec.html#id2799784
        hasContent = allowBlockCollections && readBlockSequence(state, blockIndent);
      }
    }

    if (state.tag === null) {
      if (state.anchor !== null) {
        state.anchorMap[state.anchor] = state.result;
      }

    } else if (state.tag === '?') {
      // Implicit resolving is not allowed for non-scalar types, and '?'
      // non-specific tag is only automatically assigned to plain scalars.
      //
      // We only need to check kind conformity in case user explicitly assigns '?'
      // tag, for example like this: "!<?> [0]"
      //
      if (state.result !== null && state.kind !== 'scalar') {
        throwError(state, 'unacceptable node kind for !<?> tag; it should be "scalar", not "' + state.kind + '"');
      }

      for (typeIndex = 0, typeQuantity = state.implicitTypes.length; typeIndex < typeQuantity; typeIndex += 1) {
        type = state.implicitTypes[typeIndex];

        if (type.resolve(state.result)) { // `state.result` updated in resolver if matched
          state.result = type.construct(state.result);
          state.tag = type.tag;
          if (state.anchor !== null) {
            state.anchorMap[state.anchor] = state.result;
          }
          break;
        }
      }
    } else if (state.tag !== '!') {
      if (_hasOwnProperty$1.call(state.typeMap[state.kind || 'fallback'], state.tag)) {
        type = state.typeMap[state.kind || 'fallback'][state.tag];
      } else {
        // looking for multi type
        type = null;
        typeList = state.typeMap.multi[state.kind || 'fallback'];

        for (typeIndex = 0, typeQuantity = typeList.length; typeIndex < typeQuantity; typeIndex += 1) {
          if (state.tag.slice(0, typeList[typeIndex].tag.length) === typeList[typeIndex].tag) {
            type = typeList[typeIndex];
            break;
          }
        }
      }

      if (!type) {
        throwError(state, 'unknown tag !<' + state.tag + '>');
      }

      if (state.result !== null && type.kind !== state.kind) {
        throwError(state, 'unacceptable node kind for !<' + state.tag + '> tag; it should be "' + type.kind + '", not "' + state.kind + '"');
      }

      if (!type.resolve(state.result, state.tag)) { // `state.result` updated in resolver if matched
        throwError(state, 'cannot resolve a node with !<' + state.tag + '> explicit tag');
      } else {
        state.result = type.construct(state.result, state.tag);
        if (state.anchor !== null) {
          state.anchorMap[state.anchor] = state.result;
        }
      }
    }

    if (state.listener !== null) {
      state.listener('close', state);
    }
    return state.tag !== null ||  state.anchor !== null || hasContent;
  }

  function readDocument(state) {
    var documentStart = state.position,
        _position,
        directiveName,
        directiveArgs,
        hasDirectives = false,
        ch;

    state.version = null;
    state.checkLineBreaks = state.legacy;
    state.tagMap = Object.create(null);
    state.anchorMap = Object.create(null);

    while ((ch = state.input.charCodeAt(state.position)) !== 0) {
      skipSeparationSpace(state, true, -1);

      ch = state.input.charCodeAt(state.position);

      if (state.lineIndent > 0 || ch !== 0x25/* % */) {
        break;
      }

      hasDirectives = true;
      ch = state.input.charCodeAt(++state.position);
      _position = state.position;

      while (ch !== 0 && !is_WS_OR_EOL(ch)) {
        ch = state.input.charCodeAt(++state.position);
      }

      directiveName = state.input.slice(_position, state.position);
      directiveArgs = [];

      if (directiveName.length < 1) {
        throwError(state, 'directive name must not be less than one character in length');
      }

      while (ch !== 0) {
        while (is_WHITE_SPACE(ch)) {
          ch = state.input.charCodeAt(++state.position);
        }

        if (ch === 0x23/* # */) {
          do { ch = state.input.charCodeAt(++state.position); }
          while (ch !== 0 && !is_EOL(ch));
          break;
        }

        if (is_EOL(ch)) break;

        _position = state.position;

        while (ch !== 0 && !is_WS_OR_EOL(ch)) {
          ch = state.input.charCodeAt(++state.position);
        }

        directiveArgs.push(state.input.slice(_position, state.position));
      }

      if (ch !== 0) readLineBreak(state);

      if (_hasOwnProperty$1.call(directiveHandlers, directiveName)) {
        directiveHandlers[directiveName](state, directiveName, directiveArgs);
      } else {
        throwWarning(state, 'unknown document directive "' + directiveName + '"');
      }
    }

    skipSeparationSpace(state, true, -1);

    if (state.lineIndent === 0 &&
        state.input.charCodeAt(state.position)     === 0x2D/* - */ &&
        state.input.charCodeAt(state.position + 1) === 0x2D/* - */ &&
        state.input.charCodeAt(state.position + 2) === 0x2D/* - */) {
      state.position += 3;
      skipSeparationSpace(state, true, -1);

    } else if (hasDirectives) {
      throwError(state, 'directives end mark is expected');
    }

    composeNode(state, state.lineIndent - 1, CONTEXT_BLOCK_OUT, false, true);
    skipSeparationSpace(state, true, -1);

    if (state.checkLineBreaks &&
        PATTERN_NON_ASCII_LINE_BREAKS.test(state.input.slice(documentStart, state.position))) {
      throwWarning(state, 'non-ASCII line breaks are interpreted as content');
    }

    state.documents.push(state.result);

    if (state.position === state.lineStart && testDocumentSeparator(state)) {

      if (state.input.charCodeAt(state.position) === 0x2E/* . */) {
        state.position += 3;
        skipSeparationSpace(state, true, -1);
      }
      return;
    }

    if (state.position < (state.length - 1)) {
      throwError(state, 'end of the stream or a document separator is expected');
    } else {
      return;
    }
  }


  function loadDocuments(input, options) {
    input = String(input);
    options = options || {};

    if (input.length !== 0) {

      // Add tailing `\n` if not exists
      if (input.charCodeAt(input.length - 1) !== 0x0A/* LF */ &&
          input.charCodeAt(input.length - 1) !== 0x0D/* CR */) {
        input += '\n';
      }

      // Strip BOM
      if (input.charCodeAt(0) === 0xFEFF) {
        input = input.slice(1);
      }
    }

    var state = new State$1(input, options);

    var nullpos = input.indexOf('\0');

    if (nullpos !== -1) {
      state.position = nullpos;
      throwError(state, 'null byte is not allowed in input');
    }

    // Use 0 as string terminator. That significantly simplifies bounds check.
    state.input += '\0';

    while (state.input.charCodeAt(state.position) === 0x20/* Space */) {
      state.lineIndent += 1;
      state.position += 1;
    }

    while (state.position < (state.length - 1)) {
      readDocument(state);
    }

    return state.documents;
  }


  function loadAll$1(input, iterator, options) {
    if (iterator !== null && typeof iterator === 'object' && typeof options === 'undefined') {
      options = iterator;
      iterator = null;
    }

    var documents = loadDocuments(input, options);

    if (typeof iterator !== 'function') {
      return documents;
    }

    for (var index = 0, length = documents.length; index < length; index += 1) {
      iterator(documents[index]);
    }
  }


  function load$1(input, options) {
    var documents = loadDocuments(input, options);

    if (documents.length === 0) {
      /*eslint-disable no-undefined*/
      return undefined;
    } else if (documents.length === 1) {
      return documents[0];
    }
    throw new exception('expected a single document in the stream, but found more');
  }


  var loadAll_1 = loadAll$1;
  var load_1    = load$1;

  var loader = {
  	loadAll: loadAll_1,
  	load: load_1
  };

  /*eslint-disable no-use-before-define*/





  var _toString       = Object.prototype.toString;
  var _hasOwnProperty = Object.prototype.hasOwnProperty;

  var CHAR_BOM                  = 0xFEFF;
  var CHAR_TAB                  = 0x09; /* Tab */
  var CHAR_LINE_FEED            = 0x0A; /* LF */
  var CHAR_CARRIAGE_RETURN      = 0x0D; /* CR */
  var CHAR_SPACE                = 0x20; /* Space */
  var CHAR_EXCLAMATION          = 0x21; /* ! */
  var CHAR_DOUBLE_QUOTE         = 0x22; /* " */
  var CHAR_SHARP                = 0x23; /* # */
  var CHAR_PERCENT              = 0x25; /* % */
  var CHAR_AMPERSAND            = 0x26; /* & */
  var CHAR_SINGLE_QUOTE         = 0x27; /* ' */
  var CHAR_ASTERISK             = 0x2A; /* * */
  var CHAR_COMMA                = 0x2C; /* , */
  var CHAR_MINUS                = 0x2D; /* - */
  var CHAR_COLON                = 0x3A; /* : */
  var CHAR_EQUALS               = 0x3D; /* = */
  var CHAR_GREATER_THAN         = 0x3E; /* > */
  var CHAR_QUESTION             = 0x3F; /* ? */
  var CHAR_COMMERCIAL_AT        = 0x40; /* @ */
  var CHAR_LEFT_SQUARE_BRACKET  = 0x5B; /* [ */
  var CHAR_RIGHT_SQUARE_BRACKET = 0x5D; /* ] */
  var CHAR_GRAVE_ACCENT         = 0x60; /* ` */
  var CHAR_LEFT_CURLY_BRACKET   = 0x7B; /* { */
  var CHAR_VERTICAL_LINE        = 0x7C; /* | */
  var CHAR_RIGHT_CURLY_BRACKET  = 0x7D; /* } */

  var ESCAPE_SEQUENCES = {};

  ESCAPE_SEQUENCES[0x00]   = '\\0';
  ESCAPE_SEQUENCES[0x07]   = '\\a';
  ESCAPE_SEQUENCES[0x08]   = '\\b';
  ESCAPE_SEQUENCES[0x09]   = '\\t';
  ESCAPE_SEQUENCES[0x0A]   = '\\n';
  ESCAPE_SEQUENCES[0x0B]   = '\\v';
  ESCAPE_SEQUENCES[0x0C]   = '\\f';
  ESCAPE_SEQUENCES[0x0D]   = '\\r';
  ESCAPE_SEQUENCES[0x1B]   = '\\e';
  ESCAPE_SEQUENCES[0x22]   = '\\"';
  ESCAPE_SEQUENCES[0x5C]   = '\\\\';
  ESCAPE_SEQUENCES[0x85]   = '\\N';
  ESCAPE_SEQUENCES[0xA0]   = '\\_';
  ESCAPE_SEQUENCES[0x2028] = '\\L';
  ESCAPE_SEQUENCES[0x2029] = '\\P';

  var DEPRECATED_BOOLEANS_SYNTAX = [
    'y', 'Y', 'yes', 'Yes', 'YES', 'on', 'On', 'ON',
    'n', 'N', 'no', 'No', 'NO', 'off', 'Off', 'OFF'
  ];

  var DEPRECATED_BASE60_SYNTAX = /^[-+]?[0-9_]+(?::[0-9_]+)+(?:\.[0-9_]*)?$/;

  function compileStyleMap(schema, map) {
    var result, keys, index, length, tag, style, type;

    if (map === null) return {};

    result = {};
    keys = Object.keys(map);

    for (index = 0, length = keys.length; index < length; index += 1) {
      tag = keys[index];
      style = String(map[tag]);

      if (tag.slice(0, 2) === '!!') {
        tag = 'tag:yaml.org,2002:' + tag.slice(2);
      }
      type = schema.compiledTypeMap['fallback'][tag];

      if (type && _hasOwnProperty.call(type.styleAliases, style)) {
        style = type.styleAliases[style];
      }

      result[tag] = style;
    }

    return result;
  }

  function encodeHex(character) {
    var string, handle, length;

    string = character.toString(16).toUpperCase();

    if (character <= 0xFF) {
      handle = 'x';
      length = 2;
    } else if (character <= 0xFFFF) {
      handle = 'u';
      length = 4;
    } else if (character <= 0xFFFFFFFF) {
      handle = 'U';
      length = 8;
    } else {
      throw new exception('code point within a string may not be greater than 0xFFFFFFFF');
    }

    return '\\' + handle + common.repeat('0', length - string.length) + string;
  }


  var QUOTING_TYPE_SINGLE = 1,
      QUOTING_TYPE_DOUBLE = 2;

  function State(options) {
    this.schema        = options['schema'] || _default;
    this.indent        = Math.max(1, (options['indent'] || 2));
    this.noArrayIndent = options['noArrayIndent'] || false;
    this.skipInvalid   = options['skipInvalid'] || false;
    this.flowLevel     = (common.isNothing(options['flowLevel']) ? -1 : options['flowLevel']);
    this.styleMap      = compileStyleMap(this.schema, options['styles'] || null);
    this.sortKeys      = options['sortKeys'] || false;
    this.lineWidth     = options['lineWidth'] || 80;
    this.noRefs        = options['noRefs'] || false;
    this.noCompatMode  = options['noCompatMode'] || false;
    this.condenseFlow  = options['condenseFlow'] || false;
    this.quotingType   = options['quotingType'] === '"' ? QUOTING_TYPE_DOUBLE : QUOTING_TYPE_SINGLE;
    this.forceQuotes   = options['forceQuotes'] || false;
    this.replacer      = typeof options['replacer'] === 'function' ? options['replacer'] : null;

    this.implicitTypes = this.schema.compiledImplicit;
    this.explicitTypes = this.schema.compiledExplicit;

    this.tag = null;
    this.result = '';

    this.duplicates = [];
    this.usedDuplicates = null;
  }

  // Indents every line in a string. Empty lines (\n only) are not indented.
  function indentString(string, spaces) {
    var ind = common.repeat(' ', spaces),
        position = 0,
        next = -1,
        result = '',
        line,
        length = string.length;

    while (position < length) {
      next = string.indexOf('\n', position);
      if (next === -1) {
        line = string.slice(position);
        position = length;
      } else {
        line = string.slice(position, next + 1);
        position = next + 1;
      }

      if (line.length && line !== '\n') result += ind;

      result += line;
    }

    return result;
  }

  function generateNextLine(state, level) {
    return '\n' + common.repeat(' ', state.indent * level);
  }

  function testImplicitResolving(state, str) {
    var index, length, type;

    for (index = 0, length = state.implicitTypes.length; index < length; index += 1) {
      type = state.implicitTypes[index];

      if (type.resolve(str)) {
        return true;
      }
    }

    return false;
  }

  // [33] s-white ::= s-space | s-tab
  function isWhitespace(c) {
    return c === CHAR_SPACE || c === CHAR_TAB;
  }

  // Returns true if the character can be printed without escaping.
  // From YAML 1.2: "any allowed characters known to be non-printable
  // should also be escaped. [However,] This isn’t mandatory"
  // Derived from nb-char - \t - #x85 - #xA0 - #x2028 - #x2029.
  function isPrintable(c) {
    return  (0x00020 <= c && c <= 0x00007E)
        || ((0x000A1 <= c && c <= 0x00D7FF) && c !== 0x2028 && c !== 0x2029)
        || ((0x0E000 <= c && c <= 0x00FFFD) && c !== CHAR_BOM)
        ||  (0x10000 <= c && c <= 0x10FFFF);
  }

  // [34] ns-char ::= nb-char - s-white
  // [27] nb-char ::= c-printable - b-char - c-byte-order-mark
  // [26] b-char  ::= b-line-feed | b-carriage-return
  // Including s-white (for some reason, examples doesn't match specs in this aspect)
  // ns-char ::= c-printable - b-line-feed - b-carriage-return - c-byte-order-mark
  function isNsCharOrWhitespace(c) {
    return isPrintable(c)
      && c !== CHAR_BOM
      // - b-char
      && c !== CHAR_CARRIAGE_RETURN
      && c !== CHAR_LINE_FEED;
  }

  // [127]  ns-plain-safe(c) ::= c = flow-out  ⇒ ns-plain-safe-out
  //                             c = flow-in   ⇒ ns-plain-safe-in
  //                             c = block-key ⇒ ns-plain-safe-out
  //                             c = flow-key  ⇒ ns-plain-safe-in
  // [128] ns-plain-safe-out ::= ns-char
  // [129]  ns-plain-safe-in ::= ns-char - c-flow-indicator
  // [130]  ns-plain-char(c) ::=  ( ns-plain-safe(c) - “:” - “#” )
  //                            | ( /* An ns-char preceding */ “#” )
  //                            | ( “:” /* Followed by an ns-plain-safe(c) */ )
  function isPlainSafe(c, prev, inblock) {
    var cIsNsCharOrWhitespace = isNsCharOrWhitespace(c);
    var cIsNsChar = cIsNsCharOrWhitespace && !isWhitespace(c);
    return (
      // ns-plain-safe
      inblock ? // c = flow-in
        cIsNsCharOrWhitespace
        : cIsNsCharOrWhitespace
          // - c-flow-indicator
          && c !== CHAR_COMMA
          && c !== CHAR_LEFT_SQUARE_BRACKET
          && c !== CHAR_RIGHT_SQUARE_BRACKET
          && c !== CHAR_LEFT_CURLY_BRACKET
          && c !== CHAR_RIGHT_CURLY_BRACKET
    )
      // ns-plain-char
      && c !== CHAR_SHARP // false on '#'
      && !(prev === CHAR_COLON && !cIsNsChar) // false on ': '
      || (isNsCharOrWhitespace(prev) && !isWhitespace(prev) && c === CHAR_SHARP) // change to true on '[^ ]#'
      || (prev === CHAR_COLON && cIsNsChar); // change to true on ':[^ ]'
  }

  // Simplified test for values allowed as the first character in plain style.
  function isPlainSafeFirst(c) {
    // Uses a subset of ns-char - c-indicator
    // where ns-char = nb-char - s-white.
    // No support of ( ( “?” | “:” | “-” ) /* Followed by an ns-plain-safe(c)) */ ) part
    return isPrintable(c) && c !== CHAR_BOM
      && !isWhitespace(c) // - s-white
      // - (c-indicator ::=
      // “-” | “?” | “:” | “,” | “[” | “]” | “{” | “}”
      && c !== CHAR_MINUS
      && c !== CHAR_QUESTION
      && c !== CHAR_COLON
      && c !== CHAR_COMMA
      && c !== CHAR_LEFT_SQUARE_BRACKET
      && c !== CHAR_RIGHT_SQUARE_BRACKET
      && c !== CHAR_LEFT_CURLY_BRACKET
      && c !== CHAR_RIGHT_CURLY_BRACKET
      // | “#” | “&” | “*” | “!” | “|” | “=” | “>” | “'” | “"”
      && c !== CHAR_SHARP
      && c !== CHAR_AMPERSAND
      && c !== CHAR_ASTERISK
      && c !== CHAR_EXCLAMATION
      && c !== CHAR_VERTICAL_LINE
      && c !== CHAR_EQUALS
      && c !== CHAR_GREATER_THAN
      && c !== CHAR_SINGLE_QUOTE
      && c !== CHAR_DOUBLE_QUOTE
      // | “%” | “@” | “`”)
      && c !== CHAR_PERCENT
      && c !== CHAR_COMMERCIAL_AT
      && c !== CHAR_GRAVE_ACCENT;
  }

  // Simplified test for values allowed as the last character in plain style.
  function isPlainSafeLast(c) {
    // just not whitespace or colon, it will be checked to be plain character later
    return !isWhitespace(c) && c !== CHAR_COLON;
  }

  // Same as 'string'.codePointAt(pos), but works in older browsers.
  function codePointAt(string, pos) {
    var first = string.charCodeAt(pos), second;
    if (first >= 0xD800 && first <= 0xDBFF && pos + 1 < string.length) {
      second = string.charCodeAt(pos + 1);
      if (second >= 0xDC00 && second <= 0xDFFF) {
        // https://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        return (first - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;
      }
    }
    return first;
  }

  // Determines whether block indentation indicator is required.
  function needIndentIndicator(string) {
    var leadingSpaceRe = /^\n* /;
    return leadingSpaceRe.test(string);
  }

  var STYLE_PLAIN   = 1,
      STYLE_SINGLE  = 2,
      STYLE_LITERAL = 3,
      STYLE_FOLDED  = 4,
      STYLE_DOUBLE  = 5;

  // Determines which scalar styles are possible and returns the preferred style.
  // lineWidth = -1 => no limit.
  // Pre-conditions: str.length > 0.
  // Post-conditions:
  //    STYLE_PLAIN or STYLE_SINGLE => no \n are in the string.
  //    STYLE_LITERAL => no lines are suitable for folding (or lineWidth is -1).
  //    STYLE_FOLDED => a line > lineWidth and can be folded (and lineWidth != -1).
  function chooseScalarStyle(string, singleLineOnly, indentPerLevel, lineWidth,
    testAmbiguousType, quotingType, forceQuotes, inblock) {

    var i;
    var char = 0;
    var prevChar = null;
    var hasLineBreak = false;
    var hasFoldableLine = false; // only checked if shouldTrackWidth
    var shouldTrackWidth = lineWidth !== -1;
    var previousLineBreak = -1; // count the first line correctly
    var plain = isPlainSafeFirst(codePointAt(string, 0))
            && isPlainSafeLast(codePointAt(string, string.length - 1));

    if (singleLineOnly || forceQuotes) {
      // Case: no block styles.
      // Check for disallowed characters to rule out plain and single.
      for (i = 0; i < string.length; char >= 0x10000 ? i += 2 : i++) {
        char = codePointAt(string, i);
        if (!isPrintable(char)) {
          return STYLE_DOUBLE;
        }
        plain = plain && isPlainSafe(char, prevChar, inblock);
        prevChar = char;
      }
    } else {
      // Case: block styles permitted.
      for (i = 0; i < string.length; char >= 0x10000 ? i += 2 : i++) {
        char = codePointAt(string, i);
        if (char === CHAR_LINE_FEED) {
          hasLineBreak = true;
          // Check if any line can be folded.
          if (shouldTrackWidth) {
            hasFoldableLine = hasFoldableLine ||
              // Foldable line = too long, and not more-indented.
              (i - previousLineBreak - 1 > lineWidth &&
               string[previousLineBreak + 1] !== ' ');
            previousLineBreak = i;
          }
        } else if (!isPrintable(char)) {
          return STYLE_DOUBLE;
        }
        plain = plain && isPlainSafe(char, prevChar, inblock);
        prevChar = char;
      }
      // in case the end is missing a \n
      hasFoldableLine = hasFoldableLine || (shouldTrackWidth &&
        (i - previousLineBreak - 1 > lineWidth &&
         string[previousLineBreak + 1] !== ' '));
    }
    // Although every style can represent \n without escaping, prefer block styles
    // for multiline, since they're more readable and they don't add empty lines.
    // Also prefer folding a super-long line.
    if (!hasLineBreak && !hasFoldableLine) {
      // Strings interpretable as another type have to be quoted;
      // e.g. the string 'true' vs. the boolean true.
      if (plain && !forceQuotes && !testAmbiguousType(string)) {
        return STYLE_PLAIN;
      }
      return quotingType === QUOTING_TYPE_DOUBLE ? STYLE_DOUBLE : STYLE_SINGLE;
    }
    // Edge case: block indentation indicator can only have one digit.
    if (indentPerLevel > 9 && needIndentIndicator(string)) {
      return STYLE_DOUBLE;
    }
    // At this point we know block styles are valid.
    // Prefer literal style unless we want to fold.
    if (!forceQuotes) {
      return hasFoldableLine ? STYLE_FOLDED : STYLE_LITERAL;
    }
    return quotingType === QUOTING_TYPE_DOUBLE ? STYLE_DOUBLE : STYLE_SINGLE;
  }

  // Note: line breaking/folding is implemented for only the folded style.
  // NB. We drop the last trailing newline (if any) of a returned block scalar
  //  since the dumper adds its own newline. This always works:
  //    • No ending newline => unaffected; already using strip "-" chomping.
  //    • Ending newline    => removed then restored.
  //  Importantly, this keeps the "+" chomp indicator from gaining an extra line.
  function writeScalar(state, string, level, iskey, inblock) {
    state.dump = (function () {
      if (string.length === 0) {
        return state.quotingType === QUOTING_TYPE_DOUBLE ? '""' : "''";
      }
      if (!state.noCompatMode) {
        if (DEPRECATED_BOOLEANS_SYNTAX.indexOf(string) !== -1 || DEPRECATED_BASE60_SYNTAX.test(string)) {
          return state.quotingType === QUOTING_TYPE_DOUBLE ? ('"' + string + '"') : ("'" + string + "'");
        }
      }

      var indent = state.indent * Math.max(1, level); // no 0-indent scalars
      // As indentation gets deeper, let the width decrease monotonically
      // to the lower bound min(state.lineWidth, 40).
      // Note that this implies
      //  state.lineWidth ≤ 40 + state.indent: width is fixed at the lower bound.
      //  state.lineWidth > 40 + state.indent: width decreases until the lower bound.
      // This behaves better than a constant minimum width which disallows narrower options,
      // or an indent threshold which causes the width to suddenly increase.
      var lineWidth = state.lineWidth === -1
        ? -1 : Math.max(Math.min(state.lineWidth, 40), state.lineWidth - indent);

      // Without knowing if keys are implicit/explicit, assume implicit for safety.
      var singleLineOnly = iskey
        // No block styles in flow mode.
        || (state.flowLevel > -1 && level >= state.flowLevel);
      function testAmbiguity(string) {
        return testImplicitResolving(state, string);
      }

      switch (chooseScalarStyle(string, singleLineOnly, state.indent, lineWidth,
        testAmbiguity, state.quotingType, state.forceQuotes && !iskey, inblock)) {

        case STYLE_PLAIN:
          return string;
        case STYLE_SINGLE:
          return "'" + string.replace(/'/g, "''") + "'";
        case STYLE_LITERAL:
          return '|' + blockHeader(string, state.indent)
            + dropEndingNewline(indentString(string, indent));
        case STYLE_FOLDED:
          return '>' + blockHeader(string, state.indent)
            + dropEndingNewline(indentString(foldString(string, lineWidth), indent));
        case STYLE_DOUBLE:
          return '"' + escapeString(string) + '"';
        default:
          throw new exception('impossible error: invalid scalar style');
      }
    }());
  }

  // Pre-conditions: string is valid for a block scalar, 1 <= indentPerLevel <= 9.
  function blockHeader(string, indentPerLevel) {
    var indentIndicator = needIndentIndicator(string) ? String(indentPerLevel) : '';

    // note the special case: the string '\n' counts as a "trailing" empty line.
    var clip =          string[string.length - 1] === '\n';
    var keep = clip && (string[string.length - 2] === '\n' || string === '\n');
    var chomp = keep ? '+' : (clip ? '' : '-');

    return indentIndicator + chomp + '\n';
  }

  // (See the note for writeScalar.)
  function dropEndingNewline(string) {
    return string[string.length - 1] === '\n' ? string.slice(0, -1) : string;
  }

  // Note: a long line without a suitable break point will exceed the width limit.
  // Pre-conditions: every char in str isPrintable, str.length > 0, width > 0.
  function foldString(string, width) {
    // In folded style, $k$ consecutive newlines output as $k+1$ newlines—
    // unless they're before or after a more-indented line, or at the very
    // beginning or end, in which case $k$ maps to $k$.
    // Therefore, parse each chunk as newline(s) followed by a content line.
    var lineRe = /(\n+)([^\n]*)/g;

    // first line (possibly an empty line)
    var result = (function () {
      var nextLF = string.indexOf('\n');
      nextLF = nextLF !== -1 ? nextLF : string.length;
      lineRe.lastIndex = nextLF;
      return foldLine(string.slice(0, nextLF), width);
    }());
    // If we haven't reached the first content line yet, don't add an extra \n.
    var prevMoreIndented = string[0] === '\n' || string[0] === ' ';
    var moreIndented;

    // rest of the lines
    var match;
    while ((match = lineRe.exec(string))) {
      var prefix = match[1], line = match[2];
      moreIndented = (line[0] === ' ');
      result += prefix
        + (!prevMoreIndented && !moreIndented && line !== ''
          ? '\n' : '')
        + foldLine(line, width);
      prevMoreIndented = moreIndented;
    }

    return result;
  }

  // Greedy line breaking.
  // Picks the longest line under the limit each time,
  // otherwise settles for the shortest line over the limit.
  // NB. More-indented lines *cannot* be folded, as that would add an extra \n.
  function foldLine(line, width) {
    if (line === '' || line[0] === ' ') return line;

    // Since a more-indented line adds a \n, breaks can't be followed by a space.
    var breakRe = / [^ ]/g; // note: the match index will always be <= length-2.
    var match;
    // start is an inclusive index. end, curr, and next are exclusive.
    var start = 0, end, curr = 0, next = 0;
    var result = '';

    // Invariants: 0 <= start <= length-1.
    //   0 <= curr <= next <= max(0, length-2). curr - start <= width.
    // Inside the loop:
    //   A match implies length >= 2, so curr and next are <= length-2.
    while ((match = breakRe.exec(line))) {
      next = match.index;
      // maintain invariant: curr - start <= width
      if (next - start > width) {
        end = (curr > start) ? curr : next; // derive end <= length-2
        result += '\n' + line.slice(start, end);
        // skip the space that was output as \n
        start = end + 1;                    // derive start <= length-1
      }
      curr = next;
    }

    // By the invariants, start <= length-1, so there is something left over.
    // It is either the whole string or a part starting from non-whitespace.
    result += '\n';
    // Insert a break if the remainder is too long and there is a break available.
    if (line.length - start > width && curr > start) {
      result += line.slice(start, curr) + '\n' + line.slice(curr + 1);
    } else {
      result += line.slice(start);
    }

    return result.slice(1); // drop extra \n joiner
  }

  // Escapes a double-quoted string.
  function escapeString(string) {
    var result = '';
    var char = 0;
    var escapeSeq;

    for (var i = 0; i < string.length; char >= 0x10000 ? i += 2 : i++) {
      char = codePointAt(string, i);
      escapeSeq = ESCAPE_SEQUENCES[char];

      if (!escapeSeq && isPrintable(char)) {
        result += string[i];
        if (char >= 0x10000) result += string[i + 1];
      } else {
        result += escapeSeq || encodeHex(char);
      }
    }

    return result;
  }

  function writeFlowSequence(state, level, object) {
    var _result = '',
        _tag    = state.tag,
        index,
        length,
        value;

    for (index = 0, length = object.length; index < length; index += 1) {
      value = object[index];

      if (state.replacer) {
        value = state.replacer.call(object, String(index), value);
      }

      // Write only valid elements, put null instead of invalid elements.
      if (writeNode(state, level, value, false, false) ||
          (typeof value === 'undefined' &&
           writeNode(state, level, null, false, false))) {

        if (_result !== '') _result += ',' + (!state.condenseFlow ? ' ' : '');
        _result += state.dump;
      }
    }

    state.tag = _tag;
    state.dump = '[' + _result + ']';
  }

  function writeBlockSequence(state, level, object, compact) {
    var _result = '',
        _tag    = state.tag,
        index,
        length,
        value;

    for (index = 0, length = object.length; index < length; index += 1) {
      value = object[index];

      if (state.replacer) {
        value = state.replacer.call(object, String(index), value);
      }

      // Write only valid elements, put null instead of invalid elements.
      if (writeNode(state, level + 1, value, true, true, false, true) ||
          (typeof value === 'undefined' &&
           writeNode(state, level + 1, null, true, true, false, true))) {

        if (!compact || _result !== '') {
          _result += generateNextLine(state, level);
        }

        if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
          _result += '-';
        } else {
          _result += '- ';
        }

        _result += state.dump;
      }
    }

    state.tag = _tag;
    state.dump = _result || '[]'; // Empty sequence if no valid values.
  }

  function writeFlowMapping(state, level, object) {
    var _result       = '',
        _tag          = state.tag,
        objectKeyList = Object.keys(object),
        index,
        length,
        objectKey,
        objectValue,
        pairBuffer;

    for (index = 0, length = objectKeyList.length; index < length; index += 1) {

      pairBuffer = '';
      if (_result !== '') pairBuffer += ', ';

      if (state.condenseFlow) pairBuffer += '"';

      objectKey = objectKeyList[index];
      objectValue = object[objectKey];

      if (state.replacer) {
        objectValue = state.replacer.call(object, objectKey, objectValue);
      }

      if (!writeNode(state, level, objectKey, false, false)) {
        continue; // Skip this pair because of invalid key;
      }

      if (state.dump.length > 1024) pairBuffer += '? ';

      pairBuffer += state.dump + (state.condenseFlow ? '"' : '') + ':' + (state.condenseFlow ? '' : ' ');

      if (!writeNode(state, level, objectValue, false, false)) {
        continue; // Skip this pair because of invalid value.
      }

      pairBuffer += state.dump;

      // Both key and value are valid.
      _result += pairBuffer;
    }

    state.tag = _tag;
    state.dump = '{' + _result + '}';
  }

  function writeBlockMapping(state, level, object, compact) {
    var _result       = '',
        _tag          = state.tag,
        objectKeyList = Object.keys(object),
        index,
        length,
        objectKey,
        objectValue,
        explicitPair,
        pairBuffer;

    // Allow sorting keys so that the output file is deterministic
    if (state.sortKeys === true) {
      // Default sorting
      objectKeyList.sort();
    } else if (typeof state.sortKeys === 'function') {
      // Custom sort function
      objectKeyList.sort(state.sortKeys);
    } else if (state.sortKeys) {
      // Something is wrong
      throw new exception('sortKeys must be a boolean or a function');
    }

    for (index = 0, length = objectKeyList.length; index < length; index += 1) {
      pairBuffer = '';

      if (!compact || _result !== '') {
        pairBuffer += generateNextLine(state, level);
      }

      objectKey = objectKeyList[index];
      objectValue = object[objectKey];

      if (state.replacer) {
        objectValue = state.replacer.call(object, objectKey, objectValue);
      }

      if (!writeNode(state, level + 1, objectKey, true, true, true)) {
        continue; // Skip this pair because of invalid key.
      }

      explicitPair = (state.tag !== null && state.tag !== '?') ||
                     (state.dump && state.dump.length > 1024);

      if (explicitPair) {
        if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
          pairBuffer += '?';
        } else {
          pairBuffer += '? ';
        }
      }

      pairBuffer += state.dump;

      if (explicitPair) {
        pairBuffer += generateNextLine(state, level);
      }

      if (!writeNode(state, level + 1, objectValue, true, explicitPair)) {
        continue; // Skip this pair because of invalid value.
      }

      if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
        pairBuffer += ':';
      } else {
        pairBuffer += ': ';
      }

      pairBuffer += state.dump;

      // Both key and value are valid.
      _result += pairBuffer;
    }

    state.tag = _tag;
    state.dump = _result || '{}'; // Empty mapping if no valid pairs.
  }

  function detectType(state, object, explicit) {
    var _result, typeList, index, length, type, style;

    typeList = explicit ? state.explicitTypes : state.implicitTypes;

    for (index = 0, length = typeList.length; index < length; index += 1) {
      type = typeList[index];

      if ((type.instanceOf  || type.predicate) &&
          (!type.instanceOf || ((typeof object === 'object') && (object instanceof type.instanceOf))) &&
          (!type.predicate  || type.predicate(object))) {

        if (explicit) {
          if (type.multi && type.representName) {
            state.tag = type.representName(object);
          } else {
            state.tag = type.tag;
          }
        } else {
          state.tag = '?';
        }

        if (type.represent) {
          style = state.styleMap[type.tag] || type.defaultStyle;

          if (_toString.call(type.represent) === '[object Function]') {
            _result = type.represent(object, style);
          } else if (_hasOwnProperty.call(type.represent, style)) {
            _result = type.represent[style](object, style);
          } else {
            throw new exception('!<' + type.tag + '> tag resolver accepts not "' + style + '" style');
          }

          state.dump = _result;
        }

        return true;
      }
    }

    return false;
  }

  // Serializes `object` and writes it to global `result`.
  // Returns true on success, or false on invalid object.
  //
  function writeNode(state, level, object, block, compact, iskey, isblockseq) {
    state.tag = null;
    state.dump = object;

    if (!detectType(state, object, false)) {
      detectType(state, object, true);
    }

    var type = _toString.call(state.dump);
    var inblock = block;
    var tagStr;

    if (block) {
      block = (state.flowLevel < 0 || state.flowLevel > level);
    }

    var objectOrArray = type === '[object Object]' || type === '[object Array]',
        duplicateIndex,
        duplicate;

    if (objectOrArray) {
      duplicateIndex = state.duplicates.indexOf(object);
      duplicate = duplicateIndex !== -1;
    }

    if ((state.tag !== null && state.tag !== '?') || duplicate || (state.indent !== 2 && level > 0)) {
      compact = false;
    }

    if (duplicate && state.usedDuplicates[duplicateIndex]) {
      state.dump = '*ref_' + duplicateIndex;
    } else {
      if (objectOrArray && duplicate && !state.usedDuplicates[duplicateIndex]) {
        state.usedDuplicates[duplicateIndex] = true;
      }
      if (type === '[object Object]') {
        if (block && (Object.keys(state.dump).length !== 0)) {
          writeBlockMapping(state, level, state.dump, compact);
          if (duplicate) {
            state.dump = '&ref_' + duplicateIndex + state.dump;
          }
        } else {
          writeFlowMapping(state, level, state.dump);
          if (duplicate) {
            state.dump = '&ref_' + duplicateIndex + ' ' + state.dump;
          }
        }
      } else if (type === '[object Array]') {
        if (block && (state.dump.length !== 0)) {
          if (state.noArrayIndent && !isblockseq && level > 0) {
            writeBlockSequence(state, level - 1, state.dump, compact);
          } else {
            writeBlockSequence(state, level, state.dump, compact);
          }
          if (duplicate) {
            state.dump = '&ref_' + duplicateIndex + state.dump;
          }
        } else {
          writeFlowSequence(state, level, state.dump);
          if (duplicate) {
            state.dump = '&ref_' + duplicateIndex + ' ' + state.dump;
          }
        }
      } else if (type === '[object String]') {
        if (state.tag !== '?') {
          writeScalar(state, state.dump, level, iskey, inblock);
        }
      } else if (type === '[object Undefined]') {
        return false;
      } else {
        if (state.skipInvalid) return false;
        throw new exception('unacceptable kind of an object to dump ' + type);
      }

      if (state.tag !== null && state.tag !== '?') {
        // Need to encode all characters except those allowed by the spec:
        //
        // [35] ns-dec-digit    ::=  [#x30-#x39] /* 0-9 */
        // [36] ns-hex-digit    ::=  ns-dec-digit
        //                         | [#x41-#x46] /* A-F */ | [#x61-#x66] /* a-f */
        // [37] ns-ascii-letter ::=  [#x41-#x5A] /* A-Z */ | [#x61-#x7A] /* a-z */
        // [38] ns-word-char    ::=  ns-dec-digit | ns-ascii-letter | “-”
        // [39] ns-uri-char     ::=  “%” ns-hex-digit ns-hex-digit | ns-word-char | “#”
        //                         | “;” | “/” | “?” | “:” | “@” | “&” | “=” | “+” | “$” | “,”
        //                         | “_” | “.” | “!” | “~” | “*” | “'” | “(” | “)” | “[” | “]”
        //
        // Also need to encode '!' because it has special meaning (end of tag prefix).
        //
        tagStr = encodeURI(
          state.tag[0] === '!' ? state.tag.slice(1) : state.tag
        ).replace(/!/g, '%21');

        if (state.tag[0] === '!') {
          tagStr = '!' + tagStr;
        } else if (tagStr.slice(0, 18) === 'tag:yaml.org,2002:') {
          tagStr = '!!' + tagStr.slice(18);
        } else {
          tagStr = '!<' + tagStr + '>';
        }

        state.dump = tagStr + ' ' + state.dump;
      }
    }

    return true;
  }

  function getDuplicateReferences(object, state) {
    var objects = [],
        duplicatesIndexes = [],
        index,
        length;

    inspectNode(object, objects, duplicatesIndexes);

    for (index = 0, length = duplicatesIndexes.length; index < length; index += 1) {
      state.duplicates.push(objects[duplicatesIndexes[index]]);
    }
    state.usedDuplicates = new Array(length);
  }

  function inspectNode(object, objects, duplicatesIndexes) {
    var objectKeyList,
        index,
        length;

    if (object !== null && typeof object === 'object') {
      index = objects.indexOf(object);
      if (index !== -1) {
        if (duplicatesIndexes.indexOf(index) === -1) {
          duplicatesIndexes.push(index);
        }
      } else {
        objects.push(object);

        if (Array.isArray(object)) {
          for (index = 0, length = object.length; index < length; index += 1) {
            inspectNode(object[index], objects, duplicatesIndexes);
          }
        } else {
          objectKeyList = Object.keys(object);

          for (index = 0, length = objectKeyList.length; index < length; index += 1) {
            inspectNode(object[objectKeyList[index]], objects, duplicatesIndexes);
          }
        }
      }
    }
  }

  function dump$1(input, options) {
    options = options || {};

    var state = new State(options);

    if (!state.noRefs) getDuplicateReferences(input, state);

    var value = input;

    if (state.replacer) {
      value = state.replacer.call({ '': value }, '', value);
    }

    if (writeNode(state, 0, value, true, true)) return state.dump + '\n';

    return '';
  }

  var dump_1 = dump$1;

  var dumper = {
  	dump: dump_1
  };
  var load                = loader.load;
  var dump                = dumper.dump;

  var entrySuffix = '-entry-node';
  var exitSuffix = '-exit-node';
  var rooId = 'root';
  var doReference = '/do';
  var forReference = '/for';
  var catchReference = '/catch';
  var branchReference = '/fork/branches';
  var tryReference = '/try';
  /**
   * Enumeration of possible node types in a graph.
   */
  exports.GraphNodeType = void 0;
  (function (GraphNodeType) {
      GraphNodeType["Root"] = "root";
      GraphNodeType["Start"] = "start";
      GraphNodeType["End"] = "end";
      GraphNodeType["Entry"] = "entry";
      GraphNodeType["Exit"] = "exit";
      GraphNodeType["Call"] = "call";
      GraphNodeType["Catch"] = "catch";
      GraphNodeType["Do"] = "do";
      GraphNodeType["Emit"] = "emit";
      GraphNodeType["For"] = "for";
      GraphNodeType["Fork"] = "fork";
      GraphNodeType["Listen"] = "listen";
      GraphNodeType["Raise"] = "raise";
      GraphNodeType["Run"] = "run";
      GraphNodeType["Set"] = "set";
      GraphNodeType["Switch"] = "switch";
      GraphNodeType["Try"] = "try";
      GraphNodeType["TryCatch"] = "try-catch";
      GraphNodeType["Wait"] = "wait";
  })(exports.GraphNodeType || (exports.GraphNodeType = {}));
  /**
   * Enumeration of possible workflow flow directives.
   */
  var FlowDirective;
  (function (FlowDirective) {
      FlowDirective["Exit"] = "exit";
      FlowDirective["End"] = "end";
      FlowDirective["Continue"] = "continue";
  })(FlowDirective || (FlowDirective = {}));
  /**
   * Converts an array of TaskItem objects into a Map for easy lookup.
   *
   * @param tasksList An array of TaskItem objects.
   * @returns A map where keys are task names and values are Task objects.
   */
  function mapTasks(tasksList) {
      return (tasksList || []).reduce(function (acc, item) {
          var _a = Object.entries(item)[0], key = _a[0], task = _a[1];
          acc.set(key, task);
          return acc;
      }, new Map());
  }
  /**
   * Initializes a graph with default entry and exit nodes.
   *
   * @param type The type of the graph node.
   * @param id Unique identifier for the graph.
   * @param label Optional label for the graph.
   * @param parent Optional parent graph if this is a subgraph.
   * @returns A newly created Graph instance.
   */
  function initGraph(type, id, label, parent) {
      if (id === void 0) { id = rooId; }
      if (label === void 0) { label = undefined; }
      if (parent === void 0) { parent = undefined; }
      var entryNode = {
          type: id === rooId ? exports.GraphNodeType.Start : exports.GraphNodeType.Entry,
          id: "".concat(id).concat(entrySuffix),
      };
      var exitNode = {
          type: id === rooId ? exports.GraphNodeType.End : exports.GraphNodeType.Exit,
          id: "".concat(id).concat(exitSuffix),
      };
      var graph = {
          id: id,
          label: label,
          type: type,
          parent: parent,
          entryNode: entryNode,
          exitNode: exitNode,
          nodes: [entryNode, exitNode],
          edges: [],
      };
      if (parent)
          parent.nodes.push(graph);
      return graph;
  }
  /**
   * Constructs a graph representation based on the given workflow.
   *
   * @param workflow The workflow to be converted into a graph structure.
   * @returns A graph representation of the workflow.
   */
  function buildGraph(workflow) {
      var graph = initGraph(exports.GraphNodeType.Root);
      buildTransitions(graph.entryNode, {
          graph: graph,
          reference: doReference,
          taskList: mapTasks(workflow.do),
          taskReference: doReference,
      });
      return graph;
  }
  /**
   * Gets the next task to be executed in the workflow
   * @param tasksList The list of task to resolve the next task from
   * @param taskName The current task name, if any
   * @param transition A specific transition, if any
   * @returns
   */
  function getNextTask(tasksList, taskName, transition) {
      if (taskName === void 0) { taskName = undefined; }
      if (transition === void 0) { transition = undefined; }
      if (!(tasksList === null || tasksList === void 0 ? void 0 : tasksList.size))
          throw new Error('The task list cannot be empty. No tasks list to get the next task from.');
      var currentTask = tasksList.get(taskName || '');
      transition = transition || (currentTask === null || currentTask === void 0 ? void 0 : currentTask.then) || '';
      if (transition == FlowDirective.End || transition == FlowDirective.Exit) {
          return {
              name: transition,
              index: -1,
          };
      }
      var index = 0;
      if (transition && transition != FlowDirective.Continue) {
          index = Array.from(tasksList.keys()).indexOf(transition);
      }
      else if (currentTask) {
          index = Array.from(tasksList.values()).indexOf(currentTask) + 1;
          if (index >= tasksList.size) {
              return {
                  name: FlowDirective.End,
                  index: -1,
              };
          }
      }
      var taskEntry = Array.from(tasksList.entries())[index];
      return {
          index: index,
          name: taskEntry[0],
          task: taskEntry[1],
      };
  }
  /**
   * Builds the provided transition from the source node
   * @param sourceNode The node to build the transition from
   * @param transition The transition to follow
   * @param context The context in which the transition is built
   */
  function buildTransition(sourceNode, transition, context) {
      var exitAnchor = sourceNode.exitNode || sourceNode;
      if (transition.index != -1) {
          var destinationNode = buildTaskNode(__assign(__assign({}, context), { taskReference: "".concat(context.reference, "/").concat(transition.index, "/").concat(transition.name), taskName: transition.name }));
          buildEdge(context.graph, exitAnchor, destinationNode.entryNode || destinationNode, transition.label);
      }
      else if (transition.name === FlowDirective.Exit) {
          buildEdge(context.graph, exitAnchor, context.graph.exitNode, transition.label);
      }
      else if (transition.name === FlowDirective.End) {
          buildEdge(context.graph, exitAnchor, context.graph.exitNode, transition.label);
      }
      else
          throw new Error('Invalid transition');
  }
  /**
   * Builds all the possible transitions from the provided node in the provided context
   * @param sourceNode The node to build the transitions from
   * @param context The context in which the transitions are built
   */
  function buildTransitions(sourceNode, context) {
      var _a, _b;
      var transitions = [];
      var nextTransition = getNextTask(context.taskList, context.taskName);
      transitions.push(nextTransition);
      while ((_a = nextTransition === null || nextTransition === void 0 ? void 0 : nextTransition.task) === null || _a === void 0 ? void 0 : _a.if) {
          nextTransition.label = (_b = nextTransition === null || nextTransition === void 0 ? void 0 : nextTransition.task) === null || _b === void 0 ? void 0 : _b.if;
          nextTransition = getNextTask(context.taskList, nextTransition.name, FlowDirective.Continue);
          transitions.push(nextTransition);
      }
      transitions
          .filter(function (transition, index) {
          return transitions.findIndex(function (t) { return t.index === transition.index && t.name === transition.name && t.task === transition.task; }) === index;
      })
          .forEach(function (transition) { return buildTransition(sourceNode, transition, context); });
  }
  /**
   * Builds a graph representation of a task
   * @param context The context to build the graph/node for
   * @returns A graph or node for the provided context
   */
  function buildTaskNode(context) {
      var task = context.taskList.get(context.taskName);
      if (!task)
          throw new Error("Unabled to find the task '".concat(context.taskName, "' in the current context"));
      if (task.call)
          return buildCallTaskNode(task, context);
      if (task.catch)
          return buildTryCatchTaskNode(task, context);
      if (task.emit)
          return buildEmitTaskNode(task, context);
      if (task.for)
          return buildForTaskNode(task, context);
      if (task.fork)
          return buildForkTaskNode(task, context);
      if (task.listen)
          return buildListenTaskNode(task, context);
      if (task.raise)
          return buildRaiseTaskNode(task, context);
      if (task.run)
          return buildRunTaskNode(task, context);
      if (task.set)
          return buildSetTaskNode(task, context);
      if (task.switch)
          return buildSwitchTaskNode(task, context);
      if (task.wait)
          return buildWaitTaskNode(task, context);
      if (task.do)
          return buildDoTaskNode(task, context);
      throw new Error("Unable to defined task type of task named '".concat(context.taskName, "'"));
  }
  /**
   * Builds a graph node with the provided type and context
   * @param type The type of the node
   * @param context The context to build the graph node for
   * @returns A graph node for the provided context
   */
  function buildGenericTaskNode(type, context) {
      var node = {
          type: type,
          id: context.taskReference,
          label: context.taskName,
      };
      context.graph.nodes.push(node);
      buildTransitions(node, context);
      return node;
  }
  /**
   * Builds a graph node for the provided call task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildCallTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Call, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds a graph for the provided do task
   * @param task The task to build the graph for
   * @param context The context to build the graph for
   * @returns A graph for the provided task
   */
  function buildDoTaskNode(task, context) {
      var subgraph = initGraph(exports.GraphNodeType.Do, context.taskReference, context.taskName, context.graph);
      var doContext = __assign(__assign({}, context), { graph: subgraph, reference: context.taskReference + doReference, taskList: mapTasks(task.do), taskName: null });
      buildTransitions(subgraph.entryNode, doContext);
      buildTransitions(subgraph, context);
      return subgraph;
  }
  /**
   * Builds a graph node for the provided emit task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildEmitTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Emit, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds a graph for the provided for task
   * @param task The task to build the graph for
   * @param context The context to build the graph for
   * @returns A graph for the provided task
   */
  function buildForTaskNode(task, context) {
      var subgraph = initGraph(exports.GraphNodeType.For, context.taskReference, context.taskName, context.graph);
      var forContext = __assign(__assign({}, context), { graph: subgraph, reference: subgraph.id + forReference + doReference, taskList: mapTasks(task.do), taskName: null });
      buildTransitions(subgraph.entryNode, forContext);
      buildTransitions(subgraph, context);
      return subgraph;
  }
  /**
   * Builds a graph for the provided fork task
   * @param task The task to build the graph for
   * @param context The context to build the graph for
   * @returns A graph for the provided task
   */
  function buildForkTaskNode(task, context) {
      var _a, _b;
      var subgraph = initGraph(exports.GraphNodeType.Fork, context.taskReference, context.taskName, context.graph);
      for (var i = 0, c = ((_a = task.fork) === null || _a === void 0 ? void 0 : _a.branches.length) || 0; i < c; i++) {
          var branchItem = (_b = task.fork) === null || _b === void 0 ? void 0 : _b.branches[i];
          if (!branchItem)
              continue;
          var branchName = Object.entries(branchItem)[0][0];
          var branchContext = __assign(__assign({}, context), { graph: subgraph, reference: "".concat(context.taskReference).concat(branchReference), taskList: mapTasks([branchItem]), taskReference: "".concat(context.taskReference).concat(branchReference, "/").concat(i, "/").concat(branchName), taskName: branchName });
          var branchNode = buildTaskNode(branchContext);
          buildEdge(subgraph, subgraph.entryNode, branchNode.entryNode || branchNode);
          buildEdge(subgraph, branchNode.exitNode || branchNode, subgraph.exitNode);
      }
      buildTransitions(subgraph, context);
      return subgraph;
  }
  /**
   * Builds a graph node for the provided listen task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildListenTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Listen, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds a graph node for the provided rasie task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildRaiseTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Raise, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds a graph node for the provided run task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildRunTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Run, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds a graph node for the provided set task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildSetTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Set, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds a graph node for the provided switch task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildSwitchTaskNode(task, context) {
      var _a;
      var node = buildGenericTaskNode(exports.GraphNodeType.Switch, context);
      var hasDefaultCase = false;
      (_a = task.switch) === null || _a === void 0 ? void 0 : _a.forEach(function (switchItem) {
          var _a = Object.entries(switchItem)[0], caseName = _a[0], switchCase = _a[1];
          if (!switchCase.when)
              hasDefaultCase = true;
          var transition = getNextTask(context.taskList, context.taskName, switchCase.then);
          transition.label = caseName;
          buildTransition(node, transition, context);
      });
      if (!hasDefaultCase) {
          buildTransitions(node, context);
      }
      return node;
  }
  /**
   * Builds a graph for the provided try/catch task
   * @param task The task to build the graph for
   * @param context The context to build the graph for
   * @returns A graph for the provided task
   */
  function buildTryCatchTaskNode(task, context) {
      var _a, _b;
      var containerSubgraph = initGraph(exports.GraphNodeType.TryCatch, context.taskReference, context.taskName, context.graph);
      var trySubgraph = initGraph(exports.GraphNodeType.Try, context.taskReference + tryReference, context.taskName + ' (try)', containerSubgraph);
      buildEdge(containerSubgraph, containerSubgraph.entryNode, trySubgraph.entryNode);
      var tryContext = __assign(__assign({}, context), { graph: trySubgraph, reference: trySubgraph.id, taskList: mapTasks(task.try), taskName: null });
      buildTransitions(trySubgraph.entryNode, tryContext);
      if (!((_b = (_a = task.catch) === null || _a === void 0 ? void 0 : _a.do) === null || _b === void 0 ? void 0 : _b.length)) {
          var catchNode = {
              type: exports.GraphNodeType.Catch,
              id: context.taskReference + catchReference,
              label: context.taskName + ' (catch)',
          };
          containerSubgraph.nodes.push(catchNode);
          buildEdge(containerSubgraph, trySubgraph.exitNode, catchNode);
          buildEdge(containerSubgraph, catchNode, containerSubgraph.exitNode);
      }
      else {
          var catchSubgraph = initGraph(exports.GraphNodeType.Catch, context.taskReference + catchReference + doReference, context.taskName + ' (catch)', containerSubgraph);
          buildEdge(containerSubgraph, trySubgraph.exitNode, catchSubgraph.entryNode);
          var catchContext = __assign(__assign({}, context), { graph: catchSubgraph, reference: catchSubgraph.id, taskList: mapTasks(task.catch.do), taskName: null });
          buildTransitions(catchSubgraph.entryNode, catchContext);
          buildEdge(containerSubgraph, catchSubgraph.exitNode, containerSubgraph.exitNode);
      }
      buildTransitions(containerSubgraph, context);
      return containerSubgraph;
  }
  /**
   * Builds a graph node for the provided wait task
   * @param task The task to build the graph node for
   * @param context The context to build the graph node for
   * @returns A graph node for the provided task
   */
  function buildWaitTaskNode(task, context) {
      var node = buildGenericTaskNode(exports.GraphNodeType.Wait, context);
      // TODO: add some details about the task?
      return node;
  }
  /**
   * Builds an edge between two elements
   * @param graph The graph element containing the nodes
   * @param source The origin node
   * @param destination The destination node
   * @param label The edge label, if any
   */
  function buildEdge(graph, source, destination, label) {
      if (label === void 0) { label = ''; }
      var edge = graph.edges.find(function (e) { return e.sourceId === source.id && e.destinationId === destination.id; });
      if (edge) {
          if (label) {
              edge.label = edge.label + (edge.label ? ' / ' : '') + label;
          }
          return edge;
      }
      edge = {
          label: label,
          id: "".concat(source.id, "-").concat(destination.id).concat(label ? "-".concat(label) : ''),
          sourceId: source.id,
          destinationId: destination.id,
      };
      graph.edges.push(edge);
  }

  /**
   * Adds indentation to each line of the provided code
   * @param code The code to indent
   * @returns The indented code
   */
  var indent = function (code) {
      return code
          .split('\n')
          .map(function (line) { return "    ".concat(line); })
          .join('\n');
  };
  /**
   * Converts a graph to Mermaid code
   * @param graph The graph to convert
   * @returns The converted graph
   */
  function convertGraphToCode(graph) {
      var isRoot = graph.id === 'root';
      var code = "".concat(isRoot ? 'flowchart TD' : "subgraph ".concat(graph.id, " [\"").concat(graph.label || graph.id, "\"]"), "\n").concat(indent(graph.nodes.map(function (node) { return convertNodeToCode(node); }).join('\n')), "\n").concat(indent(graph.edges.map(function (edge) { return convertEdgeToCode(edge); }).join('\n')), "\n").concat(isRoot ? '' : 'end');
      return code;
  }
  /**
   * Converts a node to Mermaid code
   * @param node The node to convert
   * @returns The converted node
   */
  function convertNodeToCode(node) {
      var _a;
      var code = '';
      if ((_a = node.nodes) === null || _a === void 0 ? void 0 : _a.length) {
          code = convertGraphToCode(node);
      }
      else {
          code = node.id;
          switch (node.type) {
              case exports.GraphNodeType.Entry:
              case exports.GraphNodeType.Exit:
                  code += ':::hidden';
                  break;
              case exports.GraphNodeType.Start:
                  code += '(( ))'; // alt '@{ shape: circle, label: " "}';
                  break;
              case exports.GraphNodeType.End:
                  code += '((( )))'; // alt '@{ shape: dbl-circ, label: " "}';
                  break;
              default:
                  code += "[\"".concat(node.label, "\"]"); // alt `@{ label: "${node.label}" }`
          }
      }
      return code;
  }
  /**
   * Converts an edge to Mermaid code
   * @param edge The edge to convert
   * @returns The converted edge
   */
  function convertEdgeToCode(edge) {
      var ignoreEndArrow = !edge.destinationId.startsWith('root') &&
          (edge.destinationId.endsWith('-entry-node') || edge.destinationId.endsWith('-exit-node'));
      var code = "".concat(edge.sourceId, " ").concat(edge.label ? "--\"".concat(edge.label, "\"") : '', "--").concat(ignoreEndArrow ? '-' : '>', " ").concat(edge.destinationId);
      return code;
  }
  /**
   * Converts the provided workflow to Mermaid code
   * @param workflow The workflow to convert
   * @returns The Mermaid diagram
   */
  function convertToMermaidCode(workflow) {
      var graph = buildGraph(workflow);
      return (convertGraphToCode(graph) +
          "\n\nclassDef hidden display: none;");
  }
  /**
   * Represents a Mermaid diagram generator for a given workflow.
   * This class takes a workflow definition and converts it into a Mermaid.js-compatible diagram.
   */
  var MermaidDiagram = /** @class */ (function () {
      function MermaidDiagram(workflow) {
          this.workflow = workflow;
      }
      /**
       * Generates the Mermaid code representation of the workflow.
       * @returns The Mermaid diagram source code as a string.
       */
      MermaidDiagram.prototype.sourceCode = function () {
          return convertToMermaidCode(this.workflow);
      };
      return MermaidDiagram;
  }());

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * Represents a Workflow with methods for validation normalization, and serialization.
   * Inherits from ObjectHydrator which provides functionality for hydrating the state based on a model.
   */
  var Workflow = /** @class */ (function (_super) {
      __extends(Workflow, _super);
      /**
       * Instanciates a new instance of the Workflow class.
       * Initializes properties based on the provided model if it is an object.
       *
       * @param model - Optional partial model object to initialize the Workflow.
       */
      function Workflow(model) {
          var _a, _b;
          var _this = _super.call(this, model) || this;
          var self = _this;
          if (isObject$1(model)) {
              if (typeof model.document === 'object')
                  self.document = new _Document(model.document);
              if (typeof model.input === 'object')
                  self.input = new _Input(model.input);
              if (typeof model.use === 'object')
                  self.use = new _Use(model.use);
              if (typeof model.do === 'object')
                  self.do = new _TaskList(model.do);
              if (typeof model.timeout === 'object')
                  self.timeout = new _WorkflowTimeout(model.timeout);
              if (typeof model.output === 'object')
                  self.output = new _Output(model.output);
              if (typeof model.schedule === 'object')
                  self.schedule = new _Schedule(model.schedule);
          }
          (_b = (_a = getLifecycleHooks('Workflow')) === null || _a === void 0 ? void 0 : _a.constructor) === null || _b === void 0 ? void 0 : _b.call(_a, _this);
          return _this;
      }
      /**
       * Validates the current instance of the Workflow.
       * Throws if invalid.
       */
      Workflow.prototype.validate = function (workflow) {
          var copy = new Workflow(this);
          validate('Workflow', copy, workflow);
      };
      /**
       * Normalizes the current instance of the Workflow.
       * Creates a copy of the Workflow, invokes normalization hooks if available, and returns the normalized copy.
       *
       * @returns A normalized version of the Workflow instance.
       */
      Workflow.prototype.normalize = function () {
          var _a, _b;
          var copy = new Workflow(this);
          return ((_b = (_a = getLifecycleHooks('Workflow')) === null || _a === void 0 ? void 0 : _a.normalize) === null || _b === void 0 ? void 0 : _b.call(_a, copy)) || copy;
      };
      Workflow.deserialize = function (text) {
          var model = load(text);
          validate('Workflow', model);
          return new Workflow(model);
      };
      Workflow.serialize = function (model, format, normalize) {
          if (format === void 0) { format = 'yaml'; }
          if (normalize === void 0) { normalize = true; }
          var workflow = new Workflow(model);
          workflow.validate();
          var normalized = normalize ? workflow.normalize() : workflow;
          if (format === 'json') {
              return JSON.stringify(normalized);
          }
          return dump(normalized);
      };
      Workflow.toGraph = function (model) {
          return buildGraph(model);
      };
      Workflow.toMermaidCode = function (model) {
          return convertToMermaidCode(model);
      };
      /**
       * Serializes the workflow to YAML or JSON
       * @param format The format, 'yaml' or 'json', default is 'yaml'
       * @param normalize If the workflow should be normalized before serialization, default true
       * @returns A string representation of the workflow
       */
      Workflow.prototype.serialize = function (format, normalize) {
          if (format === void 0) { format = 'yaml'; }
          if (normalize === void 0) { normalize = true; }
          return Workflow.serialize(this, format, normalize);
      };
      /**
       * Creates a directed graph representation of the workflow
       * @returns A directed graph of the workflow
       */
      Workflow.prototype.toGraph = function () {
          return Workflow.toGraph(this);
      };
      /**
       * Generates the MermaidJS code corresponding to the workflow
       * @returns The MermaidJS code
       */
      Workflow.prototype.toMermaidCode = function () {
          return Workflow.toMermaidCode(this);
      };
      return Workflow;
  }(ObjectHydrator));
  var _Workflow = Workflow;

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  var Classes = {
      AllEventConsumptionStrategy: _AllEventConsumptionStrategy,
      AllEventConsumptionStrategyConfiguration: _AllEventConsumptionStrategyConfiguration,
      AnyEventConsumptionStrategy: _AnyEventConsumptionStrategy,
      AnyEventConsumptionStrategyConfiguration: _AnyEventConsumptionStrategyConfiguration,
      AnyEventConsumptionStrategyUntil: _AnyEventConsumptionStrategyUntil,
      AnyEventUntilConsumed: _AnyEventUntilConsumed,
      AsyncApiArguments: _AsyncApiArguments,
      AuthenticationPolicy: _AuthenticationPolicy,
      AuthenticationPolicyReference: _AuthenticationPolicyReference,
      BasicAuthenticationPolicy: _BasicAuthenticationPolicy,
      BasicAuthenticationPolicyConfiguration: _BasicAuthenticationPolicyConfiguration,
      BasicAuthenticationProperties: _BasicAuthenticationProperties,
      BearerAuthenticationPolicy: _BearerAuthenticationPolicy,
      BearerAuthenticationPolicyConfiguration: _BearerAuthenticationPolicyConfiguration,
      BearerAuthenticationProperties: _BearerAuthenticationProperties,
      CallAsyncAPI: _CallAsyncAPI,
      CallFunction: _CallFunction,
      CallGRPC: _CallGRPC,
      CallHTTP: _CallHTTP,
      CallOpenAPI: _CallOpenAPI,
      CallTask: _CallTask,
      Catalog: _Catalog,
      CatchErrors: _CatchErrors,
      ConstantBackoff: _ConstantBackoff,
      Container: _Container,
      ContainerEnvironment: _ContainerEnvironment,
      ContainerLifetime: _ContainerLifetime,
      ContainerPorts: _ContainerPorts,
      ContainerVolumes: _ContainerVolumes,
      DigestAuthenticationPolicy: _DigestAuthenticationPolicy,
      DigestAuthenticationPolicyConfiguration: _DigestAuthenticationPolicyConfiguration,
      DigestAuthenticationProperties: _DigestAuthenticationProperties,
      Document: _Document,
      DoTask: _DoTask,
      Duration: _Duration,
      DurationInline: _DurationInline,
      EmitEventDefinition: _EmitEventDefinition,
      EmitEventWith: _EmitEventWith,
      EmitTask: _EmitTask,
      EmitTaskConfiguration: _EmitTaskConfiguration,
      Endpoint: _Endpoint,
      EndpointConfiguration: _EndpointConfiguration,
      EndpointUri: _EndpointUri,
      Error: _Error,
      ErrorFilter: _ErrorFilter,
      ErrorInstance: _ErrorInstance,
      ErrorType: _ErrorType,
      EventConsumptionStrategy: _EventConsumptionStrategy,
      EventData: _EventData,
      EventDataschema: _EventDataschema,
      EventFilter: _EventFilter,
      EventFilterCorrelate: _EventFilterCorrelate,
      EventSource: _EventSource,
      EventTime: _EventTime,
      ExponentialBackOff: _ExponentialBackOff,
      Export: _Export,
      ExportAs: _ExportAs,
      Extension: _Extension,
      ExtensionItem: _ExtensionItem,
      ExternalResource: _ExternalResource,
      ExternalScript: _ExternalScript,
      FlowDirective: _FlowDirective,
      ForkTask: _ForkTask,
      ForkTaskConfiguration: _ForkTaskConfiguration,
      ForTask: _ForTask,
      ForTaskConfiguration: _ForTaskConfiguration,
      FunctionArguments: _FunctionArguments,
      GRPCArguments: _GRPCArguments,
      HTTPArguments: _HTTPArguments,
      HTTPBody: _HTTPBody,
      HTTPHeaders: _HTTPHeaders,
      HTTPQuery: _HTTPQuery,
      InlineScript: _InlineScript,
      Input: _Input,
      InputFrom: _InputFrom,
      LinearBackoff: _LinearBackoff,
      ListenTask: _ListenTask,
      ListenTaskConfiguration: _ListenTaskConfiguration,
      OAuth2AutenthicationData: _OAuth2AutenthicationData,
      OAuth2AutenthicationDataAudiences: _OAuth2AutenthicationDataAudiences,
      OAuth2AutenthicationDataClient: _OAuth2AutenthicationDataClient,
      OAuth2AutenthicationDataScopes: _OAuth2AutenthicationDataScopes,
      OAuth2AuthenticationPolicy: _OAuth2AuthenticationPolicy,
      OAuth2AuthenticationPolicyConfiguration: _OAuth2AuthenticationPolicyConfiguration,
      OAuth2AuthenticationPropertiesEndpoints: _OAuth2AuthenticationPropertiesEndpoints,
      OAuth2ConnectAuthenticationProperties: _OAuth2ConnectAuthenticationProperties,
      OAuth2Issuers: _OAuth2Issuers,
      OAuth2TokenDefinition: _OAuth2TokenDefinition,
      OAuth2TokenRequest: _OAuth2TokenRequest,
      OneEventConsumptionStrategy: _OneEventConsumptionStrategy,
      OpenAPIArguments: _OpenAPIArguments,
      OpenIdConnectAuthenticationPolicy: _OpenIdConnectAuthenticationPolicy,
      OpenIdConnectAuthenticationPolicyConfiguration: _OpenIdConnectAuthenticationPolicyConfiguration,
      OpenIdConnectAuthenticationProperties: _OpenIdConnectAuthenticationProperties,
      Output: _Output,
      OutputAs: _OutputAs,
      RaiseTask: _RaiseTask,
      RaiseTaskConfiguration: _RaiseTaskConfiguration,
      RaiseTaskError: _RaiseTaskError,
      ReferenceableAuthenticationPolicy: _ReferenceableAuthenticationPolicy,
      RetryBackoff: _RetryBackoff,
      RetryLimit: _RetryLimit,
      RetryLimitAttempt: _RetryLimitAttempt,
      RetryPolicy: _RetryPolicy,
      RetryPolicyJitter: _RetryPolicyJitter,
      RunContainer: _RunContainer,
      RunScript: _RunScript,
      RunShell: _RunShell,
      RunTask: _RunTask,
      RunTaskConfiguration: _RunTaskConfiguration,
      RuntimeExpression: _RuntimeExpression,
      RunWorkflow: _RunWorkflow,
      Schedule: _Schedule,
      Schema: _Schema,
      SchemaExternal: _SchemaExternal,
      SchemaInline: _SchemaInline,
      Script: _Script,
      SecretBasedAuthenticationPolicy: _SecretBasedAuthenticationPolicy,
      SetTask: _SetTask,
      SetTaskConfiguration: _SetTaskConfiguration,
      Shell: _Shell,
      ShellArguments: _ShellArguments,
      ShellEnvironment: _ShellEnvironment,
      SubflowConfiguration: _SubflowConfiguration,
      SubflowInput: _SubflowInput,
      SubscriptionIterator: _SubscriptionIterator,
      SwitchCase: _SwitchCase,
      SwitchItem: _SwitchItem,
      SwitchTask: _SwitchTask,
      SwitchTaskConfiguration: _SwitchTaskConfiguration,
      Task: _Task,
      TaskBase: _TaskBase,
      TaskBaseIf: _TaskBaseIf,
      TaskItem: _TaskItem,
      TaskList: _TaskList,
      TaskMetadata: _TaskMetadata,
      TaskTimeout: _TaskTimeout,
      Timeout: _Timeout,
      TryTask: _TryTask,
      TryTaskCatch: _TryTaskCatch,
      TryTaskCatchRetry: _TryTaskCatchRetry,
      UriTemplate: _UriTemplate,
      Use: _Use,
      UseAuthentications: _UseAuthentications,
      UseCatalogs: _UseCatalogs,
      UseErrors: _UseErrors,
      UseExtensions: _UseExtensions,
      UseFunctions: _UseFunctions,
      UseRetries: _UseRetries,
      UseSecrets: _UseSecrets,
      UseTimeouts: _UseTimeouts,
      WaitTask: _WaitTask,
      WithEvent: _WithEvent,
      WithGRPCArguments: _WithGRPCArguments,
      WithGRPCService: _WithGRPCService,
      WithOpenAPIParameters: _WithOpenAPIParameters,
      Workflow: _Workflow,
      WorkflowMetadata: _WorkflowMetadata,
      WorkflowTags: _WorkflowTags,
      WorkflowTimeout: _WorkflowTimeout,
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AllEventConsumptionStrategy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AllEventConsumptionStrategyIntersection} The built object
   */
  function buildingFn$2v(model, options) {
      var instance = new Classes.AllEventConsumptionStrategy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AllEventConsumptionStrategyIntersection`
   * @returns {Builder<AllEventConsumptionStrategyIntersection, AllEventConsumptionStrategyIntersection>} A builder for `AllEventConsumptionStrategyIntersection`
   */
  var allEventConsumptionStrategyBuilder = function (model) {
      return builder(model, buildingFn$2v);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.AllEventConsumptionStrategyConfiguration} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {AllEventConsumptionStrategyConfigurationIntersection} The built array
   */
  function buildingFn$2u(model, options) {
      var instance = new Classes.AllEventConsumptionStrategyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize
          ? instance.normalize()
          : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AllEventConsumptionStrategyConfigurationIntersection`
   * @returns {ArrayBuilder<Specification.EventFilter, AllEventConsumptionStrategyConfigurationIntersection>} A builder for `AllEventConsumptionStrategyConfigurationIntersection`
   */
  var allEventConsumptionStrategyConfigurationBuilder = function (model) {
      return arrayBuilder(model, buildingFn$2u);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AnyEventConsumptionStrategy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AnyEventConsumptionStrategyIntersection} The built object
   */
  function buildingFn$2t(model, options) {
      var instance = new Classes.AnyEventConsumptionStrategy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AnyEventConsumptionStrategyIntersection`
   * @returns {Builder<AnyEventConsumptionStrategyIntersection, AnyEventConsumptionStrategyIntersection>} A builder for `AnyEventConsumptionStrategyIntersection`
   */
  var anyEventConsumptionStrategyBuilder = function (model) {
      return builder(model, buildingFn$2t);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.AnyEventConsumptionStrategyConfiguration} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {AnyEventConsumptionStrategyConfigurationIntersection} The built array
   */
  function buildingFn$2s(model, options) {
      var instance = new Classes.AnyEventConsumptionStrategyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize
          ? instance.normalize()
          : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AnyEventConsumptionStrategyConfigurationIntersection`
   * @returns {ArrayBuilder<Specification.EventFilter, AnyEventConsumptionStrategyConfigurationIntersection>} A builder for `AnyEventConsumptionStrategyConfigurationIntersection`
   */
  var anyEventConsumptionStrategyConfigurationBuilder = function (model) {
      return arrayBuilder(model, buildingFn$2s);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AnyEventConsumptionStrategyUntil} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AnyEventConsumptionStrategyUntilIntersection} The built object
   */
  function buildingFn$2r(model, options) {
      var instance = new Classes.AnyEventConsumptionStrategyUntil(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AnyEventConsumptionStrategyUntilIntersection`
   * @returns {Builder<AnyEventConsumptionStrategyUntilIntersection, AnyEventConsumptionStrategyUntilIntersection>} A builder for `AnyEventConsumptionStrategyUntilIntersection`
   */
  var anyEventConsumptionStrategyUntilBuilder = function (model) {
      return builder(model, buildingFn$2r);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AnyEventUntilConsumed} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AnyEventUntilConsumedIntersection} The built object
   */
  function buildingFn$2q(model, options) {
      var instance = new Classes.AnyEventUntilConsumed(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AnyEventUntilConsumedIntersection`
   * @returns {Builder<AnyEventUntilConsumedIntersection, AnyEventUntilConsumedIntersection>} A builder for `AnyEventUntilConsumedIntersection`
   */
  var anyEventUntilConsumedBuilder = function (model) {
      return builder(model, buildingFn$2q);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AsyncApiArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AsyncApiArgumentsIntersection} The built object
   */
  function buildingFn$2p(model, options) {
      var instance = new Classes.AsyncApiArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AsyncApiArgumentsIntersection`
   * @returns {Builder<AsyncApiArgumentsIntersection, AsyncApiArgumentsIntersection>} A builder for `AsyncApiArgumentsIntersection`
   */
  var asyncApiArgumentsBuilder = function (model) {
      return builder(model, buildingFn$2p);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AuthenticationPolicyIntersection} The built object
   */
  function buildingFn$2o(model, options) {
      var instance = new Classes.AuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AuthenticationPolicyIntersection`
   * @returns {Builder<AuthenticationPolicyIntersection, AuthenticationPolicyIntersection>} A builder for `AuthenticationPolicyIntersection`
   */
  var authenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$2o);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.AuthenticationPolicyReference} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {AuthenticationPolicyReferenceIntersection} The built object
   */
  function buildingFn$2n(model, options) {
      var instance = new Classes.AuthenticationPolicyReference(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `AuthenticationPolicyReferenceIntersection`
   * @returns {Builder<AuthenticationPolicyReferenceIntersection, AuthenticationPolicyReferenceIntersection>} A builder for `AuthenticationPolicyReferenceIntersection`
   */
  var authenticationPolicyReferenceBuilder = function (model) {
      return builder(model, buildingFn$2n);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.BasicAuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {BasicAuthenticationPolicyIntersection} The built object
   */
  function buildingFn$2m(model, options) {
      var instance = new Classes.BasicAuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `BasicAuthenticationPolicyIntersection`
   * @returns {Builder<BasicAuthenticationPolicyIntersection, BasicAuthenticationPolicyIntersection>} A builder for `BasicAuthenticationPolicyIntersection`
   */
  var basicAuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$2m);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.BasicAuthenticationPolicyConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {BasicAuthenticationPolicyConfigurationIntersection} The built object
   */
  function buildingFn$2l(model, options) {
      var instance = new Classes.BasicAuthenticationPolicyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `BasicAuthenticationPolicyConfigurationIntersection`
   * @returns {Builder<BasicAuthenticationPolicyConfigurationIntersection, BasicAuthenticationPolicyConfigurationIntersection>} A builder for `BasicAuthenticationPolicyConfigurationIntersection`
   */
  var basicAuthenticationPolicyConfigurationBuilder = function (model) {
      return builder(model, buildingFn$2l);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.BasicAuthenticationProperties} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {BasicAuthenticationPropertiesIntersection} The built object
   */
  function buildingFn$2k(model, options) {
      var instance = new Classes.BasicAuthenticationProperties(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `BasicAuthenticationPropertiesIntersection`
   * @returns {Builder<BasicAuthenticationPropertiesIntersection, BasicAuthenticationPropertiesIntersection>} A builder for `BasicAuthenticationPropertiesIntersection`
   */
  var basicAuthenticationPropertiesBuilder = function (model) {
      return builder(model, buildingFn$2k);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.BearerAuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {BearerAuthenticationPolicyIntersection} The built object
   */
  function buildingFn$2j(model, options) {
      var instance = new Classes.BearerAuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `BearerAuthenticationPolicyIntersection`
   * @returns {Builder<BearerAuthenticationPolicyIntersection, BearerAuthenticationPolicyIntersection>} A builder for `BearerAuthenticationPolicyIntersection`
   */
  var bearerAuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$2j);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.BearerAuthenticationPolicyConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {BearerAuthenticationPolicyConfigurationIntersection} The built object
   */
  function buildingFn$2i(model, options) {
      var instance = new Classes.BearerAuthenticationPolicyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `BearerAuthenticationPolicyConfigurationIntersection`
   * @returns {Builder<BearerAuthenticationPolicyConfigurationIntersection, BearerAuthenticationPolicyConfigurationIntersection>} A builder for `BearerAuthenticationPolicyConfigurationIntersection`
   */
  var bearerAuthenticationPolicyConfigurationBuilder = function (model) {
      return builder(model, buildingFn$2i);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.BearerAuthenticationProperties} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {BearerAuthenticationPropertiesIntersection} The built object
   */
  function buildingFn$2h(model, options) {
      var instance = new Classes.BearerAuthenticationProperties(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `BearerAuthenticationPropertiesIntersection`
   * @returns {Builder<BearerAuthenticationPropertiesIntersection, BearerAuthenticationPropertiesIntersection>} A builder for `BearerAuthenticationPropertiesIntersection`
   */
  var bearerAuthenticationPropertiesBuilder = function (model) {
      return builder(model, buildingFn$2h);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CallAsyncAPI} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CallAsyncAPIIntersection} The built object
   */
  function buildingFn$2g(model, options) {
      var instance = new Classes.CallAsyncAPI(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CallAsyncAPIIntersection`
   * @returns {Builder<CallAsyncAPIIntersection, CallAsyncAPIIntersection>} A builder for `CallAsyncAPIIntersection`
   */
  var callAsyncAPIBuilder = function (model) {
      return builder(model, buildingFn$2g);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CallFunction} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CallFunctionIntersection} The built object
   */
  function buildingFn$2f(model, options) {
      var instance = new Classes.CallFunction(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CallFunctionIntersection`
   * @returns {Builder<CallFunctionIntersection, CallFunctionIntersection>} A builder for `CallFunctionIntersection`
   */
  var callFunctionBuilder = function (model) {
      return builder(model, buildingFn$2f);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CallGRPC} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CallGRPCIntersection} The built object
   */
  function buildingFn$2e(model, options) {
      var instance = new Classes.CallGRPC(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CallGRPCIntersection`
   * @returns {Builder<CallGRPCIntersection, CallGRPCIntersection>} A builder for `CallGRPCIntersection`
   */
  var callGRPCBuilder = function (model) {
      return builder(model, buildingFn$2e);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CallHTTP} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CallHTTPIntersection} The built object
   */
  function buildingFn$2d(model, options) {
      var instance = new Classes.CallHTTP(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CallHTTPIntersection`
   * @returns {Builder<CallHTTPIntersection, CallHTTPIntersection>} A builder for `CallHTTPIntersection`
   */
  var callHTTPBuilder = function (model) {
      return builder(model, buildingFn$2d);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CallOpenAPI} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CallOpenAPIIntersection} The built object
   */
  function buildingFn$2c(model, options) {
      var instance = new Classes.CallOpenAPI(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CallOpenAPIIntersection`
   * @returns {Builder<CallOpenAPIIntersection, CallOpenAPIIntersection>} A builder for `CallOpenAPIIntersection`
   */
  var callOpenAPIBuilder = function (model) {
      return builder(model, buildingFn$2c);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CallTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CallTaskIntersection} The built object
   */
  function buildingFn$2b(model, options) {
      var instance = new Classes.CallTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CallTaskIntersection`
   * @returns {Builder<CallTaskIntersection, CallTaskIntersection>} A builder for `CallTaskIntersection`
   */
  var callTaskBuilder = function (model) {
      return builder(model, buildingFn$2b);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Catalog} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CatalogIntersection} The built object
   */
  function buildingFn$2a(model, options) {
      var instance = new Classes.Catalog(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CatalogIntersection`
   * @returns {Builder<CatalogIntersection, CatalogIntersection>} A builder for `CatalogIntersection`
   */
  var catalogBuilder = function (model) {
      return builder(model, buildingFn$2a);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.CatchErrors} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {CatchErrorsIntersection} The built object
   */
  function buildingFn$29(model, options) {
      var instance = new Classes.CatchErrors(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `CatchErrorsIntersection`
   * @returns {Builder<CatchErrorsIntersection, CatchErrorsIntersection>} A builder for `CatchErrorsIntersection`
   */
  var catchErrorsBuilder = function (model) {
      return builder(model, buildingFn$29);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ConstantBackoff} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ConstantBackoffIntersection} The built object
   */
  function buildingFn$28(model, options) {
      var instance = new Classes.ConstantBackoff(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ConstantBackoffIntersection`
   * @returns {Builder<ConstantBackoffIntersection, ConstantBackoffIntersection>} A builder for `ConstantBackoffIntersection`
   */
  var constantBackoffBuilder = function (model) {
      return builder(model, buildingFn$28);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Container} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ContainerIntersection} The built object
   */
  function buildingFn$27(model, options) {
      var instance = new Classes.Container(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ContainerIntersection`
   * @returns {Builder<ContainerIntersection, ContainerIntersection>} A builder for `ContainerIntersection`
   */
  var containerBuilder = function (model) {
      return builder(model, buildingFn$27);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ContainerEnvironment} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ContainerEnvironmentIntersection} The built object
   */
  function buildingFn$26(model, options) {
      var instance = new Classes.ContainerEnvironment(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ContainerEnvironmentIntersection`
   * @returns {Builder<ContainerEnvironmentIntersection, ContainerEnvironmentIntersection>} A builder for `ContainerEnvironmentIntersection`
   */
  var containerEnvironmentBuilder = function (model) {
      return builder(model, buildingFn$26);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ContainerLifetime} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ContainerLifetimeIntersection} The built object
   */
  function buildingFn$25(model, options) {
      var instance = new Classes.ContainerLifetime(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ContainerLifetimeIntersection`
   * @returns {Builder<ContainerLifetimeIntersection, ContainerLifetimeIntersection>} A builder for `ContainerLifetimeIntersection`
   */
  var containerLifetimeBuilder = function (model) {
      return builder(model, buildingFn$25);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ContainerPorts} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ContainerPortsIntersection} The built object
   */
  function buildingFn$24(model, options) {
      var instance = new Classes.ContainerPorts(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ContainerPortsIntersection`
   * @returns {Builder<ContainerPortsIntersection, ContainerPortsIntersection>} A builder for `ContainerPortsIntersection`
   */
  var containerPortsBuilder = function (model) {
      return builder(model, buildingFn$24);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ContainerVolumes} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ContainerVolumesIntersection} The built object
   */
  function buildingFn$23(model, options) {
      var instance = new Classes.ContainerVolumes(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ContainerVolumesIntersection`
   * @returns {Builder<ContainerVolumesIntersection, ContainerVolumesIntersection>} A builder for `ContainerVolumesIntersection`
   */
  var containerVolumesBuilder = function (model) {
      return builder(model, buildingFn$23);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.DigestAuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DigestAuthenticationPolicyIntersection} The built object
   */
  function buildingFn$22(model, options) {
      var instance = new Classes.DigestAuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DigestAuthenticationPolicyIntersection`
   * @returns {Builder<DigestAuthenticationPolicyIntersection, DigestAuthenticationPolicyIntersection>} A builder for `DigestAuthenticationPolicyIntersection`
   */
  var digestAuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$22);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.DigestAuthenticationPolicyConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DigestAuthenticationPolicyConfigurationIntersection} The built object
   */
  function buildingFn$21(model, options) {
      var instance = new Classes.DigestAuthenticationPolicyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DigestAuthenticationPolicyConfigurationIntersection`
   * @returns {Builder<DigestAuthenticationPolicyConfigurationIntersection, DigestAuthenticationPolicyConfigurationIntersection>} A builder for `DigestAuthenticationPolicyConfigurationIntersection`
   */
  var digestAuthenticationPolicyConfigurationBuilder = function (model) {
      return builder(model, buildingFn$21);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.DigestAuthenticationProperties} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DigestAuthenticationPropertiesIntersection} The built object
   */
  function buildingFn$20(model, options) {
      var instance = new Classes.DigestAuthenticationProperties(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DigestAuthenticationPropertiesIntersection`
   * @returns {Builder<DigestAuthenticationPropertiesIntersection, DigestAuthenticationPropertiesIntersection>} A builder for `DigestAuthenticationPropertiesIntersection`
   */
  var digestAuthenticationPropertiesBuilder = function (model) {
      return builder(model, buildingFn$20);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Document} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DocumentIntersection} The built object
   */
  function buildingFn$1$(model, options) {
      var instance = new Classes.Document(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DocumentIntersection`
   * @returns {Builder<DocumentIntersection, DocumentIntersection>} A builder for `DocumentIntersection`
   */
  var documentBuilder = function (model) {
      return builder(model, buildingFn$1$);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.DoTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DoTaskIntersection} The built object
   */
  function buildingFn$1_(model, options) {
      var instance = new Classes.DoTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DoTaskIntersection`
   * @returns {Builder<DoTaskIntersection, DoTaskIntersection>} A builder for `DoTaskIntersection`
   */
  var doTaskBuilder = function (model) {
      return builder(model, buildingFn$1_);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Duration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DurationIntersection} The built object
   */
  function buildingFn$1Z(model, options) {
      var instance = new Classes.Duration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DurationIntersection`
   * @returns {Builder<DurationIntersection, DurationIntersection>} A builder for `DurationIntersection`
   */
  var durationBuilder = function (model) {
      return builder(model, buildingFn$1Z);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.DurationInline} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {DurationInlineIntersection} The built object
   */
  function buildingFn$1Y(model, options) {
      var instance = new Classes.DurationInline(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `DurationInlineIntersection`
   * @returns {Builder<DurationInlineIntersection, DurationInlineIntersection>} A builder for `DurationInlineIntersection`
   */
  var durationInlineBuilder = function (model) {
      return builder(model, buildingFn$1Y);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EmitEventDefinition} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EmitEventDefinitionIntersection} The built object
   */
  function buildingFn$1X(model, options) {
      var instance = new Classes.EmitEventDefinition(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EmitEventDefinitionIntersection`
   * @returns {Builder<EmitEventDefinitionIntersection, EmitEventDefinitionIntersection>} A builder for `EmitEventDefinitionIntersection`
   */
  var emitEventDefinitionBuilder = function (model) {
      return builder(model, buildingFn$1X);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EmitEventWith} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EmitEventWithIntersection} The built object
   */
  function buildingFn$1W(model, options) {
      var instance = new Classes.EmitEventWith(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EmitEventWithIntersection`
   * @returns {Builder<EmitEventWithIntersection, EmitEventWithIntersection>} A builder for `EmitEventWithIntersection`
   */
  var emitEventWithBuilder = function (model) {
      return builder(model, buildingFn$1W);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EmitTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EmitTaskIntersection} The built object
   */
  function buildingFn$1V(model, options) {
      var instance = new Classes.EmitTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EmitTaskIntersection`
   * @returns {Builder<EmitTaskIntersection, EmitTaskIntersection>} A builder for `EmitTaskIntersection`
   */
  var emitTaskBuilder = function (model) {
      return builder(model, buildingFn$1V);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EmitTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EmitTaskConfigurationIntersection} The built object
   */
  function buildingFn$1U(model, options) {
      var instance = new Classes.EmitTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EmitTaskConfigurationIntersection`
   * @returns {Builder<EmitTaskConfigurationIntersection, EmitTaskConfigurationIntersection>} A builder for `EmitTaskConfigurationIntersection`
   */
  var emitTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$1U);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Endpoint} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EndpointIntersection} The built object
   */
  function buildingFn$1T(model, options) {
      var instance = new Classes.Endpoint(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EndpointIntersection`
   * @returns {Builder<EndpointIntersection, EndpointIntersection>} A builder for `EndpointIntersection`
   */
  var endpointBuilder = function (model) {
      return builder(model, buildingFn$1T);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EndpointConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EndpointConfigurationIntersection} The built object
   */
  function buildingFn$1S(model, options) {
      var instance = new Classes.EndpointConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EndpointConfigurationIntersection`
   * @returns {Builder<EndpointConfigurationIntersection, EndpointConfigurationIntersection>} A builder for `EndpointConfigurationIntersection`
   */
  var endpointConfigurationBuilder = function (model) {
      return builder(model, buildingFn$1S);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EndpointUri} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EndpointUriIntersection} The built object
   */
  function buildingFn$1R(model, options) {
      var instance = new Classes.EndpointUri(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EndpointUriIntersection`
   * @returns {Builder<EndpointUriIntersection, EndpointUriIntersection>} A builder for `EndpointUriIntersection`
   */
  var endpointUriBuilder = function (model) {
      return builder(model, buildingFn$1R);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Error} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ErrorIntersection} The built object
   */
  function buildingFn$1Q(model, options) {
      var instance = new Classes.Error(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ErrorIntersection`
   * @returns {Builder<ErrorIntersection, ErrorIntersection>} A builder for `ErrorIntersection`
   */
  var errorBuilder = function (model) {
      return builder(model, buildingFn$1Q);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ErrorFilter} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ErrorFilterIntersection} The built object
   */
  function buildingFn$1P(model, options) {
      var instance = new Classes.ErrorFilter(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ErrorFilterIntersection`
   * @returns {Builder<ErrorFilterIntersection, ErrorFilterIntersection>} A builder for `ErrorFilterIntersection`
   */
  var errorFilterBuilder = function (model) {
      return builder(model, buildingFn$1P);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ErrorInstance} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ErrorInstanceIntersection} The built object
   */
  function buildingFn$1O(model, options) {
      var instance = new Classes.ErrorInstance(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ErrorInstanceIntersection`
   * @returns {Builder<ErrorInstanceIntersection, ErrorInstanceIntersection>} A builder for `ErrorInstanceIntersection`
   */
  var errorInstanceBuilder = function (model) {
      return builder(model, buildingFn$1O);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ErrorType} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ErrorTypeIntersection} The built object
   */
  function buildingFn$1N(model, options) {
      var instance = new Classes.ErrorType(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ErrorTypeIntersection`
   * @returns {Builder<ErrorTypeIntersection, ErrorTypeIntersection>} A builder for `ErrorTypeIntersection`
   */
  var errorTypeBuilder = function (model) {
      return builder(model, buildingFn$1N);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventConsumptionStrategy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventConsumptionStrategyIntersection} The built object
   */
  function buildingFn$1M(model, options) {
      var instance = new Classes.EventConsumptionStrategy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventConsumptionStrategyIntersection`
   * @returns {Builder<EventConsumptionStrategyIntersection, EventConsumptionStrategyIntersection>} A builder for `EventConsumptionStrategyIntersection`
   */
  var eventConsumptionStrategyBuilder = function (model) {
      return builder(model, buildingFn$1M);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventData} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventDataIntersection} The built object
   */
  function buildingFn$1L(model, options) {
      var instance = new Classes.EventData(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventDataIntersection`
   * @returns {Builder<EventDataIntersection, EventDataIntersection>} A builder for `EventDataIntersection`
   */
  var eventDataBuilder = function (model) {
      return builder(model, buildingFn$1L);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventDataschema} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventDataschemaIntersection} The built object
   */
  function buildingFn$1K(model, options) {
      var instance = new Classes.EventDataschema(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventDataschemaIntersection`
   * @returns {Builder<EventDataschemaIntersection, EventDataschemaIntersection>} A builder for `EventDataschemaIntersection`
   */
  var eventDataschemaBuilder = function (model) {
      return builder(model, buildingFn$1K);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventFilter} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventFilterIntersection} The built object
   */
  function buildingFn$1J(model, options) {
      var instance = new Classes.EventFilter(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventFilterIntersection`
   * @returns {Builder<EventFilterIntersection, EventFilterIntersection>} A builder for `EventFilterIntersection`
   */
  var eventFilterBuilder = function (model) {
      return builder(model, buildingFn$1J);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventFilterCorrelate} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventFilterCorrelateIntersection} The built object
   */
  function buildingFn$1I(model, options) {
      var instance = new Classes.EventFilterCorrelate(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventFilterCorrelateIntersection`
   * @returns {Builder<EventFilterCorrelateIntersection, EventFilterCorrelateIntersection>} A builder for `EventFilterCorrelateIntersection`
   */
  var eventFilterCorrelateBuilder = function (model) {
      return builder(model, buildingFn$1I);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventSource} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventSourceIntersection} The built object
   */
  function buildingFn$1H(model, options) {
      var instance = new Classes.EventSource(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventSourceIntersection`
   * @returns {Builder<EventSourceIntersection, EventSourceIntersection>} A builder for `EventSourceIntersection`
   */
  var eventSourceBuilder = function (model) {
      return builder(model, buildingFn$1H);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.EventTime} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {EventTimeIntersection} The built object
   */
  function buildingFn$1G(model, options) {
      var instance = new Classes.EventTime(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `EventTimeIntersection`
   * @returns {Builder<EventTimeIntersection, EventTimeIntersection>} A builder for `EventTimeIntersection`
   */
  var eventTimeBuilder = function (model) {
      return builder(model, buildingFn$1G);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ExponentialBackOff} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExponentialBackOffIntersection} The built object
   */
  function buildingFn$1F(model, options) {
      var instance = new Classes.ExponentialBackOff(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExponentialBackOffIntersection`
   * @returns {Builder<ExponentialBackOffIntersection, ExponentialBackOffIntersection>} A builder for `ExponentialBackOffIntersection`
   */
  var exponentialBackOffBuilder = function (model) {
      return builder(model, buildingFn$1F);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Export} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExportIntersection} The built object
   */
  function buildingFn$1E(model, options) {
      var instance = new Classes.Export(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExportIntersection`
   * @returns {Builder<ExportIntersection, ExportIntersection>} A builder for `ExportIntersection`
   */
  var exportBuilder = function (model) {
      return builder(model, buildingFn$1E);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ExportAs} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExportAsIntersection} The built object
   */
  function buildingFn$1D(model, options) {
      var instance = new Classes.ExportAs(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExportAsIntersection`
   * @returns {Builder<ExportAsIntersection, ExportAsIntersection>} A builder for `ExportAsIntersection`
   */
  var exportAsBuilder = function (model) {
      return builder(model, buildingFn$1D);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Extension} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExtensionIntersection} The built object
   */
  function buildingFn$1C(model, options) {
      var instance = new Classes.Extension(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExtensionIntersection`
   * @returns {Builder<ExtensionIntersection, ExtensionIntersection>} A builder for `ExtensionIntersection`
   */
  var extensionBuilder = function (model) {
      return builder(model, buildingFn$1C);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ExtensionItem} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExtensionItemIntersection} The built object
   */
  function buildingFn$1B(model, options) {
      var instance = new Classes.ExtensionItem(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExtensionItemIntersection`
   * @returns {Builder<ExtensionItemIntersection, ExtensionItemIntersection>} A builder for `ExtensionItemIntersection`
   */
  var extensionItemBuilder = function (model) {
      return builder(model, buildingFn$1B);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ExternalResource} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExternalResourceIntersection} The built object
   */
  function buildingFn$1A(model, options) {
      var instance = new Classes.ExternalResource(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExternalResourceIntersection`
   * @returns {Builder<ExternalResourceIntersection, ExternalResourceIntersection>} A builder for `ExternalResourceIntersection`
   */
  var externalResourceBuilder = function (model) {
      return builder(model, buildingFn$1A);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ExternalScript} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ExternalScriptIntersection} The built object
   */
  function buildingFn$1z(model, options) {
      var instance = new Classes.ExternalScript(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ExternalScriptIntersection`
   * @returns {Builder<ExternalScriptIntersection, ExternalScriptIntersection>} A builder for `ExternalScriptIntersection`
   */
  var externalScriptBuilder = function (model) {
      return builder(model, buildingFn$1z);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.FlowDirective} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {FlowDirectiveIntersection} The built object
   */
  function buildingFn$1y(model, options) {
      var instance = new Classes.FlowDirective(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `FlowDirectiveIntersection`
   * @returns {Builder<FlowDirectiveIntersection, FlowDirectiveIntersection>} A builder for `FlowDirectiveIntersection`
   */
  var flowDirectiveBuilder = function (model) {
      return builder(model, buildingFn$1y);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ForkTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ForkTaskIntersection} The built object
   */
  function buildingFn$1x(model, options) {
      var instance = new Classes.ForkTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ForkTaskIntersection`
   * @returns {Builder<ForkTaskIntersection, ForkTaskIntersection>} A builder for `ForkTaskIntersection`
   */
  var forkTaskBuilder = function (model) {
      return builder(model, buildingFn$1x);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ForkTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ForkTaskConfigurationIntersection} The built object
   */
  function buildingFn$1w(model, options) {
      var instance = new Classes.ForkTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ForkTaskConfigurationIntersection`
   * @returns {Builder<ForkTaskConfigurationIntersection, ForkTaskConfigurationIntersection>} A builder for `ForkTaskConfigurationIntersection`
   */
  var forkTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$1w);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ForTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ForTaskIntersection} The built object
   */
  function buildingFn$1v(model, options) {
      var instance = new Classes.ForTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ForTaskIntersection`
   * @returns {Builder<ForTaskIntersection, ForTaskIntersection>} A builder for `ForTaskIntersection`
   */
  var forTaskBuilder = function (model) {
      return builder(model, buildingFn$1v);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ForTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ForTaskConfigurationIntersection} The built object
   */
  function buildingFn$1u(model, options) {
      var instance = new Classes.ForTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ForTaskConfigurationIntersection`
   * @returns {Builder<ForTaskConfigurationIntersection, ForTaskConfigurationIntersection>} A builder for `ForTaskConfigurationIntersection`
   */
  var forTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$1u);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.FunctionArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {FunctionArgumentsIntersection} The built object
   */
  function buildingFn$1t(model, options) {
      var instance = new Classes.FunctionArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `FunctionArgumentsIntersection`
   * @returns {Builder<FunctionArgumentsIntersection, FunctionArgumentsIntersection>} A builder for `FunctionArgumentsIntersection`
   */
  var functionArgumentsBuilder = function (model) {
      return builder(model, buildingFn$1t);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.GRPCArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {GRPCArgumentsIntersection} The built object
   */
  function buildingFn$1s(model, options) {
      var instance = new Classes.GRPCArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `GRPCArgumentsIntersection`
   * @returns {Builder<GRPCArgumentsIntersection, GRPCArgumentsIntersection>} A builder for `GRPCArgumentsIntersection`
   */
  var gRPCArgumentsBuilder = function (model) {
      return builder(model, buildingFn$1s);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.HTTPArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {HTTPArgumentsIntersection} The built object
   */
  function buildingFn$1r(model, options) {
      var instance = new Classes.HTTPArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `HTTPArgumentsIntersection`
   * @returns {Builder<HTTPArgumentsIntersection, HTTPArgumentsIntersection>} A builder for `HTTPArgumentsIntersection`
   */
  var hTTPArgumentsBuilder = function (model) {
      return builder(model, buildingFn$1r);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.HTTPBody} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {HTTPBodyIntersection} The built object
   */
  function buildingFn$1q(model, options) {
      var instance = new Classes.HTTPBody(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `HTTPBodyIntersection`
   * @returns {Builder<HTTPBodyIntersection, HTTPBodyIntersection>} A builder for `HTTPBodyIntersection`
   */
  var hTTPBodyBuilder = function (model) {
      return builder(model, buildingFn$1q);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.HTTPHeaders} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {HTTPHeadersIntersection} The built object
   */
  function buildingFn$1p(model, options) {
      var instance = new Classes.HTTPHeaders(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `HTTPHeadersIntersection`
   * @returns {Builder<HTTPHeadersIntersection, HTTPHeadersIntersection>} A builder for `HTTPHeadersIntersection`
   */
  var hTTPHeadersBuilder = function (model) {
      return builder(model, buildingFn$1p);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.HTTPQuery} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {HTTPQueryIntersection} The built object
   */
  function buildingFn$1o(model, options) {
      var instance = new Classes.HTTPQuery(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `HTTPQueryIntersection`
   * @returns {Builder<HTTPQueryIntersection, HTTPQueryIntersection>} A builder for `HTTPQueryIntersection`
   */
  var hTTPQueryBuilder = function (model) {
      return builder(model, buildingFn$1o);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.InlineScript} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {InlineScriptIntersection} The built object
   */
  function buildingFn$1n(model, options) {
      var instance = new Classes.InlineScript(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `InlineScriptIntersection`
   * @returns {Builder<InlineScriptIntersection, InlineScriptIntersection>} A builder for `InlineScriptIntersection`
   */
  var inlineScriptBuilder = function (model) {
      return builder(model, buildingFn$1n);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Input} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {InputIntersection} The built object
   */
  function buildingFn$1m(model, options) {
      var instance = new Classes.Input(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `InputIntersection`
   * @returns {Builder<InputIntersection, InputIntersection>} A builder for `InputIntersection`
   */
  var inputBuilder = function (model) {
      return builder(model, buildingFn$1m);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.InputFrom} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {InputFromIntersection} The built object
   */
  function buildingFn$1l(model, options) {
      var instance = new Classes.InputFrom(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `InputFromIntersection`
   * @returns {Builder<InputFromIntersection, InputFromIntersection>} A builder for `InputFromIntersection`
   */
  var inputFromBuilder = function (model) {
      return builder(model, buildingFn$1l);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.LinearBackoff} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {LinearBackoffIntersection} The built object
   */
  function buildingFn$1k(model, options) {
      var instance = new Classes.LinearBackoff(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `LinearBackoffIntersection`
   * @returns {Builder<LinearBackoffIntersection, LinearBackoffIntersection>} A builder for `LinearBackoffIntersection`
   */
  var linearBackoffBuilder = function (model) {
      return builder(model, buildingFn$1k);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ListenTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ListenTaskIntersection} The built object
   */
  function buildingFn$1j(model, options) {
      var instance = new Classes.ListenTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ListenTaskIntersection`
   * @returns {Builder<ListenTaskIntersection, ListenTaskIntersection>} A builder for `ListenTaskIntersection`
   */
  var listenTaskBuilder = function (model) {
      return builder(model, buildingFn$1j);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ListenTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ListenTaskConfigurationIntersection} The built object
   */
  function buildingFn$1i(model, options) {
      var instance = new Classes.ListenTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ListenTaskConfigurationIntersection`
   * @returns {Builder<ListenTaskConfigurationIntersection, ListenTaskConfigurationIntersection>} A builder for `ListenTaskConfigurationIntersection`
   */
  var listenTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$1i);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2AutenthicationData} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AutenthicationDataIntersection} The built object
   */
  function buildingFn$1h(model, options) {
      var instance = new Classes.OAuth2AutenthicationData(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AutenthicationDataIntersection`
   * @returns {Builder<OAuth2AutenthicationDataIntersection, OAuth2AutenthicationDataIntersection>} A builder for `OAuth2AutenthicationDataIntersection`
   */
  var oAuth2AutenthicationDataBuilder = function (model) {
      return builder(model, buildingFn$1h);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.OAuth2AutenthicationDataAudiences} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AutenthicationDataAudiencesIntersection} The built array
   */
  function buildingFn$1g(model, options) {
      var instance = new Classes.OAuth2AutenthicationDataAudiences(model);
      if (options.validate)
          instance.validate();
      return (options.normalize
          ? instance.normalize()
          : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AutenthicationDataAudiencesIntersection`
   * @returns {ArrayBuilder<string, OAuth2AutenthicationDataAudiencesIntersection>} A builder for `OAuth2AutenthicationDataAudiencesIntersection`
   */
  var oAuth2AutenthicationDataAudiencesBuilder = function (model) {
      return arrayBuilder(model, buildingFn$1g);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2AutenthicationDataClient} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AutenthicationDataClientIntersection} The built object
   */
  function buildingFn$1f(model, options) {
      var instance = new Classes.OAuth2AutenthicationDataClient(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AutenthicationDataClientIntersection`
   * @returns {Builder<OAuth2AutenthicationDataClientIntersection, OAuth2AutenthicationDataClientIntersection>} A builder for `OAuth2AutenthicationDataClientIntersection`
   */
  var oAuth2AutenthicationDataClientBuilder = function (model) {
      return builder(model, buildingFn$1f);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.OAuth2AutenthicationDataScopes} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AutenthicationDataScopesIntersection} The built array
   */
  function buildingFn$1e(model, options) {
      var instance = new Classes.OAuth2AutenthicationDataScopes(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AutenthicationDataScopesIntersection`
   * @returns {ArrayBuilder<string, OAuth2AutenthicationDataScopesIntersection>} A builder for `OAuth2AutenthicationDataScopesIntersection`
   */
  var oAuth2AutenthicationDataScopesBuilder = function (model) {
      return arrayBuilder(model, buildingFn$1e);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2AuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AuthenticationPolicyIntersection} The built object
   */
  function buildingFn$1d(model, options) {
      var instance = new Classes.OAuth2AuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AuthenticationPolicyIntersection`
   * @returns {Builder<OAuth2AuthenticationPolicyIntersection, OAuth2AuthenticationPolicyIntersection>} A builder for `OAuth2AuthenticationPolicyIntersection`
   */
  var oAuth2AuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$1d);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2AuthenticationPolicyConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AuthenticationPolicyConfigurationIntersection} The built object
   */
  function buildingFn$1c(model, options) {
      var instance = new Classes.OAuth2AuthenticationPolicyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AuthenticationPolicyConfigurationIntersection`
   * @returns {Builder<OAuth2AuthenticationPolicyConfigurationIntersection, OAuth2AuthenticationPolicyConfigurationIntersection>} A builder for `OAuth2AuthenticationPolicyConfigurationIntersection`
   */
  var oAuth2AuthenticationPolicyConfigurationBuilder = function (model) {
      return builder(model, buildingFn$1c);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2AuthenticationPropertiesEndpoints} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2AuthenticationPropertiesEndpointsIntersection} The built object
   */
  function buildingFn$1b(model, options) {
      var instance = new Classes.OAuth2AuthenticationPropertiesEndpoints(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2AuthenticationPropertiesEndpointsIntersection`
   * @returns {Builder<OAuth2AuthenticationPropertiesEndpointsIntersection, OAuth2AuthenticationPropertiesEndpointsIntersection>} A builder for `OAuth2AuthenticationPropertiesEndpointsIntersection`
   */
  var oAuth2AuthenticationPropertiesEndpointsBuilder = function (model) {
      return builder(model, buildingFn$1b);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2ConnectAuthenticationProperties} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2ConnectAuthenticationPropertiesIntersection} The built object
   */
  function buildingFn$1a(model, options) {
      var instance = new Classes.OAuth2ConnectAuthenticationProperties(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2ConnectAuthenticationPropertiesIntersection`
   * @returns {Builder<OAuth2ConnectAuthenticationPropertiesIntersection, OAuth2ConnectAuthenticationPropertiesIntersection>} A builder for `OAuth2ConnectAuthenticationPropertiesIntersection`
   */
  var oAuth2ConnectAuthenticationPropertiesBuilder = function (model) {
      return builder(model, buildingFn$1a);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.OAuth2Issuers} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2IssuersIntersection} The built array
   */
  function buildingFn$19(model, options) {
      var instance = new Classes.OAuth2Issuers(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2IssuersIntersection`
   * @returns {ArrayBuilder<string, OAuth2IssuersIntersection>} A builder for `OAuth2IssuersIntersection`
   */
  var oAuth2IssuersBuilder = function (model) {
      return arrayBuilder(model, buildingFn$19);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2TokenDefinition} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2TokenDefinitionIntersection} The built object
   */
  function buildingFn$18(model, options) {
      var instance = new Classes.OAuth2TokenDefinition(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2TokenDefinitionIntersection`
   * @returns {Builder<OAuth2TokenDefinitionIntersection, OAuth2TokenDefinitionIntersection>} A builder for `OAuth2TokenDefinitionIntersection`
   */
  var oAuth2TokenDefinitionBuilder = function (model) {
      return builder(model, buildingFn$18);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OAuth2TokenRequest} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OAuth2TokenRequestIntersection} The built object
   */
  function buildingFn$17(model, options) {
      var instance = new Classes.OAuth2TokenRequest(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OAuth2TokenRequestIntersection`
   * @returns {Builder<OAuth2TokenRequestIntersection, OAuth2TokenRequestIntersection>} A builder for `OAuth2TokenRequestIntersection`
   */
  var oAuth2TokenRequestBuilder = function (model) {
      return builder(model, buildingFn$17);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OneEventConsumptionStrategy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OneEventConsumptionStrategyIntersection} The built object
   */
  function buildingFn$16(model, options) {
      var instance = new Classes.OneEventConsumptionStrategy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OneEventConsumptionStrategyIntersection`
   * @returns {Builder<OneEventConsumptionStrategyIntersection, OneEventConsumptionStrategyIntersection>} A builder for `OneEventConsumptionStrategyIntersection`
   */
  var oneEventConsumptionStrategyBuilder = function (model) {
      return builder(model, buildingFn$16);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OpenAPIArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OpenAPIArgumentsIntersection} The built object
   */
  function buildingFn$15(model, options) {
      var instance = new Classes.OpenAPIArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OpenAPIArgumentsIntersection`
   * @returns {Builder<OpenAPIArgumentsIntersection, OpenAPIArgumentsIntersection>} A builder for `OpenAPIArgumentsIntersection`
   */
  var openAPIArgumentsBuilder = function (model) {
      return builder(model, buildingFn$15);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OpenIdConnectAuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OpenIdConnectAuthenticationPolicyIntersection} The built object
   */
  function buildingFn$14(model, options) {
      var instance = new Classes.OpenIdConnectAuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OpenIdConnectAuthenticationPolicyIntersection`
   * @returns {Builder<OpenIdConnectAuthenticationPolicyIntersection, OpenIdConnectAuthenticationPolicyIntersection>} A builder for `OpenIdConnectAuthenticationPolicyIntersection`
   */
  var openIdConnectAuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$14);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OpenIdConnectAuthenticationPolicyConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OpenIdConnectAuthenticationPolicyConfigurationIntersection} The built object
   */
  function buildingFn$13(model, options) {
      var instance = new Classes.OpenIdConnectAuthenticationPolicyConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OpenIdConnectAuthenticationPolicyConfigurationIntersection`
   * @returns {Builder<OpenIdConnectAuthenticationPolicyConfigurationIntersection, OpenIdConnectAuthenticationPolicyConfigurationIntersection>} A builder for `OpenIdConnectAuthenticationPolicyConfigurationIntersection`
   */
  var openIdConnectAuthenticationPolicyConfigurationBuilder = function (model) {
      return builder(model, buildingFn$13);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OpenIdConnectAuthenticationProperties} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OpenIdConnectAuthenticationPropertiesIntersection} The built object
   */
  function buildingFn$12(model, options) {
      var instance = new Classes.OpenIdConnectAuthenticationProperties(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OpenIdConnectAuthenticationPropertiesIntersection`
   * @returns {Builder<OpenIdConnectAuthenticationPropertiesIntersection, OpenIdConnectAuthenticationPropertiesIntersection>} A builder for `OpenIdConnectAuthenticationPropertiesIntersection`
   */
  var openIdConnectAuthenticationPropertiesBuilder = function (model) {
      return builder(model, buildingFn$12);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Output} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OutputIntersection} The built object
   */
  function buildingFn$11(model, options) {
      var instance = new Classes.Output(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OutputIntersection`
   * @returns {Builder<OutputIntersection, OutputIntersection>} A builder for `OutputIntersection`
   */
  var outputBuilder = function (model) {
      return builder(model, buildingFn$11);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.OutputAs} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {OutputAsIntersection} The built object
   */
  function buildingFn$10(model, options) {
      var instance = new Classes.OutputAs(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `OutputAsIntersection`
   * @returns {Builder<OutputAsIntersection, OutputAsIntersection>} A builder for `OutputAsIntersection`
   */
  var outputAsBuilder = function (model) {
      return builder(model, buildingFn$10);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RaiseTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RaiseTaskIntersection} The built object
   */
  function buildingFn$$(model, options) {
      var instance = new Classes.RaiseTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RaiseTaskIntersection`
   * @returns {Builder<RaiseTaskIntersection, RaiseTaskIntersection>} A builder for `RaiseTaskIntersection`
   */
  var raiseTaskBuilder = function (model) {
      return builder(model, buildingFn$$);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RaiseTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RaiseTaskConfigurationIntersection} The built object
   */
  function buildingFn$_(model, options) {
      var instance = new Classes.RaiseTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RaiseTaskConfigurationIntersection`
   * @returns {Builder<RaiseTaskConfigurationIntersection, RaiseTaskConfigurationIntersection>} A builder for `RaiseTaskConfigurationIntersection`
   */
  var raiseTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$_);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RaiseTaskError} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RaiseTaskErrorIntersection} The built object
   */
  function buildingFn$Z(model, options) {
      var instance = new Classes.RaiseTaskError(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RaiseTaskErrorIntersection`
   * @returns {Builder<RaiseTaskErrorIntersection, RaiseTaskErrorIntersection>} A builder for `RaiseTaskErrorIntersection`
   */
  var raiseTaskErrorBuilder = function (model) {
      return builder(model, buildingFn$Z);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ReferenceableAuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ReferenceableAuthenticationPolicyIntersection} The built object
   */
  function buildingFn$Y(model, options) {
      var instance = new Classes.ReferenceableAuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ReferenceableAuthenticationPolicyIntersection`
   * @returns {Builder<ReferenceableAuthenticationPolicyIntersection, ReferenceableAuthenticationPolicyIntersection>} A builder for `ReferenceableAuthenticationPolicyIntersection`
   */
  var referenceableAuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$Y);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RetryBackoff} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RetryBackoffIntersection} The built object
   */
  function buildingFn$X(model, options) {
      var instance = new Classes.RetryBackoff(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RetryBackoffIntersection`
   * @returns {Builder<RetryBackoffIntersection, RetryBackoffIntersection>} A builder for `RetryBackoffIntersection`
   */
  var retryBackoffBuilder = function (model) {
      return builder(model, buildingFn$X);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RetryLimit} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RetryLimitIntersection} The built object
   */
  function buildingFn$W(model, options) {
      var instance = new Classes.RetryLimit(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RetryLimitIntersection`
   * @returns {Builder<RetryLimitIntersection, RetryLimitIntersection>} A builder for `RetryLimitIntersection`
   */
  var retryLimitBuilder = function (model) {
      return builder(model, buildingFn$W);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RetryLimitAttempt} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RetryLimitAttemptIntersection} The built object
   */
  function buildingFn$V(model, options) {
      var instance = new Classes.RetryLimitAttempt(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RetryLimitAttemptIntersection`
   * @returns {Builder<RetryLimitAttemptIntersection, RetryLimitAttemptIntersection>} A builder for `RetryLimitAttemptIntersection`
   */
  var retryLimitAttemptBuilder = function (model) {
      return builder(model, buildingFn$V);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RetryPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RetryPolicyIntersection} The built object
   */
  function buildingFn$U(model, options) {
      var instance = new Classes.RetryPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RetryPolicyIntersection`
   * @returns {Builder<RetryPolicyIntersection, RetryPolicyIntersection>} A builder for `RetryPolicyIntersection`
   */
  var retryPolicyBuilder = function (model) {
      return builder(model, buildingFn$U);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RetryPolicyJitter} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RetryPolicyJitterIntersection} The built object
   */
  function buildingFn$T(model, options) {
      var instance = new Classes.RetryPolicyJitter(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RetryPolicyJitterIntersection`
   * @returns {Builder<RetryPolicyJitterIntersection, RetryPolicyJitterIntersection>} A builder for `RetryPolicyJitterIntersection`
   */
  var retryPolicyJitterBuilder = function (model) {
      return builder(model, buildingFn$T);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RunContainer} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RunContainerIntersection} The built object
   */
  function buildingFn$S(model, options) {
      var instance = new Classes.RunContainer(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RunContainerIntersection`
   * @returns {Builder<RunContainerIntersection, RunContainerIntersection>} A builder for `RunContainerIntersection`
   */
  var runContainerBuilder = function (model) {
      return builder(model, buildingFn$S);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RunScript} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RunScriptIntersection} The built object
   */
  function buildingFn$R(model, options) {
      var instance = new Classes.RunScript(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RunScriptIntersection`
   * @returns {Builder<RunScriptIntersection, RunScriptIntersection>} A builder for `RunScriptIntersection`
   */
  var runScriptBuilder = function (model) {
      return builder(model, buildingFn$R);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RunShell} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RunShellIntersection} The built object
   */
  function buildingFn$Q(model, options) {
      var instance = new Classes.RunShell(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RunShellIntersection`
   * @returns {Builder<RunShellIntersection, RunShellIntersection>} A builder for `RunShellIntersection`
   */
  var runShellBuilder = function (model) {
      return builder(model, buildingFn$Q);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RunTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RunTaskIntersection} The built object
   */
  function buildingFn$P(model, options) {
      var instance = new Classes.RunTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RunTaskIntersection`
   * @returns {Builder<RunTaskIntersection, RunTaskIntersection>} A builder for `RunTaskIntersection`
   */
  var runTaskBuilder = function (model) {
      return builder(model, buildingFn$P);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RunTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RunTaskConfigurationIntersection} The built object
   */
  function buildingFn$O(model, options) {
      var instance = new Classes.RunTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RunTaskConfigurationIntersection`
   * @returns {Builder<RunTaskConfigurationIntersection, RunTaskConfigurationIntersection>} A builder for `RunTaskConfigurationIntersection`
   */
  var runTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$O);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RuntimeExpression} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RuntimeExpressionIntersection} The built object
   */
  function buildingFn$N(model, options) {
      var instance = new Classes.RuntimeExpression(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RuntimeExpressionIntersection`
   * @returns {Builder<RuntimeExpressionIntersection, RuntimeExpressionIntersection>} A builder for `RuntimeExpressionIntersection`
   */
  var runtimeExpressionBuilder = function (model) {
      return builder(model, buildingFn$N);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.RunWorkflow} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {RunWorkflowIntersection} The built object
   */
  function buildingFn$M(model, options) {
      var instance = new Classes.RunWorkflow(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `RunWorkflowIntersection`
   * @returns {Builder<RunWorkflowIntersection, RunWorkflowIntersection>} A builder for `RunWorkflowIntersection`
   */
  var runWorkflowBuilder = function (model) {
      return builder(model, buildingFn$M);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Schedule} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ScheduleIntersection} The built object
   */
  function buildingFn$L(model, options) {
      var instance = new Classes.Schedule(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ScheduleIntersection`
   * @returns {Builder<ScheduleIntersection, ScheduleIntersection>} A builder for `ScheduleIntersection`
   */
  var scheduleBuilder = function (model) {
      return builder(model, buildingFn$L);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Schema} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SchemaIntersection} The built object
   */
  function buildingFn$K(model, options) {
      var instance = new Classes.Schema(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SchemaIntersection`
   * @returns {Builder<SchemaIntersection, SchemaIntersection>} A builder for `SchemaIntersection`
   */
  var schemaBuilder = function (model) {
      return builder(model, buildingFn$K);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SchemaExternal} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SchemaExternalIntersection} The built object
   */
  function buildingFn$J(model, options) {
      var instance = new Classes.SchemaExternal(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SchemaExternalIntersection`
   * @returns {Builder<SchemaExternalIntersection, SchemaExternalIntersection>} A builder for `SchemaExternalIntersection`
   */
  var schemaExternalBuilder = function (model) {
      return builder(model, buildingFn$J);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SchemaInline} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SchemaInlineIntersection} The built object
   */
  function buildingFn$I(model, options) {
      var instance = new Classes.SchemaInline(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SchemaInlineIntersection`
   * @returns {Builder<SchemaInlineIntersection, SchemaInlineIntersection>} A builder for `SchemaInlineIntersection`
   */
  var schemaInlineBuilder = function (model) {
      return builder(model, buildingFn$I);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Script} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ScriptIntersection} The built object
   */
  function buildingFn$H(model, options) {
      var instance = new Classes.Script(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ScriptIntersection`
   * @returns {Builder<ScriptIntersection, ScriptIntersection>} A builder for `ScriptIntersection`
   */
  var scriptBuilder = function (model) {
      return builder(model, buildingFn$H);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SecretBasedAuthenticationPolicy} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SecretBasedAuthenticationPolicyIntersection} The built object
   */
  function buildingFn$G(model, options) {
      var instance = new Classes.SecretBasedAuthenticationPolicy(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SecretBasedAuthenticationPolicyIntersection`
   * @returns {Builder<SecretBasedAuthenticationPolicyIntersection, SecretBasedAuthenticationPolicyIntersection>} A builder for `SecretBasedAuthenticationPolicyIntersection`
   */
  var secretBasedAuthenticationPolicyBuilder = function (model) {
      return builder(model, buildingFn$G);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SetTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SetTaskIntersection} The built object
   */
  function buildingFn$F(model, options) {
      var instance = new Classes.SetTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SetTaskIntersection`
   * @returns {Builder<SetTaskIntersection, SetTaskIntersection>} A builder for `SetTaskIntersection`
   */
  var setTaskBuilder = function (model) {
      return builder(model, buildingFn$F);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SetTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SetTaskConfigurationIntersection} The built object
   */
  function buildingFn$E(model, options) {
      var instance = new Classes.SetTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SetTaskConfigurationIntersection`
   * @returns {Builder<SetTaskConfigurationIntersection, SetTaskConfigurationIntersection>} A builder for `SetTaskConfigurationIntersection`
   */
  var setTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$E);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Shell} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ShellIntersection} The built object
   */
  function buildingFn$D(model, options) {
      var instance = new Classes.Shell(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ShellIntersection`
   * @returns {Builder<ShellIntersection, ShellIntersection>} A builder for `ShellIntersection`
   */
  var shellBuilder = function (model) {
      return builder(model, buildingFn$D);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ShellArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ShellArgumentsIntersection} The built object
   */
  function buildingFn$C(model, options) {
      var instance = new Classes.ShellArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ShellArgumentsIntersection`
   * @returns {Builder<ShellArgumentsIntersection, ShellArgumentsIntersection>} A builder for `ShellArgumentsIntersection`
   */
  var shellArgumentsBuilder = function (model) {
      return builder(model, buildingFn$C);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.ShellEnvironment} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {ShellEnvironmentIntersection} The built object
   */
  function buildingFn$B(model, options) {
      var instance = new Classes.ShellEnvironment(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `ShellEnvironmentIntersection`
   * @returns {Builder<ShellEnvironmentIntersection, ShellEnvironmentIntersection>} A builder for `ShellEnvironmentIntersection`
   */
  var shellEnvironmentBuilder = function (model) {
      return builder(model, buildingFn$B);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SubflowConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SubflowConfigurationIntersection} The built object
   */
  function buildingFn$A(model, options) {
      var instance = new Classes.SubflowConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SubflowConfigurationIntersection`
   * @returns {Builder<SubflowConfigurationIntersection, SubflowConfigurationIntersection>} A builder for `SubflowConfigurationIntersection`
   */
  var subflowConfigurationBuilder = function (model) {
      return builder(model, buildingFn$A);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SubflowInput} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SubflowInputIntersection} The built object
   */
  function buildingFn$z(model, options) {
      var instance = new Classes.SubflowInput(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SubflowInputIntersection`
   * @returns {Builder<SubflowInputIntersection, SubflowInputIntersection>} A builder for `SubflowInputIntersection`
   */
  var subflowInputBuilder = function (model) {
      return builder(model, buildingFn$z);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SubscriptionIterator} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SubscriptionIteratorIntersection} The built object
   */
  function buildingFn$y(model, options) {
      var instance = new Classes.SubscriptionIterator(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SubscriptionIteratorIntersection`
   * @returns {Builder<SubscriptionIteratorIntersection, SubscriptionIteratorIntersection>} A builder for `SubscriptionIteratorIntersection`
   */
  var subscriptionIteratorBuilder = function (model) {
      return builder(model, buildingFn$y);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SwitchCase} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SwitchCaseIntersection} The built object
   */
  function buildingFn$x(model, options) {
      var instance = new Classes.SwitchCase(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SwitchCaseIntersection`
   * @returns {Builder<SwitchCaseIntersection, SwitchCaseIntersection>} A builder for `SwitchCaseIntersection`
   */
  var switchCaseBuilder = function (model) {
      return builder(model, buildingFn$x);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SwitchItem} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SwitchItemIntersection} The built object
   */
  function buildingFn$w(model, options) {
      var instance = new Classes.SwitchItem(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SwitchItemIntersection`
   * @returns {Builder<SwitchItemIntersection, SwitchItemIntersection>} A builder for `SwitchItemIntersection`
   */
  var switchItemBuilder = function (model) {
      return builder(model, buildingFn$w);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SwitchTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SwitchTaskIntersection} The built object
   */
  function buildingFn$v(model, options) {
      var instance = new Classes.SwitchTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SwitchTaskIntersection`
   * @returns {Builder<SwitchTaskIntersection, SwitchTaskIntersection>} A builder for `SwitchTaskIntersection`
   */
  var switchTaskBuilder = function (model) {
      return builder(model, buildingFn$v);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.SwitchTaskConfiguration} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {SwitchTaskConfigurationIntersection} The built object
   */
  function buildingFn$u(model, options) {
      var instance = new Classes.SwitchTaskConfiguration(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `SwitchTaskConfigurationIntersection`
   * @returns {Builder<SwitchTaskConfigurationIntersection, SwitchTaskConfigurationIntersection>} A builder for `SwitchTaskConfigurationIntersection`
   */
  var switchTaskConfigurationBuilder = function (model) {
      return builder(model, buildingFn$u);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Task} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TaskIntersection} The built object
   */
  function buildingFn$t(model, options) {
      var instance = new Classes.Task(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskIntersection`
   * @returns {Builder<TaskIntersection, TaskIntersection>} A builder for `TaskIntersection`
   */
  var taskBuilder = function (model) {
      return builder(model, buildingFn$t);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TaskBase} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TaskBaseIntersection} The built object
   */
  function buildingFn$s(model, options) {
      var instance = new Classes.TaskBase(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskBaseIntersection`
   * @returns {Builder<TaskBaseIntersection, TaskBaseIntersection>} A builder for `TaskBaseIntersection`
   */
  var taskBaseBuilder = function (model) {
      return builder(model, buildingFn$s);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TaskBaseIf} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TaskBaseIfIntersection} The built object
   */
  function buildingFn$r(model, options) {
      var instance = new Classes.TaskBaseIf(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskBaseIfIntersection`
   * @returns {Builder<TaskBaseIfIntersection, TaskBaseIfIntersection>} A builder for `TaskBaseIfIntersection`
   */
  var taskBaseIfBuilder = function (model) {
      return builder(model, buildingFn$r);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TaskItem} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TaskItemIntersection} The built object
   */
  function buildingFn$q(model, options) {
      var instance = new Classes.TaskItem(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskItemIntersection`
   * @returns {Builder<TaskItemIntersection, TaskItemIntersection>} A builder for `TaskItemIntersection`
   */
  var taskItemBuilder = function (model) {
      return builder(model, buildingFn$q);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.TaskList} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {TaskListIntersection} The built array
   */
  function buildingFn$p(model, options) {
      var instance = new Classes.TaskList(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskListIntersection`
   * @returns {ArrayBuilder<Specification.TaskItem, TaskListIntersection>} A builder for `TaskListIntersection`
   */
  var taskListBuilder = function (model) {
      return arrayBuilder(model, buildingFn$p);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TaskMetadata} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TaskMetadataIntersection} The built object
   */
  function buildingFn$o(model, options) {
      var instance = new Classes.TaskMetadata(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskMetadataIntersection`
   * @returns {Builder<TaskMetadataIntersection, TaskMetadataIntersection>} A builder for `TaskMetadataIntersection`
   */
  var taskMetadataBuilder = function (model) {
      return builder(model, buildingFn$o);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TaskTimeout} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TaskTimeoutIntersection} The built object
   */
  function buildingFn$n(model, options) {
      var instance = new Classes.TaskTimeout(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TaskTimeoutIntersection`
   * @returns {Builder<TaskTimeoutIntersection, TaskTimeoutIntersection>} A builder for `TaskTimeoutIntersection`
   */
  var taskTimeoutBuilder = function (model) {
      return builder(model, buildingFn$n);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Timeout} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TimeoutIntersection} The built object
   */
  function buildingFn$m(model, options) {
      var instance = new Classes.Timeout(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TimeoutIntersection`
   * @returns {Builder<TimeoutIntersection, TimeoutIntersection>} A builder for `TimeoutIntersection`
   */
  var timeoutBuilder = function (model) {
      return builder(model, buildingFn$m);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TryTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TryTaskIntersection} The built object
   */
  function buildingFn$l(model, options) {
      var instance = new Classes.TryTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TryTaskIntersection`
   * @returns {Builder<TryTaskIntersection, TryTaskIntersection>} A builder for `TryTaskIntersection`
   */
  var tryTaskBuilder = function (model) {
      return builder(model, buildingFn$l);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TryTaskCatch} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TryTaskCatchIntersection} The built object
   */
  function buildingFn$k(model, options) {
      var instance = new Classes.TryTaskCatch(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TryTaskCatchIntersection`
   * @returns {Builder<TryTaskCatchIntersection, TryTaskCatchIntersection>} A builder for `TryTaskCatchIntersection`
   */
  var tryTaskCatchBuilder = function (model) {
      return builder(model, buildingFn$k);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.TryTaskCatchRetry} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {TryTaskCatchRetryIntersection} The built object
   */
  function buildingFn$j(model, options) {
      var instance = new Classes.TryTaskCatchRetry(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `TryTaskCatchRetryIntersection`
   * @returns {Builder<TryTaskCatchRetryIntersection, TryTaskCatchRetryIntersection>} A builder for `TryTaskCatchRetryIntersection`
   */
  var tryTaskCatchRetryBuilder = function (model) {
      return builder(model, buildingFn$j);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UriTemplate} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UriTemplateIntersection} The built object
   */
  function buildingFn$i(model, options) {
      var instance = new Classes.UriTemplate(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UriTemplateIntersection`
   * @returns {Builder<UriTemplateIntersection, UriTemplateIntersection>} A builder for `UriTemplateIntersection`
   */
  var uriTemplateBuilder = function (model) {
      return builder(model, buildingFn$i);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Use} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseIntersection} The built object
   */
  function buildingFn$h(model, options) {
      var instance = new Classes.Use(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseIntersection`
   * @returns {Builder<UseIntersection, UseIntersection>} A builder for `UseIntersection`
   */
  var useBuilder = function (model) {
      return builder(model, buildingFn$h);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UseAuthentications} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseAuthenticationsIntersection} The built object
   */
  function buildingFn$g(model, options) {
      var instance = new Classes.UseAuthentications(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseAuthenticationsIntersection`
   * @returns {Builder<UseAuthenticationsIntersection, UseAuthenticationsIntersection>} A builder for `UseAuthenticationsIntersection`
   */
  var useAuthenticationsBuilder = function (model) {
      return builder(model, buildingFn$g);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UseCatalogs} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseCatalogsIntersection} The built object
   */
  function buildingFn$f(model, options) {
      var instance = new Classes.UseCatalogs(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseCatalogsIntersection`
   * @returns {Builder<UseCatalogsIntersection, UseCatalogsIntersection>} A builder for `UseCatalogsIntersection`
   */
  var useCatalogsBuilder = function (model) {
      return builder(model, buildingFn$f);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UseErrors} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseErrorsIntersection} The built object
   */
  function buildingFn$e(model, options) {
      var instance = new Classes.UseErrors(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseErrorsIntersection`
   * @returns {Builder<UseErrorsIntersection, UseErrorsIntersection>} A builder for `UseErrorsIntersection`
   */
  var useErrorsBuilder = function (model) {
      return builder(model, buildingFn$e);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.UseExtensions} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {UseExtensionsIntersection} The built array
   */
  function buildingFn$d(model, options) {
      var instance = new Classes.UseExtensions(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseExtensionsIntersection`
   * @returns {ArrayBuilder<Specification.ExtensionItem, UseExtensionsIntersection>} A builder for `UseExtensionsIntersection`
   */
  var useExtensionsBuilder = function (model) {
      return arrayBuilder(model, buildingFn$d);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UseFunctions} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseFunctionsIntersection} The built object
   */
  function buildingFn$c(model, options) {
      var instance = new Classes.UseFunctions(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseFunctionsIntersection`
   * @returns {Builder<UseFunctionsIntersection, UseFunctionsIntersection>} A builder for `UseFunctionsIntersection`
   */
  var useFunctionsBuilder = function (model) {
      return builder(model, buildingFn$c);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UseRetries} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseRetriesIntersection} The built object
   */
  function buildingFn$b(model, options) {
      var instance = new Classes.UseRetries(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseRetriesIntersection`
   * @returns {Builder<UseRetriesIntersection, UseRetriesIntersection>} A builder for `UseRetriesIntersection`
   */
  var useRetriesBuilder = function (model) {
      return builder(model, buildingFn$b);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying array
   * @param {Specification.UseSecrets} model The proxied array
   * @param {BuildOptions} options The build options to use
   * @returns {UseSecretsIntersection} The built array
   */
  function buildingFn$a(model, options) {
      var instance = new Classes.UseSecrets(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseSecretsIntersection`
   * @returns {ArrayBuilder<string, UseSecretsIntersection>} A builder for `UseSecretsIntersection`
   */
  var useSecretsBuilder = function (model) {
      return arrayBuilder(model, buildingFn$a);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.UseTimeouts} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {UseTimeoutsIntersection} The built object
   */
  function buildingFn$9(model, options) {
      var instance = new Classes.UseTimeouts(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `UseTimeoutsIntersection`
   * @returns {Builder<UseTimeoutsIntersection, UseTimeoutsIntersection>} A builder for `UseTimeoutsIntersection`
   */
  var useTimeoutsBuilder = function (model) {
      return builder(model, buildingFn$9);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WaitTask} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WaitTaskIntersection} The built object
   */
  function buildingFn$8(model, options) {
      var instance = new Classes.WaitTask(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WaitTaskIntersection`
   * @returns {Builder<WaitTaskIntersection, WaitTaskIntersection>} A builder for `WaitTaskIntersection`
   */
  var waitTaskBuilder = function (model) {
      return builder(model, buildingFn$8);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WithEvent} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WithEventIntersection} The built object
   */
  function buildingFn$7(model, options) {
      var instance = new Classes.WithEvent(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WithEventIntersection`
   * @returns {Builder<WithEventIntersection, WithEventIntersection>} A builder for `WithEventIntersection`
   */
  var withEventBuilder = function (model) {
      return builder(model, buildingFn$7);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WithGRPCArguments} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WithGRPCArgumentsIntersection} The built object
   */
  function buildingFn$6(model, options) {
      var instance = new Classes.WithGRPCArguments(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WithGRPCArgumentsIntersection`
   * @returns {Builder<WithGRPCArgumentsIntersection, WithGRPCArgumentsIntersection>} A builder for `WithGRPCArgumentsIntersection`
   */
  var withGRPCArgumentsBuilder = function (model) {
      return builder(model, buildingFn$6);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WithGRPCService} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WithGRPCServiceIntersection} The built object
   */
  function buildingFn$5(model, options) {
      var instance = new Classes.WithGRPCService(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WithGRPCServiceIntersection`
   * @returns {Builder<WithGRPCServiceIntersection, WithGRPCServiceIntersection>} A builder for `WithGRPCServiceIntersection`
   */
  var withGRPCServiceBuilder = function (model) {
      return builder(model, buildingFn$5);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WithOpenAPIParameters} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WithOpenAPIParametersIntersection} The built object
   */
  function buildingFn$4(model, options) {
      var instance = new Classes.WithOpenAPIParameters(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WithOpenAPIParametersIntersection`
   * @returns {Builder<WithOpenAPIParametersIntersection, WithOpenAPIParametersIntersection>} A builder for `WithOpenAPIParametersIntersection`
   */
  var withOpenAPIParametersBuilder = function (model) {
      return builder(model, buildingFn$4);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.Workflow} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WorkflowIntersection} The built object
   */
  function buildingFn$3(model, options) {
      var instance = new Classes.Workflow(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WorkflowIntersection`
   * @returns {Builder<WorkflowIntersection, WorkflowIntersection>} A builder for `WorkflowIntersection`
   */
  var workflowBuilder = function (model) {
      return builder(model, buildingFn$3);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WorkflowMetadata} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WorkflowMetadataIntersection} The built object
   */
  function buildingFn$2(model, options) {
      var instance = new Classes.WorkflowMetadata(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WorkflowMetadataIntersection`
   * @returns {Builder<WorkflowMetadataIntersection, WorkflowMetadataIntersection>} A builder for `WorkflowMetadataIntersection`
   */
  var workflowMetadataBuilder = function (model) {
      return builder(model, buildingFn$2);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WorkflowTags} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WorkflowTagsIntersection} The built object
   */
  function buildingFn$1(model, options) {
      var instance = new Classes.WorkflowTags(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WorkflowTagsIntersection`
   * @returns {Builder<WorkflowTagsIntersection, WorkflowTagsIntersection>} A builder for `WorkflowTagsIntersection`
   */
  var workflowTagsBuilder = function (model) {
      return builder(model, buildingFn$1);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */
  /**
   * The internal function used by the builder proxy to validate and return its underlying object
   * @param {Specification.WorkflowTimeout} model The proxied object
   * @param {BuildOptions} options The build options to use
   * @returns {WorkflowTimeoutIntersection} The built object
   */
  function buildingFn(model, options) {
      var instance = new Classes.WorkflowTimeout(model);
      if (options.validate)
          instance.validate();
      return (options.normalize ? instance.normalize() : instance);
  }
  /**
   * A factory to create a builder proxy for the type `WorkflowTimeoutIntersection`
   * @returns {Builder<WorkflowTimeoutIntersection, WorkflowTimeoutIntersection>} A builder for `WorkflowTimeoutIntersection`
   */
  var workflowTimeoutBuilder = function (model) {
      return builder(model, buildingFn);
  };

  /*
   * Copyright 2021-Present The Serverless Workflow Specification Authors
   *
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   */

  var specification = /*#__PURE__*/Object.freeze({
    __proto__: null
  });

  exports.Classes = Classes;
  exports.MermaidDiagram = MermaidDiagram;
  exports.Specification = specification;
  exports.allEventConsumptionStrategyBuilder = allEventConsumptionStrategyBuilder;
  exports.allEventConsumptionStrategyConfigurationBuilder = allEventConsumptionStrategyConfigurationBuilder;
  exports.anyEventConsumptionStrategyBuilder = anyEventConsumptionStrategyBuilder;
  exports.anyEventConsumptionStrategyConfigurationBuilder = anyEventConsumptionStrategyConfigurationBuilder;
  exports.anyEventConsumptionStrategyUntilBuilder = anyEventConsumptionStrategyUntilBuilder;
  exports.anyEventUntilConsumedBuilder = anyEventUntilConsumedBuilder;
  exports.asyncApiArgumentsBuilder = asyncApiArgumentsBuilder;
  exports.authenticationPolicyBuilder = authenticationPolicyBuilder;
  exports.authenticationPolicyReferenceBuilder = authenticationPolicyReferenceBuilder;
  exports.basicAuthenticationPolicyBuilder = basicAuthenticationPolicyBuilder;
  exports.basicAuthenticationPolicyConfigurationBuilder = basicAuthenticationPolicyConfigurationBuilder;
  exports.basicAuthenticationPropertiesBuilder = basicAuthenticationPropertiesBuilder;
  exports.bearerAuthenticationPolicyBuilder = bearerAuthenticationPolicyBuilder;
  exports.bearerAuthenticationPolicyConfigurationBuilder = bearerAuthenticationPolicyConfigurationBuilder;
  exports.bearerAuthenticationPropertiesBuilder = bearerAuthenticationPropertiesBuilder;
  exports.buildGraph = buildGraph;
  exports.callAsyncAPIBuilder = callAsyncAPIBuilder;
  exports.callFunctionBuilder = callFunctionBuilder;
  exports.callGRPCBuilder = callGRPCBuilder;
  exports.callHTTPBuilder = callHTTPBuilder;
  exports.callOpenAPIBuilder = callOpenAPIBuilder;
  exports.callTaskBuilder = callTaskBuilder;
  exports.catalogBuilder = catalogBuilder;
  exports.catchErrorsBuilder = catchErrorsBuilder;
  exports.constantBackoffBuilder = constantBackoffBuilder;
  exports.containerBuilder = containerBuilder;
  exports.containerEnvironmentBuilder = containerEnvironmentBuilder;
  exports.containerLifetimeBuilder = containerLifetimeBuilder;
  exports.containerPortsBuilder = containerPortsBuilder;
  exports.containerVolumesBuilder = containerVolumesBuilder;
  exports.convertToMermaidCode = convertToMermaidCode;
  exports.digestAuthenticationPolicyBuilder = digestAuthenticationPolicyBuilder;
  exports.digestAuthenticationPolicyConfigurationBuilder = digestAuthenticationPolicyConfigurationBuilder;
  exports.digestAuthenticationPropertiesBuilder = digestAuthenticationPropertiesBuilder;
  exports.doTaskBuilder = doTaskBuilder;
  exports.documentBuilder = documentBuilder;
  exports.durationBuilder = durationBuilder;
  exports.durationInlineBuilder = durationInlineBuilder;
  exports.emitEventDefinitionBuilder = emitEventDefinitionBuilder;
  exports.emitEventWithBuilder = emitEventWithBuilder;
  exports.emitTaskBuilder = emitTaskBuilder;
  exports.emitTaskConfigurationBuilder = emitTaskConfigurationBuilder;
  exports.endpointBuilder = endpointBuilder;
  exports.endpointConfigurationBuilder = endpointConfigurationBuilder;
  exports.endpointUriBuilder = endpointUriBuilder;
  exports.errorBuilder = errorBuilder;
  exports.errorFilterBuilder = errorFilterBuilder;
  exports.errorInstanceBuilder = errorInstanceBuilder;
  exports.errorTypeBuilder = errorTypeBuilder;
  exports.eventConsumptionStrategyBuilder = eventConsumptionStrategyBuilder;
  exports.eventDataBuilder = eventDataBuilder;
  exports.eventDataschemaBuilder = eventDataschemaBuilder;
  exports.eventFilterBuilder = eventFilterBuilder;
  exports.eventFilterCorrelateBuilder = eventFilterCorrelateBuilder;
  exports.eventSourceBuilder = eventSourceBuilder;
  exports.eventTimeBuilder = eventTimeBuilder;
  exports.exponentialBackOffBuilder = exponentialBackOffBuilder;
  exports.exportAsBuilder = exportAsBuilder;
  exports.exportBuilder = exportBuilder;
  exports.extensionBuilder = extensionBuilder;
  exports.extensionItemBuilder = extensionItemBuilder;
  exports.externalResourceBuilder = externalResourceBuilder;
  exports.externalScriptBuilder = externalScriptBuilder;
  exports.flowDirectiveBuilder = flowDirectiveBuilder;
  exports.forTaskBuilder = forTaskBuilder;
  exports.forTaskConfigurationBuilder = forTaskConfigurationBuilder;
  exports.forkTaskBuilder = forkTaskBuilder;
  exports.forkTaskConfigurationBuilder = forkTaskConfigurationBuilder;
  exports.functionArgumentsBuilder = functionArgumentsBuilder;
  exports.gRPCArgumentsBuilder = gRPCArgumentsBuilder;
  exports.hTTPArgumentsBuilder = hTTPArgumentsBuilder;
  exports.hTTPBodyBuilder = hTTPBodyBuilder;
  exports.hTTPHeadersBuilder = hTTPHeadersBuilder;
  exports.hTTPQueryBuilder = hTTPQueryBuilder;
  exports.inlineScriptBuilder = inlineScriptBuilder;
  exports.inputBuilder = inputBuilder;
  exports.inputFromBuilder = inputFromBuilder;
  exports.linearBackoffBuilder = linearBackoffBuilder;
  exports.listenTaskBuilder = listenTaskBuilder;
  exports.listenTaskConfigurationBuilder = listenTaskConfigurationBuilder;
  exports.oAuth2AutenthicationDataAudiencesBuilder = oAuth2AutenthicationDataAudiencesBuilder;
  exports.oAuth2AutenthicationDataBuilder = oAuth2AutenthicationDataBuilder;
  exports.oAuth2AutenthicationDataClientBuilder = oAuth2AutenthicationDataClientBuilder;
  exports.oAuth2AutenthicationDataScopesBuilder = oAuth2AutenthicationDataScopesBuilder;
  exports.oAuth2AuthenticationPolicyBuilder = oAuth2AuthenticationPolicyBuilder;
  exports.oAuth2AuthenticationPolicyConfigurationBuilder = oAuth2AuthenticationPolicyConfigurationBuilder;
  exports.oAuth2AuthenticationPropertiesEndpointsBuilder = oAuth2AuthenticationPropertiesEndpointsBuilder;
  exports.oAuth2ConnectAuthenticationPropertiesBuilder = oAuth2ConnectAuthenticationPropertiesBuilder;
  exports.oAuth2IssuersBuilder = oAuth2IssuersBuilder;
  exports.oAuth2TokenDefinitionBuilder = oAuth2TokenDefinitionBuilder;
  exports.oAuth2TokenRequestBuilder = oAuth2TokenRequestBuilder;
  exports.oneEventConsumptionStrategyBuilder = oneEventConsumptionStrategyBuilder;
  exports.openAPIArgumentsBuilder = openAPIArgumentsBuilder;
  exports.openIdConnectAuthenticationPolicyBuilder = openIdConnectAuthenticationPolicyBuilder;
  exports.openIdConnectAuthenticationPolicyConfigurationBuilder = openIdConnectAuthenticationPolicyConfigurationBuilder;
  exports.openIdConnectAuthenticationPropertiesBuilder = openIdConnectAuthenticationPropertiesBuilder;
  exports.outputAsBuilder = outputAsBuilder;
  exports.outputBuilder = outputBuilder;
  exports.raiseTaskBuilder = raiseTaskBuilder;
  exports.raiseTaskConfigurationBuilder = raiseTaskConfigurationBuilder;
  exports.raiseTaskErrorBuilder = raiseTaskErrorBuilder;
  exports.referenceableAuthenticationPolicyBuilder = referenceableAuthenticationPolicyBuilder;
  exports.retryBackoffBuilder = retryBackoffBuilder;
  exports.retryLimitAttemptBuilder = retryLimitAttemptBuilder;
  exports.retryLimitBuilder = retryLimitBuilder;
  exports.retryPolicyBuilder = retryPolicyBuilder;
  exports.retryPolicyJitterBuilder = retryPolicyJitterBuilder;
  exports.runContainerBuilder = runContainerBuilder;
  exports.runScriptBuilder = runScriptBuilder;
  exports.runShellBuilder = runShellBuilder;
  exports.runTaskBuilder = runTaskBuilder;
  exports.runTaskConfigurationBuilder = runTaskConfigurationBuilder;
  exports.runWorkflowBuilder = runWorkflowBuilder;
  exports.runtimeExpressionBuilder = runtimeExpressionBuilder;
  exports.scheduleBuilder = scheduleBuilder;
  exports.schemaBuilder = schemaBuilder;
  exports.schemaExternalBuilder = schemaExternalBuilder;
  exports.schemaInlineBuilder = schemaInlineBuilder;
  exports.scriptBuilder = scriptBuilder;
  exports.secretBasedAuthenticationPolicyBuilder = secretBasedAuthenticationPolicyBuilder;
  exports.setTaskBuilder = setTaskBuilder;
  exports.setTaskConfigurationBuilder = setTaskConfigurationBuilder;
  exports.shellArgumentsBuilder = shellArgumentsBuilder;
  exports.shellBuilder = shellBuilder;
  exports.shellEnvironmentBuilder = shellEnvironmentBuilder;
  exports.subflowConfigurationBuilder = subflowConfigurationBuilder;
  exports.subflowInputBuilder = subflowInputBuilder;
  exports.subscriptionIteratorBuilder = subscriptionIteratorBuilder;
  exports.switchCaseBuilder = switchCaseBuilder;
  exports.switchItemBuilder = switchItemBuilder;
  exports.switchTaskBuilder = switchTaskBuilder;
  exports.switchTaskConfigurationBuilder = switchTaskConfigurationBuilder;
  exports.taskBaseBuilder = taskBaseBuilder;
  exports.taskBaseIfBuilder = taskBaseIfBuilder;
  exports.taskBuilder = taskBuilder;
  exports.taskItemBuilder = taskItemBuilder;
  exports.taskListBuilder = taskListBuilder;
  exports.taskMetadataBuilder = taskMetadataBuilder;
  exports.taskTimeoutBuilder = taskTimeoutBuilder;
  exports.timeoutBuilder = timeoutBuilder;
  exports.tryTaskBuilder = tryTaskBuilder;
  exports.tryTaskCatchBuilder = tryTaskCatchBuilder;
  exports.tryTaskCatchRetryBuilder = tryTaskCatchRetryBuilder;
  exports.uriTemplateBuilder = uriTemplateBuilder;
  exports.useAuthenticationsBuilder = useAuthenticationsBuilder;
  exports.useBuilder = useBuilder;
  exports.useCatalogsBuilder = useCatalogsBuilder;
  exports.useErrorsBuilder = useErrorsBuilder;
  exports.useExtensionsBuilder = useExtensionsBuilder;
  exports.useFunctionsBuilder = useFunctionsBuilder;
  exports.useRetriesBuilder = useRetriesBuilder;
  exports.useSecretsBuilder = useSecretsBuilder;
  exports.useTimeoutsBuilder = useTimeoutsBuilder;
  exports.validate = validate;
  exports.waitTaskBuilder = waitTaskBuilder;
  exports.withEventBuilder = withEventBuilder;
  exports.withGRPCArgumentsBuilder = withGRPCArgumentsBuilder;
  exports.withGRPCServiceBuilder = withGRPCServiceBuilder;
  exports.withOpenAPIParametersBuilder = withOpenAPIParametersBuilder;
  exports.workflowBuilder = workflowBuilder;
  exports.workflowMetadataBuilder = workflowMetadataBuilder;
  exports.workflowTagsBuilder = workflowTagsBuilder;
  exports.workflowTimeoutBuilder = workflowTimeoutBuilder;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
